/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"
#include "qdutil.h"

extern "C" QplugReturnStatus QarDIVIDE  ( QplugArgList *);

QplugReturnStatus QarDIVIDE  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;

// pointers for all possible input types

QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var out_ptr;
QdObject_var inv_ptr;

char * attr_ptr;
char * string_buf;
char * out_name;

status = QPLUG_SUCCESS;


//	Unpack the input data objects

 in1_ptr = (*call_list)[0];

 in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("Ratio");
    SetSlotText(0,out_name);
 }
  
// Test validity of inputs...

qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)   
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  } 

// trap divide by vector or array

 if( !QuIsScalar(in2_ptr) ) {
    QplugAppendTextDisplay (
        "Arithmetic>> Only divide by scalars or scalar sequences allowed\n");
    return QPLUG_FAILURE; 
 }
 
// do arithmetic...
  try{
    out_ptr = in1_ptr / in2_ptr;
  }
  catch(Exception &e){
    QplugAppendTextDisplay (e.msg().c_str());
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE;
  }
  
  if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
  }

 // Add timeline: Either timeline is safe if it exists
 
 QdTimeSeq_var tt = get_timetags(in1_ptr);
 if (tt.is_nil()) tt = get_timetags(in2_ptr);
 if (!tt.is_nil()) set_timetags(out_ptr, tt);


 // Set QSAS essential attriutes
 
  // find inverses first
  
  // SI_conversion
    attr_ptr = QarSIConv_inverse(in2_ptr);
    inv_ptr = in1_ptr->create(); // this is empty as we only need metadata
    
    if(attr_ptr != NULL){
       QuSetTxtAttr("SI_conversion", attr_ptr, inv_ptr);
       delete[] attr_ptr;
    }
 
 
 // UNITS
 
   attr_ptr = QuGetAttrText("UNITS", in2_ptr);
   if( strcmp(attr_ptr, "") != 0){
     string_buf = (char *) malloc(strlen(attr_ptr)+6);
     strcpy(string_buf,"(");
     strcat(string_buf, attr_ptr);
     strcat(string_buf,")^-1");
     QuSetTxtAttr("UNITS", string_buf, inv_ptr);
     free (string_buf);
   }
 
 // use inverse for attributes 
 
 // SI_conversion
 
 if ( (attr_ptr = QarSIConv_product(in1_ptr, inv_ptr)) != NULL) {
   QuSetTxtAttr("SI_conversion", attr_ptr, out_ptr);
   delete[] attr_ptr;
 }

 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame (must come from first object for divide)

 attr_ptr = QuGetAttrText("Frame", in1_ptr);
   QuSetTxtAttr("Frame", attr_ptr, out_ptr);

 
 // UNITS
 
 if ( (attr_ptr = QarUNITS_product(in1_ptr, inv_ptr)) != NULL) {
   QuSetTxtAttr("UNITS", attr_ptr, out_ptr);
   delete[] attr_ptr;
 } 
 

 // DEPEND_i etc.
 
 bool OK = copyDimXrefs(in1_ptr, out_ptr);
 if (OK) QplugAppendTextDisplay ("Dimensional Xrefs copied to new object\n");
 
  //	..add  the output series to the list

  call_list->push_back(out_ptr);


  QplugAppendTextDisplay ("Divide: Done\n");
  
  
return status;

}

