/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"


int  QarTestConformal(QdObject_var in1_ptr, 
                      QdObject_var in2_ptr){


 if(in1_ptr.is_nil() || in2_ptr.is_nil() ) return QAR_ERR;
 
 
// if one item is constant don't need to join

 
 if( in1_ptr->sequence_size() != 1 && in2_ptr->sequence_size() !=1 ){
   
    if (QarTestJoined(in1_ptr, in2_ptr) != QAR_OK)   return QAR_ERR;
   
 }
 // only get here if operation is possible
 
  return QAR_OK;

} // end QarTestConformal

// ***************************************************

int  QarTestConformalForVecProds(QdObject_var in1_ptr, 
                                 QdObject_var in2_ptr){
                                 
 
  // NOTE that test on joined TS is already done in TestConformal
   
  if (!QuIsThreeVector(in1_ptr) || !QuIsThreeVector(in2_ptr)) {
      return QAR_ERR;
  }          

   // only get here if operation is possible
   return QarAreFramesSame(in1_ptr, in2_ptr) ;
                     
} // end QarTestConformalForVecProds


// *************************************************************************

int QarTestJoined (QdObject_var do1, 
                   QdObject_var do2){

  if(do1->sequence_size() != do2->sequence_size()) return QAR_ERR;

  // both are sequences so must be joined if they have time tags
 
  QdTimeSeq_var tt1 =  get_timetags(do1);
  QdTimeSeq_var tt2 =  get_timetags(do2);
 
  // if one is not time sequence only care about length
 
  if(tt1.is_nil() || tt2.is_nil()) return QAR_OK;
  
  if( (*tt1) == (*tt2) ) return QAR_OK;
  
  if (tt1->sequence_size() != tt2->sequence_size()) return QAR_ERR;
  
  for(int i=0; i<tt1->sequence_size(); i++){
    // allow for 1 msec jitter in double rep for epoch
    if( fabs(tt1[i].get_time_rep() - tt2[i].get_time_rep()) > .0015 ) return QAR_ERR;
  }
  
  return QAR_OK;

} // end QarTestJoined


// *************************************************************************

int QarAreFramesSame(QdObject_var ptr1, 
                     QdObject_var ptr2){

 char * F1, * F2;
 const char * attr_ptr;

  attr_ptr = QuGetFrameAttr(ptr1);  // this gets CSDS or CAA frame info
       
  if ( strcmp(attr_ptr, "unknown>na") == 0 ) return QAR_WARN; // no frame found
  
  F1 = QplugNewSTR(QuToUpper(attr_ptr));
  
  attr_ptr = QuGetFrameAttr(ptr2);
  if ( strcmp(attr_ptr, "unknown>na") == 0 ) {
    free (F1);
    return QAR_WARN;
  }
  F2 = QplugNewSTR(QuToUpper(attr_ptr));
    
  if(strcmp(F1, F2) != 0) {
    free (F1);
    free (F2);
    return QAR_ERR;
  }
  
  free (F1);
  free (F2);
  return QAR_OK;
  
} // end QarAreFramesSame

// *************************************************************************

bool QarSameBaseSI(const char *SIc1, const char *SIc2){
 std::string base_str1;
 std::string base_str2;
 char * ptr1, * ptr2;
 
  base_str1 = std::string(QunitGetBaseSI(SIc1)); 
           
  base_str2 = std::string(QunitGetBaseSI(SIc2));
  ptr1 = strstr(base_str1.c_str(), ">");
  ptr2 = strstr(base_str2.c_str(), ">");

  if(ptr1 == NULL || ptr2 == NULL || strcmp(ptr1, ptr2) !=0 ) return false;
  
  return true;
  
}

int QarAreUnitsSame(QdObject_var ptr1, 
                    QdObject_var ptr2){

 char * SIc1, *SIc2;
 char * SI_ptr;
 std::string base_str1;
 std::string base_str2;

 int status;

  status = QAR_OK;
  
  SIc1 = QuGetAttrText("SI_conversion", ptr1);
  if (strcmp(SIc1, "") == 0)  status = QAR_WARN;

  SIc2 = QuGetAttrText("SI_conversion", ptr2);
  if (strcmp(SIc2, "") == 0)  status = QAR_WARN;

  if(status != QAR_WARN){
    base_str1 = std::string(QunitGetBaseSI(SIc1));  
    base_str2 = std::string(QunitGetBaseSI(SIc2));

    if(base_str1 != base_str2) status = QAR_ERR;
  
  }
   
  return status;
  
} // end QarAreUnitsSame


// *************************************************************************

double QarConvFactor(const char * SIc){

  char * SIcopy = QplugNewSTR(SIc);
  char * ptr;
  double value=0.0;
  ptr = strstr(SIcopy, ">");
  if (ptr != NULL){
    ptr[0] = '\0';
    value = (double) Qar_atof(SIcopy);
  }
  
  free(SIcopy);
  return value;
  
}

double Qar_atof(const char * vtxt){

  double value=0.0;
  char *ptr = strstr(vtxt, "10^");
  if (ptr != NULL){
    double power = atof(&(ptr[3]));
    ptr[0] = '\0'; 
    value = (double) atof(vtxt);
    if(value < 1e-13) value = 1.0; // if vtxt of form "10^6" value is zero since there is no mantissa
    value *= pow(10., power);
  }
  else
      value = (double) atof(vtxt);

  return value;
  
}



// *************************************************************************

QdObject_var QarConvertUnitsTo(QdObject_var ptr, const char * SIcTarget, const char* newUnits){

 std::string SIc;
 char * SI_ptr;
 QdObject_var newObj;
 double conversion;
 double convTarget;
 
 QdObject_var xref;
 char temp_string[30];
 double val_double;
 QString val_string;
 
  SIc = string(QuGetAttrText("SI_conversion", ptr));
  if (strcmp(SIc.c_str(), "") == 0)  return newObj;  // nil object return
  if( !QarSameBaseSI(SIc.c_str(), SIcTarget) ) {
    QuMsgBar("input object not in same units as ", SIcTarget, true);
    return newObj;
  }

  convTarget = QarConvFactor(SIcTarget);
  
   if( convTarget < 1.0e-60 ) {
    QuMsgBar("target units too small ", SIcTarget, true);
    return newObj;
  }
 
  // Conversion factor
  conversion = QarConvFactor(SIc.c_str());
  conversion  = conversion / convTarget;
  QdRScalar_var conv_var = new QdRScalar(conversion);
  newObj = ptr * conv_var;

  newObj->copy_xrefs_from(ptr);
  newObj->change_xref("SI_conversion", new QdString(SIcTarget));
  
  // Now convert the parameters and xrefs that have to be converted
  try
  {
      xref=newObj->get_xref("DELTA_PLUS");
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref("DELTA_PLUS", xref);
  }
  catch(Exception& e)
  {
      val_string=QString(QuGetAttrText("DELTA_PLUS", newObj));
      if(!val_string.isEmpty())
      {
          val_double=val_string.toDouble();
          val_double*=conversion;
          sprintf(temp_string, "%e", val_double);
          QuSetTxtAttr("DELTA_PLUS", temp_string, newObj);
      }
  }
  
  try
  {
      xref=newObj->get_xref("DELTA_MINUS");
      xref=QarConvertUnitsTo(xref, SIcTarget, newUnits);
      newObj->change_xref("DELTA_MINUS", xref);
  }
  catch(Exception& e)
  {
      val_string=QString(QuGetAttrText("DELTA_MINUS", newObj));
      if(!val_string.isEmpty())
      {
          val_double=val_string.toDouble();
          val_double*=conversion;
          sprintf(temp_string, "%e", val_double);
          QuSetTxtAttr("DELTA_MINUS", temp_string, newObj);
      }
  }
  
  val_string=QString(QuGetAttrText("SCALEMIN", newObj));
  if(!val_string.isEmpty())
  {
    val_double=val_string.toDouble();
    val_double*=conversion;
    sprintf(temp_string, "%e", val_double);
    QuSetTxtAttr("SCALEMIN", temp_string, newObj);
  }
  
  val_string=QString(QuGetAttrText("SCALEMAX", newObj));
  if(!val_string.isEmpty())
  {
      val_double=val_string.toDouble();
      val_double*=conversion;
      sprintf(temp_string, "%e", val_double);
      QuSetTxtAttr("SCALEMAX", temp_string, newObj);
  }
  
  val_string=QString(QuGetAttrText("VALIDMIN", newObj));
  if(!val_string.isEmpty())
  {
      val_double=val_string.toDouble();
      val_double*=conversion;
      sprintf(temp_string, "%e", val_double);
      QuSetTxtAttr("VALIDMIN", temp_string, newObj);
  }
  
  val_string=QString(QuGetAttrText("VALIDMAX", newObj));
  if(!val_string.isEmpty())
  {
      val_double=val_string.toDouble();
      val_double*=conversion;
      sprintf(temp_string, "%e", val_double);
      QuSetTxtAttr("VALIDMAX", temp_string, newObj);
  }
  
  val_string=QString(QuGetAttrText("FILLVAL", newObj));
  if(!val_string.isEmpty())
  {
      val_double=val_string.toDouble();
      val_double*=conversion;
      sprintf(temp_string, "%e", val_double);
      QuSetTxtAttr("FILLVAL", temp_string, newObj);
  }
  
  //int index=QString(SIcTarget).find('>');
  if(strlen(newUnits)>0)
  {
      //QString unit=QString(SIcTarget).right(QString(SIcTarget).length()-index-1);
      QuSetTxtAttr("UNITS", newUnits, newObj);
  }
  
//   char * checkAttr = QuGetAttrText("SI_conversion", newObj);
//   cout << checkArr << endl;
  return newObj;
  
} // end QarConvertUnitsTo

// *************************************************************************

 char * QarFrame_product(QdObject_var in1_ptr, 
                        QdObject_var in2_ptr){

 char *frame;
 static char *frame_scalar = "scalar>na";
 static char *unk_frame =  "vector>_??_??";
 
 // This is used for non-vector operation multiply
 // input may be rotation matrix 
 
 bool isRotn1 = false;
 const char *fr = QuGetOrder(in1_ptr);
 if (strcmp(QuToUpper(fr), "ROT") == 0) isRotn1 = true;
 
 bool isRotn2 = false;
 fr = QuGetOrder(in2_ptr);
 if (strcmp(QuToUpper(fr), "ROT") == 0) isRotn2 = true;
 
 // if vector  times scalar use frame of vector...
  
  bool vec_1=QuIsThreeVector(in1_ptr);
  bool vec_2=QuIsThreeVector(in2_ptr);

  if(vec_1 && vec_2)  // both 3 vectors
    frame=frame_scalar;
  else if(vec_1 && isRotn2){
    frame = unk_frame; 
    QuMsgBar("CAUTION: ROTATION MATRIX should pre-multiply", true);
  }
  else if(vec_1)
    frame = QuGetFrameAttr( in1_ptr);
  else if(vec_2 && isRotn1)
    frame = QuGetAttrText("ToFrame", in1_ptr);
  else if(vec_2)
    frame = QuGetFrameAttr( in2_ptr);
  else if(isRotn1 && isRotn2){
    frame = QuGetFrameAttr( in2_ptr);           // R2 is applied 1st to vector, so from Frame from R2
    // ToFrame will be from R1 
  }
  else  // scalar
    frame = QuGetFrameAttr(in1_ptr, in2_ptr); // will use frame if found else scalar
 
  
  return frame;
  
} // end QarFrame_product

// ***************************************************
char * QarUNITS_product(QdObject_var in1_ptr, 
                        QdObject_var in2_ptr){

char * units;
char * units1, * units2;
int length;
 
 // note QuGetAttrTextn returns a valid empty string if xref not found
 
  units1 = QuGetAttrText("UNITS", in1_ptr);
  units2 = QuGetAttrText("UNITS", in2_ptr);
  
  length = strlen(units1);
  length += strlen(units2);

  units = new char[length+2];
 
  strcpy(units, units1);
  strcat(units," ");
  strcat(units, units2);

  return units;
  
} // end QarUNITS_product

// ***************************************************

char * QarUNITS_power(QdObject_var in1_ptr, double constB){

char * units;
char * units1;
char  exponent[20];

   units1 = QuGetAttrText("UNITS", in1_ptr);
   sprintf(&(exponent[0]),"%g",(double) constB);
   units = new char[strlen(units1)+strlen(exponent)+4];
   strcpy(units,"(");
   strcat(units, units1);
   strcat(units,")^");
   strcat(units, exponent);
 
  return units;
   
} // end QarUNITS_power

// *************************************************************************


char * QarSIConv_product(QdObject_var in1_ptr, 
                         QdObject_var in2_ptr){

 char * siconv1, * siconv2, * siconv;
 char * base_str;
 float value1, value2, value;
 char * ptr1, * ptr2;
 char * SI_ptr;
 static char * null_unit = "1 > ()";

  char * txt1 = QuGetAttrText("SI_conversion", in1_ptr);
  if(strcmp(txt1, "") == 0) {
    siconv1 = new char[strlen(null_unit)+1] ;
    strcpy(siconv1, null_unit);
  }
  else{
    siconv1 = new char[strlen(txt1)+1] ;
    strcpy(siconv1, txt1);  
  }
  
  char * txt2 = QuGetAttrText("SI_conversion", in2_ptr);
  if(strcmp(txt2, "") == 0) {
    siconv2 = new char[strlen(null_unit)+1] ;
    strcpy(siconv2, null_unit);
  }
  else{
    siconv2 = new char[strlen(txt2)+1] ;
    strcpy(siconv2, txt2);  
  }

  if( (ptr1 = strtok(siconv1, ">")) == NULL) return NULL;
  value1 = atof(ptr1);
  ptr1 += strlen(ptr1)+1;
  
  if( (ptr2 = strtok(siconv2, ">")) == NULL) return NULL;
  value2 = atof(ptr2);
  ptr2 += strlen(ptr2)+1;
  value = value1 * value2;

  base_str = new char[strlen(ptr1)+strlen(ptr2)+20];
   
  sprintf(base_str, "%16g >", value);

  strcat(base_str, ptr1); 
  strcat(base_str, " "); 
  strcat(base_str, ptr2);
    
  SI_ptr =  QunitGetTidySI(base_str);
  siconv = new char[strlen(SI_ptr)+1];
  strcpy(siconv,SI_ptr);
  
//   printf("siC1= %s, \nsiC2= %s, \nbase= %s \nsic= %s\n", txt1,txt2,base_str,siconv); fflush(stdout);

  delete[] siconv1;
  delete[] siconv2;
  delete[] base_str;
  
  return siconv;
  
} // end QarSIConv_product

// *************************************************************************

char * QarSIConv_inverse(QdObject_var in1_ptr){

 char * siconv;
 char * buffer;
 char * SI_ptr;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);
     
  SI_ptr =  QunitGetPowerSI(siconv, -1);
  if(strcmp(SI_ptr, "") == 0) return NULL;

  buffer = new char[strlen(SI_ptr)+1];
  strcpy(buffer,SI_ptr);
    
  return buffer;
  
} // end QarSIConv_inverse

// *************************************************************************

char * QarSIConv_power(QdObject_var in1_ptr, double constB){

 char * siconv;
 char * buffer;
 char * SI_ptr;

  siconv = QuGetAttrText("SI_conversion", in1_ptr);

  SI_ptr =  QunitGetPowerSI(siconv, constB);
  if(strcmp(SI_ptr, "") == 0) return NULL;

  buffer = new char[strlen(SI_ptr)+1];
  strcpy(buffer,SI_ptr);
    
  return buffer;
  
} // end QarSIConv_power

// *************************************************************************

int QarHasUnits(QdObject_var in_ptr){

 int flag;
 char * siconv;
 
  siconv = QuGetAttrText("SI_conversion", in_ptr);
  if (strcmp(siconv, "") == 0) return 0;
  
  flag = QunitHasUnits (siconv);
  
  return flag;
}




// *************************************************************************
