#include "qcalc.h"
#include "qwui.h"
#include "qdutil.h"

using namespace QSAS;

extern qwuiWindow* qsasWindow;

void QcalcListView::renameSelectedItemAlias()
{
	if ( QuSlotDebug() )
	{
		printf("QcalcListView::renameSelectedItemAlias %ld\n", this);
		fflush(stdout);
	}

	// this is implemented only on the QT selection since
	// we can only handle one selected item

	QcalcListVirtualItem *selected_item;

	selected_item = dynamic_cast<QcalcListVirtualItem*> (this->selectedItem());
	if ( selected_item == NULL ) return;

	reNameItemAlias( selected_item) ;
}


bool QcalcListView::reNameItemAlias( QcalcListVirtualItem* item )
{
	// launch a dialog box to change an item's name
	// insist it is unique
	if ( item == NULL ) return ( false );

	QString userNameChoice = QString("");
	bool    ok             = false;
	QString qstrOldAliase  = QString("");

	// change text of this item
	// launch getText box old alias

	qstrOldAliase  = QString(item->text( Alias_ ).latin1());
	userNameChoice = QuTextInputDialog(	"Rename",
						"Enter new alias",
				           qstrOldAliase.latin1(),
						 &ok,
						 item->listView() );

	if ( ( ok ) && ( !userNameChoice.isEmpty() ) )
	{
		while ( ( ok ) && ( item->AliasAlreadyUsed( userNameChoice ) ) )
		{
			userNameChoice = QuTextInputDialog( 	"Rename",
								"Alias used",
								 userNameChoice.latin1(),
								 &ok,
								 item->listView() );

		}

		if ( ok )
		{
			item->reNameAlias(userNameChoice);
			return true;
		}
	}

	return false;

}

void QcalcListView::deleteAllItems()
{
	if  ( QuSlotDebug() )
		printf( "QcalcListView::deleteAllItems %ld\n", this );

	this->selectAll();
	this->deleteListItems();
}

void QcalcListView::deleteListItems()
{
	if  ( QuSlotDebug() )
		printf( "QcalcListView::deleteListItems %ld\n", this );

	vector<QcalcListVirtualItem *> folderList; 
  	
	QcalcListVirtualItem* qCurrItem = NULL;
	// ensure that selectionChanged() is not emitted while deleting
	clearSelection();
	
	int N = localList.size();
	
	for ( int i = 0; i < N; i++ )
	{
		qCurrItem = dynamic_cast< QcalcListVirtualItem* >( localList[ i ] );
		if ( qCurrItem == NULL ) continue;

		if ( qCurrItem->getEntryType() == ListTypeGroup)
      		 folderList.push_back(qCurrItem);

		if ( qCurrItem->getEntryType() == ListTypeEntry)
		{
			QcalcTab* qTab = qCurrItem->m_pcalcTab;
			if ( qTab && owner && owner->tabWidget )
			{
					// Remove its associated tab
					owner->tabWidget->removePage( qTab );
					delete qTab;
					qCurrItem->m_pcalcTab = NULL;
					delete qCurrItem;
					qCurrItem = NULL;
			}
		}
	}
	
	this->owner->_TagSource->reValidate();
	
	// delete remaining empty folders
  	for(int i = 0; i < folderList.size(); i++)
  	{
		qCurrItem = dynamic_cast< QcalcListVirtualItem* >( folderList[ i ] );
		if ( qCurrItem == NULL ) continue;
    		if( qCurrItem->firstChild() == NULL ) {
        		// empty folder, so ok to delete
        		delete qCurrItem;
		}
	}	
	localList.clear();
	
	// listSize() returns the number of entry type items in the list
    if(listSize() == 0) this->insertVirtualItem();  // if list is empty create empty item
    else emit listItemDeleted(); // if list not empty update linkage
	
	if(this->currentItem()) 
	{
		setSelectedList((QguListItem *) this->currentItem()); 
	}			
	
}


void QcalcListView::insertEntries( QguListItem *dropAt, QguListItem *item )
{
  // call for non-WL widgets to insert WL context as item
  // when creating entry from drag or copy off WL
  // if item is a folder, it gets ignored
	if  ( QuSlotDebug() )
		printf( "QcalcListView::insertEntries list=%ld, entry=%ld\n", this, item );

	QcalcListVirtualItem * newItem;
	if ( item == NULL )
	{
		return;
	}
	if ( item )
	{
		if ( item->nameContext().isNull() )
		{
			return;
		}

		if ( item->getWLcontext().isNull() )
		{
			return;
		}
	}

	if ( item->getEntryType() == ListTypeEntry )
	{
		if ( dropAt == 0 )
		{
			newItem = new QcalcListVirtualItem(  this,
						             item->nameContext(),
						             item->getWLcontext(),
				                             ListTypeEntry  );
		}
		else if ( dropAt->getEntryType() == ListTypeEntry )
		{
			if ( dropAt->parent() )
			{

				newItem = new QcalcListVirtualItem(   ( QguListItem * )dropAt->parent(),
								       item->nameContext(),
								       item->getWLcontext(),
								       ListTypeEntry );

				if ( newItem != NULL )
				{
					newItem->moveItem( dropAt );
				}
			}
			else
			{

				newItem = new QcalcListVirtualItem(
								       this,
								       item->nameContext(),
								       item->getWLcontext(),
								       ListTypeEntry );
			}
		}
		else
		{

			newItem = new QcalcListVirtualItem(
						         dropAt,
						         item->nameContext(),
						         item->getWLcontext(),
						         ListTypeEntry   );
		}

		if ( newItem )
		{
			newItem->setArrayFlags();
		}

		newItem->setChoices();
	}

}

void QcalcListView::insertVirtualItem()
{
#ifdef QT_VERSION
#if QT_VERSION <300
	QcalcListVirtualItem* newItem =  new QcalcListVirtualItem   ( this, 
									"????",  // Not Know at this stage
									"",
									ListTypeEntry );
	newItem->m_pcalcTab->objAliase_field->setText( newItem->text(Alias_) );
#else
	QguListItem*          afterItem = dynamic_cast< QguListItem* >( this->lastItem() );
	QcalcListVirtualItem* newItem =  new QcalcListVirtualItem   ( this, 
									"????",  // Not Know at this stage
									"",
									ListTypeEntry,
									afterItem );
	newItem->m_pcalcTab->objAliase_field->setText( newItem->text(Alias_) );
#endif // end of #if
#endif // end of #ifdef

}


void QcalcListView::paste()
{
	if ( QuSlotDebug() )
		printf( "QcalcListView::paste %ld\n", this );

	QuMsgBar(" ", false);

	QString text;

	// do you mean me?
// 	if ( !( this->hasFocus() ) )
// 	{
// 		emit pasteDetected( );
// 		return;
// 	}

	QguListItem *dropAtItem = ( QguListItem * )selectedItem();

#if QT_VERSION < 0x040000
	if ( QTextDrag::decode( QApplication::clipboard()->data(), text ) )
	{
#else
	if (QApplication::clipboard()->mimeData()->hasText() )
	{
		text=QApplication::clipboard()->mimeData()->text();	
#endif
		QguListItem * item = qsasWindow->WorkingList->getItemByName(text);
		if ( item )
		{
			insertEntries( dropAtItem, item );
			// Refresh combo box contents for operand A and B
			
		}
		else
		{
			QuMsgBar( "failed to paste item", text.latin1(), false );
		}

	}

	emit listModified();
	validateWLitems();

}

void QcalcListView::validateWLitems(){

 if(QuSlotDebug())
 {
  printf("QcalcListView::validateWLitems %ld\n", this);
  fflush(stdout);
  }
 
 bool listValid = true;
 
   	Q_LIST_VIEW_ITEM_ITERATOR it( this );
	while ( it.current() ) {
		QcalcListVirtualItem *item = dynamic_cast <QcalcListVirtualItem *> (it.current());
		QString ctxt = item->getWLcontext();

		if(item->m_eObjType == WORKING_LIST_OBJECT && qsasWindow->WorkingList->getItemByName(ctxt) == NULL ){
			listValid = false;
			item->setColour(listValid, false);
		}
		else{
			item->setColour(listValid, true);
		}
		++it;
	}


}

void QcalcListView::propagateSelection(const char * compString, const char * compType){

 if(QuSlotDebug())
 {
	printf("QcalcListView::propagateSelection %ld %s\n", this, compString);
	fflush(stdout);
 }

	for ( int i = 0; i < localList.size(); i++ )
	{
		QcalcListVirtualItem * item = (QcalcListVirtualItem *)localList[i];
		
		// ignore virtual items
		if(item->m_eObjType == WORKING_LIST_OBJECT && item != this->currentItem()) 
		{
			// only modify items of the correct object type
			QguWLinput * objSlot = item->m_pcalcTab->operandStack[0]->slot;
			
			if ( strcmp(compType , "VECTOR" ) == 0){
				if(item->isVector){
					QString baseName = objSlot->textObject();
					objSlot->setText(baseName + QString(compString));
					objSlot->reValidate(); // propagate change to rest of GUI
				}
			}
			else if ( strcmp(compType , "ARRAY") == 0){
				if(  !(item->virtualArrDim.is_nil()) && !(this->selectedObjDim.is_nil()) ) {
					if(*(item->virtualArrDim) == *(this->selectedObjDim) ) {
						QString baseName = objSlot->textObject();
						objSlot->setText(baseName + QString(compString));
						objSlot->reValidate(); // propagate change to rest of GUI
					}
				}
			}
			else if ( strcmp(compType , "TIME" ) == 0){
				if(objSlot->acceptTimeObject){
					QString baseName = objSlot->textObject();
					objSlot->setText(baseName + QString(compString));
					objSlot->reValidate(); // propagate change to rest of GUI
				}
			}
			else if (strcmp(compType , "XREF") == 0){
				cout << "Multiple selection is ignored for Xref (attribute) menu choice" << endl;
			}
		}
	}
}


void QcalcListView::propagateDrop(const char * objString){

 if(QuSlotDebug())
 {
	printf("QcalcListView::propagateDrop  %ld %s\n", this, objString);
	fflush(stdout);
 }

	for ( int i = 0; i < localList.size(); i++ )
	{
		QcalcListVirtualItem * item = (QcalcListVirtualItem *)localList[i];

		// ignore virtual items
		if(item->m_eObjType == WORKING_LIST_OBJECT && item != this->currentItem()) 
		{
			// only modify items of the correct object type
			QguWLinput * objSlot = item->m_pcalcTab->operandStack[0]->slot;
			QString compString = compText(objSlot->text());

			QdObject_var droppedObj = QuGetObjByName(objString);
			QdTimeInterval_var tiObj = get_timeinterval(droppedObj);
			
			if(objSlot->acceptTimeObject ){
				if( !tiObj.is_nil() ){
					objSlot->setText(objString + compString);
					objSlot->reValidate(); // propagate change to rest of GUI
				}
			}
			else if( item->isVector && QuIsThreeVector(droppedObj)) {
					objSlot->setText(objString + compString);
					objSlot->reValidate(); // propagate change to rest of GUI
			}
			else if( !(item->virtualArrDim.is_nil()) && !(this->selectedObjDim.is_nil()) ) {
				if(*(item->virtualArrDim) == *(this->selectedObjDim) ) {
					objSlot->setText(objString + compString);
					objSlot->reValidate(); // propagate change to rest of GUI
				}
			}
		}
	}
}

void QcalcListView::makeGroup()
{

	if ( QuSlotDebug() )
		printf("QjuiListView::makeGroup %ld\n", this);

	// this function is overloaded

	QcalcListVirtualItem *newFolder;
	bool           ok;

	// make new group item in folder or parent of selected item
		// if nothing is selected put at root (if allowed)
	if ( localList.size() == 0 )
	{
		if ( this->rootIsLocked() )
		{
			// root is locked, ie Working List, so there must be a first child
			newFolder = new QcalcListVirtualItem( (QguListItem*)this->firstChild(),
									        QString("Group"),
						                    QString(""),
						                    ListTypeGroup,
						                    0 );
		}
		else
		{
			newFolder = new QcalcListVirtualItem( this,
									        QString("Group"),
						                    QString(""),
						                    ListTypeGroup,
						                    0 );
		}

		clearSelection();
		newFolder->setSelected( true );
		ok = reNameItem( newFolder );  // launch re-name text box
		if (!ok)
		{
			delete newFolder;
		}

		return;
	}


	// else something is selected, find where to put folder
	QguListItem *commonRoot = findCommonFolder();

	if (commonRoot != NULL )
	{
		newFolder = new QcalcListVirtualItem(  commonRoot,
						    QString( "Group"),
				                    QString( ""),
				                    ListTypeGroup );
	}
	else if ( !this->rootIsLocked() )
	{
		newFolder = new QcalcListVirtualItem( this,
					    QString( "Group" ),
				                    QString( "" ),
				                    ListTypeGroup );
	}
	else
	{
		return;  // cannot find where to put folder that is allowed
	}

	ok = reNameItem( newFolder );  // launch re-name text box
	if ( !ok )
	{
		delete newFolder;
		return;
	}
	// move selected items to this new folder

	for ( int i = 0; i < localList.size(); i++ )
	{
		// move item from selection list
		QguListItem * movingItem = (QguListItem *)localList[i];
		QguListItem * parentItem = (QguListItem *)movingItem->parent();

		// move from old folder or listView
		if ( parentItem == 0 )
		{
			this->takeItem( movingItem );
		}
		else
		{
			parentItem->takeItem( movingItem );
		}

		newFolder->insertItem( movingItem );

	}

	clearSelectedList();
}


void QcalcListView::tabChanged( QWidget* qwdgt )
{
	if  ( QuSlotDebug() )
		printf("QcalcListView::tabChanged, tab %ld of list %ld \n", qwdgt, this);

	QcalcTab* qTab = dynamic_cast< QcalcTab* >( qwdgt );
	if ( qTab == NULL ) return;

	if ( qTab->virtualItem == NULL ) return;

	this->setSelectedList( qTab->virtualItem );
	this->ensureItemVisible( qTab->virtualItem );
}





