// -----------------------------------------------------------------------------------------
//
// Performs a generic Analysis of a Single Nest size
//
// SEH, Sep 16, 2003
// 
// Inputs :
//     B = magnetic field   (n x 3) 
//     V = plasma velocity  (n x 3) 
//     E = electric field   (n x 3)
//     P = pressure tensor  (n x 3 x 3)
//     Q = heat flux vec    (n x 3)
//     N = density          (n)
//     nestsize = number of points in data sets
//     constrain = 1 for Bn=0, 2 for (<V> - U)n etc - see below.
//     alpha = slope of walen correlation coeff (used for Alfvenic constrain)
//
// SEH, 12 Sep 2003
//      06 Nov 2003, added SCID and constrain type on output
//      14 May 2005, added alpha - the Walenslope for constraining
//      31 May 2006, added MVA dB/dt
// -----------------------------------------------------------------------------------------

#include "GenericMinVar.h"
#include "GenericResidue.h"


int SingleAnalysis(QdRMatrixSeq_var V, QdRMatrixSeq_var B, QdRMatrixSeq_var dB, QdRMatrixSeq_var P, 
		   QdRMatrixSeq_var E, QdRScalarSeq_var N, QdRMatrixSeq_var H, 
		   QdTimeSeq_var ttJoined,  QdRMatrix_var ExWeights,
		   int constrained, double alpha, char *SCc_ID, bool doType[10],int nestsize,
		   QplugArgList *call_list)
{
  printf("SingleGeneric > \n");
  fflush(stdout);
  bool GenerateSpreadSheet = true;
  bool GeneratePolarOutput = true;
  FILE *f1;
  int i,j,k;
  QdRMatrix_var ProjMx  = new QdRMatrix(Dim(3,3), 0.0);
  QdRMatrix_var Evec = new QdRMatrix(Dim(3,3), 0.0);  
  QdRMatrix_var EvecMVAB = new QdRMatrix(Dim(3,3), 0.0);      // save MVAB eigenvecs for later use in constrain
  QdRMatrix_var Evalues = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var Dummy =  new QdRMatrix(Dim(3), 0.0);  
  QdRMatrix_var U = new QdRMatrix(Dim(3), 0.0);        
  char buf[255],typebuf[16], constrbuf[4]="";
  double un=0.0, vn,van,bn,ucis=0.0,deltat,N_ave=0.0;

  QdTime t_start = (*ttJoined)[0];
  QdTime t_stop = (*ttJoined)[nestsize-1];
  QdTime t_center = t_start + (t_stop - t_start) / 2;

  QdRMatrixSeq_var Valf = new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0));  
  if (!QuIsTSRegular(ttJoined,&deltat));

  QdRMatrix_var B_singleave = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var V_singleave = new QdRMatrix(Dim(3), 0.0);
  QdRMatrix_var Va_singleave = new QdRMatrix(Dim(3), 0.0);
  
  // ----- SEH 18 Sep 2007 - to plot intrinsic E-field -----
  QdRMatrixSeq_var Erperp =  new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0)); 
  set_timetags((QdObject_var) Erperp, ttJoined);
  QuSetTxtAttr("Frame","vector>gse_xyz",(QdObject_var) Erperp);
  QuSetTxtAttr("UNITS","mV/m",(QdObject_var) Erperp);
  QuSetTxtAttr("SI_conversion","1.0e-3 > V m^-1",(QdObject_var) Erperp);
  
  QdRMatrixSeq_var Erpar  =  new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0)); 
  QuSetTxtAttr("Frame","vector>gse_xyz",(QdObject_var) Erpar);
  QuSetTxtAttr("UNITS","mV/m",(QdObject_var) Erpar);
  QuSetTxtAttr("SI_conversion","1.0e-3 > V m^-1",(QdObject_var) Erpar);
  set_timetags((QdObject_var) Erpar, ttJoined);
  // ------ eo SEH 18 Sep 2007 --------------------


  // --- send some info (time, nestsize etc to output ----------------

  QDateTime dt = QDateTime::currentDateTime();
  sprintf(buf,"# Date performed : %s\n",dt.toString().latin1());
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  sprintf(buf,"# t = %s <-- %s --> %s (%d samples)\n",t_start.iso_srep().c_str()+11,t_center.iso_srep().c_str()+11,t_stop.iso_srep().c_str()+11,nestsize);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);
  
  sprintf(buf,"# Nestsize N = %d\n",nestsize);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  sprintf(buf,"# Given Walenslope = %5.3f\n",alpha);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  
 //  try{  
//     printf("\n");
//     printf("  k                 B                              V                      N           Pxx      Pyy      Pzz      Pxy      Pxz      Pyz\n");
//     printf("                  [nT]                          [km/s]               [1/cm^3]                            [keV/cm^3]  \n");
//     printf("--------------------------------------------------------------------------------------------------------------------------------------\n");
//     for (k=0; k<nestsize; k++) {
//       printf("%3d  %8.3f %8.3f %8.3f       ",k+1,(float)(*B)[k][0], (float)(*B)[k][1], (float)(*B)[k][2]);
//       printf("%8.3f %8.3f %8.3f      ",(float)(*V)[k][0], (float)(*V)[k][1], (float)(*V)[k][2]);
//       printf("%7.4f      ",(float)(*N)[k]);
//       for (i=0; i<3; i++) printf("%7.2f  ",(double)(*P)[k][i][i]);
//       printf("%7.2f  %7.2f  %7.2f\n",(float)(*P)[k][0][1], (float)(*P)[k][0][2], (float)(*P)[k][1][2]);
//     }
//   }
//  catch(Exception& e) {
//    printf("exception in SingleGeneric printout..\n");
//  }
  

  // ------ out to spreadsheet  -----------------------

  char *filename="/tmp/outGenericSpread.csv";
  
  if (GenerateSpreadSheet) {
    if (!fopen(filename,"r")) {
      f1 = fopen(filename,"w");
      fprintf(f1,"ID, tstart, tstop, nestsize, ");
      if (doType[0]) fprintf(f1,"b_ave_x,  b_ave_y,  b_ave_z,  n_mva_x,  n_mva_y,  n_mva_z,  Bn,  dBn,  [B], "); 
      if (doType[2]) fprintf(f1,"v_cis_x,  v_cis_y,  v_cis_z,  v_cis_n,  N,        [N],  ");
      if (doType[2]) fprintf(f1,"v_ht_x,   v_ht_y,   v_ht_z,   v_ht_n,   vht'n,    VAn,      wal_sl,    wal_cc,   ");
      if (doType[3]) fprintf(f1,"n_mfr_x,  n_mfr_y,  n_mfr_z,  u_mfr_x,  u_mfr_y,  u_mfr_z,  u_mfr_n,   du_mfr_n,   vmfr'n,   dvmfr'n,  VAn,  Bn,  dBn,");
      if (doType[4]) fprintf(f1,"n_mmr_x,  n_mmr_y,  n_mmr_z,  u_mmr_x,  u_mmr_y,  u_mmr_z,  u_mmr_n,   du_mmr_n,   vmmr'n,   dvmmr'n,  VAn,  Bn,  dBn,");
      if (doType[5]) fprintf(f1,"n_mlmr_x, n_mlmr_y, n_nlmr_z, u_mlmr_x, u_mlmr_y, u_mlmr_z, u_mlmr_n, ");
      if (doType[7]) fprintf(f1,"n_mter_x, n_mter_y, n_mter_z, u_mter_x, u_mter_y, u_mter_z, u_mter_n, ");
      if (doType[6]) fprintf(f1,"n_mer_x,  n_mer_y,  n_mer_z,  u_mer_x,  u_mer_y,  u_mer_z,  u_mer_n, ");
      if (doType[8]) fprintf(f1,"n_mvav_x, n_mvav_y, n_mvav_z, u_mvav_x, u_mvav_y, u_mvav_z, u_mvav_n,  du_mvav_n,  vmvav'n,   dvmvav'n,  VAn,  Bn,  dBn,");
                     fprintf(f1,"n_com_x,  n_com_y,  n_com_z,  u_com_x,  u_com_y,  u_com_z,  u_com_n,   du_com_n,   vcom'n,    dvcom'n,   VAn,  Bn,  dBn,");
		     fprintf(f1,"VAx,      VAy,      VAz\n");
      fclose(f1);
    }
    f1 = fopen(filename,"a+");
    fprintf(f1,"IDNO, %s , %s , %d , ",t_start.iso_srep().c_str(),t_stop.iso_srep().c_str(),nestsize);
    fclose(f1);
  }
  

  QdRMatrix_var Q_COM = new QdRMatrix(Dim(3,3), 0.0); 
  QdRMatrix_var U_COM = new QdRMatrix(Dim(3), 0.0); 
  QdRMatrix_var dYij  = new QdRMatrix(Dim(3,3), 0.0);       // matrix to hold the three angular errors from eq 8.23 for MVA

  QdRMatrix_var Normal = new QdRMatrix(Dim(3), 0.0);       
  QdRMatrix_var V_res = new QdRMatrix(Dim(3), 0.0);       
  QdRMatrix_var E_ave = new QdRMatrix(Dim(3), 0.0);  
  QdRMatrix_var E_res = new QdRMatrix(Dim(3), 0.0);  
  QdRMatrix_var E_tang= new QdRMatrix(Dim(3), 0.0);  


  double mu_0 = 1.25664e-06;     // modified mu_0 (due to B in nT and dens in 1/ccm)
  double rho,m_p= 1.67e-27;      // rho = mass dens with proton_mass m_p
  double dBx3, sigma_Un, sigma_Vcis;
  double wl, cc, weight_k, weight_sum;
  QdRMatrix_var Q_MVAB;


  // ------ start calculating the Q-matrices - establish EvecMVAB for constraining --------------
  
  if (doType[0]) {
    printf("trying MVAB to establish constraints\n");
	fflush(stdout);
    set_timetags((QdObject_var) B,ttJoined);
    Q_MVAB = Q_MVAB_calc(B,V,U,Valf,N, &wl, &cc, nestsize, 0);
    try{
      GenericMinVar minvar(0,*Q_MVAB,*ProjMx);  
      *EvecMVAB = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; }

    Report(Evalues, EvecMVAB, TRUE, "MVAB", 0, alpha, SCc_ID);                 // unconstrained
    if (constrained > 0) sprintf(constrbuf,"C%1d",constrained);
  }  


  // ------------- 0 - MVAB -------------------------------

  bool doV = !V.is_nil();
  bool doN = !N.is_nil();
  bool doB = !B.is_nil();
  if (doType[0]) {
    printf("trying MVAB\n");
	fflush(stdout);
    sprintf(typebuf,"%s (%d)",SCc_ID,nestsize);
    try{
      for (k=0; k<nestsize; k++) {
	for (i=0; i<3; i++) {      
	  (*B_singleave)[i] += (double)(*B)[k][i];
	  if (doV) (*V_singleave)[i] += (double)(*V)[k][i];
	  //	  (*Va_singleave)[i] += alpha*(double)(*Valf)[k][i];
	}
	if (doN) N_ave += (double)(*N)[k];
      }
      for (i=0; i<3; i++) {      
	(*B_singleave)[i] /= (double) nestsize;
     }
    } catch(Exception& e) {
      printf("exception in SingGeneric averaging\n");
	  fflush(stdout);
      //      return 1; 
    }
    
    
    //  sprintf(typebuf,"MVAB%s",constrbuf);
    sprintf(typebuf,"%s (%d)",SCc_ID,nestsize);
    Q_MVAB = Q_MVAB_calc(B, V, U, Valf, N, &wl, &cc, nestsize,doType[2]);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MVAB,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) {
      printf("exception in SingGeneric averaging\n");
	  fflush(stdout);
      return 1; 
    }
   
    printf("here...\n");
	fflush(stdout);
    if (doV) {
      if (doN) N_ave /= (double) nestsize;
      for (i=0; i<3; i++) {
	(*V_singleave)[i] /= (double) nestsize;
	if (doN)(*Va_singleave)[i] = alpha * 1.0e-3 * (double) (*B_singleave)[i] * 1e-9 / sqrt(mu_0 * N_ave *1e06 * m_p);
      }
    }

    weight_k = (*ExWeights)[0] * weight_k_calc(Evalues);
    weight_sum = weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MVAB)[i][j] / ( (double)(*Q_MVAB)[0][0] +  (double)(*Q_MVAB)[1][1] + (double)(*Q_MVAB)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    }
    
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 0); 
    if (doV) sigma_Un = sigma(V,Evec,nestsize);  // calculate standarddeviation in <V> dot n_COM
    //  double sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
    if (doV) sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MVAB,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    
    un =  bn = vn = van = 0.0;

    for (i=0; i<3; i++) {
      un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
      bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
      if (doV) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
      if (doV && doN) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];
    }
    
    Report(Evalues, Evec, TRUE, "MVAB",constrained, alpha, typebuf);
    
//     printf("Details about covariance matrix :\n");
//     Q_MVAB->display();
//     printf("weight_MVAB = %10.4f\n\n",weight_k);
//     printf("Accumulated Q_COM : (after MVAB)\n- - - - - - - - - - - - - - - - - - - - - - \n"); Q_COM->display(); 
    
    
    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[0],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! V_HT   = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, 0.00);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van, sigma_Un);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    
    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s\n",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    double bjump = jumpvalue(B);
    double njump = jumpvalue(N);
    if (GenerateSpreadSheet) {
      FILE *f1 = fopen(filename,"a+");
      fprintf(f1,"%g , %g , %g , ",(double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2]); 
      fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
      fprintf(f1,"%g , %g , %g , ", bn, dBx3, bjump);
      fprintf(f1,"%g , %g , %g , ",(double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2]);
      fprintf(f1,"%g , %g , %g , ",vn, N_ave, njump);
      fprintf(f1,"%g , %g , %g , ",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3);
      fprintf(f1,"%g , %g , %g , ",un, vn-un, van);
      fprintf(f1,"%g , %g , ",wl, cc);
      fclose(f1);
    }
  }




  // ------------- 1 - MVAdB/dt -------------------------------

  if (doType[1]) {
    printf("trying MVAdB/dt\n");
	fflush(stdout);
    sprintf(typebuf,"%s (%d)",SCc_ID,nestsize);
    QdRMatrix_var Udummy = new QdRMatrix(Dim(3), 0.0);        
    QdRMatrixSeq_var Valfdummy = new QdRMatrixSeq(nestsize,RMatrix(Dim(3), 0.0));  
    QdRMatrix_var Q_MVADB = Q_MVAB_calc(dB, V, Udummy, Valfdummy, N, &wl, &cc, nestsize,doType[2]);
    ProjMx = Projection(dB, V, Udummy, Valf, EvecMVAB, alpha, nestsize, constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MVADB,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; }
    
    for (i=0; i<3; i++) {
      (*Va_singleave)[i] = alpha * 1.0e-3 * (double) (*B_singleave)[i] * 1e-9 / sqrt(mu_0 * N_ave *1e06 * m_p);
    }
    
    weight_k = (*ExWeights)[1] * weight_k_calc(Evalues);
    weight_sum = weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MVADB)[i][j] / ( (double)(*Q_MVADB)[0][0] +  (double)(*Q_MVADB)[1][1] + (double)(*Q_MVADB)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    }
    
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVADB, 0.0, nestsize, 0); 
    if (doV) sigma_Un = sigma(V,Evec,nestsize);  // calculate standarddeviation in <V> dot n_COM
    if (doV) sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MVADB,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    
    
    un =  bn = vn = van = 0.0;

    for (i=0; i<3; i++) {
      un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
      bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
      if (doV) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
      if (doV && doN) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];
    }
    
    
    Report(Evalues, Evec, TRUE, "MVAdBdT",constrained, alpha, typebuf);
    
    printf("Details about covariance matrix :\n");
	fflush(stdout);
    Q_MVADB->display();
    printf("weight_MVAdB/dt = %10.4f\n\n",weight_k);
    printf("Accumulated Q_COM : (after MVAdB/dT)\n- - - - - - - - - - - - - - - - - - - - - - - \n"); 
	fflush(stdout);
    Q_COM->display(); 
    
    
    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[0],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! V_HT   = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, 0.00);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van, sigma_Un);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    
    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s\n",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    
    
    if (GenerateSpreadSheet) {
      FILE *f1 = fopen(filename,"a");
      fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
      fprintf(f1,"%g , %g , %g , ",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3);
      fprintf(f1,"%g , %g , %g , ",un, wl, cc);
      fclose(f1);
    }
  }




  // ------------- 3 - MFR -------------------------------

  if (doType[3]) {
    printf("trying MFR\n");
	fflush(stdout);
    sprintf(typebuf,"%s (%d)",SCc_ID,nestsize);
    QdRMatrix_var Q_MFR = Q_MFR_calc(B,V,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize,constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MFR,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; } 
  
  
    weight_k = (*ExWeights)[2]  * weight_k_calc(Evalues);
    weight_sum += weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MFR)[i][j] / ( (double)(*Q_MFR)[0][0] +  (double)(*Q_MFR)[1][1] + (double)(*Q_MFR)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
      (*Normal)[i] = (double)(*Evec)[0][i]; 
      (*V_res)[i] = (double)(*U)[i]/1e3 - (double)(*V_singleave)[i];
    }


    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MFR,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MFR,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    //sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n

    un = 0.0;   for (i=0; i<3; i++) un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
    bn = 0.0;   for (i=0; i<3; i++) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
    vn = 0.0;   for (i=0; i<3; i++) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
    van = 0.0;  for (i=0; i<3; i++) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];

    Report(Evalues, Evec, TRUE, "MFR",constrained, alpha, typebuf);

    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[1],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MFR  = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
 
    // --------- SEH 12.09.2007 ----------------
    if (1 ==1 ) {
      QdRMatrix_var E_vecn = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_vecn_ave = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_tan2 = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_tan2_ave =  new QdRMatrix(Dim(3), 0.0);  
      QdRMatrix_var Vtemp = new QdRMatrix(Dim(3), 0.0);        
      QdRMatrix_var Btemp = new QdRMatrix(Dim(3), 0.0); 
      
      for (i=0; i<nestsize; i++) {
	for (int j=0; j<3; j++) {
	  (*Vtemp)[j] = (double)(*V)[i][j]*1e3;   // to get mV/m
	  (*Btemp)[j] = (double)(*B)[i][j]*1e-9;
	}
	
	// E_vecn = CrossVec((ScaleVec(un*1e3,Normal) - Vtemp),Btemp);
	// printf("E_vecn[%2d] =  [%10.6f  %10.6f  %10.6f] ",i, 1e3*(double)(*E_vecn)[0], 1e3*(double)(*E_vecn)[1], 1e3*(double)(*E_vecn)[2]);
	
	E_vecn = CrossVec(new QdRMatrix(*U-*Vtemp),Btemp);
	E_tan2 = ScaleVec(DotVec(Normal,E_vecn),Normal);
	
	for (int j=0; j<3; j++) {
	  (*Erperp)[i][j] = 1e3 * ((double)(*E_vecn)[j] - (double)(*E_tan2)[j]);
	  (*Erpar)[i][j]  = 1e3 * (double)(*E_vecn)[j];
	  (*E_tan2_ave)[j] += (double)(*Erperp)[i][j] / (double)nestsize;
	  (*E_vecn_ave)[j] += (double)(*Erpar)[i][j] / (double)nestsize;
	}
	//      printf("E_vecn[%2d] =  [%10.6f  %10.6f  %10.6f] ",i, 1e3*(double)(*E_vecn)[0], 1e3*(double)(*E_vecn)[1], 1e3*(double)(*E_vecn)[2]);
	//      printf("E_tan =   [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*Erperp)[i][0], (double)(*Erperp)[i][1], (double)(*Erperp)[i][2]);
      }
      
      printf("<E_tan>    =  [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*E_tan2_ave)[0], (double)(*E_tan2_ave)[1], (double)(*E_tan2_ave)[2]);
      printf("<E_vecn>   =  [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*E_vecn_ave)[0], (double)(*E_vecn_ave)[1], (double)(*E_vecn_ave)[2]);
      fflush(stdout);
	  
      QdRMatrix_var Q_ETAN = Q_GEN_calc(Erperp);
      try{
	GenericMinVar minvar(0,*Q_ETAN,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) 
	{return 1; } 
      
      Q_ETAN->display();
      
      int order[3]={0,1,2};
      for (int i=0; i<3; i++){
	sprintf(buf,"%12.6g  [%10.6g, %10.6g, %10.6g]\n", 
		(double)(*Evalues)[order[i]],
		(double)(*Evec)[order[i]][0],
		(double)(*Evec)[order[i]][1], 
		(double)(*Evec)[order[i]][2]);
	QplugAppendTextDisplay(buf);
	printf("%s",buf);
      }
    }
    // --------- e.o. SEH 12.09.2007 ----------------
    
     if (GenerateSpreadSheet) {
      FILE *f1 = fopen(filename,"a");
      fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
      fprintf(f1,"%g , %g , %g , ",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3);
      fprintf(f1,"%g , %g , %g , %g ,",un, sigma_Un/1e3,  vn-un, sigma_Vcis);
      fprintf(f1,"%g , %g , %g ,", van, bn, dBx3);
      fclose(f1);
    }
  }


  // ------------- 4 - MMR -------------------------------

  if (doType[4]) {
    printf("trying MMR\n");
	fflush(stdout);
    QdRMatrix_var Q_MMR = Q_MMR_calc(V,N,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MMR,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; } 
  
    weight_k = (*ExWeights)[3]  * weight_k_calc(Evalues);
    weight_sum += weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MMR)[i][j] / ( (double)(*Q_MMR)[0][0] +  (double)(*Q_MMR)[1][1] + (double)(*Q_MMR)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    }
  
    N_ave /= (double) nestsize;
    for (i=0; i<3; i++) {
      if (doB) (*Va_singleave)[i] = alpha * 1.0e-3 * (double) (*B_singleave)[i] * 1e-9 / sqrt(mu_0 * N_ave *1e06 * m_p);
    }

    if (doB) dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MMR,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    if (doB) sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MMR,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

    un =  bn = vn = van = 0.0;

    for (i=0; i<3; i++) {
      un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
      if (doB) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
      vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
      if (doB) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];
      (*Normal)[i] = (double)(*Evec)[0][i]; 
      (*V_res)[i] = (double)(*V_singleave)[i] - (double)(*U)[i]/1e3;
    }

    Report(Evalues, Evec, TRUE, "MMR",constrained, alpha, typebuf);

    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[2],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MMR  = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    // --------- SEH 12.09.2007 ----------------
    
    if (1 == 0) {
      QdRMatrix_var E_vecn = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_vecn_ave = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_tan2 = new QdRMatrix(Dim(3), 0.0);   
      QdRMatrix_var E_tan2_ave =  new QdRMatrix(Dim(3), 0.0);  
      QdRMatrix_var Vtemp = new QdRMatrix(Dim(3), 0.0);        
      QdRMatrix_var Btemp = new QdRMatrix(Dim(3), 0.0); 
      
      for (i=0; i<nestsize; i++) {
	for (int j=0; j<3; j++) {
	  (*Vtemp)[j] = (double)(*V)[i][j]*1e3;   // to get mV/m
	  (*Btemp)[j] = (double)(*B)[i][j]*1e-9;
	}
	
	// E_vecn = CrossVec((ScaleVec(un*1e3,Normal) - Vtemp),Btemp);
	// printf("E_vecn[%2d] =  [%10.6f  %10.6f  %10.6f] ",i, 1e3*(double)(*E_vecn)[0], 1e3*(double)(*E_vecn)[1], 1e3*(double)(*E_vecn)[2]);
	
	E_vecn = CrossVec(new QdRMatrix(*U-*Vtemp),Btemp);
	E_tan2 = ScaleVec(DotVec(Normal,E_vecn),Normal);
	
	for (int j=0; j<3; j++) {
	  (*Erperp)[i][j] = 1e3 * ((double)(*E_vecn)[j] - (double)(*E_tan2)[j]);
	  (*Erpar)[i][j]  = 1e3 * (double)(*E_vecn)[j];
	  (*E_tan2_ave)[j] += (double)(*Erperp)[i][j] / (double)nestsize;
	  (*E_vecn_ave)[j] += (double)(*Erpar)[i][j] / (double)nestsize;
	}
	//      printf("E_vecn[%2d] =  [%10.6f  %10.6f  %10.6f] ",i, 1e3*(double)(*E_vecn)[0], 1e3*(double)(*E_vecn)[1], 1e3*(double)(*E_vecn)[2]);
	//      printf("E_tan =   [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*Erperp)[i][0], (double)(*Erperp)[i][1], (double)(*Erperp)[i][2]);
      }
      
      printf("<E_tan>    =  [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*E_tan2_ave)[0], (double)(*E_tan2_ave)[1], (double)(*E_tan2_ave)[2]);
      printf("<E_vecn>   =  [%10.6f  %10.6f  %10.6f] mV/m\n", (double)(*E_vecn_ave)[0], (double)(*E_vecn_ave)[1], (double)(*E_vecn_ave)[2]);
      fflush(stdout);
	  
      QdRMatrix_var Q_ETAN = Q_GEN_calc(Erperp);
      try{
	GenericMinVar minvar(0,*Q_ETAN,*ProjMx);  
	*Evec = minvar.eigenvectors().duplicate(SHALLOW);
	*Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
      }
      catch(Exception& e) 
	{return 1; } 
      
      Q_ETAN->display();
  
      int order[3]={0,1,2};
      for (int i=0; i<3; i++){
	sprintf(buf,"%12.6g  [%10.6g, %10.6g, %10.6g]\n", 
		(double)(*Evalues)[order[i]],
		(double)(*Evec)[order[i]][0],
		(double)(*Evec)[order[i]][1], 
		(double)(*Evec)[order[i]][2]);
	QplugAppendTextDisplay(buf);
	printf("%s",buf);
      }
    }  

  // --------- e.o. SEH 12.09.2007 ----------------

    
    if (GenerateSpreadSheet) {
      FILE *f1 = fopen(filename,"a");
      fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
      fprintf(f1,"%g , %g , %g , ",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3);
      fprintf(f1,"%g , %g , %g , %g ,",un, sigma_Un/1e3,  vn-un, sigma_Vcis);
      fprintf(f1,"%g , %g , %g , ", van, bn, dBx3);
      fclose(f1);
    }
  }
  



  // ------------- 5 -  MLMR -------------------------------
  
  if (doType[5]) {
    printf("trying MLMR\n");
	fflush(stdout);
    QdRMatrix_var Q_MLMR = Q_MLMR_calc(V,B,P,N,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MLMR,*ProjMx);
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; }
    
    weight_k = (*ExWeights)[4]  * weight_k_calc(Evalues);
    weight_sum += weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MLMR)[i][j] / ( (double)(*Q_MLMR)[0][0] +  (double)(*Q_MLMR)[1][1] + (double)(*Q_MLMR)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
      (*Normal)[i] = (double)(*Evec)[0][i]; 
      (*V_res)[i] = (double)(*V_singleave)[i] - (double)(*U)[i]/1e3;
    }
  
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MLMR,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    //  sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
    sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MLMR,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

  
    un = 0.0;   for (i=0; i<3; i++) un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
    bn = 0.0;   for (i=0; i<3; i++) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
    vn = 0.0;   for (i=0; i<3; i++) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
    van = 0.0;  for (i=0; i<3; i++) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];

    Report(Evalues, Evec, TRUE, "MLMR",constrained, alpha, typebuf);
     sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[3],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MLMR = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf); 

    // --------- SEH 12.09.2007 ----------------
    E_res = ScaleVec(1e-3,CrossVec(B_singleave,V_res));
    E_tang = ScaleVec(1e-3, new QdRMatrix(*E_res - *(ScaleVec(DotVec(E_res,Normal),Normal))));
    sprintf(buf,"! <E_res>= [%10.3f  %10.3f  %10.3f] \n",
	    (double)(*E_res)[0],(double)(*E_res)[1],(double)(*E_res)[2]);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
 
    sprintf(buf,"! <E_tan>= [%10.3f  %10.3f  %10.3f] \n",
	    (double)(*E_tang)[0],(double)(*E_tang)[1],(double)(*E_tang)[2]);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
 
   // --------- e.o. SEH 12.09.2007 ----------------
    if (GenerateSpreadSheet) ToSpreadsheet(filename, Evec, U, 0, false);
  }
  

 // ------------- 6 - MER -------------------------------
  
  if (doType[6]) {
    printf("trying MER\n");
	fflush(stdout);
    QdRMatrix_var Q_MER = Q_MER_calc(V,B,P,N,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize,constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MER,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {
	sprintf(buf,"Exception in SingleCalc MER\n");
	printf("%s",buf);
	QplugAppendTextDisplay(buf);      
	return 1; 
      } 

    weight_k = (*ExWeights)[6] * weight_k_calc(Evalues);
    weight_sum += weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MER)[i][j] / ( (double)(*Q_MER)[0][0] +  (double)(*Q_MER)[1][1] + (double)(*Q_MER)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    } 

    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MER,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    //  sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
    sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MER,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

    un = 0.0;   for (i=0; i<3; i++) un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
    bn = 0.0;   for (i=0; i<3; i++) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
    vn = 0.0;   for (i=0; i<3; i++) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
    van = 0.0;  for (i=0; i<3; i++) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];

    Report(Evalues, Evec, TRUE, "MER",constrained, alpha, typebuf);
//     printf("Details about covariance matrix :\n");
//     Q_MER->display();
//     printf("weight_MER = %10.4f\n",weight_k);
//     printf("Accumulated Q_COM : (after MER)\n- - - - - - - - - - - - - - - - - - - - - - \n"); Q_COM->display(); 

    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[5],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MER  = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",(double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    if (GenerateSpreadSheet) ToSpreadsheet(filename, Evec, U, 0, false);
  }
  

  // ------------- 7 - MTER -------------------------------
 
  if (doType[7]) {  
    printf("trying MTER\n");
	fflush(stdout);
    QdRMatrix_var Q_MTER = Q_MTER_calc(V,B,P,E,N,H,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize,constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MTER,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      {return 1; } 
  
    weight_k = (*ExWeights)[5]  * weight_k_calc(Evalues);
    weight_sum += weight_k;
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MTER)[i][j] / ( (double)(*Q_MTER)[0][0] +  (double)(*Q_MTER)[1][1] + (double)(*Q_MTER)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    }
  
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MTER,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    //  sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
    sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MTER,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

    un = 0.0;   for (i=0; i<3; i++) un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
    bn = 0.0;   for (i=0; i<3; i++) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
    vn = 0.0;   for (i=0; i<3; i++) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
    van = 0.0;  for (i=0; i<3; i++) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];

    Report(Evalues, Evec, TRUE, "MTER",constrained, alpha, typebuf);
    //   printf("Details about covariance matrix :\n");
    //   Q_MTER->display();
    //   printf("weight_MTER = %10.4f\n",weight_k);
    //   printf("Accumulated Q_COM : (after MTER)\n- - - - - - - - - - - - - - - - - - - - - - \n"); Q_COM->display(); 

    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[4],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MTER = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",
	    (double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    if (GenerateSpreadSheet) ToSpreadsheet(filename, Evec,U, 0, false);
  }
  
  
  

  // ------------- 8 - MVAV -------------------------------
  
  if (doType[8]) {
    printf("trying MVAV\n");
	fflush(stdout);
    sprintf(typebuf,"%s (%d)",SCc_ID,nestsize);
    QdRMatrix_var Q_MVAV = Q_MVAV_calc(V,U,nestsize);
    ProjMx = Projection(B, V, U, Valf, EvecMVAB, alpha, nestsize, constrained, N_ave);
    try{
      GenericMinVar minvar(constrained,*Q_MVAV,*ProjMx);  
      *Evec = minvar.eigenvectors().duplicate(SHALLOW);
      *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
    }
    catch(Exception& e) 
      { 
	sprintf(buf,"Exception in SingleCalc MVAV\n");
	printf("%s",buf);
	QplugAppendTextDisplay(buf);      
	return 1; 
      } 

    try{
      for (k=0; k<nestsize; k++) {
	for (i=0; i<3; i++) {      
	  (*V_singleave)[i] += (double)(*V)[k][i];
	}
      }
      for (i=0; i<3; i++)  (*V_singleave)[i] /= (float) nestsize;            
    } catch(Exception& e) {
      printf("exception in SingGeneric MVAV averaging\n");
	  fflush(stdout);
      return 1; 
    }

    weight_k = (*ExWeights)[7] * weight_k_calc(Evalues);
    weight_sum += weight_k;
    printf("MVAV Weighs %f  %f\n",weight_sum, weight_k);
	fflush(stdout);
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
	(*Q_COM)[i][j] += weight_k * (double)(*Q_MVAV)[i][j] / ( (double)(*Q_MVAV)[0][0] +  (double)(*Q_MVAV)[1][1] + (double)(*Q_MVAV)[2][2]);
      }
      (*U_COM)[i] += weight_k * (double) (*U)[i];
    }

    printf("calc avg../n");
	fflush(stdout);
    un = bn = vn = van = 0.0;

    for (i=0; i<3; i++) {
      un += (double)(*U)[i] * (double)(*Evec)[0][i] / 1e3;
      if (doType[0]) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
      vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
      van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];
    }

    dBx3 = 0.0;
    if (doType[0]) dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
    sigma_Un = err_824(U, Evec, Evalues, dYij, Q_MVAV,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
    //  sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
    sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_MVAV,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

    Report(Evalues, Evec, TRUE, "MVAV",constrained, alpha, typebuf);
//     printf("Details about covariance matrix :\n");
//     Q_MVAV->display();
//     printf("weight_MVAV = %10.4f\n",weight_k);
//     printf("Accumulated Q_COM : (after MVAV)\n- - - - - - - - - - - - - - - - - - - - - - \n"); Q_COM->display(); 

    sprintf(buf,"! Given weight, calculated weight =  %5.3f   %5.3f\n",(double)(*ExWeights)[6],weight_k);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! V_MVAV = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3, un, sigma_Un/1e3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n =%10.2f +/- %8.4f)\n",
	    (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);

    sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	    (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
    printf("%s",buf);
	fflush(stdout);
    QplugAppendTextDisplay(buf);
    if (GenerateSpreadSheet) {
      FILE *f1 = fopen(filename,"a");
      fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
      fprintf(f1,"%g , %g , %g , ",(double)(*U)[0]/1e3,(double)(*U)[1]/1e3,(double)(*U)[2]/1e3);
      fprintf(f1,"%g , %g , %g , %g ,",un, sigma_Un/1e3,  vn-un, sigma_Vcis);
      fprintf(f1,"%g , %g , %g , ", van, bn, dBx3);
      fclose(f1);
    }
  }
  
  // ------------------ COM -------------------------------

  
  for (i=0; i<3; i++) {
    for (j=0; j<3; j++) {
      (*Q_COM)[i][j] /= weight_sum;
    }
    (*U_COM)[i] /=  weight_sum;
  }

  ProjMx = Projection(B, V, U_COM, Valf, EvecMVAB, alpha, nestsize,constrained, N_ave);
  try{
    GenericMinVar minvar(constrained,*Q_COM,*ProjMx);  
    *Evec = minvar.eigenvectors().duplicate(SHALLOW);
    *Evalues = minvar.eigenvalues().duplicate(SHALLOW); 
  }
  catch(Exception& e) 
    {
      QplugAppendTextDisplay("Exception error in MVA of Q_COM");
      return 1; 
    } 

  un = bn = vn = van = 0.00;
  
  for (i=0; i<3; i++) {
    un += (double)(*U_COM)[i]/1e3 * (double)(*Evec)[0][i];
    if (doType[0]) bn += (double)(*B_singleave)[i] * (double)(*Evec)[0][i];
    if (doType[8]) vn += (double)(*V_singleave)[i] * (double)(*Evec)[0][i];
    if (doType[8]) van += (double)(*Va_singleave)[i] * (double)(*Evec)[0][i];
  }

  Report(Evalues, Evec, TRUE, "COM",constrained, alpha, typebuf);
  //  printf("Details about covariance matrix :\n");
  //Q_COM->display();
  //printf("weight_COM = %10.4f\n",weight_sum);
  //printf("Accumulated Q_COM :\n"); 
  //Q_COM->display(); 

  dBx3 = 0.0;
  if (doType[0]) {
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 0); 
    dBx3 = err_824(B_singleave, Evec, Evalues, dYij, Q_MVAB, 0.0, nestsize, 1); 
  }
  sigma_Un = err_824(U, Evec, Evalues, dYij, Q_COM,  0.0, nestsize, 3);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]
  //  sigma_Vcis = sigma(V, Evec, nestsize);  // calculate standarddeviation in <V> dot n
  sigma_Vcis = err_824(V_singleave, Evec, Evalues, dYij, Q_COM,  0.0, nestsize, 1);         // calculater |dUn| error after eq 8.23/8.24 - first term from [ni Q_MVAij nj]

  sprintf(buf,"! V_COM  = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	  (double)(*U_COM)[0]/1e3,(double)(*U_COM)[1]/1e3,(double)(*U_COM)[2]/1e3, un, sigma_Un/1e3);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  sprintf(buf,"! <Vcis> = [%10.2f  %10.2f  %10.2f] (V'n = %10.2f +/- %8.4f)\n",
	  (double)(*V_singleave)[0],(double)(*V_singleave)[1],(double)(*V_singleave)[2], vn-un, sigma_Vcis);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  sprintf(buf,"! <w_Va> = [%10.2f  %10.2f  %10.2f] (%10.2f)\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2], van);
  printf("%s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);

  sprintf(buf,"! <Bvec> = [%10.2f  %10.2f  %10.2f] (%10.2f +/- %8.4f)\n",
	  (double)(*B_singleave)[0],(double)(*B_singleave)[1],(double)(*B_singleave)[2], bn, dBx3);
  printf("-- %s",buf);
  fflush(stdout);
  QplugAppendTextDisplay(buf);
  if (GenerateSpreadSheet) {
    FILE *f1 = fopen(filename,"a");
    fprintf(f1,"%g , %g , %g , ",(double)(*Evec)[0][0], (double)(*Evec)[0][1], (double)(*Evec)[0][2]);
    fprintf(f1,"%g , %g , %g , ",(double)(*U_COM)[0]/1e3,(double)(*U_COM)[1]/1e3,(double)(*U_COM)[2]/1e3);
    fprintf(f1,"%g , %g , %g , %g ,",un, sigma_Un/1e3,  vn-un, sigma_Vcis);
    fprintf(f1,"%g , %g , %g ,", van, bn, dBx3);
    fprintf(f1,"%g , %g , %g\n",(double)(*Va_singleave)[0],(double)(*Va_singleave)[1],(double)(*Va_singleave)[2]);
    fclose(f1);
  }
  


  QplugAppendTextDisplay("\n");

 for (int i=0; i<10; i++)
 {
	printf("[%10.3f  %10.3f %10.3f]\n",(double)(*Erperp)[i][0],(double)(*Erperp)[i][0],(double)(*Erperp)[i][0]);
	fflush(stdout);
 }
 call_list->push_back(Erperp);
 call_list->push_back(Erpar);
 
  return (0);
} 




