/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "MinVar.h"
#include "qdutil.h"
#include "qpui.h"

using namespace QSAS;

void mvplot(QdRMatrixSeq_var vts_out, 
           QdRMatrix_var Eigenvalues, QdRMatrix_var Rmatx)
{
int i;
char stringret[10];
double day_epoch;
PLFLT max1, max2, min1, min2, int1, int2, diff, hold;
PLFLT *secs;
PLFLT *mvmax, *mvint, *mvmin;
int yr, mth, dy, hr, mn, sc, msc;
int pgid;  /* hold qsas id to restore on return */
char str1[200], str2[200], str3[200];
PLFLT xval[4], yval[9];
int np;
PLFLT line_x[2], line_y[2];

QdTimeSeq_var tt = get_timetags(vts_out);
if (tt.is_nil() ) return;

np = tt->size();

secs = new PLFLT[np];
  (tt[0]).split_ymd(yr, mth, dy, hr, mn, sc, msc);
  
hr = 0; mn = 0; sc = 0; msc = 0;
QdTime_var t_0 = new QdTime(yr, mth, dy, 
                           (double)hr, (double)mn, (double)sc);
day_epoch = cdfEpoch(*t_0);


for(i=0;i<np;i++)
  secs[i] = (PLFLT)( ((double)(cdfEpoch(tt[i])) - day_epoch)/1000.0);

// unpack vts_out and cast to float (find array bounds as we go)

mvmax = new PLFLT[np];
mvint = new PLFLT[np];
mvmin = new PLFLT[np];

mvmin[0] = vts_out[0][0];
mvint[0] = vts_out[0][1];
mvmax[0] = vts_out[0][2];

max1=mvmax[0];int1=mvint[0];min1=mvmin[0];
max2=mvmax[0];int2=mvint[0];min2=mvmin[0];

for(i=1;i<np;i++){
  mvmin[i] = vts_out[i][0];
  mvint[i] = vts_out[i][1];
  mvmax[i] = vts_out[i][2];
  if(mvmin[i] < min1) min1=mvmin[i];
  if(mvint[i] < int1) int1=mvint[i];
  if(mvmax[i] < max1) max1=mvmax[i];
  if(mvmin[i] > min2) min2=mvmin[i];
  if(mvint[i] > int2) int2=mvint[i];
  if(mvmax[i] > max2) max2=mvmax[i];
}
diff = max2-max1;
diff = diff/10.0;
max1 -= diff;
max2 += diff;
diff = (max2-max1)/2.0;
hold = int1;
int1 = (hold + ((int2-hold)/2.0)) - diff;
int2 = (hold + ((int2-hold)/2.0)) + diff;
hold = min1;
min1 = (hold + ((min2-hold)/2.0)) - diff;
min2 = (hold + ((min2-hold)/2.0)) + diff;

// xval[0] = 0.7; xval[1] = 3.7; xval[2] = 6.7; xval[3] = 7.3;
// yval[0] = 0.75; yval[1] = 2.5; yval[2] = 3.5; yval[3] = 4.5; yval[4] = 5.5;
// yval[5] = 6.5; yval[6] = 9.5; yval[7] = 10.0; yval[8] = 10.5;
double fact_x=.75/5.8;
double fact_y=.75/5.8/1.414;
xval[0] = 0.7*fact_x; xval[1] = 3.7*fact_x; xval[2] = 6.7*fact_x; xval[3] = 7.3*fact_x;
yval[0] = 0.75*fact_y; yval[1] = 2.5*fact_y; yval[2] = 3.5*fact_y; yval[3] = 4.5*fact_y; yval[4] = 5.5*fact_y;
yval[5] = 6.5*fact_y; yval[6] = 9.5*fact_y; yval[7] = 10.0*fact_y; yval[8] = 10.5*fact_y;

// get name, units, frame for plot:

// cpgqid(&pgid);  /* save id of currently selected pgplot dev */
// 
// switch (pgdev_f){
// case(PGDEV_NONE):
//    return;
//    break;
// case(PGDEV_XWIN):  /* X window */
//   if(cpgopen("101/xserv")<=0){  /* try win 101 */
//      if(cpgopen("/xserv")<=0){  /* try any win */
//         printf("MinVar: Failed to open win101 or any xwin\n");
//         if (pgid != 0) cpgslct(pgid);
// 		return;
//      }
//   }
//   cpgpap(5.8, 1.414);
//   cpgsch(0.82);
//   for(i=0;i<4;i++)
//     xval[i] *= 0.75;
//   for(i=0;i<9;i++)
//     yval[i] *= 0.75;
//   break;
// case(PGDEV_PS): 
// default:           /* Postscript or gif file */
//    QString nameUsed="";
//    if(Qpgopen(nameUsed, filename, 'V')<=0){
//         printf("MinVar: Failed to open ps files, %s\n", nameUsed.latin1());
//         if (pgid != 0) cpgslct(pgid);
//         return; 
//    }
//    cpgsch(1.2);
//    break;
// 
// }     /* end switch  */
int plot_id;
PlotWindow * win= newPlotWindow(500, (int)(500.*1.414), &plot_id);
win->setSmooth(true);
pladv(0);
plcol0(0);
plschr(QPUI_CHAR_HEIGHT, 0.6);
/* Both */

plwid(QtWidth2PlWidth(1.));
/*
cpgvsiz(xval[0], xval[1], yval[5], yval[6]);
cpgswin(int1, int2, max1, max2);
cpgbox("bcmts", 0.0, 0, "bcnts", 0.0, 0);
cpgline(np, mvint, mvmax);*/

plvpor(xval[0], xval[1], yval[5], yval[6]);
plwind(int1, int2, max1, max2);
plbox("bcmts", 0., 0, "bcnts", 0., 0);
win->setSmooth(false);
plline(np, mvint, mvmax);

/*

cpgpt(1, &mvint[0], &mvmax[0], 4);
cpgpt(1, &mvint[0], &mvmax[0], 17);
cpglab("", "Maximum", "Intermediate");
*/

plpoin(1, &mvint[0], &mvmax[0], 4);
plpoin(1, &mvint[0], &mvmax[0], 17);
win->setSmooth(true);
// pllab("", "Maximum", "Intermediate");
plmtex("t", 3., 0.5, 0.5, "Intermediate");
plmtex("l", 3., 0.5, 0.5, "Maximum");

/*

cpgsls(4);
cpgmove(int1, 0.0);
cpgdraw(int2, 0.0);
cpgmove(0.0, max1);
cpgdraw(0.0, max2);
cpgsls(1);
*/
win->setSmooth(false);
pllsty(4);
line_x[0]=int1; line_x[1]=int2;
line_y[0]=0.; line_y[1]=0.;
plline(2, line_x, line_y);
line_x[0]=0.; line_x[1]=0.;
line_y[0]=max1; line_y[1]=max2;
plline(2, line_x, line_y);
pllsty(1);
win->setSmooth(true);

/*
cpgvsiz(xval[1], xval[2], yval[5], yval[6]);
cpgswin(min1, min2, max1, max2);
cpgbox("bcmts", 0.0, 0, "bcmts", 0.0, 0);
cpgline(np, mvmin, mvmax);
cpgpt(1, &mvmin[0], &mvmax[0], 4);
cpgpt(1, &mvmin[0], &mvmax[0], 17);
*/

plvpor(xval[1], xval[2], yval[5], yval[6]);
plwind(min1, min2, max1, max2);
plbox("bcmts", 0., 0, "bcmts", 0., 0);
win->setSmooth(false);
plline(np, mvmin, mvmax);
plpoin(1, &mvmin[0], &mvmax[0], 4);
plpoin(1, &mvmin[0], &mvmax[0], 17);

/*

cpglab("", "", "Minimum");
cpgsls(4);
cpgmove(min1, 0.0);
cpgdraw(min2, 0.0);
cpgmove(0.0, max1);
cpgdraw(0.0, max2);
cpgsls(1);
cpgvsiz(xval[2], xval[3], yval[5], yval[6]);
cpgswin(0.0, 1.0, 0.0, 1.0);
cpgptxt(1.0, 0.5, 90.0, 0.5, "Maximum");
*/

// pllab("", "", "Minimum");
pllsty(4);
line_x[0]=min1; line_x[1]=min2;
line_y[0]=0.; line_y[1]=0.;
plline(2, line_x, line_y);
line_x[0]=0.; line_x[1]=0.;
line_y[0]=max1; line_y[1]=max2;
plline(2, line_x, line_y);
pllsty(1);
// plwind(0., 1., 0., 1.);
// plptex(1., 1., 0., 1., .5, "Maximum");
win->setSmooth(true);
plmtex("t", 3., 0.5, 0.5, "Minimum");
plmtex("r", 3., 0.5, 0.5, "Maximum");

/*
cpgvsiz(xval[0], xval[1], yval[2], yval[5]);
cpgswin(int1, int2, min1, min2);
cpgbox("bcnts", 0.0, 0, "bcnts", 0.0, 0);
cpgline(np, mvint, mvmin);
cpgpt(1, &mvint[0], &mvmin[0], 4);
cpgpt(1, &mvint[0], &mvmin[0], 17);*/

plvpor(xval[0], xval[1], yval[2], yval[5]);
plwind(int1, int2, min1, min2);
plbox("bcnts", 0., 0, "bcnts", 0., 0);
win->setSmooth(false);
plline(np, mvint, mvmin);
plpoin(1, &mvint[0], &mvmin[0], 4);
plpoin(1, &mvint[0], &mvmin[0], 17);

/*

cpglab("Intermediate", "Minimum", "");

cpgsls(4);
cpgmove(int1, 0.0);
cpgdraw(int2, 0.0);
cpgmove(0.0, min1);
cpgdraw(0.0, min2);
cpgsls(1);*/

// pllab("Intermediate", "Minimum", "");
win->setSmooth(true);
plmtex("l", 3., 0.5, 0.5, "Intermediate");
plmtex("b", 3., 0.5, 0.5, "Minimum");
win->setSmooth(false);

pllsty(4);
line_x[0]=int1; line_x[1]=int2;
line_y[0]=0.; line_y[1]=0.;
plline(2, line_x, line_y);
line_x[0]=0.; line_x[1]=0.;
line_y[0]=min1; line_y[1]=min2;
plline(2, line_x, line_y);
pllsty(1);

/*
cpgvsiz(xval[1], xval[2], yval[4], yval[5]);
cpgswin(secs[0], secs[np-1], max1, max2);
cpgtbox("zbcts", 0.0, 0, "bcmts", 0.0, 0);
cpgline(np, secs, mvmax);
cpgvsiz(xval[2], xval[3], yval[4], yval[5]);
cpgswin(0.0, 1.0, 0.0, 1.0);
cpgptxt(1.0, 0.5, 90.0, 0.5, "Max.");*/
win->setSmooth(true);

char fmt_ticks[50];
if(QSAS_profiles->current()->TimeRep=="Colon")
{
	sprintf(fmt_ticks, "%%H:%%M");
}
else
{
	sprintf(fmt_ticks, "%%H#uh#d%%M");
}
pltimefmt(fmt_ticks);

plvpor(xval[1], xval[2], yval[4], yval[5]);
plwind(secs[0], secs[np-1], max1, max2);
plbox("dbcts", 0., 0, "bcmts", 0., 0);
win->setSmooth(false);
plline(np, secs, mvmax);
// plvpor(xval[2], xval[3], yval[4], yval[5]);
// plwind(0.0, 1.0, 0.0, 1.0);
// plptex(1.0, 1., 0., 1., 0.5, "Max.");
win->setSmooth(true);
plmtex("r", 3., 0.5, 0.5, "Max.");

/*cpgvsiz(xval[1], xval[2], yval[3], yval[4]);
cpgswin(secs[0], secs[np-1], int1, int2);
cpgtbox("zbcts", 0.0, 0, "bcmts", 0.0, 0);
cpgline(np, secs, mvint);
cpgvsiz(xval[2], xval[3], yval[3], yval[4]);
cpgswin(0.0, 1.0, 0.0, 1.0);
cpgptxt(1.0, 0.5, 90.0, 0.5, "Int.");*/

plvpor(xval[1], xval[2], yval[3], yval[4]);
plwind(secs[0], secs[np-1], int1, int2);
plbox("dbcts", 0., 0, "bcmts", 0., 0);
win->setSmooth(false);
plline(np, secs, mvint);
// plvpor(xval[2], xval[3], yval[3], yval[4]);
// plwind(0.0, 1.0, 0.0, 1.0);
// plptex(1.0, 1., 0., 1., 0.5, "Int.");
win->setSmooth(true);
plmtex("r", 3., 0.5, 0.5, "Int.");

/*cpgvsiz(xval[1], xval[2], yval[2], yval[3]);
cpgswin(secs[0], secs[np-1], min1, min2);
cpgtbox("zfbcnts", 0.0, 0, "bcmts", 0.0, 0);
cpgline(np, secs, mvmin);*/

plvpor(xval[1], xval[2], yval[2], yval[3]);
plwind(secs[0], secs[np-1], min1, min2);
plbox("dfbcnts", 0., 0, "bcmts", 0., 0);
win->setSmooth(false);
plline(np, secs, mvmin);

/*cpglab("Universal Time", "", "");
cpgvsiz(xval[2], xval[3], yval[2], yval[3]);
cpgswin(0.0, 1.0, 0.0, 1.0);
cpgptxt(1.0, 0.5, 90.0, 0.5, "Min.");*/

// plvpor(xval[2], xval[3], yval[2], yval[3]);
// plwind(0.0, 1.0, 0.0, 1.0);
// plptex(1.0, 1., 0., 1., 0.5, "Min.");
win->setSmooth(true);
plmtex("r", 3., 0.5, 0.5, "Min.");
plmtex("b", 3., 0.5, 0.5, "Universal Time");

/*cpgvsiz(xval[0], xval[2], yval[7], yval[8]);
cpgswin(0.0, 1.0, 0.0, 1.0);

char *name = QuGetAttrText("FIELDNAM", vts_out);
sprintf(str1, "Variance Analysis: %s", name);

char *SC_id = QuGetAttrText("SC_id", vts_out);
if(strcmp(SC_id,"") != 0){
  sprintf(str2, " Spacecraft: %s", SC_id);
  strcat(str1, str2);
}
cpgptxt(0.5, 0.6, 0.0, 0.5, str1);*/

plvpor(0., 1., yval[7], yval[8]);
plwind(0.0, 1.0, 0.0, 1.0);

char *name = QuGetAttrText("FIELDNAM", vts_out);
sprintf(str1, "Variance Analysis: %s", name);

char *SC_id = QuGetAttrText("SC_id", vts_out);
if(strcmp(SC_id,"") != 0){
	sprintf(str2, " Spacecraft: %s", SC_id);
	strcat(str1, str2);
}
plschr(QPUI_CHAR_HEIGHT, 0.8);
plptex(0.5, 0.6, 1., 0., 0.5, str1);
plschr(QPUI_CHAR_HEIGHT, 0.6);

/*cpgvsiz(xval[0], xval[2], yval[0], yval[1]);
cpgswin(0.0, 1.0, 0.0, 7.0);
sprintf(str1, "Time Interval (UT): ");
strcat(str1,  (tt[0].iso_srep()).c_str());
strcat(str1, " - ");
strcpy(str2, (tt[np-1].iso_srep()).c_str());
for(i=0;i<13;i++){
  str3[i] = str2[i+11];
}
strcat(str1, str3);
cpgptxt(0.5, 6.0, 0.0, 0.5, str1);*/

plvpor(xval[0], xval[2], yval[0], yval[1]);
plwind(0.0, 1.0, 0.0, 7.0);
sprintf(str1, "Time Interval (UT): ");
strcat(str1,  (tt[0].iso_srep()).c_str());
strcat(str1, " - ");
strcpy(str2, (tt[np-1].iso_srep()).c_str());
for(i=0;i<13;i++){
	str3[i] = str2[i+11];
}
strcat(str1, str3);
plptex(0.5, 6.5, 1., 0.0, 0.5, str1);

/*

char *units = QuGetAttrText("UNITS", vts_out);
sprintf(str1, "Units: %s  Frame: ", units);

char *frame = QuGetAttrText("Frame", vts_out);
strcat(str1, frame);

SC_id = QuGetAttrText("SC_id", vts_out);
if(strcmp(SC_id,"") != 0){
  sprintf(str2, " Spacecraft: %s", SC_id);
  strcat(str1, str2);
}

cpgptxt(0.5, 4.5, 0.0, 0.5, str1);
*/

char *units = QuGetAttrText("UNITS", vts_out);
sprintf(str1, "Units: %s  Frame: ", units);

char *frame = QuGetAttrText("Frame", vts_out);
strcat(str1, frame);

SC_id = QuGetAttrText("SC_id", vts_out);
if(strcmp(SC_id,"") != 0){
	sprintf(str2, " Spacecraft: %s", SC_id);
	strcat(str1, str2);
}

plptex(0.5, 5.0, 1., 0., 0.5, str1);


/*
cpgptxt(0.25, 3.0, 0.0, 0.5, "\\(2137)");
cpgptxt(0.60, 3.0, 0.0, 0.5, "Direction");
for(i=0;i<3;i++){
  hold = 2.0 - (float)i;
  sprintf(str1, "%.3g", (double)(*Eigenvalues)[i]);
  cpgptxt(0.25, hold, 0.0, 0.5, str1);
  sprintf(str1, "(%6.3f, %6.3f, %6.3f)", 
   (double)(*Rmatx)[i][0], (double)(*Rmatx)[i][1], (double)(*Rmatx)[i][2]);
  cpgptxt(0.60, hold, 0.0, 0.5, str1);
}

cpgclos();*/

plptex(0.25, 3.5, 1., 0., 0.5, "#gl");
plptex(0.60, 3.5, 1., 0., 0.5, "Direction");
for(i=0;i<3;i++){
	hold = 2.5 - (float)i;
	sprintf(str1, "%.3g", (double)(*Eigenvalues)[i]);
	plptex(0.25, hold, 1., 0.0, 0.5, str1);
	sprintf(str1, "(%6.3f, %6.3f, %6.3f)", 
		(double)(*Rmatx)[i][0], (double)(*Rmatx)[i][1], (double)(*Rmatx)[i][2]);
	plptex(0.60, hold, 1., 0.0, 0.5, str1);
}

// if(pgid != 0) cpgslct(pgid); /* restore pgplot device selected prior to call */

delete [] secs;
delete [] mvmax;
delete [] mvint;
delete [] mvmin;
}

/*----------------------------------------------------------------------*/

