/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/* PROJECT: Cluster UK CDHF
*
* COMPONENT:    TSSubset
* 
* MODULE:	QSAS Subsetting of Time Series
*
* LANGUAGE:	C++
*
* FUNCTION:
*
* PURPOSE: Subset a time series (any type)
* 
* DESCRIPTION:
*
* ARGUMENTS:
*    IN:
*     ANY_TIME_SERIES    ts_in                 input vector time series 
*                                                  data object
*     TIME_INTERVAL         start_end             time interval data object
*                                                  giving start and end times
*                                                  surrounding output series
*    OUT:
*        ANY_TIME_SERIES ts_out
* GLOBALS:
*
* RETURN:
*    TYPE:     QplugReturnStatus
*    VALUES:   see qplug_if.h
*
* FUNCTIONS_USED:	 
*
* ALGORITHM:   pretty obvious
*
* HISTORY:		Steve Schwartz, 1996
*  Amended to use Qfui interface instead of Qai - A R Hare, 12 Sep 96.
*  Amended to use Qplug interface instead of Qfui - SJ Schwartz 21 May 97
*  Combined Scalar and Vector TS Subset using  "ANY_TIME_SERIES" args 21/5/97
*  Fixed memory leaks on failure and in attr. handling   27/6/97
*
***********************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "Qdos.h"
#include "qdutil.h"

using namespace QSAS;

extern "C" QplugReturnStatus  TSSubset (QplugArgList *);

extern "C" {
#include "TSSubset.h"

}

QplugReturnStatus  TSSubset  ( QplugArgList *call_list )
{

QdTimeInterval_var start_end;
QdObject_var ts_in;
QdTimeSeq_var tt_in;
QdObject_var ts_out;
QdTimeSeq_var tt_out;

TSSubsetErrorStatus	okstart, okend;
int attribute_ok, iatt;                       // attribute shufflers
int n_min_attrs_s=5;
char  *min_attr_s_ptr[]={"FIELDNAM", "Frame", "SI_conversion", "UNITS", "LABLAXIS"};         // LABLAXIS isn't a minimal attribute, but very useful
int n_min_attrs_v=6;
char  *min_attr_v_ptr[]={"FIELDNAM", "Frame", "SI_conversion", "UNITS", "LABLAXIS", "LABL_PTR_1"};         
      // LABLAXIS isn't a minimal attribute, but very useful; LABL_PTR_1 for vec

char out_string[150];

double tstart,tend; 				  /* start, end times     */
int    ndata_in, ndata_out, inew, iold;           /* counters for arrays  */
                                                  /* arrays new'ed below  */
double *in_data_x, *in_data_y, *in_data_z;    // arrays for input vector series 
double *in_data;                              // array for input scalar series
double *in_tags;

int istart, iend;          // indices to 1st and last point within subinterval
double *out_data_x, *out_data_y, *out_data_z;   // actual output arrays
double *out_data;
double *out_tags;

//
//	...first, unpack the input data objects
//

//	..first item is input Time Series

  ts_in = (*call_list)[0];
  tt_in = get_timetags(ts_in);

 if(tt_in.is_nil()) {
    QplugAppendTextDisplay ("TSSubset>> ERROR - Object has no time tags\n");
	return QPLUG_FAILURE; 
 }
 if  ( !tt_in->is_monotonic_increasing() && !tt_in->is_monotonic_decreasing()  ){
     QplugAppendTextDisplay ("TSSubset>> ERROR - Time Tags not monotonic\n");
	 return QPLUG_FAILURE; 
 }      

//	2nd item is time interval for start and stop


 QdObject_var do1 = (* call_list)[1];

 start_end = get_timeinterval(do1);
 
 // modified for using QuTakeSlice
 vector <string> dataObjectNames;
 
 dimTriple sliceTriple(3);
 sliceTriple[0] = 0; // subset
 if( ! QuGetSliceFromIvl( ts_in, start_end,  sliceTriple)) {
	 QplugAppendTextDisplay ("TSSubset>> ERROR - cannot find data records in this interval\n");
	 return QPLUG_FAILURE;
 }
 
 ts_out = QuTakeSlice(dataObjectNames,  ts_in,  sliceTriple, FALSE);
  
  call_list->push_back(ts_out);


QplugAppendTextDisplay ("TSSubset >> data object created - exiting successfully\n");
return QPLUG_SUCCESS;

}  // end TSSubset
