/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/**********************************************************************
*
*  QSAS Plugin to shift timetags of a time series
*
*
*  Inputs:  Time Series
*           dt          time (secs) to ADD to each timetag
*
*  Output:  Cloned Time series with shifted tags and all xrefs
*
*  History:  sjs September 2001
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"

using namespace QSAS;

// local prototype
extern "C" QplugReturnStatus TimeShift (QplugArgList *);

QplugReturnStatus TimeShift  ( QplugArgList *call_list )
{

// call-list arguments

QdObject_var tsin_obj;
QdRScalar_var dt_obj;
QdObject_var tsout_obj;

double dt;

// local arguments
QdTimeSeq_var tagsin, tagsout;

//	Unpack the input data objects
//
//	..first input item is  time series

//  set pointer to objects and check objects

tsin_obj = QdObject_var::narrow((*call_list)[0]);
dt_obj   = QdRScalar_var::narrow((*call_list)[1]);

if (tsin_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input TS is nil\n");
   return QPLUG_FAILURE;
}
if (dt_obj.is_nil()){
   QplugAppendTextDisplay("TimeShift> Input dt is nil\n");
   return QPLUG_FAILURE;
}
if (!tsin_obj->xref_exists(TIME_TAGS)){
   QplugAppendTextDisplay("TimeShift> Input TS not a time series!\n");
   return QPLUG_FAILURE;
}

// get timetags  and get ready
tagsin = get_timetags(tsin_obj);
dt  = (double) (*dt_obj);

// copy input tags, object, and xrefs
tagsout = (QdTimeSeq_var)Factory::clone(tagsin);
tsout_obj = (QdObject_var)Factory::clone(tsin_obj);
tsout_obj->copy_xrefs_from(tsin_obj);

// do shift
*tagsout += Seconds(dt);

set_timetags(tsout_obj,tagsout);
char modattribute[200];
sprintf(modattribute,"Tags shifted by  %g  seconds",dt);
QuSetTxtAttr("Mods",modattribute,tsout_obj);

call_list->push_back(tsout_obj);

QplugAppendTextDisplay ("TimeShift: Done\n");
  
return QPLUG_SUCCESS;

}

