//  QbTimeEditor.h
//
// Time Editor Widget
//
// July 2000: David Burgess
//

#ifndef QBTIME_WIDGETS_H
#define QBTIME_WIDGETS_H

#include <qobject.h>
#include "QdTime.h"     // QDOS
#include "KVDataBase.h"
#include "Viostream.h"
#include "TimeUtils.h"

#include <qvalidator.h>
#include "qt_version.h"
#include <qobject.h>
#include <qtooltip.h>

#include <qdragobject.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qdatetime.h>
#include <qlayout.h>
#include <qspinbox.h>
#include <qpushbutton.h>

class QLabel;
class QLineEdit;

class QbDurationWidget:public QLineEdit
{
	Q_OBJECT

	public:
		QbDurationWidget(QWidget *);
		
		bool getDuration(QDOS::Time &);
		
		void setDuration(const QDOS::Time&);
		void setActive(bool);
	signals:
		void getFocus();
	protected slots:
		void focusInEvent(QFocusEvent *);
		
		void dropEvent(QDropEvent *event);
		
		void dragEnterEvent(QDragEnterEvent *event);
		
	protected:
		bool active;
		QColor activeColor, inactiveColor;
		QColor activeText, inactiveText;

		QRegExpValidator * validator;
};

class QbDateTimeWidget:public QLineEdit
{
	Q_OBJECT
			
	public:
		QbDateTimeWidget ( QWidget * parent = 0 );

		QbDateTimeWidget ( const QDateTime & datetime, QWidget * parent = 0 );
		
		QDOS::Time time();
		
		void setTime(const QDOS::Time&);
		
		void setActive(bool);
		
		bool edited() {return hasBeenEdited;}
		
		double value();
		
		void setDouble(double value);
		
	signals:
		void getFocus();
		void valueChanged();
		void textEdited(const QString &);
		
	protected slots:
		void focusInEvent(QFocusEvent *);
		void slotValueChanged();
		
		void dropEvent(QDropEvent *event);
		
		void dragEnterEvent(QDragEnterEvent *event);
		
		void keyPressEvent(QKeyEvent *);
		
		void wheelEvent(QWheelEvent *);
		
		void droppedObjStart();
		void droppedObjCentre();
		void droppedObjEnd();
		void increaseCurrent()
		{
			addToCurrent(1);
		}
		
		void decreaseCurrent()
		{
			addToCurrent(-1);
		}
		void mouseDoubleClickEvent ( QMouseEvent * event );
	protected:
				
		QRegExpValidator * validator;
		bool active;
		QColor activeColor, inactiveColor;
		QColor activeText, inactiveText;
		bool hasBeenEdited;
		QDOS::Time previousTime;
		
		QDOS::QdObject_var droppedObject;
		
		void addToCurrent(int delta=0);
		QPushButton* buttonUp;
		QPushButton* buttonDown;
};

class QbTimeIntervalWidget:public QWidget
{
	Q_OBJECT
			
	public:
		QbTimeIntervalWidget(QWidget* parent);
		
		QbDateTimeWidget * _start_time_field;
		QbDateTimeWidget * _end_time_field;
		QbDateTimeWidget * _centre_time_field;
		QbDurationWidget* _duration_field;
		
		void setTimeInterval(QDOS::QdObject_var obj);
		void setTimeInterval(const QDOS::TimeInterval& obj);
		
		QDOS::QdTimeInterval_var getTimeInterval();
		
		QDOS::Time getDuration();
		
	signals:
		
		void intervalUpdated();
		
	public slots:
		
		void dropEvent(QDropEvent *event);
		void startFocus();
		void endFocus();
		void centreFocus();
		void durationFocus();
		void dragEnterEvent(QDragEnterEvent *event);

		void updateFields();
		
	private:
		
		QGridLayout * layout;
		int current, previous;
		
		void updateStartEnd();
		void updateStartCentre();
		void updateStartDuration();
		void updateEndCentre();
		void updateEndDuration();
		void updateCentreDuration();
		void updateSelection();

		bool allowUpdate;
};


#endif // QBTIME_WIDGETS_H
