//  DataSelector.cc
//
// Data Selection Widget
//
// July 2000: David Burgess
//

#include "qt_version.h"
#include "Factory.h"
#include "qdutil.h"
#include "qwui.h"
#include <sys/stat.h>

#if QT_VERSION < 0x040000
#include <qheader.h>
#include "DataSelector.h"
#else
#include <Q3Header>
#include "DataSelector_qt4.h"
#endif

#include <stdlib.h> // for drand

#define DA_PIXEL_WIDTH 5
#define DA_PIXEL_HEIGHT 12
#define DCM_MODE 0664

//TODO rewrite a clean Qt4 version

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release;

void DataBaseListView::resizeEvent ( QResizeEvent * e)
{
   Q_LIST_VIEW::resizeEvent(e);
   //int diff_sz=e->size().width()-e->oldSize().width();
   //if(diff_sz)
   //{
   //    int w=columnWidth(1);
   //    w=min(w+diff_sz,viewport()->width()-(columnWidth(0)+leftMargin()+rightMargin()));
   //    w=max(w,200);      
   //    setColumnWidth(1,w);
   //    w=columnWidth(1);
   //    qApp->processEvents();
   //    _timer->start(1000,TRUE);
   //}
}

void DataBaseListView::update_da()
{
  emit da_resized();
}

QPixmap DataSelector::data_availability_pixmap(const vector<bool>& avail )
{
  const int unit_width=DA_PIXEL_WIDTH;
  const int unit_height=DA_PIXEL_HEIGHT;
  int n=avail.size();
  int w=n*unit_width+2;
  const int h=unit_height+4;

  QColorGroup cg = QColorGroup(qApp->palette().active());

  QPixmap pm(w,h);

  QPainter p(&pm);
  p.setPen(Qt::NoPen);

  int x=0;
  p.fillRect(0,0,w,h,cg.brush(QColorGroup::Light));

  for(int i=0;i<n;i++)
  {
    if( avail[i] )
    {
      p.fillRect(x+2,2,unit_width-2,unit_height, QColor("#118811"));
    }
    else
    {
        //p.fillRect(x+2,2,unit_width-2,unit_height,\
        //    cg.brush(QColorGroup::Button));
        //p.fillRect(x+2,2,unit_width-2,unit_height,\
        //    cg.brush(QColorGroup::Dark));
        p.fillRect(x+2,2,unit_width-2,unit_height,\
            cg.brush(QColorGroup::Midlight));
    }
    x+=unit_width;
  }

  p.end();
  
  return pm;
}

DataSelector::DataSelector( QWidget *parent, const char *name ):\
  QWidget(parent,name),_open_dcm(0),_time_updated(false)
#if QT_VERSION < 0x040000
{
   _top_gl = new QGridLayout( this, 3, 2 );
   _top_gl->setSpacing(4);
   _top_gl->setMargin(4);
   _top_gl->setColStretch(0,20);
   _top_gl->setColStretch(1,10);

  // splitter to manage database list view and data selection widget

   _top_splitter = new QSplitter( this );
   _top_gl->addMultiCellWidget( _top_splitter, 0,0,0,1 );

// databases list view of available data sources

  _database_listview = new DataBaseListView( _top_splitter, "database_listview");
  _database_listview->addColumn("Data Source");
  _database_listview->addColumn("Data Availability");
  _database_listview->setRootIsDecorated(true);
  _database_listview->setSelectionMode(Q_LIST_VIEW::Extended);
  _database_listview->setColumnWidthMode(1,Q_LIST_VIEW::Manual);
  _database_listview->setColumnWidth(1,200);
  _database_listview->setColumnWidth(0,200);
  _database_listview->setTreeStepSize(10);
  _database_listview->setMinimumSize(_database_listview->sizeHint());
  QSize database_listview_sz=_database_listview->size();
  int w=_database_listview->columnWidth(1);
  _davail_sz=(w-2)/DA_PIXEL_WIDTH;

  connect( _database_listview, SIGNAL(doubleClicked(QListViewItem*)),
           this, SLOT(item_double_click(QListViewItem*)) );
           
  connect( _database_listview, SIGNAL(rightButtonPressed(QListViewItem *, \
          const QPoint &, int)), this, SLOT(popup_datasource_menu(QListViewItem *, \
    const QPoint &, int)));
 
  Q_HEADER* datasource_list_header = _database_listview->header();
  datasource_list_header->setClickEnabled(true,1);
  connect( datasource_list_header, SIGNAL(clicked(int)),
           this, SLOT(data_availability_clicked(int)) );  
  
  connect( _database_listview, SIGNAL(selectionChanged()),
           this, SLOT(update_database_selection()) );

  QGroupBox* selection_groupbox =
    new QGroupBox( 1, Qt::Vertical, _top_splitter,
   "selection_groupbox");
  selection_groupbox->setTitle("Data Selection");

  QWidget* dataselection_widget =
     new QWidget(selection_groupbox,"dataselection_widget");

  QGridLayout* dataselection_layout = new QGridLayout( dataselection_widget, 6, 2);
  dataselection_layout->setSpacing(4);
  dataselection_layout->setColStretch(1,10);

  // add to data selection list button
  _add_button =
    new QPushButton( ">> Add >>", dataselection_widget, "_add_button" );
  _add_button->setMinimumSize(_add_button->sizeHint());
  _add_button->setEnabled(false);
  dataselection_layout->addWidget( _add_button, 0, 0 );
  connect( _add_button, SIGNAL(clicked()),
           this, SLOT(add_to_selection_list()) );

  // remove from data selection list button
  _remove_button =
		  new QPushButton( "Remove <<", dataselection_widget, "_remove_button" );
  _remove_button->setMinimumSize(_remove_button->sizeHint());
  _remove_button->setEnabled(false);
  dataselection_layout->addWidget( _remove_button, 1, 0 );
  connect( _remove_button, SIGNAL(clicked()),
           this, SLOT(remove_from_selection_list()) );

  // clear all items from data selection list button
  _clear_button =
		  new QPushButton( "Clear <<", dataselection_widget, "_clear_button" );
  _clear_button->setMinimumSize(_clear_button->sizeHint());
  _clear_button->setEnabled(false);
  dataselection_layout->addWidget( _clear_button, 2, 0 );
  connect( _clear_button, SIGNAL(clicked()),
           this, SLOT(clear_selection_list()) );

  // list of selected data 
  _data_selection_listview =
    new DataSelectionListView( this, dataselection_widget, "data_selection_listview");
  _data_selection_listview->addColumn("Name");
  _data_selection_listview->addColumn("DataSource");
  _data_selection_listview->setMinimumSize(_data_selection_listview->sizeHint());
  connect( _data_selection_listview, SIGNAL(selectionChanged()),
           this, SLOT(update_selected_data_display()) );
  connect( _data_selection_listview, SIGNAL(rightButtonPressed(QListViewItem *, \
    const QPoint &, int)), this, SLOT(popup_dataselection_menu(QListViewItem *, \
    const QPoint &, int)));

  dataselection_layout->addMultiCellWidget( _data_selection_listview, 0,4,1,1 );

  // Tabbed INFO/OPTIONS/FILES widget
  QTabWidget* info_tabs_widget = new QTabWidget( dataselection_widget);
  info_tabs_widget->setMargin( 4 );
  dataselection_layout->addMultiCellWidget( info_tabs_widget, 5, 5, 0, 1 );

  // Tabbed INFO page

  Q_V_BOX* selection_info = new Q_V_BOX( info_tabs_widget );

  QHBox* coverage_line = new QHBox( selection_info );
  QLabel *coverage_label = new QLabel("Data Coverage:", coverage_line, 
  "coverage_label");
  _data_coverage =new QProgressBar( 100, coverage_line, "data_coverage" );

  
  _data_info=new QTextEdit( selection_info, "data_info" );
  _data_info->setWordWrap( QTextEdit::WidgetWidth );
 
  QToolTip::add( _data_info, "Textual info on selection item" );
  _data_info->setReadOnly(true);

  info_tabs_widget->addTab( selection_info, "Info");

  // Tabbed Options page
  QWidget* options_widget = new QWidget(info_tabs_widget,"options_widget");
  QGridLayout* options_grid = new QGridLayout(options_widget,2,2);

  QLabel* option1_label = new QLabel("Option 1:",\
    options_widget,"option1_label");
  options_grid->addWidget( option1_label, 0, 0, Qt::AlignRight );
  QLineEdit* option1 = new QLineEdit( options_widget, "option1" );  
  option1->setText("Option 1 goes here");
  options_grid->addWidget( option1, 0, 1 );

  QLabel* option2_label = new QLabel("Option 2:",\
    options_widget,"option2_label");
  options_grid->addWidget( option2_label, 1, 0, Qt::AlignRight );
  QLineEdit* option2 = new QLineEdit( options_widget, "option2" );  
  option2->setText("Option 2 goes here");
  options_grid->addWidget( option2, 1, 1 );
  options_widget->setEnabled(false);
  info_tabs_widget->addTab( options_widget, "Options");
  info_tabs_widget->setTabEnabled(options_widget,false);

  // Tabbed Data Files  page
  _data_files_list =
    new QListBox(info_tabs_widget,"data_files_list");

  QToolTip::add( _data_files_list, "Files for selection item" );
  info_tabs_widget->addTab( _data_files_list, "Files");

  // Clipboard
  _clipboard=QApplication::clipboard();
  //connect(_clipboard,SIGNAL(dataChanged()),this,SLOT(clipboardChanged()));

  // Add the Data Selection widget to the top level layout
  // QGroupBox* timeeditor_groupbox =
  // new QGroupBox( 2, Qt::Vertical, "Time Interval", this, \
  //   "timeeditor_groupbox");
  // _time_editor = new QbTimeEditor( 0,timeeditor_groupbox, "time_editor");
  _time_editor = new QbTimeIntervalEditor( 0,this, "time_editor");
  _time_editor->adjustSize();

//   connect(_time_editor,SIGNAL(time_changed()),this,SLOT(check_time()) );
  connect(_time_editor,SIGNAL(editorHasInput()),this,SLOT(check_time()) );
  connect(this,SIGNAL(time_checked()),_time_editor,SIGNAL(time_updated()));
  connect( _time_editor, SIGNAL(time_updated()),this, \
    SLOT(update_data_availability()) );
  connect( _time_editor, SIGNAL(time_updated()),this, \
    SLOT(update_selected_data_display()) );
  connect( _time_editor, SIGNAL(time_updated()),this, \
    SLOT(time_updated()) );
  connect(_database_listview,SIGNAL(da_resized()),this,\
    SLOT(update_data_availability()));
 
  _top_gl->addMultiCellWidget( _time_editor, 1,1,0,1 );
  
  QHBoxLayout *buttons_layout=new QHBoxLayout();
  QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);
  
  _get_data_button = new QPushButton( "Get Data", this,\
    "_get_data_button" );
  _get_data_button->setMinimumSize(_get_data_button->sizeHint());
  _get_data_button->setEnabled(false);
  connect( _get_data_button, SIGNAL(clicked()), this, SLOT(get_data()) );
  buttons_layout->addWidget(_get_data_button);
  
  QPushButton *quit = new QPushButton( "Dismiss", this, "quit" );
  quit->setMinimumSize(quit->sizeHint());
  connect( quit, SIGNAL(clicked()), this, SLOT(quit()) );
  buttons_layout->addWidget(quit);
  
  _top_gl->addLayout(buttons_layout,2,1);
 adjustSize(); 

}
#else
{
   _top_gl = new QGridLayout( this, 3, 2 );

   _top_gl->setSpacing(4);
   _top_gl->setMargin(4);
   _top_gl->setColStretch(0,20);
   _top_gl->setColStretch(1,10);

  // splitter to manage database list view and data selection widget

   _top_splitter = new QSplitter( this );
   _top_gl->addMultiCellWidget( _top_splitter, 0,0,0,1 );

// databases list view of available data sources

   _database_listview = new DataBaseListView( _top_splitter, "database_listview");
   _database_listview->addColumn("Data Source");
   _database_listview->addColumn("Data Availability");
   _database_listview->setRootIsDecorated(true);
   _database_listview->setSelectionMode(Q_LIST_VIEW::Extended);
   _database_listview->setColumnWidthMode(1,Q_LIST_VIEW::Manual);
   _database_listview->setColumnWidth(1,200);
   _database_listview->setColumnWidth(0,200);
   _database_listview->setTreeStepSize(10);
   _database_listview->setMinimumSize(_database_listview->sizeHint());
   QSize database_listview_sz=_database_listview->size();
   int w=_database_listview->columnWidth(1);
   _davail_sz=(w-2)/DA_PIXEL_WIDTH;

   connect( _database_listview, SIGNAL(doubleClicked(Q3ListViewItem*)),
	    this, SLOT(item_double_click(Q3ListViewItem*)) );
           
   connect( _database_listview, SIGNAL(rightButtonPressed(Q3ListViewItem *, \
		   const QPoint &, int)), this, SLOT(popup_datasource_menu(Q3ListViewItem *, \
				   const QPoint &, int)));
 
   Q_HEADER* datasource_list_header = _database_listview->header();
   datasource_list_header->setClickEnabled(true,1);
   connect( datasource_list_header, SIGNAL(clicked(int)),
	    this, SLOT(data_availability_clicked(int)) );  
  
   connect( _database_listview, SIGNAL(selectionChanged()),
	    this, SLOT(update_database_selection()) );

   QGroupBox* selection_groupbox=new QGroupBox(_top_splitter);
   selection_groupbox->setTitle("Data Selection");

//   QWidget* dataselection_widget =
//      new QWidget(selection_groupbox,"dataselection_widget");

   QGridLayout* dataselection_layout = new QGridLayout( /*dataselection_widget*/selection_groupbox, 6, 2);
   dataselection_layout->setSpacing(4);
   dataselection_layout->setColStretch(1,10);

  // add to data selection list button
   _add_button =
		   new QPushButton( ">> Add >>", /*dataselection_widget*/selection_groupbox, "_add_button" );
   _add_button->setMinimumSize(_add_button->sizeHint());
   _add_button->setEnabled(false);
   dataselection_layout->addWidget( _add_button, 0, 0 );
   connect( _add_button, SIGNAL(clicked()),
	    this, SLOT(add_to_selection_list()) );

  // remove from data selection list button
   _remove_button =
		   new QPushButton( "Remove <<", /*dataselection_widget*/selection_groupbox, "_remove_button" );
   _remove_button->setMinimumSize(_remove_button->sizeHint());
   _remove_button->setEnabled(false);
   dataselection_layout->addWidget( _remove_button, 1, 0 );
   connect( _remove_button, SIGNAL(clicked()),
	    this, SLOT(remove_from_selection_list()) );

  // clear all items from data selection list button
   _clear_button =
		   new QPushButton( "Clear <<", /*dataselection_widget*/selection_groupbox, "_clear_button" );
   _clear_button->setMinimumSize(_clear_button->sizeHint());
   _clear_button->setEnabled(false);
   dataselection_layout->addWidget( _clear_button, 2, 0 );
   connect( _clear_button, SIGNAL(clicked()),
	    this, SLOT(clear_selection_list()) );

  // list of selected data 
   _data_selection_listview =
		   new DataSelectionListView( this, \
      /*dataselection_widget*/selection_groupbox, "data_selection_listview");
   _data_selection_listview->addColumn("Name");
   _data_selection_listview->addColumn("DataSource");
   _data_selection_listview->setMinimumSize(_data_selection_listview->sizeHint());
   connect( _data_selection_listview, SIGNAL(selectionChanged()),
	    this, SLOT(update_selected_data_display()) );
   connect( _data_selection_listview, SIGNAL(rightButtonPressed(Q3ListViewItem *, \
		   const QPoint &, int)), this, SLOT(popup_dataselection_menu(Q3ListViewItem *, \
				   const QPoint &, int)));

   dataselection_layout->addMultiCellWidget( _data_selection_listview, 0,4,1,1 );

  // Tabbed INFO/OPTIONS/FILES widget
   QTabWidget* info_tabs_widget = new QTabWidget( /*dataselection_widget*/selection_groupbox );
   info_tabs_widget->setMargin( 4 );
   dataselection_layout->addMultiCellWidget( info_tabs_widget, 5, 5, 0, 1 );

  // Tabbed INFO page

   Q_V_BOX* selection_info = new Q_V_BOX( info_tabs_widget );

   QVBoxLayout * selection_info_layout=new QVBoxLayout;
   selection_info_layout->setSpacing(0);
   selection_info_layout->setMargin(0);
  
   QWidget* coverage_line = new QWidget( selection_info );
   selection_info_layout->addWidget(coverage_line);
   QHBoxLayout * coverage_layout=new QHBoxLayout(coverage_line);
#if QT_VERSION < 0x040300
   coverage_layout->setMargin(0);
#else
   coverage_layout->setContentsMargins(0,0,0,0);
#endif

   QLabel *coverage_label = new QLabel("Data Coverage:", coverage_line, 
				       "coverage_label");
   _data_coverage =new QProgressBar(this);//( 100, coverage_line, "data_coverage" );
   _data_coverage->setMaximum(100);
   coverage_layout->addWidget(coverage_label);
   coverage_layout->addWidget(_data_coverage);


  
   _data_info=new QTextEdit( selection_info, "data_info" );

   _data_info->setWordWrapMode(QTextOption::WrapAnywhere);
   selection_info_layout->addWidget(_data_info);
 
   QToolTip::add( _data_info, "Textual info on selection item" );
   _data_info->setReadOnly(true);

   selection_info->setLayout(selection_info_layout);

   info_tabs_widget->addTab( selection_info, "Info");

  // Tabbed Options page
   QWidget* options_widget = new QWidget(info_tabs_widget,"options_widget");
   QGridLayout* options_grid = new QGridLayout(options_widget,2,2);

   QLabel* option1_label = new QLabel("Option 1:",\
		   options_widget,"option1_label");
   options_grid->addWidget( option1_label, 0, 0, Qt::AlignRight );
   QLineEdit* option1 = new QLineEdit( options_widget, "option1" );  
   option1->setText("Option 1 goes here");
   options_grid->addWidget( option1, 0, 1 );

   QLabel* option2_label = new QLabel("Option 2:",\
		   options_widget,"option2_label");
   options_grid->addWidget( option2_label, 1, 0, Qt::AlignRight );
   QLineEdit* option2 = new QLineEdit( options_widget, "option2" );  
   option2->setText("Option 2 goes here");
   options_grid->addWidget( option2, 1, 1 );
   options_widget->setEnabled(false);
   info_tabs_widget->addTab( options_widget, "Options");
   info_tabs_widget->setTabEnabled(options_widget,false);

  // Tabbed Data Files  page
   _data_files_list =
		   new QListWidget(info_tabs_widget);
   QToolTip::add( _data_files_list, "Files for selection item" );
   info_tabs_widget->addTab( _data_files_list, "Files");

  // Clipboard
   _clipboard=QApplication::clipboard();
  //connect(_clipboard,SIGNAL(dataChanged()),this,SLOT(clipboardChanged()));

  // Add the Data Selection widget to the top level layout
   _time_editor = new QbTimeIntervalEditor( 0,this, "time_editor");
   _time_editor->adjustSize();

   connect(_time_editor,SIGNAL(editorHasInput()),this,SLOT(check_time()) );
   connect(this,SIGNAL(time_checked()),_time_editor,SIGNAL(time_updated()));
   connect( _time_editor, SIGNAL(time_updated()),this, \
		   SLOT(update_data_availability()) );
   connect( _time_editor, SIGNAL(time_updated()),this, \
		   SLOT(update_selected_data_display()) );
   connect( _time_editor, SIGNAL(time_updated()),this, \
		   SLOT(time_updated()) );
   connect(_database_listview,SIGNAL(da_resized()),this,\
		   SLOT(update_data_availability()));
 
   _top_gl->addMultiCellWidget( _time_editor, 1,1,0,1 );
  
   QHBoxLayout *buttons_layout=new QHBoxLayout();
   QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);
  
   _get_data_button = new QPushButton( "Get Data", this,\
		   "_get_data_button" );
   _get_data_button->setMinimumSize(_get_data_button->sizeHint());
   _get_data_button->setEnabled(false);
   connect( _get_data_button, SIGNAL(clicked()), this, SLOT(get_data()) );
   buttons_layout->addWidget(_get_data_button);
  
   QPushButton *quit = new QPushButton( "Dismiss", this, "quit" );
   quit->setMinimumSize(quit->sizeHint());
   connect( quit, SIGNAL(clicked()), this, SLOT(quit()) );
   buttons_layout->addWidget(quit);
  
   _top_gl->addLayout(buttons_layout,2,1);
   adjustSize(); 

}
#endif

// void DataBaseListView::contentsMousePressEvent(QMouseEvent* event)
// {
//   //record the position
//   if(event->button()==_dragdropbutton)
//     _presspos=event->pos();
//  
//   Q_LIST_VIEW::contentsMousePressEvent(event);
// }

// void DataBaseListView::contentsMouseMoveEvent(QMouseEvent* event)
// {
//   bool drag_event=false;
//   if(event->state()==_dragdropbutton) // && _dragdropbutton==event->button())
//   {
//     if(abs(event->pos().x() - _presspos.x() ) >=3 || 
//       abs(event->pos().y() - _presspos.y() ) >=3 )
//     {
//       drag_event=true;
//       DcmDataSourceListViewItem* drag_item=\
//         dynamic_cast<DcmDataSourceListViewItem*>(\
//         itemAt(_presspos-QPoint(0,contentsY())));
//       if(drag_item)
//       {
//         DcmDataSourceDrag* drag=new DcmDataSourceDrag(\
//           drag_item->database_name()+string(DATABASE_DELIM)+\
//             drag_item->name(),this);
//         drag->dragCopy();
//       }
//     }
//   }
//     
//   Q_LIST_VIEW::contentsMouseMoveEvent(event);
//   if(drag_event)
//     selectAll(false);
// }

// void DataSelector::copy_item_to_clipboard(DcmDataSourceListViewItem* item)
// {
//   string copy_txt=item->database_name()+string(DATABASE_DELIM)+item->name();
//   _clipboard->setText(copy_txt.c_str());
// }

void DataSelector::item_double_click( Q_LIST_VIEW_ITEM* item )
{ add_item_to_selection( item );}

void DataSelector::add_item_to_selection( Q_LIST_VIEW_ITEM* node)
{
  string item_name = node->text(0).latin1();
  if( node->childCount() == 0 )
  {
    bool is_selected = node->isSelected();
    
    if( is_selected )
    {
      DcmDataSourceListViewItem* db_lv_item =
        dynamic_cast<DcmDataSourceListViewItem*>(node);
      if(db_lv_item)
      {
         QDOS::Name n(db_lv_item->name());
         n.delete_component(n.num_components()-1);
         string datasource=db_lv_item->database_name()+string(DATABASE_DELIM)+\
           n.srep();
         _data_selection_listview->add_item(item_name,datasource,\
           db_lv_item->data_base());
      }
      // clear item in database view
      // nb: don't use node->setSelected(bool)
      _database_listview->setSelected( node, false );
    }
  
  } else
  {
    if( node->isSelected() )
      _database_listview->setSelected( node, false );
  }
}

void DataSelector::recursive_add_to_selection( Q_LIST_VIEW_ITEM* node)
{
  add_item_to_selection( node );

  if( node->childCount() != 0 )
  {
    bool folder_has_children_is_open = node->isOpen();
    bool is_selected = node->isSelected();
    
    // only descend down tree if folder is open and has children

    if( folder_has_children_is_open )
    {
      node = node->firstChild();
      while( node )
      {
        recursive_add_to_selection( node );
        node = node->nextSibling();
      }
    }
  }
}

void DataSelector::database_listview_expand(bool open)
{
  if( _database_listview->childCount() == 0 )
    return;

  Q_LIST_VIEW_ITEM* node = _database_listview->firstChild();
  while( node )
  {
    recursive_expand( node,open );
    node = node->nextSibling();
  }

}

void DataSelector::recursive_expand( Q_LIST_VIEW_ITEM* node,bool open)
{
  node->setOpen(open);

  if( node->childCount() != 0 )
  {
    node = node->firstChild();
    while( node )
    {
      recursive_expand( node,open );
      node = node->nextSibling();
    }
  }
}

void DataSelector::add_to_selection_list()
{
  if( _database_listview->childCount() == 0 )
    return;

  Q_LIST_VIEW_ITEM* node = _database_listview->firstChild();
  while( node )
  {
    recursive_add_to_selection( node );
    node = node->nextSibling();
  }

}

void DataSelector::remove_from_selection_list()
{
  Q_LIST_VIEW_ITEM* item = _data_selection_listview->firstChild();
  
  while( item )
  {
     if( item->isSelected() )
     {
       delete item;
       break;
     }
     item = item->nextSibling();
  }

  update_selected_data_display();
}

void DataSelector::clear_selection_list()
{
  _data_selection_listview->clear();
  update_selected_data_display();
}

#if QT_VERSION < 0x040000
void DataSelector::popup_dataselection_menu( Q_LIST_VIEW_ITEM *item, const QPoint &p, int )
{
  QPopupMenu dataselection_listview_popup(0,"dataselection_listview_popup");
  dataselection_listview_popup.move(p);

  int paste_item=dataselection_listview_popup.insertItem(tr("&Paste"));
  QString paste_text=_clipboard->text();
  if(paste_text.isNull() || paste_text.isEmpty())
    dataselection_listview_popup.setItemEnabled(paste_item,FALSE);
  int res=dataselection_listview_popup.exec();
  if(res==paste_item)
    paste_item_from_clipboard();
}
#else
void DataSelector::popup_dataselection_menu( Q_LIST_VIEW_ITEM *item, const QPoint &p, int )
{
    /*Q3PopupMenu dataselection_listview_popup(0,"dataselection_listview_popup");
    dataselection_listview_popup.move(p);

    int paste_item=dataselection_listview_popup.insertItem(tr("&Paste"));
    QString paste_text=_clipboard->text();
    if(paste_text.isNull() || paste_text.isEmpty())
        dataselection_listview_popup.setItemEnabled(paste_item,FALSE);
    int res=dataselection_listview_popup.exec();
    if(res==paste_item)
        paste_item_from_clipboard();*/
		
	QMenu dataselection_listview_popup(this);
    dataselection_listview_popup.move(p);

    QAction* paste_item=dataselection_listview_popup.addAction(tr("&Paste"));
    QString paste_text=_clipboard->text();
    if(paste_text.isNull() || paste_text.isEmpty())
		paste_item->setEnabled(false);
    QAction* res=dataselection_listview_popup.exec();
    if(res==paste_item)
        paste_item_from_clipboard();
}
#endif

#if QT_VERSION < 0x040000
void DataSelector::popup_datasource_menu( Q_LIST_VIEW_ITEM *item, const QPoint 
&p,
				    int )
{
   DcmFileListViewItem* dcmfile_item=\
     dynamic_cast<DcmFileListViewItem*>(item);
   
   DcmDataSourceListViewItem* datasource_item =
        dynamic_cast<DcmDataSourceListViewItem*>(item);
   if(dcmfile_item || datasource_item)
   {
     _database_listview->setCurrentItem( item );
     _database_listview->setSelected( item, true );
     QPopupMenu database_listview_popup(0,"database_listview_popup");
     database_listview_popup.move(p);
     if(dcmfile_item)
     {     
       int close=database_listview_popup.insertItem(tr("&Close Database"));
       int filelist=database_listview_popup.insertItem(tr("&File List"));
       int res=database_listview_popup.exec();
       update();
       if(res==close)
         close_database(dcmfile_item);
       else if(res=filelist)
       {
         dcmfile_item->show_files(p);
         _database_listview->setSelected( item, false ); 
       }  
     }
     else if ( datasource_item ) 
     {
//        int copy=database_listview_popup.insertItem(tr("&Copy"));
       int browse=database_listview_popup.insertItem(tr("&Data Browser"));
       int filelist=database_listview_popup.insertItem(tr("&File List"));
       int metadata=database_listview_popup.insertItem(tr("&Meta Data"));
       int selectitem=database_listview_popup.insertItem(tr("&Select DataSource"));     
       int res=database_listview_popup.exec();
       update();
       if(res==metadata)
         datasource_item->show_meta_data(p);
       else if(res==browse)
           datasource_item->browse_data(p,_time_editor);
       else if(res=filelist)
       {
         QDOS::TimeInterval tintv;
         bool valid=_time_editor->get_time_interval(tintv);
         datasource_item->show_files(p,datasource_item->name(),tintv);
       }
       else if(res==selectitem)
         add_item_to_selection(item);
//        else if (res==copy)
//          copy_item_to_clipboard(datasource_item);
       _database_listview->setSelected( item, false );   
     }
   }
   // This line moved as will crash on some installations if 
   // called following close_database operation as item is deleted 
   // by this call 
   //_database_listview->setSelected( item, false );   
}
#else
void DataSelector::popup_datasource_menu( Q_LIST_VIEW_ITEM *item, const QPoint 
&p,
				    int )
{
   DcmFileListViewItem* dcmfile_item=\
     dynamic_cast<DcmFileListViewItem*>(item);
   
   DcmDataSourceListViewItem* datasource_item =
        dynamic_cast<DcmDataSourceListViewItem*>(item);
   if(dcmfile_item || datasource_item)
   {
     _database_listview->setCurrentItem( item );
     _database_listview->setSelected( item, true );
	 QMenu database_listview_popup(this);
     database_listview_popup.move(p);
     if(dcmfile_item)
     {     
       QAction* close=database_listview_popup.addAction(tr("&Close Database"));
       QAction* filelist=database_listview_popup.addAction(tr("&File List"));
       QAction* res=database_listview_popup.exec();
       update();
       if(res==close)
         close_database(dcmfile_item);
       else if(res=filelist)
       {
         dcmfile_item->show_files(p);
         _database_listview->setSelected( item, false ); 
       }  
     }
     else if ( datasource_item ) 
     {
//        QAction* copy=database_listview_popup.addAction(tr("&Copy"));
       QAction* browse=database_listview_popup.addAction(tr("&Data Browser"));
       QAction* filelist=database_listview_popup.addAction(tr("&File List"));
       QAction* metadata=database_listview_popup.addAction(tr("&Meta Data"));
       QAction* selectitem=database_listview_popup.addAction(tr("&Select DataSource"));     
       QAction* res=database_listview_popup.exec();
       update();
       if(res==metadata)
         datasource_item->show_meta_data(p);
       else if(res==browse)
           datasource_item->browse_data(p,_time_editor);
       else if(res=filelist)
       {
         QDOS::TimeInterval tintv;
         bool valid=_time_editor->get_time_interval(tintv);
         datasource_item->show_files(p,datasource_item->name(),tintv);
       }
       else if(res==selectitem)
         add_item_to_selection(item);
//        else if (res==copy)
//          copy_item_to_clipboard(datasource_item);
       _database_listview->setSelected( item, false );   
     }
   }
   // This line moved as will crash on some installations if 
   // called following close_database operation as item is deleted 
   // by this call 
   //_database_listview->setSelected( item, false );   
}
#endif

#if QT_VERSION < 0x040000
void DataSelectionListView::delete_database_entries(QDOS::CDFDataBaseDataSource_var cdf_db)
{
  if(!cdf_db.is_nil())
  {
     Q_LIST_VIEW_ITEM_ITERATOR itr(this);
     QList<DataSelectionListViewItem> lst;
     for(;itr.current();++itr)
     {
        DataSelectionListViewItem *data_select_item=\
          dynamic_cast<DataSelectionListViewItem*>(itr.current());
        if(data_select_item)
        {
           if(cdf_db==data_select_item->data_base())
             lst.append(data_select_item);
        }
     }
     for(DataSelectionListViewItem* data_select_item=lst.first();\
       data_select_item;data_select_item=lst.next())
         delete data_select_item;
  }
}
#else
void DataSelectionListView::delete_database_entries(QDOS::CDFDataBaseDataSource_var cdf_db)
{
    if(!cdf_db.is_nil())
    {
        Q_LIST_VIEW_ITEM_ITERATOR itr(this);
        QList<DataSelectionListViewItem*> lst;
        for(;itr.current();++itr)
        {
            DataSelectionListViewItem *data_select_item=\
                    dynamic_cast<DataSelectionListViewItem*>(itr.current());
            if(data_select_item)
            {
                if(cdf_db==data_select_item->data_base())
                    lst.append(data_select_item);
            }
        }
        /*for(DataSelectionListViewItem* data_select_item=lst.first();\
            data_select_item;data_select_item=lst.next())
            delete data_select_item;*/
        for(QList<DataSelectionListViewItem*>::iterator iter=lst.begin(); iter!=lst.end(); iter++)
            delete *iter;
    }
}
#endif

void DataSelector::paste_item_from_clipboard()
{
  QString paste_text=_clipboard->text();
  if(paste_text.isNull() || paste_text.isEmpty())
    return;
  string datasource_str(paste_text.latin1());
  string database_delim(DATABASE_DELIM);
  int pos=datasource_str.find(database_delim+"/");
  string database_name;
  if(pos!=string::npos)
    database_name=datasource_str.substr(0,pos);
  if( database_name.size()!=0)
  {
    try{
      QDOS::CDFDataBaseDataSource_var cdf_db=get_database(\
        database_name);
      QDOS::Name n(datasource_str);
      string col0=n[n.num_components()-1].get_id();
      n.delete_component(n.num_components()-1);
      string col1=n.srep();
      _data_selection_listview->add_item(col0,col1,cdf_db);
    }
    catch(QDOS::Exception& e)
    {;}
  }
}

void DataSelector::update_selected_data_display()
{
  _data_files_list->clear();
#if QT_VERSION < 0x040000
  _data_coverage->setProgress(0);
#else
  _data_coverage->setValue(0);
#endif
  _data_info->setText("");
  if( _data_selection_listview->childCount() == 0 )
  {
    _clear_button->setEnabled(false);
    _remove_button->setEnabled(false);
    _get_data_button->setEnabled(false);
    return;
  }
  else
  {
    _remove_button->setEnabled(true);
    _clear_button->setEnabled(true);
    _add_button->setEnabled(false);
    _get_data_button->setEnabled(true);
    Q_LIST_VIEW_ITEM* item = _data_selection_listview->firstChild();
    bool no_item_selected = true;
    QDOS::TimeInterval tintv;
    bool valid=_time_editor->get_time_interval(tintv);
    while( item && valid)
    {
       if( item->isSelected() )
       {
         no_item_selected = false;
         DataSelectionListViewItem* data_select_item=\
           dynamic_cast<DataSelectionListViewItem*>(item);
         vector<string> fnames;
         vector<string> info_txt;
         if(data_select_item)
         {
             data_select_item->get_files(tintv,fnames);
             int coverage=data_select_item->get_data_coverage(tintv);
#if QT_VERSION < 0x040000
             _data_coverage->setProgress(coverage);
#else
	     _data_coverage->setValue(coverage);
#endif
             data_select_item->get_data_info(tintv,info_txt);
         }         
         
         for(int i=0;i<fnames.size();i++)
	 {
#if QT_VERSION < 0x040000
             _data_files_list->insertItem(fnames[i].c_str());  
#else
	     _data_files_list->addItem(QString(fnames[i].c_str()));
#endif
	 }
         for(int i=0;i<info_txt.size();i++)
             (i==0)?_data_info->setText(info_txt[i].c_str()):_data_info->append(\
               info_txt[i].c_str());   
         break;
       }
       item = item->nextSibling();
    }
 
  }

}

void DataSelector::update_database_selection()
{
  _add_button->setEnabled(false);
  Q_LIST_VIEW_ITEM_ITERATOR itr(_database_listview);
  for(;itr.current();++itr)
  {
    if(itr.current()->isSelected())
    {
      _add_button->setEnabled(true);
      break;
    }
  }
}

void DataSelector::get_data()
{
  emit status_msg("Fetching data...");
  qApp->processEvents();
  int num= _data_selection_listview->childCount();
  if( num == 0 )
    return;
  QDOS::TimeInterval tintv;
  bool valid=_time_editor->get_time_interval(tintv);
  if(valid)
  {
    Q_LIST_VIEW_ITEM* item = _data_selection_listview->firstChild();
#if QT_VERSION < 0x040000
    QProgressDialog progress_dlg( "Fetching data...", "Abort", num,
                              this, "progress", TRUE );
#else
    QProgressDialog progress_dlg( "Fetching data...", "Abort", 0, num, this, Qt::Dialog);
#endif
    progress_dlg.setCaption("Get Data");
    progress_dlg.show();
    qApp->processEvents();     
    int data_i=0;
    QApplication::setOverrideCursor(Qt::waitCursor);
    while( item )
    {
#if QT_VERSION < 0x040000
       progress_dlg.setProgress(data_i++);
#else
       progress_dlg.setValue(data_i++);
#endif
       qApp->processEvents();
#if QT_VERSION < 0x040000
       if(progress_dlg.wasCancelled())
#else
       if(progress_dlg.wasCanceled())
#endif
       {
         emit status_msg("Aborted...");
         QApplication::restoreOverrideCursor();
         qApp->processEvents();
         return;
       }
       DataSelectionListViewItem* data_select_item=\
         dynamic_cast<DataSelectionListViewItem*>(item);
       if(data_select_item)
       {
         string datasource=data_select_item->name();
         try
         {
           // get data object
           QDOS::QdObject_var obj=data_select_item->get_data(tintv);
           if(!obj.is_nil())
	   {
	     obj->rename_xref(TTAGS,TIME_TAGS);
             vector<string> xrefs;
             obj->list_xref_names(xrefs);
             for(int i=0;i<xrefs.size();i++)
             {
               QdObject_var xref_obj=obj->get_xref(xrefs[i]);
               xref_obj->rename_xref(DATA_PTR_NAME,PTR_VAR_NAME);
             }
	     Name data_name(datasource);
	     string data_str=(\
			     data_name.get_component(\
				     data_name.num_components()-1)).get_id();
             char * sanitizedVarName = QuSanitizeName(data_str.c_str());
             QwuiPutOnWL(obj, sanitizedVarName, "Data");
             free(sanitizedVarName);
             string msg("Done: ");
             msg+=data_str;
             msg+=" on Working List";
             emit status_msg(msg.c_str());
	   } 
           else
             throw QDOS::VarIsNil("Data Object is nil");            
         }
         catch(QDOS::Exception& e)
         {
           emit status_msg("Error in read."); 
           QApplication::restoreOverrideCursor();
           qApp->processEvents();
           string errmsg("Error fetching Data Source "+datasource+\
             ":"+e.msg());
           switch(QMessageBox::warning(this,"Data Selection",\
             errmsg.c_str(),\
             QMessageBox::Ignore|QMessageBox::Default,\
             QMessageBox::Abort|QMessageBox::Escape))
           {
             case QMessageBox::Ignore:
               default:
               QApplication::setOverrideCursor(Qt::waitCursor);
               break;
             case QMessageBox::Abort:
               return;
           }
         } // end catch  
       }   // end if
       item = item->nextSibling();
    } // end while
#if QT_VERSION < 0x040000
    progress_dlg.setProgress(num); 
#else
    progress_dlg.setValue(num); 
#endif
    QApplication::restoreOverrideCursor();
  } // end if valid
  emit status_msg("Done.");
  emit done_signal();
  qApp->processEvents();
}

void DataSelector::check_time()
{
  static QDOS::TimeInterval old_tintv;
  QDOS::TimeInterval tintv;
  bool valid=_time_editor->get_time_interval(tintv);
  if(valid && !_database_tintv.empty() && tintv!=old_tintv)
  {
    bool reset_end=false;
    bool reset_start=false;
    if((tintv.end_abs()!=old_tintv.end_abs()) && \
      (_database_tintv.end_abs()<tintv.end_abs()))
    {
      string errmsg("End time beyond end of database");
      switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
      "Ignore","Reset"))
      {
        case 0:
        default:
          break;
        case 1:
          reset_end=true; 
          break;
       }
    }
   
    if((tintv.start_abs()!=old_tintv.start_abs()) && \
      (_database_tintv.start_abs()>tintv.start_abs()))
    {
      string errmsg("Start time before start of database");
      switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
      "Ignore","Reset"))
      {
        case 0:
        default:
          break;
        case 1:
          reset_start=true; 
          break;
       }
    }
    if(reset_start && reset_end)
    {
      QDOS::TimeInterval ntintv(_database_tintv.start_abs(),\
        _database_tintv.end_abs(),tintv.get_sense());
      old_tintv=ntintv;
      _time_editor->set_time_interval(ntintv);
    }
    else if (reset_start)
    {
      QDOS::TimeInterval ntintv;
      if(tintv.end_abs()<_database_tintv.start_abs())
        ntintv.set(_database_tintv.start_abs(),\
          _database_tintv.start_abs()+_time_editor->get_duration(),\
          tintv.get_sense());
      else
         ntintv.set(_database_tintv.start_abs(),tintv.end_abs(),\
           tintv.get_sense());
      old_tintv=ntintv;
      _time_editor->set_time_interval(ntintv);
    }
    else if (reset_end)
    {
      QDOS::TimeInterval ntintv;
      if(tintv.start_abs()>_database_tintv.end_abs())
        ntintv.set(_database_tintv.end_abs()-_time_editor->get_duration(),\
          _database_tintv.end_abs(),tintv.get_sense());
      else
        ntintv.set(tintv.start_abs(),_database_tintv.end_abs(),\
          tintv.get_sense());
      old_tintv=ntintv;
      _time_editor->set_time_interval(ntintv);
    }
    else
    {
      old_tintv=tintv;
      emit time_checked();
    }
  }
  else
    emit time_checked();
}
    
void DataSelector::update_time()
{    
  // update time interval
  if(!_database_tintv.empty())
  { 
    QDOS::Time end_time=_database_tintv.end_abs();
    QDOS::Time start_time=_database_tintv.start_abs();
    //QDOS::TimeInterval new_tintv=QDOS::TimeInterval(end_time-QDOS::Days(1),\
    //  end_time);       
    QDOS::TimeInterval new_tintv=QDOS::TimeInterval(start_time,\
      end_time);       
    _time_editor->set_time_interval(new_tintv);
    update_data_availability();
  }
}

void DataSelector::update_data_availability()
{
  // update data availability pixel maps
  // fetch time interval
  QDOS::TimeInterval tintv;
  bool valid=_time_editor->get_time_interval(tintv);

  if(valid)
  {
     QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
     qApp->processEvents(2000);
#else
     qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
  // valid time interval, so get time tags over which to determine
  // availability from current resolution setting
      int w=_database_listview->columnWidth(1);
      _davail_sz=(w-2)/DA_PIXEL_WIDTH;
      
      QDOS::Seconds resolution_seconds(\
        tintv.duration_abs()/(QDOS::Time::TRep)_davail_sz);
      
      _davail_timeseq.assign_regular(_davail_sz+1,tintv.start(),resolution_seconds);
      if(tintv.end()!=_davail_timeseq[_davail_sz])
         _davail_timeseq[_davail_sz]=tintv.end();
      vector<bool> avail(_davail_timeseq.size()-1);
      Q_LIST_VIEW_ITEM_ITERATOR itr( _database_listview );
      // iterate through all items of the listview
      for ( ; itr.current(); ++itr)
      {
          DataBaseListViewItem* item=\
              dynamic_cast<DataBaseListViewItem*>(itr.current());
          if(item)
          {
          // if current item is a data source name, then check
          // data availability
              if(item->type()==DataSelector::LVI_DATASOURCENAME)
              {
                  string item_name=item->name();
                  try{
                                     
                      for(int j=1;j<_davail_timeseq.size();j++)
                      {
                          QDOS::TimeInterval ltintv(_davail_timeseq[j-1],\
                            _davail_timeseq[j]);
                          // wants to be a generic DataSource which has a 
                          // virtual member is_data_available
                          QDOS::CDFDataBaseDataSource_var cdf_db=\
				  item->data_base();
                          if(!cdf_db.is_nil())
                          {
                            avail[j-1]=cdf_db->is_data_available(\
                              item_name,ltintv);
                          }
             
                      }
                  }
                  catch(QDOS::Exception& e)
                  {;}
                  const QPixmap& pm=data_availability_pixmap(avail);
                  item->setPixmap(1,pm);
              }
          }
      } 
      QApplication::restoreOverrideCursor();
#if QT_VERSION < 0x040000
      qApp->processEvents(2000);
#else
      qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
  } 
  else
    update_time(); 
}
       
   
void DataSelector::add_database_folder_items(
  QDOS::DataNamesTable_var folder, DataBaseListViewItem* list_view_item_parent 
)
{

  QDOS::CDFDataBaseDataSource_var cdf_db=list_view_item_parent->data_base();
  
  QDOS::TimeInterval tintv; 
  bool valid=_time_editor->get_time_interval(tintv);

  QDOS::DataNamesTable::iterator itr;
  string item_name;
  
  for( itr=folder->begin();
       itr!=folder->end();itr++)
  {
    item_name=itr.name();

    if( itr.type() == QDOS::N_CONTEXT )
    {
       DcmFolderListViewItem* new_folder =
         new DcmFolderListViewItem(
           list_view_item_parent, itr->first.c_str(), cdf_db );

       add_database_folder_items( itr->second._nc, new_folder );
       
    } else
    {
       DcmDataSourceListViewItem* new_datasource=\
            new DcmDataSourceListViewItem( list_view_item_parent, itr->first,\
           cdf_db );
    }
  }

}

void DataSelector::close_database(DcmFileListViewItem* dcmfile_item)
{
  if(dcmfile_item==0)
    return;
  string dcmfile=dcmfile_item->text(0).latin1();
  QDOS::CDFDataBaseDataSource_var cdf_db=get_database(dcmfile);
  close_database(cdf_db);
  delete dcmfile_item;
}


void DataSelector::close_database(QDOS::CDFDataBaseDataSource_var cdf_db)
{
   if(cdf_db.is_nil())
     return;
   try{
     string cdf_db_name(database_name(cdf_db));
     QDOS::ExceptionMsg msg;
     msg << "Closing " << cdf_db_name << " ...";
     emit status_msg(msg.c_str());
     QApplication::setOverrideCursor(Qt::waitCursor);
     qApp->processEvents();
     _data_selection_listview->delete_database_entries(cdf_db);
     update_selected_data_display();
     QDOS::Name dcm_n(cdf_db_name);
     _database_table.unbind(dcm_n);
     _open_dcm--;
     if(0>=_open_dcm)
     { 
       _database_tintv=QDOS::TimeInterval();
       emit database_closed();
     }
     else
     {
       DataBaseTable::recursive_iterator itr;
       for(itr=_database_table.recursive_begin();
         itr!=_database_table.recursive_end();itr++)
       {
          if(itr.type()!=QDOS::N_OBJECT)
            continue;
          QDOS::CDFDataBaseDataSource_var db=itr->second.n_object();
          if(db.is_nil())
            continue;
          QDOS::TimeInterval db_tintv;
          db->get_time_range(db_tintv);
          db_tintv=_database_tintv.int_union(db_tintv);
          if(db_tintv!=_database_tintv)
            _database_tintv=db_tintv;
       }
     }
      QApplication::restoreOverrideCursor();
      emit status_msg("");
      qApp->processEvents();
   }
   catch(QDOS::Exception e)
   { 
      QApplication::restoreOverrideCursor();
      emit status_msg("");
      qApp->processEvents();
      cout << "The following error occurred..." << endl;
      e.print_msg();
   }     
}
  

string DataSelector::binary_file(const string& dcm_filename)
{
  QFileInfo dcm_fi(dcm_filename.c_str());
  QDOS::ExceptionMsg bin_filename;
  if(dcm_fi.isSymLink())
  {
    string link_file(dcm_fi.readLink().latin1());
    QFileInfo link_fi(link_file.c_str());
    if(link_fi.isRelative())
    {
      QString dcm_dir_qs(dcm_fi.dirPath());
      if(!dcm_dir_qs.isEmpty())
        bin_filename << dcm_dir_qs.latin1() << "/";
    }
    QString link_dir_qs(link_fi.dirPath());
    if(!link_dir_qs.isEmpty())
      bin_filename << link_dir_qs.latin1() << "/";
    bin_filename << "." << link_fi.baseName().latin1() << ".dat";
    return bin_filename;
  }
  else
  {
    QString dcm_dir_qs(dcm_fi.dirPath());
    if(!dcm_dir_qs.isEmpty())
      bin_filename << dcm_dir_qs.latin1() << "/";
    bin_filename << "." << dcm_fi.baseName().latin1() << ".dat";
    return bin_filename;
  }
}

bool DataSelector::create_binary_file(const string& dcm_filename)
{
  string bin_filename=binary_file(dcm_filename);
  DataBaseTable database_table;
  emit status_msg("Reading DCM file..."); 
  QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
  qApp->processEvents(2000);
#else
  qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
  bool try_again=false;
  try
  {
    QDOS::CDFDataBaseDataSource_var cdf_db = new \
      QDOS::CDFDataBaseDataSource(dcm_filename);
    if(cdf_db->get_dcm_read_status()!=QDOS::CDFDCM_SUCCESS)
    {
      (void)QMessageBox::warning(this,"Read Database",\
	      "Parse error encountered - check console messages");
    }                          
    QDOS::Name n(database_name(dcm_filename));
    try{
      _database_table.bindx(n,cdf_db);
    }
    catch(QDOS::NameAlreadyBound& ne)
    {
      _database_table.unbind(n);
      _database_table.bindx(n,cdf_db);
    }
    database_table.bindx(n,cdf_db); 
  }
  catch(QDOS::Exception& e)
  {
    QApplication::restoreOverrideCursor();
    emit status_msg("Error in DCM read.");
    string errmsg("Could not open "+dcm_filename+": "+e.msg());
    switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
      QMessageBox::Retry|QMessageBox::Default,\
      QMessageBox::Abort|QMessageBox::Escape))
    {
      case QMessageBox::Retry:
      default:
          try_again=true;
          break;
      case QMessageBox::Abort:
          break;
    }
  }
  if(try_again)
    return create_binary_file(dcm_filename);
  emit status_msg("Creating binary file..."); 
#if QT_VERSION < 0x040000
  qApp->processEvents(2000);
#else
  qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
  QFileInfo bin_fi(bin_filename.c_str());
  if(bin_fi.exists())
  {
    if(!QFile::remove(bin_filename.c_str()))
    {
       QApplication::restoreOverrideCursor();
       QDOS::ExceptionMsg errmsg("Warning: Cannot remove binary file ");
       errmsg << bin_filename;
       errmsg << " - insufficient permission rights for current user";
       emit status_msg(errmsg.c_str());
       qApp->processEvents();
       (void)QMessageBox::warning(this,"Create Binary File",errmsg.c_str());
       emit status_msg("");
       qApp->processEvents();
       return false;
     }
  }
  try
  {
    VBostream b_os(bin_filename);
    if(0!=chmod(bin_filename.c_str(),DCM_MODE))
    {
       QApplication::restoreOverrideCursor();
       QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
       errmsg << " on binary file " << bin_filename;
       emit status_msg(errmsg.c_str());
       qApp->processEvents();
       (void)QMessageBox::warning(this,"Create Binary File",errmsg.c_str());
       emit status_msg("");
       qApp->processEvents();
       QApplication::setOverrideCursor( Qt::waitCursor );
       qApp->processEvents();
    }
    b_os << database_table;
    QApplication::restoreOverrideCursor();
    emit status_msg("");
    qApp->processEvents();
  }
  catch(QDOS::Exception& e)
  {
    QApplication::restoreOverrideCursor();
    QDOS::ExceptionMsg errmsg("Error creating binary file  ");
    errmsg << bin_filename;
    emit status_msg(errmsg.c_str());
    qApp->processEvents();
    (void)QMessageBox::warning(this,"Create Binary File",errmsg.c_str());
    emit status_msg("");
    qApp->processEvents();
    return false;
  }
  return true;
}
  
string DataSelector::database_name( const string& dcm_filename)
{
  string ext(DCM_FEXT);
  int ext_sz=std::min(ext.size(),dcm_filename.size());
  int ext_offset=dcm_filename.size()-ext_sz;
  string root_name;
  if(dcm_filename.substr(ext_offset,ext_sz)==ext)
    root_name=dcm_filename.substr(0,ext_offset);
  else
    root_name=dcm_filename;
  int offset=(root_name.substr(0,1)=="/")?1:0;
  return root_name.substr(offset);
}

QDOS::CDFDataBaseDataSource_var DataSelector::get_database( const string& \
  dcm_filename )
{
  QDOS::Name name=database_name(dcm_filename);
  QDOS::CDFDataBaseDataSource_var cdf_db=0;
  try{
    cdf_db=_database_table.resolve(name);
  }
  catch(QDOS::Exception& e)
  {
     e.print_msg();
  }
  return cdf_db;
}

void DataSelector::add_database( QDOS::CDFDataBaseDataSource_var cdf_db)
{
   if(cdf_db.is_nil())
     return;
   QDOS::TimeInterval db_tintv;
   cdf_db->get_time_range(db_tintv);
   db_tintv=_database_tintv.int_union(db_tintv);
   if(db_tintv!=_database_tintv)
     _database_tintv=db_tintv;

   QDOS::TimeInterval tintv;
   bool valid=_time_editor->get_time_interval(tintv);

   string root_name=database_name(cdf_db);  
    
   DcmFileListViewItem* root_item =
     new DcmFileListViewItem( _database_listview, root_name.c_str(), cdf_db);

   QDOS::DataNamesTable_var data_names = cdf_db->get_data_names();

   QDOS::DataNamesTable::iterator itr;
   for( itr=data_names->begin();
     itr!=data_names->end();itr++)
   {
     string item_name=itr.name();
     if( itr.type() == QDOS::N_CONTEXT )
     {
        DcmFolderListViewItem* lvi_parent =
          new DcmFolderListViewItem( root_item, itr->first.c_str(), cdf_db );
        add_database_folder_items( itr->second._nc, lvi_parent );
       
     }
     else
     {
        DcmDataSourceListViewItem* lvi_datasource_item=\
           new DcmDataSourceListViewItem( root_item, itr->first, cdf_db );
     }
   }
   if(!valid || !_time_updated)
     update_time();
   else
     update_data_availability();
   database_listview_expand(true);
}

DataSelectorWindow::DataSelectorWindow( QWidget *parent, const char *name )
: QguWindow(parent,name)
{
  QguWindow::_saveset_filter=QString("Data selection savesets (*.qs_dataSelector)");
  _data_selector = new DataSelector(this,"data_selector");
  setCentralWidget( _data_selector );
  connect( _data_selector, SIGNAL(quit_signal()), this, SLOT(hide()) );
  connect( _data_selector, SIGNAL(done_signal()), this, SLOT(done()) );

  _menu_bar = menuBar();
  _menu_bar->clear();
#if QT_VERSION < 0x040000
  QPopupMenu* file_menu = new QPopupMenu(this,"file_menu");
#else
  QMenu* file_menu = new QMenu(this);
#endif

  file_menu->clear();
  file_menu->insertItem("&Save", this, SLOT(save()), Qt::CTRL+Qt::Key_S);
  file_menu->insertItem("Save &As...", this, SLOT(saveAs()), Qt::CTRL+Qt::Key_A);
  file_menu->insertSeparator();
  file_menu->insertItem("&Include...", this, SLOT(include()), Qt::CTRL+Qt::Key_R);
  file_menu->insertItem("&Restore...", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
  file_menu->insertSeparator();
  file_menu->insertItem("&Close", this, SLOT(hide()));
  _menu_bar->insertItem("&File", file_menu );

#if QT_VERSION < 0x040000
  QPopupMenu* database_menu = new QPopupMenu(this,"database_menu");
  database_menu->clear();
  database_menu->insertItem("&Open...", \
    this, SLOT(open_database()));
  _close_menu_item=\
    database_menu->insertItem("&Close Database",_data_selector,SLOT(close_database()));
  _create_menu_item=\
    database_menu->insertItem("C&reate Database Entries",this,SLOT(create_database()));
  _merge_menu_item=\
    database_menu->insertItem("&Merge Database Entries",this,SLOT(merge_database()));
  _update_menu_item=\
    database_menu->insertItem("&Update Database Entries",this,SLOT(update_database()));
  database_menu->insertSeparator();
  database_menu->insertItem("&Expand Tree",this,SLOT(expand_tree()));
  database_menu->insertItem("Co&llapse Tree",this,SLOT(collapse_tree()));
  _menu_bar->insertItem("&Database", database_menu );
  _menu_bar->insertSeparator();
  msgBar->message("Ready");

  connect(_data_selector,SIGNAL(status_msg(const QString&)),\
    msgBar,SLOT(message(const QString&)) );
  connect(_data_selector,SIGNAL(database_closed()),\
    this,SLOT(database_closed()) );
  connect(_data_selector,SIGNAL(database_opened()),\
    this,SLOT(database_opened()) );
  this->setMinimumSize(this->sizeHint());
  _menu_bar->setItemEnabled(_close_menu_item,FALSE); 
  
#else
  QMenu* database_menu = new QMenu(this);
  database_menu->clear();
  database_menu->insertItem("&Open...", \
    this, SLOT(open_database()));
  _close_menu_item=\
    database_menu->addAction("&Close Database",_data_selector,SLOT(close_database()));
  _create_menu_item=\
    database_menu->addAction("C&reate Database Entries",this,SLOT(create_database()));
  _merge_menu_item=\
    database_menu->addAction("&Merge Database Entries",this,SLOT(merge_database()));
  _update_menu_item=\
    database_menu->addAction("&Update Database Entries",this,SLOT(update_database()));
  database_menu->addSeparator();
  database_menu->addAction("&Expand Tree",this,SLOT(expand_tree()));
  database_menu->addAction("Co&llapse Tree",this,SLOT(collapse_tree()));
  _menu_bar->insertItem("&Database", database_menu );
  _menu_bar->addSeparator();
  msgBar->message("Ready");

  connect(_data_selector,SIGNAL(status_msg(const QString&)),\
    msgBar,SLOT(message(const QString&)) );
  connect(_data_selector,SIGNAL(database_closed()),\
    this,SLOT(database_closed()) );
  connect(_data_selector,SIGNAL(database_opened()),\
    this,SLOT(database_opened()) );
  this->setMinimumSize(this->sizeHint());
  _close_menu_item->setEnabled(false); 
  
#endif

  
#if QT_VERSION < 0x040000
  QPopupMenu* help_menu=new QPopupMenu(this,"help_menu");
#else
  QMenu * help_menu=new QMenu(this);
#endif
  help_menu->insertItem("Qsas Help Topics...",this, SLOT(launch_QSAS_Help()));
  help_menu->insertItem("Help on &Data Selection...", this,SLOT(launch_DS_Help()));
  help_menu->insertSeparator();
  help_menu->insertItem("About Qsas...",qsasWindow->help_window,SLOT(about()));
  _menu_bar->insertItem("&Help",help_menu, Qt::CTRL+Qt::Key_H);
 
  this->_filename += QString("/saveSet");
  this->SAVE_EXTENSION = QString(".qs_dataSelector");
  this->_filename += this->SAVE_EXTENSION;
  this->fromSession = false;
  
  this->connect(qsasWindow, SIGNAL(saveSessionRequest(QString)),  this, SLOT(saveSession(QString)) );
  this->connect(qsasWindow, SIGNAL(restoreSessionRequest(QString)), this, SLOT(restoreSession(QString)) );
 
}

void DataSelectorWindow::launch_QSAS_Help()
{ qsasWindow->help_window->manEntry("QSAS_help_index.html"); }  

void DataSelectorWindow::launch_DS_Help()
{ qsasWindow->help_window->manEntry("QSAS_data_selection.html"); }

void DataSelectorWindow::database_closed()
{
#if QT_VERSION < 0x040000
  _menu_bar->setItemEnabled(_create_menu_item,TRUE);
  _menu_bar->setItemEnabled(_merge_menu_item,TRUE);
  _menu_bar->setItemEnabled(_close_menu_item,FALSE); 
  _menu_bar->setItemEnabled(_update_menu_item,TRUE); 
#else
	_create_menu_item->setEnabled(true);
	_merge_menu_item->setEnabled(true);
	_close_menu_item->setEnabled(false);
	_update_menu_item->setEnabled(true);
#endif
}

void DataSelectorWindow::database_opened()
{
#if QT_VERSION < 0x040000
  _menu_bar->setItemEnabled(_create_menu_item,FALSE);
  _menu_bar->setItemEnabled(_merge_menu_item,FALSE);
  _menu_bar->setItemEnabled(_close_menu_item,TRUE); 
  _menu_bar->setItemEnabled(_update_menu_item,FALSE); 
#else
	_create_menu_item->setEnabled(false);
	_merge_menu_item->setEnabled(false);
	_close_menu_item->setEnabled(true);
	_update_menu_item->setEnabled(false);
#endif
}

void DataSelectorWindow::open_database()
{ open_database("");}

void DataSelectorWindow::open_database(const string& start_with)
{
  static string root_dir;
  if(root_dir=="") 
  {
	  char * root_dir_cs=new char[QSAS_profiles->current()->ConfigPath.length()+1];
	  strcpy(root_dir_cs, QSAS_profiles->current()->ConfigPath.latin1());
#ifdef WINDOWS
    if(strlen(root_dir_cs)==0)
    {
	    delete root_dir_cs;
      root_dir_cs=getenv("USERPROFILE");
    }
#else
    if(strlen(root_dir_cs)==0)
    {
	    delete root_dir_cs;
      root_dir_cs=getenv("PWD");
    }
#endif
    root_dir=root_dir_cs;
  }

  string root_name;

  if(start_with!="")
    root_name=start_with;
  else
   root_name=root_dir;

  QString qs_dcm_filename =
    QFileDialog::getOpenFileName( root_name.c_str(), "Database Configuration(*.dcm)",
     this, "dcm_file_select_dialog", "Open Database Configuration File" );
  if ( !qs_dcm_filename.isEmpty() ) 
  {
#if QT_VERSION < 0x040000
    _menu_bar->setItemEnabled(_create_menu_item,FALSE);
    _menu_bar->setItemEnabled(_merge_menu_item,FALSE);
    _menu_bar->setItemEnabled(_close_menu_item,TRUE);
    _menu_bar->setItemEnabled(_update_menu_item,FALSE);
#else
	_create_menu_item->setEnabled(false);
	_merge_menu_item->setEnabled(false);
	_close_menu_item->setEnabled(true);
	_update_menu_item->setEnabled(false);
#endif

    string dcm_filename( qs_dcm_filename.latin1() );
    qApp->processEvents();
    QDOS::CDFDataBaseDataSource_var cdf_db=\
     _data_selector->read_database(dcm_filename);
    qApp->processEvents();
    _data_selector->add_database(cdf_db);
    QString new_root_dir=QFileInfo(qs_dcm_filename).dirPath();
    if(!new_root_dir.isEmpty())
      root_dir=new_root_dir.latin1();
  }
  
  _data_selector->update_data_availability();
}

void DataSelectorWindow::save(Vostream& v_os)
{
	string databaseName;
	Q_LIST_VIEW_ITEM * item;
	DataSelectionListViewItem * selItem;

	string guiID;
	guiID=QSAS_VERSION_ID;
	v_os << guiID;   

	int cnt = 0;
	item = _data_selector->_database_listview->firstChild();
	while (item){
		cnt++;
		item = item->nextSibling();
	}
	v_os << cnt;

   item = _data_selector->_database_listview->firstChild();
   while (item){
     QString itemText = item->text(0);
	 #ifndef WINDOWS
     itemText.prepend("/");
	 #endif
     databaseName = string(itemText.latin1());
     v_os << databaseName;
     item = item->nextSibling();
   }

   // count and save selection list
   selItem = (DataSelectionListViewItem*)_data_selector->_data_selection_listview->firstChild();
   int ct = 0;
   while (selItem){
     ct++;
     selItem = (DataSelectionListViewItem*)selItem->nextSibling();
   }

   v_os << ct;
   selItem = (DataSelectionListViewItem*)_data_selector->_data_selection_listview->firstChild();
   while (selItem){
     string itemText0 = string(selItem->text(0).latin1());
     string itemText1 = string(selItem->text(1).latin1());
     v_os << itemText0;
     v_os << itemText1;
     string db_name;
     if(!selItem->data_base().is_nil())
       selItem->data_base()->get_database_name(db_name);
     v_os << db_name;
     //v_os << selItem->data_base();
     selItem = (DataSelectionListViewItem*)selItem->nextSibling();
   }

   _data_selector->_time_editor->save(v_os);
}
void DataSelectorWindow::restore(Vistream& v_is,bool replace)
{
	string guiID; 
	v_is >> guiID;
	char *ptr;
   
	if( QuSlotDebug() ) printf("DataSelectorWindow::restore set ID =  %s\n", guiID.c_str());
  
	// QDS always used QSAS_VERSION_ID 
	setRestoreVersion(guiID.c_str());

  int cnt;

  // clear list view of open dcm files
  if(replace)
  {
    DcmFileListViewItem *nextItem =
      (DcmFileListViewItem*)_data_selector->_database_listview->firstChild();;
    while(nextItem)
    {
      DcmFileListViewItem * item = nextItem;
      nextItem = (DcmFileListViewItem*)item->nextSibling();
      _data_selector->close_database(item);
    }
  }

  // read in dcm files listed in save file
  v_is >> cnt;
  for (int i=0; i<cnt; i++)
  {
    string dcm_filename;
    v_is >> dcm_filename;

    dcm_filename+=string(DCM_FEXT);

#if QT_VERSION < 0x040000
    _menu_bar->setItemEnabled(_create_menu_item,FALSE);
    _menu_bar->setItemEnabled(_merge_menu_item,FALSE);
    _menu_bar->setItemEnabled(_close_menu_item,TRUE);
    _menu_bar->setItemEnabled(_update_menu_item,FALSE);
#else
	_create_menu_item->setEnabled(false);
	_merge_menu_item->setEnabled(false);
	_close_menu_item->setEnabled(true);
	_update_menu_item->setEnabled(false);
#endif

   string no_file_msg("Restored session contains an unrecognised database:"\
     +dcm_filename+"\nIgnore error messages");

    try{
      qApp->processEvents();
      QFileInfo dcm_fi(dcm_filename.c_str());
      if(!dcm_fi.exists())
      {
        (void)QMessageBox::warning(this,"Restore",no_file_msg.c_str());
         status(no_file_msg.c_str());
         cout << no_file_msg << endl;
      }
      else
      {
        QDOS::CDFDataBaseDataSource_var cdf_db=\
        _data_selector->read_database(dcm_filename);
        qApp->processEvents();
        _data_selector->add_database(cdf_db);
      }
    }
    catch(Exception& e)
    {
      (void)QMessageBox::warning(this,"Restore",e.msg().c_str());
      status(e.msg().c_str());
      cout << e.msg() << endl;
    }
  }

  // clear selection list view
  if(replace)
  {
    DataSelectionListViewItem *next =
      (DataSelectionListViewItem*)_data_selector->_data_selection_listview->firstChild();;
    while(next)
    {
      DataSelectionListViewItem * item = next;
      next = (DataSelectionListViewItem*)item->nextSibling();
      delete item;
    }
  }
   // read in selection

  CDFDataBaseDataSource_var cdfdb;
  int ct;
  v_is >> ct;
  for (int i=0; i<ct; i++){
    string text0;
    string text1;
    v_is >> text0;
    v_is >> text1;
    if(qsas_restore_release < 20101)
    {
       v_is >> cdfdb;
       string db_name;
       if(!cdfdb.is_nil())
       {
         cdfdb->get_database_name(db_name);
		 cdfdb=_data_selector->get_database(db_name);
       }
    }
    else
    {
      string db_name;
      v_is >> db_name;
      cdfdb=_data_selector->get_database(db_name);
    }
    _data_selector->_data_selection_listview->add_item(text0, text1, cdfdb);
  }
  _data_selector->_time_editor->restore(v_is);

}

void DataSelectorWindow::save(){

 if(QuSlotDebug())
 {
  printf("DataSelectorWindow::save %ld\n", this);
  fflush(stdout);
 }
 

    if(_filename.isEmpty())
    {
       saveAs();
       return;
    }
    
    try
    {
      VXDRostream xdr_os(_filename.latin1());
      save(xdr_os);
    }
    catch(Exception &e)
    {
       status(e.msg().c_str(),10000);   
       return;
    }
    status(_filename+" saved",10000);
    return;

}
  
void DataSelectorWindow::saveAs(){

 if(QuSlotDebug())
 {
  printf("DataSelectorWindow::saveAs %ld\n", this);
  fflush(stdout);
  }
 

   QString _filter = QString("save (*");
   _filter += SAVE_EXTENSION;
   _filter += QString(")");
   QString fn=QFileDialog::getSaveFileName(_filename, _filter, this, "save", "Select file - Save Import Settings");
   if(!fn.isEmpty())
   {
     int pos = fn.findRev(SAVE_EXTENSION);
     if (pos < 0) {
       pos = fn.findRev(".");
       if (pos > 0) fn.truncate(pos);
       fn += SAVE_EXTENSION;
     }
     _filename=fn;
     save();
   }
   else
     status("saving aborted ",10000);  
}
  
void DataSelectorWindow::saveSession(QString folder){

  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.latin1());
  
  // set session folder and save
     this->_filename = QString(folder.latin1());
     this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;
  save();

  // put file name back
  _filename = QString(fileN.latin1());

}
  
QDOS::CDFDataBaseDataSource_var DataSelector::read_database(const \
  string& dcm_filename, bool read_dcm)
{
  QDOS::CDFDataBaseDataSource_var cdf_db;
  string dcm_file;
  QFileInfo dcm_fi(dcm_filename.c_str());
  if(dcm_fi.isSymLink())
  {
     string link_file(dcm_fi.readLink().latin1());
     QFileInfo link_fi(link_file.c_str());
     if(link_fi.isRelative())
     {
        QString dcm_dir_qs(dcm_fi.dirPath());
        if(!dcm_dir_qs.isEmpty())
        {
          dcm_file=dcm_dir_qs.latin1();
          dcm_file+="/";
        }
     }
     QString link_dir_qs(link_fi.dirPath());
     if(!link_dir_qs.isEmpty())
     {
       dcm_file+=link_dir_qs.latin1();
       dcm_file+="/";
     }
     dcm_file+=link_fi.fileName().latin1();
  } 
  else
     dcm_file=dcm_filename;
         
  if(!read_dcm)
  {
    // get binary file name
    string bin_filename=binary_file(dcm_file);
    QFileInfo bin_fi(bin_filename.c_str());
    //bool read_dcm=false;
    // if binary file does not exist create
    if(bin_fi.exists())
    {
      try{
        emit status_msg("Reading binary file..."); 
        QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
        qApp->processEvents(2000);
#else
        qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
        VBistream b_is(bin_filename);
        b_is >> _database_table;
        QApplication::restoreOverrideCursor();
        emit status_msg("");
        qApp->processEvents();
        // check that no decms have been updated later than binary file
        QFileInfo bin_fi(bin_filename.c_str());
        QDateTime bin_ds=bin_fi.lastModified();
        vector<string> dcm_files;
        cdf_db=get_database(dcm_file);
        if(cdf_db.is_nil())
	{
          read_dcm=true;
	}
        else
	{
       	  cdf_db->get_dcm_filenames(dcm_files);
	}
        for(int i=0;i<dcm_files.size();i++)
        {
          QFileInfo dcm_fi(dcm_files[i].c_str());
          if(bin_ds>=dcm_fi.lastModified())
            continue;
          read_dcm=true;
          try{
           QDOS::Name dcm_n(database_name(cdf_db));
           _database_table.unbind(dcm_n);
          }
          catch(QDOS::Exception& e3)
          {;}
          break;
        }
      }
      catch(QDOS::Exception& e)
      {
	e.print_msg();      
        QApplication::restoreOverrideCursor();
        emit status_msg("");
        qApp->processEvents();
        read_dcm=true;
        try{
          QDOS::Name dcm_n(database_name(cdf_db));
          _database_table.unbind(dcm_n);
        }
        catch(QDOS::Exception& e2)
        {e2.print_msg();}
      } 
    } 
    else
      read_dcm=true;
  } 
  if(read_dcm)
  {
    (void)create_binary_file(dcm_file);
    cdf_db=get_database(dcm_file);
  }
  if(!cdf_db.is_nil())
    _open_dcm++;
  return cdf_db;
}

void DataSelector::close_database()
{ 
  //  Select dcm file to close 
  QStringList dcm_name_lst;
#if QT_VERSION < 0x040000
  QList<DcmFileListViewItem> dcm_item_lst; 
#else
  QList<DcmFileListViewItem*> dcm_item_lst; 
#endif
  Q_LIST_VIEW_ITEM_ITERATOR it(_database_listview);
  for(;it.current();it++)
  {
    DcmFileListViewItem* dcm_file_item=\
      dynamic_cast<DcmFileListViewItem*>(it.current());
    if(dcm_file_item)
    {
      dcm_name_lst << dcm_file_item->text(0);
      dcm_item_lst.append(dcm_file_item);
    }
  }
 
  QWidget *input_dialog_widget=new QWidget(this);
  QString dcm_file="";
  if(dcm_name_lst.count()>1)
  {
    bool ok=FALSE;
    dcm_file=QInputDialog::getItem(tr("Please select database to close"),\
      tr("Database = "),dcm_name_lst,0,FALSE,&ok,input_dialog_widget);
    if(!ok)
      return;
  }
  else
    dcm_file=dcm_name_lst[0];
  delete input_dialog_widget;
  if (dcm_file.isEmpty() ) 
  { 
    string errmsg("No Database(.dcm) Name Specified");
    (void)QMessageBox::warning(this,"Close Database",errmsg.c_str());
    return;
  }
  int ix=dcm_name_lst.findIndex(dcm_file);
  if(ix==-1)
    return;
  if(ix<dcm_item_lst.count())
      close_database(dcm_item_lst.at(ix));
}
 
void DataSelectorWindow::create_database()
{
  // Select data root directory
  char* data_dir_cs=getenv("DATA_DIR");
  if(!data_dir_cs)
    data_dir_cs=getenv("PWD");
  string data_dir;
  if(data_dir_cs)
    data_dir=data_dir_cs;
  QString data_dir_qs=QFileDialog::getExistingDirectory(data_dir.c_str(),this,\
    "data_dir_select_dialog","Select Data Directory",TRUE);
  if(data_dir_qs.isEmpty())
  {
    status("No Data Directory selected...");
    return;
  }
  data_dir=data_dir_qs.latin1();
  if(data_dir.substr(data_dir.size()-1,1)=="/")
        data_dir=data_dir.substr(0,data_dir.size()-1);

  char* data_name_root_cs=getenv("DATA_NAME_ROOT");
  string data_name_root;
  if(data_name_root_cs)
    data_name_root=data_name_root_cs;

  bool ok=FALSE;
  QString data_name_root_qs=QuTextInputDialog("Input Data Root Name",\
    "DATA_NAME_ROOT = ", data_name_root.c_str(),&ok,this);
  if(!ok)
  {
    status("Cancelled...");
    return;
  }

  if(!data_name_root_qs.isEmpty())
  {
    data_name_root=data_name_root_qs.latin1();
    if(data_name_root.substr(0,1)=="/")
      data_name_root=data_name_root.substr(1);
    if(data_name_root.size())
    {
      if(data_name_root.substr(data_name_root.size()-1,1)=="/")
        data_name_root=data_name_root.substr(0,data_name_root.size()-1);
    }
  }
  
  // Prompt for filename
  char * config_dir_cs=new char[QSAS_profiles->current()->ConfigPath.length()+1];
  strcpy(config_dir_cs, QSAS_profiles->current()->ConfigPath.latin1());
  if(strlen(config_dir_cs)==0)
  {
	delete config_dir_cs;
    config_dir_cs=getenv("PWD");
  }

  string config_dir;
  if(config_dir_cs)
    config_dir=config_dir_cs;

  bool try_again=true;
  QDOS::ExceptionMsg dcm_filename;
  while(try_again)
  {
    QString dcm_filename_qs=
      QFileDialog::getSaveFileName(config_dir.c_str(),"Database Configuration Map(*.dcm)",
        this,"dcm_file_select_dialog","Save As Database Configuration Map file");

    if(dcm_filename_qs.isEmpty())
    {
      string errmsg("No Database(.dcm) name specified");
      (void)QMessageBox::warning(this,"Create Database Entries",errmsg.c_str());
      return;
    }
    //dcm_filename=dcm_filename_qs.latin1();
    QFileInfo dcm_fi(dcm_filename_qs);
    QString dcm_dir_qs(dcm_fi.dirPath());
    if(!dcm_dir_qs.isEmpty())
      dcm_filename << dcm_dir_qs.latin1() << "/";
    //dcm_filename << "." << dcm_fi.baseName().latin1() << ".dcm";
    dcm_filename << dcm_fi.baseName().latin1() << ".dcm";
    if(dcm_fi.exists())
    {
      string errmsg(dcm_filename+" already exists. Overwrite?");
      switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
       "&Yes","&No","&Cancel",
       0, // Enter == button 0
       2)) // Escape == button 2
      {
        case 0:
          try_again=false;
          break;
        case 1:
          break;
        case 2:
          status("Cancelled...");
          return;
      }
    }
    else
      try_again=false;
  }

  update();
  qApp->processEvents();

  status("Generating dcm entries...");
  QApplication::setOverrideCursor(Qt::waitCursor);
 
  try{
    create_dcm_entries(data_dir,data_name_root,dcm_filename);
  }
  catch(QDOS::Exception& e)
  {
    QApplication::restoreOverrideCursor();
    return;
  }

  QApplication::restoreOverrideCursor();

  status(string("Database Entries created for "+dcm_filename).c_str());
  _data_selector->create_binary_file(dcm_filename);
  status("");
  open_database(dcm_filename);
  status("");
}

#if QT_VERSION < 0x040000
void DataSelectorWindow::create_dcm_entries(const string& data_dir_str,\
  const string& data_name_root_str,const string& dcm_filename)
{
// select all *.cdf files and create a .dcm file to contain contents
  QDir data_dir(data_dir_str.c_str());
  data_dir.setFilter(QDir::Files);
  data_dir.setSorting(QDir::Time);
  data_dir.setNameFilter("*.cdf");

  const QFileInfoList *cdf_file_list=data_dir.entryInfoList();
  QFileInfoListIterator itr(*cdf_file_list);
  QFileInfo *cdf_fi;

  double resolution=600;
 
  QFileInfo dcm_fi(dcm_filename.c_str());
  string dcm_filename_root(dcm_fi.baseName().latin1());

  string dcm_data_filename(dcm_fi.dirPath().latin1());
  dcm_data_filename+="/.";
  dcm_data_filename+=dcm_filename_root+"_data.dcm";

  FAR::FARData merge_dcm_entries;
  FAR::Record alias_rec(vector<string>((int)DCM_ALIAS_NUM_FLDS));
  alias_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_ALIAS_RECORD);
  FAR::Record merge_rec(vector<string>((int)DCM_MERGE_NUM_FLDS));
  merge_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_MERGE_RECORD);
  merge_rec[(int)DCM_MERGE_FILE_PRIORITY]="0";

  FAR::FARData data_dir_dcm_entries;
  FAR::Record data_dir_rec(vector<string>((int)DCM_DATA_DIR_NUM_FLDS));
  data_dir_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_DATA_DIR_RECORD);
  data_dir_rec[(int)DCM_DATA_DIR_NAME_FLD]="$(DATA_DIR)/.";
  data_dir_dcm_entries.add_record(data_dir_rec);
  data_dir_dcm_entries.write_to_file(dcm_data_filename,FAR::FARWRITEFILE_NEW);
  if(0!=chmod(dcm_data_filename.c_str(),DCM_MODE))
  {
     QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
     errmsg << " on " << dcm_data_filename;
     status(errmsg.c_str());
     cout << errmsg << endl;
  }

  int num=cdf_file_list->count();
  if(num)
  {
     QProgressDialog progress_dlg(\
      string("Generating dcm entries for "+data_dir_str).c_str(),\
             "Abort",num,this,"");
     progress_dlg.setCaption("Generating Database Entries");
     progress_dlg.setMinimumDuration(3000);
     progress_dlg.show();
     qApp->processEvents();
     int ind=0;

     while((cdf_fi=itr.current()))
     {
       string cdf_filename=data_dir_str+"/"+cdf_fi->fileName().latin1();
       progress_dlg.setLabelText(\
         string("Generating dcm entries for "+cdf_filename).c_str());
       progress_dlg.setProgress(ind++);
       status(string(cdf_filename+": "+int_to_string(ind)+" of "+\
         int_to_string(num)).c_str());
       qApp->processEvents();
       if(progress_dlg.wasCancelled())
       {
         QApplication::restoreOverrideCursor();
	 string errmsg("Abort?");
         switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
          QMessageBox::No|QMessageBox::Default, \
	  QMessageBox::Yes,
          QMessageBox::Cancel | QMessageBox::Escape ))
          {
            case QMessageBox::Yes:
	      status("Aborted");
              throw QDOS::Exception("Create Database Entries Aborted");
	      break;
	      
            case QMessageBox::No: 
            case QMessageBox::Cancel:
            default:
	      progress_dlg.reset();
              progress_dlg.setLabelText(\
                string("Generating dcm entries for "+cdf_filename).c_str());
	      progress_dlg.setProgress(ind);
	      progress_dlg.show();
              qApp->processEvents();

	    break;
         }
  
       }
       ++itr;
 
       FAR::FARData data_dcm_entries;
       //string cdf_filename=data_dir_str+"/"+cdf_fi->fileName().latin1();
       try{
          QDOS::CDF_TSFileDataSource cdf_ts_file(cdf_filename);
          cdf_ts_file.get_dcm_entries(data_dcm_entries,"$(DATA_NAME_ROOT)/",\
            "$(DATA_DIR)/",resolution);
          data_dcm_entries.write_to_file(dcm_data_filename,\
            FAR::FARWRITEFILE_APPEND);
       }
       catch(QDOS::Exception&e )
       {
         string errmsg("Error reading "+ cdf_filename); 
         status(errmsg.c_str());
       }
    }
    progress_dlg.setProgress(num);
    qApp->processEvents();
  
  }

  status("");
  qApp->processEvents();
  
  alias_rec[(int)DCM_ALIAS_NAME_FLD]="DATA_DIR";
  alias_rec[(int)DCM_ALIAS_VALUE_FLD]=data_dir_str;
  merge_dcm_entries.add_record(alias_rec);

  alias_rec[(int)DCM_ALIAS_NAME_FLD]="DATA_NAME_ROOT";
  alias_rec[(int)DCM_ALIAS_VALUE_FLD]=data_name_root_str;
  merge_dcm_entries.add_record(alias_rec);

  merge_rec[(int)DCM_MERGE_FILE_NAME]=dcm_data_filename;
  merge_dcm_entries.add_record(merge_rec);
  
  // now step thru' sub-directories
  data_dir.setFilter(QDir::Dirs | QDir::Readable);
  data_dir.setNameFilter("*");
  const QFileInfoList *dir_list=data_dir.entryInfoList();
  QFileInfoListIterator dir_itr(*dir_list);
  QFileInfo *dir_fi;
  while((dir_fi=dir_itr.current()))
  {
    ++dir_itr;
    if(dir_fi->fileName()=="." || dir_fi->fileName()=="..")
      ;
    else if (dir_fi->isDir())
    {
      string data_dir_i(data_dir_str+"/"+dir_fi->fileName().latin1());
      string data_name_root_i(data_name_root_str+"/"+dir_fi->fileName().latin1());
      string merge_dcm(dcm_fi.dirPath().latin1());
      merge_dcm+=("/"+dcm_filename_root)+"_"+dir_fi->fileName().latin1();
      merge_dcm+=".dcm";
      create_dcm_entries(data_dir_i,data_name_root_i,merge_dcm);
      merge_rec[(int)DCM_MERGE_FILE_NAME]=merge_dcm;
      merge_dcm_entries.add_record(merge_rec);
    }
  }
  merge_dcm_entries.write_to_file(dcm_filename,FAR::FARWRITEFILE_NEW);
  if(0!=chmod(dcm_filename.c_str(),DCM_MODE))
  {
     QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
     errmsg << " on " << dcm_filename;
     status(errmsg.c_str());
     cout << errmsg << endl;
  }
}
#else
void DataSelectorWindow::create_dcm_entries(const string& data_dir_str,\
        const string& data_name_root_str,const string& dcm_filename)
{
// select all *.cdf files and create a .dcm file to contain contents
    QDir data_dir(data_dir_str.c_str());
    data_dir.setFilter(QDir::Files);
    data_dir.setSorting(QDir::Time);
    data_dir.setNameFilter("*.cdf");

    QFileInfoList cdf_file_list=data_dir.entryInfoList();
    QFileInfoListIterator itr=cdf_file_list.begin();
    //QFileInfo *cdf_fi;

    double resolution=600;
 
    QFileInfo dcm_fi(dcm_filename.c_str());
    string dcm_filename_root(dcm_fi.baseName().latin1());

    string dcm_data_filename(dcm_fi.dirPath().latin1());
    dcm_data_filename+="/.";
    dcm_data_filename+=dcm_filename_root+"_data.dcm";

    FAR::FARData merge_dcm_entries;
    FAR::Record alias_rec(vector<string>((int)DCM_ALIAS_NUM_FLDS));
    alias_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_ALIAS_RECORD);
    FAR::Record merge_rec(vector<string>((int)DCM_MERGE_NUM_FLDS));
    merge_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_MERGE_RECORD);
    merge_rec[(int)DCM_MERGE_FILE_PRIORITY]="0";

    FAR::FARData data_dir_dcm_entries;
    FAR::Record data_dir_rec(vector<string>((int)DCM_DATA_DIR_NUM_FLDS));
    data_dir_rec[(int)DCM_RECORD_TYPE_FLD]=string(DCM_DATA_DIR_RECORD);
    data_dir_rec[(int)DCM_DATA_DIR_NAME_FLD]="$(DATA_DIR)/.";
    data_dir_dcm_entries.add_record(data_dir_rec);
    data_dir_dcm_entries.write_to_file(dcm_data_filename,FAR::FARWRITEFILE_NEW);
    if(0!=chmod(dcm_data_filename.c_str(),DCM_MODE))
    {
        QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
        errmsg << " on " << dcm_data_filename;
        status(errmsg.c_str());
        cout << errmsg << endl;
    }

    int num=cdf_file_list.count();
    if(num)
    {
        QProgressDialog progress_dlg(\
                string("Generating dcm entries for "+data_dir_str).c_str(),\
                "Abort",0,num,this,Qt::Dialog);
        progress_dlg.setCaption("Generating Database Entries");
        progress_dlg.setMinimumDuration(3000);
        progress_dlg.show();
        qApp->processEvents();
        int ind=0;

        while(/*(cdf_fi=itr.current())*/itr!=cdf_file_list.end())
        {
            string cdf_filename=data_dir_str+"/"+(*itr).fileName().latin1();
            progress_dlg.setLabelText(\
                    string("Generating dcm entries for "+cdf_filename).c_str());
            progress_dlg.setValue(ind++);
            status(string(cdf_filename+": "+int_to_string(ind)+" of "+\
                    int_to_string(num)).c_str());
            qApp->processEvents();
            if(progress_dlg.wasCanceled())
            {
                QApplication::restoreOverrideCursor();
                string errmsg("Abort?");
                switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
                       QMessageBox::No|QMessageBox::Default, \
                               QMessageBox::Yes,
                               QMessageBox::Cancel | QMessageBox::Escape ))
                {
                    case QMessageBox::Yes:
                        status("Aborted");
                        throw QDOS::Exception("Create Database Entries Aborted");
                        break;
	      
                    case QMessageBox::No: 
                    case QMessageBox::Cancel:
                    default:
                        progress_dlg.reset();
                        progress_dlg.setLabelText(\
                                string("Generating dcm entries for "+cdf_filename).c_str());
                        progress_dlg.setValue(ind);
                        progress_dlg.show();
                        qApp->processEvents();

                        break;
                }
  
            }
            ++itr;
 
            FAR::FARData data_dcm_entries;
       //string cdf_filename=data_dir_str+"/"+cdf_fi->fileName().latin1();
            try{
                QDOS::CDF_TSFileDataSource cdf_ts_file(cdf_filename);
                cdf_ts_file.get_dcm_entries(data_dcm_entries,"$(DATA_NAME_ROOT)/",\
                        "$(DATA_DIR)/",resolution);
                data_dcm_entries.write_to_file(dcm_data_filename,\
                        FAR::FARWRITEFILE_APPEND);
            }
            catch(QDOS::Exception&e )
            {
                string errmsg("Error reading "+ cdf_filename); 
                status(errmsg.c_str());
            }
        }
        progress_dlg.setValue(num);
        qApp->processEvents();
  
    }

    status("");
    qApp->processEvents();
  
    alias_rec[(int)DCM_ALIAS_NAME_FLD]="DATA_DIR";
    alias_rec[(int)DCM_ALIAS_VALUE_FLD]=data_dir_str;
    merge_dcm_entries.add_record(alias_rec);

    alias_rec[(int)DCM_ALIAS_NAME_FLD]="DATA_NAME_ROOT";
    alias_rec[(int)DCM_ALIAS_VALUE_FLD]=data_name_root_str;
    merge_dcm_entries.add_record(alias_rec);

    merge_rec[(int)DCM_MERGE_FILE_NAME]=dcm_data_filename;
    merge_dcm_entries.add_record(merge_rec);
  
  // now step thru' sub-directories
    data_dir.setFilter(QDir::Dirs | QDir::Readable);
    data_dir.setNameFilter("*");
    QList<QFileInfo> dir_list=data_dir.entryInfoList();
    QList<QFileInfo>::iterator dir_itr=dir_list.begin();

    while(dir_itr!=dir_list.end())
    {
        if((*dir_itr).fileName()=="." || (*dir_itr).fileName()=="..")
            ;
        else if ((*dir_itr).isDir())
        {
            string data_dir_i(data_dir_str+"/"+(*dir_itr).fileName().latin1());
            string data_name_root_i(data_name_root_str+"/"+(*dir_itr).fileName().latin1());
            string merge_dcm(dcm_fi.dirPath().latin1());
            merge_dcm+=("/"+dcm_filename_root)+"_"+(*dir_itr).fileName().latin1();
            merge_dcm+=".dcm";
            create_dcm_entries(data_dir_i,data_name_root_i,merge_dcm);
            merge_rec[(int)DCM_MERGE_FILE_NAME]=merge_dcm;
            merge_dcm_entries.add_record(merge_rec);
        }
        ++dir_itr;
    }
    merge_dcm_entries.write_to_file(dcm_filename,FAR::FARWRITEFILE_NEW);
    if(0!=chmod(dcm_filename.c_str(),DCM_MODE))
    {
        QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
        errmsg << " on " << dcm_filename;
        status(errmsg.c_str());
        cout << errmsg << endl;
    }
}
#endif

void DataSelectorWindow::update_database()
{
  // Save As Database Configuration (*.dcm) 
 char * root_dir_cs=new char[QSAS_profiles->current()->ConfigPath.length()+1];
  strcpy(root_dir_cs, QSAS_profiles->current()->ConfigPath.latin1());
  if(strlen(root_dir_cs)==0)
  {
	delete root_dir_cs;
    root_dir_cs=getenv("PWD");
  }

  string root_dir;
  if(root_dir_cs)
    root_dir=root_dir_cs;

  QString qs_dcm_filename =
    QFileDialog::getOpenFileName( root_dir.c_str(), "Database Configuration(*.dcm)",
     this, "dcm_file_select_dialog","Select Database For Updating" );
  
  string dcm_filename;
  if ( qs_dcm_filename.isEmpty() ) 
  { 
    string errmsg("No Database(.dcm) Name Specified");
    (void)QMessageBox::warning(this,"Update Database Entries",errmsg.c_str());
    return;
  }
  dcm_filename=qs_dcm_filename.latin1();

  bool try_again=true;
  FAR::FARWriteFileMode write_mode=FAR::FARWRITEFILE_APPEND;
  while(try_again)
  {
    std::ofstream outfile(dcm_filename.c_str(),ios::app);
    if(!outfile)
    {
      string errmsg("Could not open "+dcm_filename+" in write mode");
      switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
        QMessageBox::Retry|QMessageBox::Default,\
        QMessageBox::Abort|QMessageBox::Escape))
      {
        case QMessageBox::Retry:
        default:
          try_again=true;
          break;
        case QMessageBox::Abort:
          try_again=false;
          return;
      }
    }
    else
    {
      outfile.close();
      try_again=false;
    }
  }
  QDOS::CDFDataBaseDataSource_var cdf_db=_data_selector->read_database(\
    dcm_filename);

  // Get list of datafile names and directories
  QDOS::DataNamesTable_var data_names=cdf_db->get_datafile_names();
  _data_selector->close_database(cdf_db);

  if(data_names->is_empty())
  {
    status("No files in Database for updating!");
    return;
  }

  QStringList data_dir_lst;
  QApplication::setOverrideCursor(Qt::waitCursor);
  
  bool updated=false;

  for(QDOS::DataNamesTable::recursive_iterator itr=data_names->recursive_begin();\
    itr!=data_names->recursive_end();itr++)
  {
    if(itr.type()!=QDOS::N_OBJECT)
      continue;

    // get directory
    string data_file_name=itr.name();
    if(data_file_name.substr(0,1)!="/")
      data_file_name="/"+data_file_name;
    
    // get data dcm file
    string dcm_datafile(itr->second.n_object());
  
    QFileInfo data_dcm_fi(dcm_datafile.c_str());
    if(!data_dcm_fi.exists())  
    {
      string errmsg("No data dcm file exists for ");
      errmsg+=data_file_name;
      status(errmsg.c_str());
      cout << errmsg << endl;
      continue;
      // in this instance should create it and the merge file (or append to
      // if need be
    }

    QFileInfo data_name_fi(data_file_name.c_str());
    QDir data_dir;
    if(data_name_fi.isDir())
      data_dir=QDir(data_file_name.c_str());
    else
    {
      if(!data_name_fi.exists())
      {
        FAR::FARData data_dcm_entries;
        try
	{
          string cdf_ts_file("$(DATA_DIR)/");
	  cdf_ts_file+=data_name_fi.fileName().latin1();
          status(string("Deleting dcm entries for "+data_file_name).c_str());
	  data_dcm_entries.read_from_file(dcm_datafile);
	  int r;
	  while(-1!=\
		(r=data_dcm_entries.match_record_by_field(\
		 (int)DCM_CDF_TS_DATA_FILE_NAME_FLD,cdf_ts_file)
		)
	       )
	  {
	    data_dcm_entries.delete_record(r);
	  }
	  data_dcm_entries.write_to_file(dcm_datafile,FAR::FARWRITEFILE_NEW); 
          updated=true;
        }
        catch(QDOS::Exception& e)
        {
          string errmsg("Error deleting dcm entries for "+data_file_name);
          errmsg+=" to database file "+dcm_datafile+": ";
          errmsg+=e.msg();
          status(errmsg.c_str());
          cout << errmsg << endl;
        }
        continue;	      
      }
      else
        data_dir=data_name_fi.dir();
    }
    QString data_dir_qs=data_dir.absPath(); 
    if(-1==data_dir_lst.findIndex(data_dir_qs))
      data_dir_lst << data_dir_qs;
    else
      continue;
  
    string data_dir_str=data_dir_qs.latin1();
    if(!data_dir.exists())
    {
      string err_msg(data_dir_str+" does not exist!");
      status(err_msg.c_str());
      cout << err_msg << endl;
      continue;
    }
   
    status(string("Updating "+data_dir_str).c_str());
    
    // get list of cdf files
    data_dir.setFilter(QDir::Files);
    data_dir.setSorting(QDir::Time);
    data_dir.setNameFilter("*.cdf");
 
#if QT_VERSION < 0x040000
    const QFileInfoList *cdf_file_list=data_dir.entryInfoList();
    QFileInfoListIterator cdf_itr(*cdf_file_list);
    QFileInfo *cdf_fi;
  
    int num=cdf_file_list->count();
    qApp->processEvents();
    int ind=0;
  
    status("Checking files...");
 
    while((cdf_fi=cdf_itr.current()))
    {
      ++cdf_itr;
      string cdf_filename(cdf_fi->filePath().latin1());
      try{
        status(cdf_filename.c_str());
        int offset=0;
        if(cdf_filename.substr(0,1)=="/")
          offset=1;
        data_names->resolve(cdf_filename.substr(offset));
      }
      catch(QDOS::NamingException& ne)
      {
        FAR::FARData data_dcm_entries;
        try{
          status(string("Generating dcm entries for "+cdf_filename).c_str());
          double resolution=600;
          QDOS::CDF_TSFileDataSource cdf_ts_file(cdf_filename);
          cdf_ts_file.get_dcm_entries(data_dcm_entries,"$(DATA_NAME_ROOT)/",\
            "$(DATA_DIR)/",resolution);
          data_dcm_entries.write_to_file(dcm_datafile,FAR::FARWRITEFILE_APPEND);
          updated=true;
        }
        catch(QDOS::Exception& e)
        {
          string errmsg("Error writing dcm entries for "+cdf_filename);
          errmsg+=" to database file "+dcm_datafile+": ";
          errmsg+=e.msg();
          status(errmsg.c_str());
          cout << errmsg << endl;
        }
      }
    }
#else
    QList<QFileInfo> cdf_file_list=data_dir.entryInfoList();
    QList<QFileInfo>::iterator cdf_itr=cdf_file_list.begin();
  
    int num=cdf_file_list.count();
    qApp->processEvents();
    int ind=0;
  
    status("Checking files...");
 
    while(cdf_itr!=cdf_file_list.end())
    {
        string cdf_filename((*cdf_itr).filePath().latin1());
        try{
            status(cdf_filename.c_str());
            int offset=0;
            if(cdf_filename.substr(0,1)=="/")
                offset=1;
            data_names->resolve(cdf_filename.substr(offset));
        }
        catch(QDOS::NamingException& ne)
        {
            FAR::FARData data_dcm_entries;
            try{
                status(string("Generating dcm entries for "+cdf_filename).c_str());
                double resolution=600;
                QDOS::CDF_TSFileDataSource cdf_ts_file(cdf_filename);
                cdf_ts_file.get_dcm_entries(data_dcm_entries,"$(DATA_NAME_ROOT)/",\
                        "$(DATA_DIR)/",resolution);
                data_dcm_entries.write_to_file(dcm_datafile,FAR::FARWRITEFILE_APPEND);
                updated=true;
            }
            catch(QDOS::Exception& e)
            {
                string errmsg("Error writing dcm entries for "+cdf_filename);
                errmsg+=" to database file "+dcm_datafile+": ";
                errmsg+=e.msg();
                status(errmsg.c_str());
                cout << errmsg << endl;
            }
        }
        ++cdf_itr;
    }
#endif
  }
  QApplication::restoreOverrideCursor();
  if(updated)
  {
    status("Database updated...");
    (void)_data_selector->create_binary_file(dcm_filename);
  }
  else
    status("No updates to be made...");
  open_database(dcm_filename);
  status("");
}

void DataSelectorWindow::merge_database()
{
	char * root_dir_cs=new char[QSAS_profiles->current()->ConfigPath.length()+1];
  strcpy(root_dir_cs, QSAS_profiles->current()->ConfigPath.latin1());
  if(strlen(root_dir_cs)==0)
  {
	  delete root_dir_cs;
    root_dir_cs=getenv("PWD");
  }

  string root_dir;
  if(root_dir_cs)
    root_dir=root_dir_cs;
  
  //  Select dcm files
  QStringList s(QFileDialog::getOpenFileNames(\
    "Database Configuration Maps(*.dcm)",\
    root_dir.c_str(),this,"dcm_merge_dcm_select_dialog",\
    "Select Database Configuration Maps For Merging"));
 
  // Save As Database Configuration (*.dcm) 
  QString qs_dcm_filename =
    QFileDialog::getSaveFileName( root_dir.c_str(), "Database Configuration(*.dcm)",
     this, "dcm_file_select_dialog","Save As Database Configuration" );
  
  string dcm_filename;
  if ( qs_dcm_filename.isEmpty() ) 
  { 
    string errmsg("No Database(.dcm) Name Specified");
    (void)QMessageBox::warning(this,"Merge Databases",errmsg.c_str());
    return;
  }
  dcm_filename=qs_dcm_filename.latin1();

  bool try_again=true;
  FAR::FARWriteFileMode write_mode;
  while(try_again)
  {
    write_mode=FAR::FARWRITEFILE_APPEND;
    std::ofstream outfile(dcm_filename.c_str(),ios::app);
    if(!outfile)
    {
      write_mode=FAR::FARWRITEFILE_NEW;
      std::ofstream outfile(dcm_filename.c_str(),ios::out);
      if(!outfile)
      {
         string errmsg("Could not open "+dcm_filename+" in write mode");
         switch(QMessageBox::warning(this,"Data Selection", errmsg.c_str(),\
           QMessageBox::Retry|QMessageBox::Default,\
           QMessageBox::Abort|QMessageBox::Escape))
         {
           case QMessageBox::Retry:
           default:
             try_again=true;
             break;
           case QMessageBox::Abort:
             try_again=false;
             return;
         }
      }
      else
      {
        outfile.close();
        try_again=false;
      }
    }
    else
    {
      outfile.close();
      try_again=false;
    }
  }

  bool ok=FALSE;
  double priority=QInputDialog::getDouble(tr("Enter default file priority"),\
    tr("File Priority = "),0,0,2147483647,2,&ok,this);

  int num=s.count();  
  status("Generating merge dcm entries..."); 
#if QT_VERSION < 0x040000
  QProgressDialog progress_dlg( "Generating merge dcm entries...", "Abort", num,
    this, "progress", TRUE );
#else
  QProgressDialog progress_dlg( "Generating merge dcm entries...", "Abort", 0, num,
                                this, Qt::Dialog);
#endif
  progress_dlg.setCaption("Merge Database");
  progress_dlg.setMinimumDuration(3000);
  progress_dlg.show();     
  qApp->processEvents();
  int ind=0;
  bool cont=true;
  QApplication::setOverrideCursor(Qt::waitCursor);
 
  for(QStringList::Iterator it=s.begin();it!=s.end();++it)
  {
#if QT_VERSION < 0x040000
    progress_dlg.setProgress(ind++);
#else
    progress_dlg.setValue(ind++);
#endif
    qApp->processEvents();
#if QT_VERSION < 0x040000
    if(progress_dlg.wasCancelled())
#else
        if(progress_dlg.wasCanceled())
#endif
    {
      QApplication::restoreOverrideCursor();
      status("Aborted");
      break;
    }
    string merge_dcm_filename=(*it).latin1();
    try{
       FAR::FARData dcm_entries;
       const int rec_sz(DCM_MERGE_NUM_FLDS);
       vector<string> flds(rec_sz);
       FAR::Record rec(flds);
       rec[DCM_RECORD_TYPE_FLD]=string(DCM_MERGE_RECORD);
       rec[DCM_MERGE_FILE_PRIORITY]=QDOS::double_to_string(priority);
       rec[DCM_MERGE_FILE_NAME]=merge_dcm_filename;
       dcm_entries.add_record(rec);
       dcm_entries.write_to_file(dcm_filename,write_mode);
       if(it==s.begin())
       {
         write_mode=FAR::FARWRITEFILE_APPEND;
         if(0!=chmod(dcm_filename.c_str(),DCM_MODE))
         {
           QDOS::ExceptionMsg errmsg("Warning: Cannot set group permission rights");
           errmsg << " on " << dcm_filename;
           status(errmsg.c_str());
           cout << errmsg << endl;
         }
       }
    }
    catch(QDOS::Exception& e)
    { 
       QApplication::restoreOverrideCursor();
       status("Error in Merge Database");
       string errmsg("Error merging dcm entries for file "+merge_dcm_filename+\
         ":"+e.msg());
        switch(QMessageBox::warning(this,"Merge Database",\
          errmsg.c_str(),\
          QMessageBox::Ignore|QMessageBox::Default,\
          QMessageBox::Abort|QMessageBox::Escape))
       {
         case QMessageBox::Ignore:
         default:
           cont=true; 
           break;
         case QMessageBox::Abort:
           cont=false;
           return;
       }
       QApplication::setOverrideCursor(Qt::waitCursor);
    }
    if(!cont)
      break;
  }
  QApplication::restoreOverrideCursor();
#if QT_VERSION < 0x040000
  progress_dlg.setProgress(num); 
#else
  progress_dlg.setValue(num);
#endif
  status(string("Databases merged in "+dcm_filename).c_str());
  open_database(dcm_filename);
  status("");
}

void DataSelectorWindow::expand_tree()
{ _data_selector->database_listview_expand(true);}

void DataSelectorWindow::collapse_tree()
{ _data_selector->database_listview_expand(false);}

DcmFileListViewItem::DcmFileListViewItem(
  Q_LIST_VIEW* parent, const string& name, QDOS::CDFDataBaseDataSource_var cdf_db)
    : DataBaseListViewItem(parent,name,cdf_db)
{;}

DcmFolderListViewItem::DcmFolderListViewItem(
  Q_LIST_VIEW_ITEM* parent, const string& name, QDOS::CDFDataBaseDataSource_var 
cdf_db)
    : DataBaseListViewItem(parent,name,cdf_db)
{;}

DcmDataSourceListViewItem::DcmDataSourceListViewItem(
  Q_LIST_VIEW_ITEM* parent, const string& name, QDOS::CDFDataBaseDataSource_var 
cdf_db)
    : DataBaseListViewItem(parent,name,cdf_db),metadata_dialog(0),\
    browsedata_dialog(0)
{;}

DcmDialog::DcmDialog( QDOS::CDFDataBaseDataSource_var datasource,\
  QWidget *parent, const char *name   ) : \
  QDialog( parent, name),_datasource(datasource)
{ 
  timer_id=startTimer(500);
}


DcmFileListDialog::DcmFileListDialog( const string& dcmfile_name, \
  const vector<string>& file_names,  QDOS::CDFDataBaseDataSource_var cdf_db,\
  QWidget *parent, const char *name   ) : \
  DcmDialog( cdf_db,parent, name)
#if QT_VERSION < 0x040000
{
   QGridLayout* filelist_dialog_layout=new QGridLayout(this,2,2);
   filelist_dialog_layout->setSpacing(7);
   filelist_dialog_layout->setMargin(4);
   
   QGroupBox *filelist_groupbox=new QGroupBox(2, Qt::Vertical,\
         "File List", this,"file list groupbox");
        
   QWidget *filelist_widget=new QWidget(filelist_groupbox,"file list widget");
  
   QGridLayout* filelist_top=new QGridLayout(filelist_widget,2,2);
   
   filelist_top->setMargin(4);
   filelist_top->setSpacing(7);
       
   string dcmfile_text="DataBase Name: " + dcmfile_name;
   QLabel *dcmfile_label=new \
     QLabel(dcmfile_text.c_str(),filelist_widget);
   filelist_top->addMultiCellWidget(dcmfile_label,0,0,0,1);
   
   _filelist_listview=new Q_LIST_VIEW(filelist_widget);
   int sz=file_names.size();
   string col_txt(QDOS::double_to_string(sz)+" Files");
   _filelist_listview->addColumn(col_txt.c_str());
   for(int i=0;i<file_names.size();i++)
     new CdfFileListViewItem(_filelist_listview,file_names[i].c_str(),cdf_db);
   _filelist_listview->setMinimumSize(_filelist_listview->sizeHint());
   filelist_top->addMultiCellWidget(_filelist_listview,1,1,0,1);      
   
   if(_filelist_listview->childCount()!=0)
   {
      Q_LIST_VIEW_ITEM* top_item=_filelist_listview->firstChild();
      _filelist_listview->setCurrentItem( top_item );
      _filelist_listview->setSelected( top_item,true );
   }   
 
  connect( _filelist_listview, SIGNAL(rightButtonPressed(QListViewItem *, \
    const QPoint &,int)), this, SLOT(popup_file_menu(QListViewItem *, \
    const QPoint &,int)));

   filelist_dialog_layout->addMultiCellWidget(filelist_groupbox,0,0,0,1);  
      
   QPushButton *close = new QPushButton( "Close", this );
   close->setMinimumSize(close->sizeHint());
   connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
 
   filelist_dialog_layout->addWidget(close,1,1);      
   setCaption("File List");
}
#else
{
// 	QWidget * mainWidget=new QWidget(this);
	
	QVBoxLayout * mainLayout=new QVBoxLayout(this);
	
	QGroupBox * listWidget=new QGroupBox(this);
	QVBoxLayout * listLayout=new QVBoxLayout;
	
	listWidget->setTitle("File List");

	
	// Label
	
	string dcmfile_text="DataBase Name: " + dcmfile_name;
	QLabel *dcmfile_label=new QLabel(dcmfile_text.c_str());
	listLayout->addWidget(dcmfile_label);
	
	// List view
	
	_filelist_listview=new Q_LIST_VIEW(listWidget);
	int sz=file_names.size();
	string col_txt(QDOS::double_to_string(sz)+" Files");
	_filelist_listview->addColumn(col_txt.c_str());
	for(int i=0;i<file_names.size();i++)
		new CdfFileListViewItem(_filelist_listview,file_names[i].c_str(),cdf_db);
	_filelist_listview->setMinimumSize(_filelist_listview->sizeHint());
	listLayout->addWidget(_filelist_listview);
	
	connect( _filelist_listview, SIGNAL(rightButtonPressed(Q3ListViewItem *, \
			const QPoint &,int)), this, SLOT(popup_file_menu(Q3ListViewItem *, \
					const QPoint &,int)));
	
	listWidget->setLayout(listLayout);
	mainLayout->addWidget(listWidget);
	
	QPushButton *close = new QPushButton( "Close", this);
	close->setMinimumSize(close->sizeHint());
	connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
	mainLayout->addWidget(close);
	
// 	mainWidget->setLayout(mainLayout);
	setLayout(mainLayout);
	setCaption("File List");
	
// 	QGridLayout* filelist_dialog_layout=new QGridLayout(this,2,2);
// 	filelist_dialog_layout->setSpacing(7);
// 	filelist_dialog_layout->setMargin(4);
//    
// #if QT_VERSION < 0x040000
// 	QGroupBox *filelist_groupbox=new QGroupBox(2, Qt::Vertical,\
// 			"File List", this,"file list groupbox");
// #else
// 	QGroupBox *filelist_groupbox=new QGroupBox(2, Qt::Vertical,\
// 			"File List", this,"file list groupbox");
// #endif
//         
// 	QWidget *filelist_widget=new QWidget(filelist_groupbox,"file list widget");
//   
// 	QGridLayout* filelist_top=new QGridLayout(filelist_widget,2,2);
//    
// 	filelist_top->setMargin(4);
// 	filelist_top->setSpacing(7);
//        
// 	string dcmfile_text="DataBase Name: " + dcmfile_name;
// 	QLabel *dcmfile_label=new \
// 			QLabel(dcmfile_text.c_str(),filelist_widget);
// 	filelist_top->addMultiCellWidget(dcmfile_label,0,0,0,1);
//    
// 	_filelist_listview=new Q_LIST_VIEW(filelist_widget);
// 	int sz=file_names.size();
// 	string col_txt(QDOS::double_to_string(sz)+" Files");
// 	_filelist_listview->addColumn(col_txt.c_str());
// 	for(int i=0;i<file_names.size();i++)
// 		new CdfFileListViewItem(_filelist_listview,file_names[i].c_str(),cdf_db);
// 	_filelist_listview->setMinimumSize(_filelist_listview->sizeHint());
// 	filelist_top->addMultiCellWidget(_filelist_listview,1,1,0,1);      
//    
// 	if(_filelist_listview->childCount()!=0)
// 	{
// 		Q_LIST_VIEW_ITEM* top_item=_filelist_listview->firstChild();
// 		_filelist_listview->setCurrentItem( top_item );
// 		_filelist_listview->setSelected( top_item,true );
// 	}   
//  
// #if QT_VERSION < 0x040000
// 	connect( _filelist_listview, SIGNAL(rightButtonPressed(QListViewItem *, \
// 			const QPoint &,int)), this, SLOT(popup_file_menu(QListViewItem *, \
// 					const QPoint &,int)));
// #else
// 	connect( _filelist_listview, SIGNAL(rightButtonPressed(Q3ListViewItem *, \
// 			const QPoint &,int)), this, SLOT(popup_file_menu(Q3ListViewItem *, \
// 					const QPoint &,int)));
// #endif
// 
// 	filelist_dialog_layout->addMultiCellWidget(filelist_groupbox,0,0,0,1);  
//       
// 	QPushButton *close = new QPushButton( "Close", this );
// 	close->setMinimumSize(close->sizeHint());
// 	connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
//  
// 	filelist_dialog_layout->addWidget(close,1,1);      
// 	setCaption("File List");
}
#endif

void DcmFileListDialog::popup_file_menu( Q_LIST_VIEW_ITEM *item, \
  const QPoint &p, int) 
{
   CdfFileListViewItem *cdf_file_item=dynamic_cast<CdfFileListViewItem*>(item);
   if(cdf_file_item)
   {
     _filelist_listview->setCurrentItem( item );
     _filelist_listview->setSelected( item, true );
#if QT_VERSION < 0x040000
     QPopupMenu filelist_listview_popup(0,"filelist_listview_popup");
#else
     QMenu filelist_listview_popup(this);
#endif
     filelist_listview_popup.move(p);
     int metadata=filelist_listview_popup.insertItem(tr("&Meta Data"));
     /*int res=*/filelist_listview_popup.exec();
//      if(res==metadata)
       cdf_file_item->show_meta_data(p);
   }
   _filelist_listview->setSelected( item, false );   
}

DcmBrowseDataDialog::DcmBrowseDataDialog( const string& data_name, \
  QDOS::CDFDataBaseDataSource_var cdf_var, QbTimeIntervalEditor* time_editor,\
  QWidget *parent, const char *name   ) : \
  DcmDialog( cdf_var, parent, name),_datasource_str(data_name),\
  _time_editor(time_editor)
#if QT_VERSION < 0x040000
{
   QGridLayout* browse_dialog_layout=new QGridLayout(this,2,2);
   browse_dialog_layout->setSpacing(7);
   browse_dialog_layout->setMargin(4);
   
   QGroupBox *browsedata_groupbox=new QGroupBox(2, Qt::Vertical,\
         "Browse Data", this,"browse data groupbox");
        
   QWidget *browsedata_widget=new QWidget(browsedata_groupbox,\
     "browse data widget");
  
   QGridLayout* browsedata_top=new QGridLayout(browsedata_widget,7,2);
   
   browsedata_top->setMargin(4);
   browsedata_top->setSpacing(7);
       
   string datasource_text="Name: " + data_name;
   QLabel *datasource_label=new 
   QLabel(datasource_text.c_str(),browsedata_widget);
   browsedata_top->addMultiCellWidget(datasource_label,0,0,0,1);

   _timeinterval_lb=new QLabel(browsedata_widget,"time interval label");
   browsedata_top->addMultiCellWidget(_timeinterval_lb,1,1,0,1);

   _entry_start_label=new QLabel(browsedata_widget,"entry start label");
   _entry_start_label->setAlignment(Qt::AlignLeft);
   browsedata_top->addMultiCellWidget(_entry_start_label,2,2,0,0);

   _entry_end_label=new QLabel(browsedata_widget,"entry end label");
   _entry_end_label->setAlignment(Qt::AlignRight);
   browsedata_top->addMultiCellWidget(_entry_end_label,2,2,1,1);

   _num_entries_slider=new QScrollBar(Qt::Horizontal,browsedata_widget,\
      "num entries slider");
   browsedata_top->addMultiCellWidget(_num_entries_slider,3,3,0,1);
  
   string num_entries_txt="Entry No.: ";
   QLabel *num_entries_lb=new QLabel(num_entries_txt.c_str(),\
     browsedata_widget);
   browsedata_top->addMultiCellWidget(num_entries_lb,4,4,0,0);
 
   _num_entries_cb=new QComboBox(TRUE,browsedata_widget);
   _num_entries_cb->setMinimumSize(_num_entries_cb->sizeHint());
   _num_entries_cb->setDuplicatesEnabled(FALSE);
   
   browsedata_top->addMultiCellWidget(_num_entries_cb,4,4,1,1);  
   connect(_num_entries_cb,SIGNAL(highlighted(int)),this,\
     SLOT(browse_data_entry(int)));
   connect(_num_entries_cb,SIGNAL(highlighted(int)),this,\
     SLOT(update_slider_value(int)));
   connect(_num_entries_slider,SIGNAL(valueChanged(int)),this,\
     SLOT(update_entry_num(int)));
   
   _browsedata_tv=new QTextEdit(browsedata_widget,"browsedata textview");
   _browsedata_tv->setMinimumSize(_browsedata_tv->sizeHint());
   //_browsedata_tv->setMargin(5);
   _browsedata_tv->setWordWrap( QTextEdit::WidgetWidth );
   
   _browsedata_tv->setReadOnly(true);
   
   browsedata_top->addMultiCellWidget(_browsedata_tv,5,6,0,1);  
 
   browse_dialog_layout->addMultiCellWidget(browsedata_groupbox,0,0,0,1);  

   connect(_time_editor,SIGNAL(time_updated()),\
     this,SLOT(update_num_entries()));
      
   QPushButton *close = new QPushButton( "Close", this );
   close->setMinimumSize(close->sizeHint());
   connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
 
   browse_dialog_layout->addWidget(close,1,1);      
   setCaption("Browse Data");
   update_num_entries();
}
#else
{
	QVBoxLayout * mainLayout=new QVBoxLayout(this);
	
	QGroupBox* browseGroupbox=new QGroupBox(this);
	browseGroupbox->setTitle("Browse Data");
	QGridLayout * browseLayout=new QGridLayout;
	browseLayout->setMargin(4);
	browseLayout->setSpacing(7);
	
	string datasource_text="Name: " + data_name;
	QLabel *datasource_label=new QLabel(datasource_text.c_str(), browseGroupbox);
	browseLayout->addWidget(datasource_label, 0, 0, 1, 2);
	
	_timeinterval_lb=new QLabel;
	browseLayout->addWidget(_timeinterval_lb, 1, 0, 1, 2);
	
	_entry_start_label=new QLabel;
	browseLayout->addWidget(_entry_start_label, 2, 0, 1, 1, Qt::AlignLeft);
	
	_entry_end_label=new QLabel;
	browseLayout->addWidget(_entry_end_label, 2, 1, 1, 1, Qt::AlignRight);
	
	_num_entries_slider=new QScrollBar(Qt::Horizontal, browseGroupbox);
	browseLayout->addWidget(_num_entries_slider, 3, 0, 1, 2);
	
	string num_entries_txt="Entry No.: ";
	QLabel *num_entries_lb=new QLabel(num_entries_txt.c_str());
	browseLayout->addWidget(num_entries_lb, 4, 0, 1, 1, Qt::AlignLeft);
	
	_num_entries_cb=new QComboBox(TRUE, browseGroupbox);
	_num_entries_cb->setDuplicatesEnabled(FALSE);
	connect(_num_entries_cb,SIGNAL(highlighted(int)),this,\
			SLOT(browse_data_entry(int)));
	connect(_num_entries_cb,SIGNAL(highlighted(int)),this,\
			SLOT(update_slider_value(int)));
	connect(_num_entries_slider,SIGNAL(valueChanged(int)),this,\
			SLOT(update_entry_num(int)));
	connect(_num_entries_cb,SIGNAL(currentIndexChanged(int)),this,\
			SLOT(browse_data_entry(int)));
	browseLayout->addWidget(_num_entries_cb, 4, 1, 1, 1, Qt::AlignRight);
	
	_browsedata_tv=new QTextEdit;
	_browsedata_tv->setWordWrapMode(QTextOption::WrapAnywhere);
	_browsedata_tv->setReadOnly(true);
	browseLayout->addWidget(_browsedata_tv, 5, 0, 1, 2);
	
	
	browseGroupbox->setLayout(browseLayout);
	mainLayout->addWidget(browseGroupbox);
	
	QPushButton *close = new QPushButton( "Close");
	connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
	mainLayout->addWidget(close);
	
	connect(_time_editor,SIGNAL(time_updated()),\
			this,SLOT(update_num_entries())); 
	
	setCaption("Browse Data");
	update_num_entries();
	
}
#endif

void DcmBrowseDataDialog::update_num_entries()
{
   bool valid=_time_editor->get_time_interval(_time_interval);
   if(!valid)
   {
     try{
       datasource()->get_time_range(_datasource_str,_time_interval);
     }
     catch(QDOS::Exception& e)
     {;}
   }
   
   string timeinterval_text="Time Interval: " + _time_interval.iso_srep();
   _timeinterval_lb->setText(timeinterval_text.c_str()); 
   
   int num_entries=0;
   try{
     num_entries=datasource()->get_num_entries(_datasource_str,_time_interval);
   }
   catch(QDOS::Exception& e)
   {;}

   _num_entries_slider->setMinValue(0);
   _num_entries_slider->setMaxValue(num_entries-1);
   _entry_start_label->setNum(0);
   _entry_end_label->setNum(num_entries-1);

   _num_entries_cb->clear();
   for(int i=0;i<num_entries;i++)
   {
     _num_entries_cb->insertItem(QDOS::int_to_string(i).c_str());
     if(i==num_entries-1)
       _num_entries_cb->setCurrentItem(i);
   }
       
}

DcmMetaDataDialog::DcmMetaDataDialog( const string& dname, \
  const vector<string>& mnames,  QDOS::CDFDataBaseDataSource_var datasource,\
  QWidget *parent, const char *name  ) : \
  DcmDialog( datasource,parent, name),datasource_name(dname)
#if QT_VERSION < 0x040000
{
   QGridLayout* meta_dialog_layout=new QGridLayout(this,2,2);
   meta_dialog_layout->setSpacing(7);
   meta_dialog_layout->setMargin(4);
   
   QGroupBox *metadata_groupbox=new QGroupBox(2, Qt::Vertical,\
         "Meta Data", this,"meta data groupbox");
        
   QWidget *metadata_widget=new QWidget(metadata_groupbox,"meta data widget");
  
   QGridLayout* metadata_top=new QGridLayout(metadata_widget,2,2);
   
   metadata_top->setMargin(4);
   metadata_top->setSpacing(7);
       
   string datasource_text="Name: " + dname;
   QLabel *datasource_label=new 
   QLabel(datasource_text.c_str(),metadata_widget);
   metadata_top->addMultiCellWidget(datasource_label,0,0,0,1);
   
   QSplitter *metadata_splitter=new QSplitter(metadata_widget); 
   metadata_top->addMultiCellWidget(metadata_splitter,1,1,0,1);      
   metadata_listbox=new Q_LIST_BOX(metadata_splitter);
   for(int i=0;i<mnames.size();i++)
   {
     metadata_listbox->insertItem(mnames[i].c_str());
   }

   metadata_listbox->setMinimumSize(metadata_listbox->sizeHint());
   
   metadata_tv=new QTextEdit(metadata_splitter,"metadata textview");
   metadata_tv->setMinimumSize(metadata_tv->sizeHint());
   //metadata_tv->setMargin(5);

   metadata_tv->setWordWrap( QTextEdit::WidgetWidth );

   metadata_tv->setReadOnly(true);
   if(metadata_listbox->count()!=0)
   {
      int top_item=metadata_listbox->topItem();
      metadata_listbox->setCurrentItem( top_item );
      metadata_listbox->setSelected( top_item,true );
   }   
 
   connect(metadata_listbox,SIGNAL(selectionChanged()),this,\
     SLOT(show_meta_data()));

   meta_dialog_layout->addMultiCellWidget(metadata_groupbox,0,0,0,1);  
      
   QPushButton *close = new QPushButton( "Close", this );
   close->setMinimumSize(close->sizeHint());
   connect( close, SIGNAL(clicked()), SLOT(reject()) ); 
 
   meta_dialog_layout->addWidget(close,1,1);      
   setCaption("Meta Data");
   show_meta_data();
       
}
#else
{
	QVBoxLayout * mainLayout=new QVBoxLayout(this);
	
	QGroupBox * metaGroupbox=new QGroupBox(this);
	metaGroupbox->setTitle("Meta Data");
	QVBoxLayout * metaLayout=new QVBoxLayout;
	metaLayout->setMargin(4);
	metaLayout->setSpacing(7);
	
	string datasource_text="Name: " + dname;
	QLabel *datasource_label=new QLabel(datasource_text.c_str());
	metaLayout->addWidget(datasource_label);
	
	QSplitter *metadata_splitter=new QSplitter(metaGroupbox);
	metaLayout->addWidget(metadata_splitter);
	
	metadata_listbox=new Q_LIST_BOX(metadata_splitter);
	for(int i=0;i<mnames.size();i++)
	{
		metadata_listbox->addItem(QString(mnames[i].c_str()));
	}

	metadata_listbox->setMinimumSize(metadata_listbox->sizeHint());
   
	metadata_tv=new QTextEdit(metadata_splitter,"metadata textview");
	metadata_tv->setMinimumSize(metadata_tv->sizeHint());
	metadata_tv->setWordWrapMode(QTextOption::WrapAnywhere);
	metadata_tv->setReadOnly(true);
	if(metadata_listbox->count()!=0)
	{
		metadata_listbox->setCurrentRow(0);
	}
	
	connect(metadata_listbox, SIGNAL(currentRowChanged(int)), this, SLOT(show_meta_data()));
	
	metaGroupbox->setLayout(metaLayout);
	
	mainLayout->addWidget(metaGroupbox);

	
	QPushButton *close = new QPushButton("Close");
	connect( close, SIGNAL(clicked()), SLOT(reject()) );
	mainLayout->addWidget(close);
	
	setCaption("Meta Data");
	show_meta_data();
	
       
}
#endif

QDOS::QdObject_var  DataSelectionListViewItem::get_data(\
  const QDOS::TimeInterval& time_interval)
{
  QDOS::QdObject_var obj_out;
  if(!_cdf_db.is_nil())
  {
    try{
      QDOS::DataBuffer_var db=\
        _cdf_db->get_data_data(_datasource_name,time_interval);
      if(db.is_nil())
        return obj_out;
      obj_out=QDOS::Factory::clone(db);
      if(!obj_out.is_nil())
      {
        vector<string> metadata_names;
        _cdf_db->get_meta_names(_datasource_name,metadata_names);
        string root=_datasource_name+"/";
        for(int j=0;j<metadata_names.size();j++)
        {  
          QDOS::QdObject_var xref_obj;
          string meta_name;
          try{        
            meta_name=root+metadata_names[j];
            QDOS::DataBuffer_var xref_db=_cdf_db->get_meta_data(meta_name);
            xref_obj=QDOS::Factory::clone(xref_db);
            if(xref_obj.is_nil())
              continue;
          }
          catch(QDOS::Exception& e)
          { continue;}
          obj_out->change_xref(metadata_names[j],xref_obj);
        }
        QDOS::QdStringSeq_var file_list=new QDOS::QdStringSeq;
        get_files(time_interval,*file_list);
        obj_out->change_xref(PARENT_FILES,file_list);
      } 
        
    }
    catch(QDOS::Exception& e)
    {
    //  QApplication::restoreOverrideCursor();
    //  string errmsg(e.msg());
    //  (void)QMessageBox::warning(listView(),"Get Data",errmsg.c_str());
    //  QApplication::setOverrideCursor(Qt::waitCursor);
    }
  }
  return obj_out;
}

int DataSelectionListViewItem::get_data_coverage(\
  const QDOS::TimeInterval& time_interval)
{
  int res=0;
  if(!_cdf_db.is_nil())
  {
    QDOS::Seconds resolution(time_interval.duration_abs()/\
      (QDOS::Time::TRep)100);
    QDOS::TimeSeq timeseq;
    timeseq.assign_regular(101,time_interval.start(),resolution);
    if(time_interval.end()!=timeseq[100])
      timeseq[100]=time_interval.end(); 
    try{
      for(int j=1;j<=100;j++)
      {
        QDOS::TimeInterval ltintv(timeseq[j-1],timeseq[j]);
        if(_cdf_db->is_data_available(_datasource_name,ltintv))
          res++;
      }
    }
    catch(QDOS::Exception& e)
    {;}
  }
  return res;
}

void DcmBrowseDataDialog::browse_data_entry(int pos)
{
  int entry=0;
  QString num_entries_qs(_num_entries_cb->text(pos));
  istrstream ist(num_entries_qs.latin1());
  ist >> entry;
  int sz=_num_entries_cb->count();
  if(entry<0 || entry>=sz)
    return;
  else
  {
     if(!datasource().is_nil())
     {
       try
       {
          QApplication::setOverrideCursor(Qt::waitCursor);
          qApp->processEvents();
          static QDOS::StringRep string_rep;
          QDOS::DataBuffer_var entry_db=datasource()->get_data_data(\
            _datasource_str,_time_interval);
          QDOS::QdObject_var entry_obj=QDOS::Factory::clone(entry_db);
          string name_txt="Data Entry : " + QDOS::int_to_string(entry);
          string entry_txt=string_rep(entry_obj,entry,1);       
          _browsedata_tv->append(name_txt.c_str());
          _browsedata_tv->append(entry_txt.c_str());
          //_browsedata_tv->setCursorPosition(_browsedata_tv->numLines(),0); //TODO
          QApplication::restoreOverrideCursor();
          qApp->processEvents();
       }
       catch(QDOS::Exception& e)
       { QApplication::restoreOverrideCursor();
         qApp->processEvents();
         string errmsg(e.msg());
         (void)QMessageBox::warning(this,"Browse Data",errmsg.c_str());
       }
     }
  }
}

void DcmMetaDataDialog::show_meta_data()
{
  if( metadata_listbox->count() == 0 )
    return;
  else
  {
#if QT_VERSION < 0x040000
    int item=metadata_listbox->currentItem();
    if(item==-1)
      return;
    else
    {
      int offset=(datasource_name.substr(0,1)=="/")?1:0;
      string name=datasource_name.substr(offset)+\
        "/"+metadata_listbox->text(item).latin1();
#else
      QListWidgetItem * item=metadata_listbox->currentItem();
    if(item==NULL)
	return;
    else
    {
      int offset=(datasource_name.substr(0,1)=="/")?1:0;
      string name=datasource_name.substr(offset)+\
		      "/"+item->text().latin1();
#endif
      if(!datasource().is_nil())
      {
        try
        {
          static QDOS::StringRep string_rep;
          QDOS::DataBuffer_var xref_db=datasource()->get_meta_data(name);
          QDOS::QdObject_var xref_obj=QDOS::Factory::clone(xref_db);
          string name_txt="MetaData Name: " + name;
          metadata_tv->append(name_txt.c_str());
          string xref_txt=string_rep(xref_obj);       
          metadata_tv->append(xref_txt.c_str());
        }
        catch(QDOS::Exception& e)
        { string errmsg(e.msg());
          (void)QMessageBox::warning(this,"Meta Data",errmsg.c_str());
        }
      }
    }
  }
}

// void DataSelectionListView::dropEvent(QDropEvent* event)
// {
//   QString droptext="";
//   if(DcmDataSourceDrag::decode(event,droptext))
//   { 
//     string datasource_str=droptext.latin1();
//     string database_delim=DATABASE_DELIM;
//     int pos=datasource_str.find(database_delim);
//     string database_name;
//     if(pos!=string::npos)
//       database_name=datasource_str.substr(0,pos);
//     if(_data_selector && database_name.size()!=0)
//     {
//       try{
//         QDOS::CDFDataBaseDataSource_var cdf_db=_data_selector->get_database(\
//           database_name);
//         QDOS::Name n(datasource_str);
//         string col0=n[n.num_components()-1].get_id();
//         n.delete_component(n.num_components()-1);
//         string col1=n.srep();
//         add_item(col0,col1,cdf_db);
//       }
//       catch(QDOS::Exception& e)
//       {;}
//     }
//   }
// }

bool DataSelectionListView::add_item(string& col0, string& col1, \
  QDOS::CDFDataBaseDataSource_var cdf_db)
{
  Q_LIST_VIEW_ITEM* list_item=firstChild();
  bool duplicate=false;
  while(list_item && !duplicate)
 {
   if(list_item->text(0).latin1()==col0)
     duplicate=true;
   list_item=list_item->nextSibling();
 }
 if(!duplicate)
 {
   DataSelectionListViewItem *new_item=\
     new DataSelectionListViewItem(this,col0,col1,cdf_db);
   setSelected(new_item,true);
 }
 return duplicate;
}

