
#include "qie_caa.h"
#include "qdutil.h"
#include <dirent.h>
#ifndef WINDOWS
#include <pwd.h>
#endif

// Globals
// ------------------------------------------

FILE * fp_display ;

// ------------------------------------------

void setDebug(bool d_on){
	if(d_on) fp_display = freopen("/dev/stderr", "w", fp_display);
	else fp_display = freopen("/dev/null", "w", fp_display );
}

// ------------------------------------------

void setDebugDisplay(const char * fpath){
	fp_display = freopen(fpath, "w", fp_display);
}


// ------------------------------------------

void initDebug(){
	fp_display = fopen("/dev/null", "w");
}

// -----------------------------------------

void  QieMsg(const char * status, const char *line1, const char *line2, const char *line3  )  
{
   
   std::string msg_line;

					msg_line =  std::string(" ");
  if(line1 != NULL) msg_line += std::string(line1);
					msg_line += " ";
  if(line2 != NULL) msg_line += line2;
					msg_line += " ";
  if(line3 != NULL) msg_line += line3;
  
   if (fp_display != NULL){   // file pointer, may be /dev/NULL 
      fprintf(fp_display, "%s\n", msg_line.c_str());
   }
   
   
   if(strncmp(status, "ERROR", 5) == 0 )
	QuMsgBar( status, msg_line.c_str() , true);
   else
	QuMsgBar( status, msg_line.c_str() , false);

   return;

} 

// ------------------------------------------------

vector <std::string> QieFindHelperFiles(const char * dataFile, const char * dotExtn){

 vector <std::string> helperList;
 
 static const char * str_null = "";
 struct dirent *dir_ent;
 DIR *dir_ptr;
 int ptr;

 static std::string hdPath;
 hdPath = dataFile;
 
 ptr = hdPath.rfind("/");
 hdPath.replace(ptr, hdPath.length()-ptr, "");

 /*  search directory for a header */

 if( (dir_ptr=opendir(hdPath.c_str())) == NULL) {
    QieMsg("ERROR", "Unable to open directory " , hdPath.c_str());
	return helperList;
}

  /* list directory entries */

  while( (dir_ent=readdir(dir_ptr)) != NULL ){
    std::string hdName =  dir_ent->d_name;
    ptr = hdName.find(dotExtn);

    if( ptr != -1 && ptr == hdName.length()-4) {
        std::string hdrFound = hdPath.c_str();
        hdrFound += "/";
	hdrFound += hdName;
	helperList.push_back(hdrFound.c_str());
    }
  }

  return helperList;

}

// ------------------------------------------------

bool   QieStrIsNULL(const char *txt)
{
	if(txt == NULL) return true;
	if( strcmp(txt, "") == 0)  return true;
    return false;

} 



 // ---------------------------------------------------------

char QieGetDelim (std::string value)
{
  if (value.empty()) return ' ';
  if (value[0] == ' ') return ' ';

  if( value[0] == '\n' || value[0] == '\0') return ' ';
  if( value[0] == '\"' && value[1] != '\0') return value[1];
  if( value[0] != '\\' || value.length() < 2) return value[0];
  if(value[1] == 't') return '\t';
  if(value[1] == '\'') return '\'';
  if(value[1] == '\"') return '\"';
  if(value[1] == 'n') return '\n';

  return ' ';

} 


// ------------------------------------------------

std::string QieStripQuotes(std::string value){

  int ptr_quote = 0;  
  int Slen = value.length();
  /* remove leading spaces */
  while(value[ptr_quote] == ' ') ptr_quote++;

  /* remove leading quote */
  if(value[ptr_quote] == '\"') {
     ptr_quote++;
  }
 
  /* remove trailing spaces */
  while(value[Slen -1] == ' ' || value[Slen -1] == '\n') Slen--;

  /* remove trailing quote */
  if(value[Slen -1] == '\"') {
    Slen--;
  }
  
  return value.substr(ptr_quote, Slen - ptr_quote);

}

// ---------------------------------------------------------

const char * QieToUpper(const char *txt)
{

int i;
static char upper[TO_UPPER_LEN+1];
int s_length;

s_length = strlen(txt);

if(s_length >= TO_UPPER_LEN) {
  QieMsg("WARNING", "Truncating text string, too long for QieToUpper = ", txt);
  s_length = TO_UPPER_LEN;
}

for (i=0; i < s_length; i++){
  upper[i]=toupper(txt[i]);
}
upper[s_length] = '\0';

return &(upper[0]);
}


// ---------------------------------------------------------

bool QieIsISOtime(std::string TestStrISO){

	int ptr=0;
	int str_len = 0;
	std::string subString;

	/* trap null string */

	if(TestStrISO.empty() || TestStrISO == "") return false;

	// ISO is " 1995-01-23 02:33:17.235 " or " 1995-01-23T02:33:17.235Z " 

	// remove any leading space 

	while(ptr < TestStrISO.size() && TestStrISO[ptr] == ' ')	ptr++;
		
	//  check str is at least long enough to give seconds 
	if( TestStrISO.size() - ptr < 19 ) return false;
	
	/* parse off date - no field can be zero */

	str_len = TestStrISO.find( "-"); if(str_len == -1) return false;
	if( !QieIsNumber( TestStrISO.substr(ptr, str_len - ptr) ) ) return false;
	
	subString = TestStrISO.substr(str_len+1, TestStrISO.size() - str_len -1); // next token

	str_len = subString.find( "-"); if(str_len == -1) return false;
	if(!QieIsNumber(subString.substr(0, str_len ))) return false;
	subString = subString.substr(str_len+1, subString.size() - str_len -1); // next token

	str_len = subString.find( "T"); 
	if(str_len == -1) str_len = subString.find( " "); if(str_len == -1) return false;
	if(!QieIsNumber(subString.substr(0, str_len ))) return false;
	subString = subString.substr(str_len+1, subString.size() - str_len -1); // next token

	// parse off time portion - time fields can be zero, just test for existence 

	str_len = subString.find( ":"); if(str_len == -1) return false;  // hours
	if (!QieIsNumber(subString.substr(0, str_len )) ) return false;
	subString = subString.substr(str_len+1, subString.size() - str_len -1); // next token

	str_len = subString.find( ":"); if(str_len == -1) return false;  // minutes
	if (!QieIsNumber(subString.substr(0, str_len )) ) return false;
	subString = subString.substr(str_len+1, subString.size() - str_len -1); // next token

	// just test seconds, decimal and fractional seconds are optional

	if (!QieIsNumber(subString.substr(0, 2 )) ) return false;

	//if we get this far it can be read as a time string (might still be wrong though, e.g. 90 instead of 1990 

	return true;

}

// ---------------------------------------------------------

bool QieIsNumber(std::string ptr){
  int i;

  for( i=0; i<ptr.size(); i++){
    if(!isdigit(ptr[i]) && ptr[i] != '.') return false;
  }
  return true;
}


//-----------------------------------------------------------
//   QiSafePath
//   expands ~ and ~username to the full path
//   It can safely be reduced to
//       QieStrcpy(safepath, path, MAXPATHLEN);
//       return &(safepath[0]);
//   The only loss is the built in expansion of these abbreviations



std::string QieSafePath(const char * path)
{
	std::string safepath;
	std::string home_token;
	std::string home;

#if defined _WIN32 || defined __VMS
	safepath = std::string(path);
#else
	if(path[0] == '~'){
		home_token = std::string(path);
		int ptr = home_token.find("/");
		if(ptr != -1) {
			home = QieUserHomeDir(home_token.substr(0, ptr).c_str());
			safepath = std::string(home);
			safepath += home_token.substr(ptr, home_token.length() - ptr);
		}
		else{
			home = QieUserHomeDir(home_token.c_str());
			safepath = std::string(home);
		}
	}
	else{   /* nothing to expand */
		safepath = std::string(path);
	}

#endif
  return safepath;

} 

// ----------------------------------------------------
//  QieUserHomeDir
//  Is only used by QieSafePath above.
//  Simplification of that function as specified will remove
//  the need for this function which can be reduced to
//    return string("");

std::string QieUserHomeDir(const char * user_id)
{
	std::string homeDir;
	char *home;
	
#if defined _WIN32 || defined __VMS
	homeDir = "";
#else
	struct passwd *password_info;

	if(user_id[1] == '\0' ){
			// expand ~ on its own to user's home space
			if ( (home = getenv("HOME")) != NULL ){
				homeDir = std::string(home);
			}
			else{
				QieMsg ("ERROR", "Cannot expand ~ to user's home, set 'HOME' environment \n");
				homeDir = "";
			}
	}
	else{

		password_info = getpwnam( user_id );

		if( password_info == NULL ){
				homeDir = "";
		}
		else{
			homeDir = std::string( password_info->pw_dir );
		}

	}
#endif
  return homeDir;

} 

// utilities to track if cef 1 or 2 is default

bool cefRead2 (int set){
  static bool read_cef_2;
  if(set == -1) return read_cef_2;
  if(set == 0) {
    read_cef_2 = false;
    return read_cef_2;
  }
  if(set == 1) {
    read_cef_2 = true;
    return read_cef_2;
  }
  
}

bool cefWrite2 (int set){
  static bool write_cef_2;
  if(set == -1) return write_cef_2;
  if(set == 0) {
    write_cef_2 = false;
    return write_cef_2;
  }
  if(set == 1) {
    write_cef_2 = true;
    return write_cef_2;
  }
  
}

