// floatspinbox.cc
//
// Floating Point Spinner Class for Qt - Implementation  File
//
//


#include "floatspinbox.h"

#include <qspinbox.h>
#include <cmath>
#include <iostream>

/*!
  \class FloatSpinBox floatspinbox.h
  \brief reimplements QSpinBox for float values
 */

FloatSpinBox::FloatSpinBox( QWidget* parent, const char* name )
	: QSpinBox( int(-2e10), int(2e10), 10, parent, name ),dec(2)
{
	dVal = new QDoubleValidator( -2e8, 2e8, 2, this, "dVal" );
	setValidator( dVal );
	dVal = new QDoubleValidator( -2e8, 2e8, 2, this, "dVal" );
	setValue( 0.0 );
	setFocusPolicy(QWidget::StrongFocus);
}

FloatSpinBox::FloatSpinBox( float minValue, float maxValue, int decimals, float step, QWidget* parent, const char* name )
	: QSpinBox( int(rint(minValue * pow( 10, decimals ) ) ), \
			int( rint(maxValue * pow( 10, decimals ) ) ),
			     int( rint( step * pow( 10, decimals ) ) ), parent, name ),
				  dec( decimals )
{
	dVal = new QDoubleValidator( minValue, maxValue, 2, this, "dVal" );
	setValidator( dVal );
	setValue( 0.0 );
	setFocusPolicy(QWidget::StrongFocus);
}

FloatSpinBox::~FloatSpinBox()
{
}

bool FloatSpinBox::eventFilter(QObject *pobj, QEvent *pev)
{
	bool result=QSpinBox::eventFilter(pobj,pev);
	if(pev->type()==QEvent::FocusOut)
		emit focusOut();
	return result;
}

void FloatSpinBox::keyPressEvent(QKeyEvent *e)
{
	QSpinBox::keyPressEvent(e);
	if (e->key()==Qt::Key_Return || e->key()==Qt::Key_Enter)
		emit returnPressed();
}

int FloatSpinBox::mapTextToValue ( bool * ok )
{
	QString str=cleanText();
	float fval=str.toFloat(ok);
	return int(rint( cleanText().toFloat( ok ) * pow( 10, dec ) ));
}

QString FloatSpinBox::mapValueToText ( int value )
{
	QString s;
	s.setNum( float( value )/ pow( 10, dec ), 'f', dec );
	return s;
}

float FloatSpinBox::maxValue() const
{
	return float( QRangeControl::maxValue() ) / pow( 10, dec );
}

float FloatSpinBox::minValue() const
{
	return float( QRangeControl::maxValue() ) / pow( 10, dec );
}

void FloatSpinBox::setRange( double minValue, double maxValue )
{
	QRangeControl::setRange( int( rint( minValue *  pow( 10, dec ) ) ),
				 int( rint( maxValue *  pow( 10, dec ) ) ) );
	dVal->setRange( minValue, maxValue, 2 );
}

void FloatSpinBox::setValue( double val )
{
	QRangeControl::setValue( int( rint(val *  pow( 10, dec ) ) ) );
}

QSize FloatSpinBox::sizeHint() const
{
	QFontMetrics fm = fontMetrics();
	int h = fm.height();
	if ( h < 12 )       // ensure enough space for the button pixmaps
		h = 12;
	int w = 35;         // minimum width for the value
	int wx = fm.width( "  " );
	QString s;
	s.setNum( minValue(), 'f', dec );
	s.prepend( prefix() );
	s.append( suffix() );
	w = QMAX( w, fm.width( s ) + wx );
	s.setNum( maxValue(), 'f', dec );
	s.prepend( prefix() );
	s.append( suffix() );
	w = QMAX( w, fm.width( s ) + wx );
	s = specialValueText();
	w = QMAX( w, fm.width( s ) + wx );

	QSize r( h // buttons AND frame both sides
			+ 6 // right/left margins
			+ w, // widest value
			//frameWidth() * 2 // top/bottom frame
			8 // top/bottom margins
			+ h // font height
	       );
	return r;
}

float FloatSpinBox::value() const
{
	return float( QRangeControl::value() ) / pow( 10, dec );
}

void FloatSpinBox::valueChange()
{
	QSpinBox::valueChange();
	emit valueChanged( value() );
}
