// qpui_frame_options.cc
//
// Plot Interface Implementation - Frame Options
//
// Tony Allen
// Apr 2008

#include "qpui.h"
#include "qdutil.h"

#include <qcolordialog.h>
#include <qtooltip.h>

extern QPalette QPUI_normal_palette;


//  QpPropertiesTab Class
// -----------------------

#if QT_VERSION < 0x040000
QpPropertiesTab::QpPropertiesTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
    QpTab(tabWidget, name, QP_POSITION_TAB)
{
	// Constructor for the Properties Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_properties_top_layout = new QVBoxLayout(this);
	panel_properties_top_layout->setSpacing(4);

	
	QGroupBox *panel_position_groupbox = new QGroupBox(1, Qt::Vertical, this, "panel_position_groupbox");

	panel_position_groupbox->setTitle("Frame Position");

	QWidget* panel_position_widget = new QWidget(panel_position_groupbox, "panel_position_widget");

	QGridLayout *panel_position_layout = new QGridLayout(panel_position_widget, 9, 5);
	panel_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autolayout_lbl = new QLabel("Autoset by Page Layout ", panel_position_widget, "autolayout_lbl");
	panel_position_layout->addMultiCellWidget(_autolayout_lbl, 1, 1, 0, 1);

	QLabel* loffset_label = new QLabel("Left offset: ", panel_position_widget, "loffset_label");
	panel_position_layout->addWidget(loffset_label, 3, 0, Qt::AlignRight);
	_loffset_fld = new FloatSpinBox(0., 1., 2, .05, panel_position_widget, "loffset");
	_loffset_fld->setFixedWidth(100);
	_loffset_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_loffset_fld, 3, 1, Qt::AlignLeft);
	_loffset_fld->setValue(QPUI_PANEL_LOFFSET);
	
	connect( _loffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setLOffset(double)) );
	_loffset_fld->setDisabled(true);
	
	QLabel* width_label = new QLabel("Width: ", panel_position_widget, "width_label");
	panel_position_layout->addWidget(width_label, 3, 2, Qt::AlignRight);
	_width_fld = new FloatSpinBox(0.1, 1., 2, .05, panel_position_widget, "width");
	_width_fld->setFixedWidth(100);
	_width_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_width_fld, 3, 3, Qt::AlignLeft);
	_width_fld->setValue(QPUI_PANEL_WIDTH);
	connect( _width_fld, SIGNAL(valueChanged(double)), this, SLOT(setWidth(double)) );
	_width_fld->setDisabled(true);

	QLabel* toffset_label = new QLabel("Top offset: ", panel_position_widget, "toffset_label");
	panel_position_layout->addWidget(toffset_label, 5, 0, Qt::AlignRight);
	_toffset_fld = new FloatSpinBox(0., 1., 2,.05, panel_position_widget, "toffset");
	_toffset_fld->setFixedWidth(100);
	_toffset_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_toffset_fld, 5, 1, Qt::AlignLeft);
	_toffset_fld->setValue(QPUI_PANEL_TOFFSET);
	connect( _toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setTOffset(double)) );
	_toffset_fld->setDisabled(true);

	QLabel* height_label = new QLabel("Height: ", panel_position_widget, "height_label");
	panel_position_layout->addWidget(height_label, 5, 2, Qt::AlignRight);
	_height_fld = new FloatSpinBox(0.1, 1., 2, .05, panel_position_widget, "height");
	_height_fld->setFixedWidth(100);
	_height_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_height_fld, 5, 3, Qt::AlignLeft);
	_height_fld->setValue(QPUI_PANEL_HEIGHT);
	connect( _height_fld, SIGNAL(valueChanged(double)), this, SLOT(setHeight(double)) );
	_height_fld->setDisabled(true);

	_aspect_ratio = QPUI_PANEL_HEIGHT / QPUI_PANEL_WIDTH; // initialise aspect ratio
	_aspect_ratio_fld = new QCheckBox("Preserve aspect ratio", panel_position_widget, "aspect_ratio_fld");
	panel_position_layout->addMultiCellWidget(_aspect_ratio_fld, 7, 7, 0, 1);

	connect( _aspect_ratio_fld, SIGNAL(toggled(bool)), this, SLOT(setAspectRatio(bool)) );
	_aspect_ratio_fld->setDisabled(true);
	
	panel_position_layout->addRowSpacing(0,0);
	panel_position_layout->addRowSpacing(1, _autolayout_lbl->height());
	panel_position_layout->addRowSpacing(2,0);
	panel_position_layout->addRowSpacing(3, _loffset_fld->height());
	panel_position_layout->addRowSpacing(4,0);
	panel_position_layout->addRowSpacing(5, _toffset_fld->height());
	panel_position_layout->addRowSpacing(6,0);
	panel_position_layout->addRowSpacing(7, _aspect_ratio_fld->height());
	panel_position_layout->addRowSpacing(8,0);

	panel_position_layout->setRowStretch(0,0);
	panel_position_layout->setRowStretch(1,0);
	panel_position_layout->setRowStretch(2,0);
	panel_position_layout->setRowStretch(3,1);
	panel_position_layout->setRowStretch(4,0);
	panel_position_layout->setRowStretch(5,1);
	panel_position_layout->setRowStretch(6,0);
	panel_position_layout->setRowStretch(7,1);
	panel_position_layout->setRowStretch(8,0);

	panel_position_layout->setColStretch(0,0);
	panel_position_layout->setColStretch(1,0);
	panel_position_layout->setColStretch(2,0);
	panel_position_layout->setColStretch(3,0);
	panel_position_layout->setColStretch(4,1);

	panel_properties_top_layout->addWidget(panel_position_groupbox);

	panel_properties_top_layout->addSpacing(4);
	panel_properties_top_layout->addStretch(10);

	panel_position_widget->adjustSize();

}
#else
QpPropertiesTab::QpPropertiesTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
		QpTab(tabWidget, name, QP_POSITION_TAB)
{
	// Constructor for the Properties Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_properties_top_layout = new QVBoxLayout(this);
	panel_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	
	QGroupBox *panel_position_groupbox = new QGroupBox(this);

	panel_position_groupbox->setTitle("Frame Position");

// 	QWidget* panel_position_widget = new QWidget(panel_position_groupbox, "panel_position_widget");

	QGridLayout *panel_position_layout = new QGridLayout;
	panel_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autolayout_lbl = new QLabel("Autoset by Page Layout ");
	panel_position_layout->addMultiCellWidget(_autolayout_lbl, 1, 1, 0, 1);

	QLabel* loffset_label = new QLabel("Left offset: ");
	panel_position_layout->addWidget(loffset_label, 3, 0, Qt::AlignRight);
	_loffset_fld = new FloatSpinBox(0., 1., 2, .05, panel_position_groupbox, "loffset");
	_loffset_fld->setFixedWidth(100);
	_loffset_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_loffset_fld, 3, 1, Qt::AlignLeft);
	_loffset_fld->setValue(QPUI_PANEL_LOFFSET);
	
	connect( _loffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setLOffset(double)) );
	_loffset_fld->setDisabled(true);
	
	QLabel* width_label = new QLabel("Width: ");
	panel_position_layout->addWidget(width_label, 3, 2, Qt::AlignRight);
	_width_fld = new FloatSpinBox(0.1, 1., 2, .05, panel_position_groupbox, "width");
	_width_fld->setFixedWidth(100);
	_width_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_width_fld, 3, 3, Qt::AlignLeft);
	_width_fld->setValue(QPUI_PANEL_WIDTH);
	connect( _width_fld, SIGNAL(valueChanged(double)), this, SLOT(setWidth(double)) );
	_width_fld->setDisabled(true);

	QLabel* toffset_label = new QLabel("Top offset: ");
	panel_position_layout->addWidget(toffset_label, 5, 0, Qt::AlignRight);
	_toffset_fld = new FloatSpinBox(0., 1., 2,.05, panel_position_groupbox, "toffset");
	_toffset_fld->setFixedWidth(100);
	_toffset_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_toffset_fld, 5, 1, Qt::AlignLeft);
	_toffset_fld->setValue(QPUI_PANEL_TOFFSET);
	connect( _toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setTOffset(double)) );
	_toffset_fld->setDisabled(true);

	QLabel* height_label = new QLabel("Height: ");
	panel_position_layout->addWidget(height_label, 5, 2, Qt::AlignRight);
	_height_fld = new FloatSpinBox(0.1, 1., 2, .05, panel_position_groupbox, "height");
	_height_fld->setFixedWidth(100);
	_height_fld->setFocusPolicy(STRONG_FOCUS);
	panel_position_layout->addWidget(_height_fld, 5, 3, Qt::AlignLeft);
	_height_fld->setValue(QPUI_PANEL_HEIGHT);
	connect( _height_fld, SIGNAL(valueChanged(double)), this, SLOT(setHeight(double)) );
	_height_fld->setDisabled(true);

	_aspect_ratio = QPUI_PANEL_HEIGHT / QPUI_PANEL_WIDTH; // initialise aspect ratio
	_aspect_ratio_fld = new QCheckBox("Preserve aspect ratio");
	panel_position_layout->addMultiCellWidget(_aspect_ratio_fld, 7, 7, 0, 1);

	connect( _aspect_ratio_fld, SIGNAL(toggled(bool)), this, SLOT(setAspectRatio(bool)) );
	_aspect_ratio_fld->setDisabled(true);
	
// 	panel_position_layout->addRowSpacing(0,0);
// 	panel_position_layout->addRowSpacing(1, _autolayout_lbl->height());
// 	panel_position_layout->addRowSpacing(2,0);
// 	panel_position_layout->addRowSpacing(3, _loffset_fld->height());
// 	panel_position_layout->addRowSpacing(4,0);
// 	panel_position_layout->addRowSpacing(5, _toffset_fld->height());
// 	panel_position_layout->addRowSpacing(6,0);
// 	panel_position_layout->addRowSpacing(7, _aspect_ratio_fld->height());
// 	panel_position_layout->addRowSpacing(8,0);
// 
// 	panel_position_layout->setRowStretch(0,0);
// 	panel_position_layout->setRowStretch(1,0);
// 	panel_position_layout->setRowStretch(2,0);
// 	panel_position_layout->setRowStretch(3,1);
// 	panel_position_layout->setRowStretch(4,0);
// 	panel_position_layout->setRowStretch(5,1);
// 	panel_position_layout->setRowStretch(6,0);
// 	panel_position_layout->setRowStretch(7,1);
// 	panel_position_layout->setRowStretch(8,0);
// 
// 	panel_position_layout->setColStretch(0,0);
// 	panel_position_layout->setColStretch(1,0);
// 	panel_position_layout->setColStretch(2,0);
// 	panel_position_layout->setColStretch(3,0);
// 	panel_position_layout->setColStretch(4,1);
	
	panel_position_groupbox->setLayout(panel_position_layout);

	panel_properties_top_layout->addWidget(panel_position_groupbox);

	panel_properties_top_layout->addSpacing(4);
	panel_properties_top_layout->addStretch(10);

// 	panel_position_widget->adjustSize();

}
#endif

void QpPropertiesTab::setLOffset(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LOFFSET, 0, fChoice);
}

void QpPropertiesTab::setTOffset(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TOFFSET, 0, fChoice);
}

void QpPropertiesTab::setWidth(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(WIDTH, 0, fChoice);
   if(this->_aspect_ratio_fld->isChecked()) _height_fld->setValue(fChoice*_aspect_ratio);
   else _aspect_ratio = _height_fld->value()/fChoice;

}

void QpPropertiesTab::setHeight(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if( this->isVisible() ) I_Frame->I_ListView->setFrameOption(HEIGHT, 0, fChoice);
   if(this->_aspect_ratio_fld->isChecked()) _width_fld->setValue(fChoice/_aspect_ratio);
   else _aspect_ratio = fChoice/_width_fld->value();
}

void QpPropertiesTab::setAspectRatio(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(ASPECT_RATIO, 0, 0.0, "",  bChoice);
}




// QpTitleTab class
// ----------------

#if QT_VERSION < 0x040000
QpTitleTab::QpTitleTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
    QpTab(tabWidget, name, QP_TITLE_TAB)
{  
	// Constructor for the Title Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_title_top_layout=new QVBoxLayout(this);
	panel_title_top_layout->setSpacing(4);

	QGroupBox *panel_title_groupbox = new QGroupBox(1, Qt::Vertical, this, "panel_title_groupbox");

	panel_title_groupbox->setTitle("Title");

	QWidget* panel_title_widget = new QWidget(panel_title_groupbox, "panel_title_widget");

	QGridLayout *panel_title_layout = new QGridLayout(panel_title_widget, 17, 5);
	panel_title_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_title_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autotitle_fld = new QCheckBox("Autotitle", panel_title_widget, "autotitle");
	_autotitle_fld->setChecked(true);
	panel_title_layout->addWidget(_autotitle_fld, 1, 0, Qt::AlignLeft);

	connect( _autotitle_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoTitle(bool)) );

	QLabel* text_label = new QLabel("Text: ",panel_title_widget, "text_label");
	panel_title_layout->addWidget(text_label, 3, 0, Qt::AlignRight);
	_title_fld = new QLineEdit(panel_title_widget, "title");
	_title_fld->setMinimumWidth(500);
	panel_title_layout->addMultiCellWidget(_title_fld, 3, 3, 1, 4, Qt::AlignLeft);
	setSlotColour(false);
	_title_fld->setPalette(GUIpalette);
	connect( _title_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTitle(const QString&)) );

	QLabel* title_char_size_label = new QLabel("Character size: ", panel_title_widget, "title_char_size_label");
	panel_title_layout->addWidget(title_char_size_label, 5, 0, Qt::AlignRight);
	_title_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_title_widget, "title_char_size");
	_title_char_size_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_char_size_fld, 5, 1, Qt::AlignLeft);
	_title_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_TITLE_CHAR_SIZE_ADD);
	connect(_title_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTitleCharSize(double)));

	QLabel* title_disp_label = new QLabel("Displacement: ", panel_title_widget, "title_disp_label");
	panel_title_layout->addWidget(title_disp_label, 5, 2, Qt::AlignRight);
	_title_disp_fld = new FloatSpinBox(0., 40., 1, .5, panel_title_widget, "title_disp");
	_title_disp_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_disp_fld, 5, 3, Qt::AlignLeft);
	_title_disp_fld->setValue(QPUI_TITLE_DISP);
	connect(_title_disp_fld,SIGNAL(valueChanged(double)),this,SLOT(setTitleDisp(double)));

	panel_title_layout->setRowSpacing(0,0);
	panel_title_layout->setRowSpacing(1, _title_fld->height());
	panel_title_layout->setRowSpacing(2,0);
	panel_title_layout->setRowSpacing(3, _title_char_size_fld->height());
	panel_title_layout->setRowSpacing(4,0);

	panel_title_layout->setRowStretch(0,0);
	panel_title_layout->setRowStretch(1,1);
	panel_title_layout->setRowStretch(2,0);
	panel_title_layout->setRowStretch(3,1);
	panel_title_layout->setRowStretch(4,0);


	QGroupBox *panel_subtitle_groupbox = new QGroupBox(1, Qt::Vertical, panel_title_widget, "panel_subtitle_groupbox");

	panel_subtitle_groupbox->setTitle("Subtitle");

	QWidget* panel_subtitle_widget=new QWidget(panel_subtitle_groupbox, "panel_subtitle_widget");

	QGridLayout *panel_subtitle_layout = new QGridLayout(panel_subtitle_widget, 7, 5);
	panel_subtitle_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_subtitle_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* subtitle_text_label = new QLabel("Text: ", panel_subtitle_widget, "subtitle_text_label");
	panel_subtitle_layout->addWidget(subtitle_text_label, 1, 0, Qt::AlignRight);
	_subtitle_fld = new QLineEdit(panel_subtitle_widget, "subtitle");
	_subtitle_fld->setMinimumWidth(350);
	panel_subtitle_layout->addMultiCellWidget(_subtitle_fld, 1, 1, 1, 4, Qt::AlignLeft);
	connect( _subtitle_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setSubTitle(const QString&)) );

	QLabel* subtitle_char_size_label = new QLabel("Character size: ", panel_subtitle_widget, "subtitle_char_size_label");
	panel_subtitle_layout->addWidget(subtitle_char_size_label, 3, 0, Qt::AlignRight);
	_subtitle_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_subtitle_widget, "subtitle_char_size");
	_subtitle_char_size_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_char_size_fld, 3, 1, Qt::AlignLeft);
	_subtitle_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_SUBTITLE_CHAR_SIZE_ADD);

	connect(_subtitle_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleCharSize(double)));

	QLabel* subtitle_disp_label = new QLabel("Displacement: ", panel_subtitle_widget, "subtitle_disp_label");
	panel_subtitle_layout->addWidget(subtitle_disp_label, 3, 2, Qt::AlignRight);
	_subtitle_disp_fld = new FloatSpinBox(-10., 40., 1, .2, panel_subtitle_widget, "subtitle_disp");
	_subtitle_disp_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_disp_fld, 3, 3, Qt::AlignLeft);
	_subtitle_disp_fld->setValue(QPUI_SUBTITLE_DISP);
	connect(_subtitle_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleDisp(double)));

	panel_subtitle_layout->addRowSpacing(0,0);
	panel_subtitle_layout->addRowSpacing(1, _subtitle_fld->height());
	panel_subtitle_layout->addRowSpacing(2,0);
	panel_subtitle_layout->addRowSpacing(3, _subtitle_char_size_fld->height());
	panel_subtitle_layout->addRowSpacing(4,0);

	panel_subtitle_layout->setRowStretch(0,0);
	panel_subtitle_layout->setRowStretch(1,1);
	panel_subtitle_layout->setRowStretch(2,0);
	panel_subtitle_layout->setRowStretch(3,1);
	panel_subtitle_layout->setRowStretch(4,0);

	panel_title_layout->addMultiCellWidget(panel_subtitle_groupbox, 7, 11, 0, 4);
	
	QLabel* justification_label = new QLabel("Justification: ", panel_title_widget, "justification_label");
	panel_title_layout->addWidget(justification_label, 13, 0, Qt::AlignRight);
	_title_just_fld = new QComboBox(false, panel_title_widget, "title_just");
	_title_just_fld->insertItem("centre", QPUI_HCENTRE);
	_title_just_fld->insertItem("left", QPUI_LEFT);
	_title_just_fld->insertItem("right", QPUI_RIGHT);
	QFontMetrics title_just_fm(_title_just_fld->font());
	int title_just_w = title_just_fm.width("centre");
	int title_just_h = title_just_fm.height();
	_title_just_fld->setMinimumSize(title_just_w, title_just_h);
#if QT_VERSION < 0x040000
	_title_just_fld->listBox()->setMinimumSize(title_just_w, title_just_h);
#else
        _title_just_fld->setMinimumSize(title_just_w, title_just_h);
#endif
	QToolTip::add( _title_just_fld, "Select title justification");
	panel_title_layout->addWidget(_title_just_fld, 13, 1, Qt::AlignLeft);

	connect( _title_just_fld, SIGNAL(activated(int)),this, SLOT(setTitleJust(int)) );

	QLabel* font_label = new QLabel("Font: ", panel_title_widget, "font_label");
	panel_title_layout->addWidget(font_label, 15, 0, Qt::AlignRight);
	_title_font_fld = new FontButton(panel_title_widget);
	
	QToolTip::add( _title_font_fld, "Select title font");
	panel_title_layout->addWidget(_title_font_fld, 15, 1, Qt::AlignLeft);

	connect( _title_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setTitleFamily(int)) );
	connect( _title_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setTitleStyle(bool)) );
	connect( _title_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setTitleWeight(bool)) );

	QLabel* colour_label = new QLabel("Colour: ", panel_title_widget, "colour_label");
	panel_title_layout->addWidget(colour_label, 15, 2, Qt::AlignRight);
	_title_colour_fld = new QPushButton("Colour...", panel_title_widget, "title_colour");
	_title_colour_fld->setFixedWidth(100);
	_title_colour_fld->setFixedHeight(30);
	QFont f = QFont(qApp->font());
	f.setBold(TRUE);
	_title_colour_fld->setFont(f);
	panel_title_layout->addWidget(_title_colour_fld, 15, 3, Qt::AlignLeft);

	connect(_title_colour_fld, SIGNAL(clicked()), this, SLOT(setTitleColour()));

	panel_title_layout->addRowSpacing(13, _title_just_fld->height());
	panel_title_layout->addRowSpacing(14,0);
	panel_title_layout->addRowSpacing(15, _title_font_fld->height());
	panel_title_layout->addRowSpacing(16,0);

	panel_title_layout->setRowStretch(13,1);
	panel_title_layout->setRowStretch(14,0);
	panel_title_layout->setRowStretch(15,1);
	panel_title_layout->setRowStretch(16,0);

	panel_title_top_layout->addWidget(panel_title_groupbox);

	panel_title_top_layout->addSpacing(4);
	panel_title_top_layout->addStretch(10);
	panel_title_widget->adjustSize();
}
#else
QpTitleTab::QpTitleTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
		QpTab(tabWidget, name, QP_TITLE_TAB)
{  
	// Constructor for the Title Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_title_top_layout=new QVBoxLayout(this);
	panel_title_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_title_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *panel_title_groupbox = new QGroupBox(this);

	panel_title_groupbox->setTitle("Title");

// 	QWidget* panel_title_widget = new QWidget(panel_title_groupbox, "panel_title_widget");

	QGridLayout *panel_title_layout = new QGridLayout;
	panel_title_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_title_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_autotitle_fld = new QCheckBox("Autotitle");
	_autotitle_fld->setChecked(true);
	panel_title_layout->addWidget(_autotitle_fld, 1, 0, Qt::AlignLeft);

	connect( _autotitle_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoTitle(bool)) );

	QLabel* text_label = new QLabel("Text: ");
	panel_title_layout->addWidget(text_label, 3, 0, Qt::AlignRight);
	_title_fld = new QLineEdit;
	_title_fld->setMinimumWidth(500);
	panel_title_layout->addMultiCellWidget(_title_fld, 3, 3, 1, 4, Qt::AlignLeft);
	setSlotColour(false);
	_title_fld->setPalette(GUIpalette);
	connect( _title_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setTitle(const QString&)) );

	QLabel* title_char_size_label = new QLabel("Character size: ");
	panel_title_layout->addWidget(title_char_size_label, 5, 0, Qt::AlignRight);
	_title_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_title_groupbox, "title_char_size");
	_title_char_size_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_char_size_fld, 5, 1, Qt::AlignLeft);
	_title_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_TITLE_CHAR_SIZE_ADD);
	connect(_title_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTitleCharSize(double)));

	QLabel* title_disp_label = new QLabel("Displacement: ");
	panel_title_layout->addWidget(title_disp_label, 5, 2, Qt::AlignRight);
	_title_disp_fld = new FloatSpinBox(0., 40., 1, .5, panel_title_groupbox, "title_disp");
	_title_disp_fld->setFixedWidth(100);
	panel_title_layout->addWidget(_title_disp_fld, 5, 3, Qt::AlignLeft);
	_title_disp_fld->setValue(QPUI_TITLE_DISP);
	connect(_title_disp_fld,SIGNAL(valueChanged(double)),this,SLOT(setTitleDisp(double)));

// 	panel_title_layout->setRowSpacing(0,0);
// 	panel_title_layout->setRowSpacing(1, _title_fld->height());
// 	panel_title_layout->setRowSpacing(2,0);
// 	panel_title_layout->setRowSpacing(3, _title_char_size_fld->height());
// 	panel_title_layout->setRowSpacing(4,0);
// 
// 	panel_title_layout->setRowStretch(0,0);
// 	panel_title_layout->setRowStretch(1,1);
// 	panel_title_layout->setRowStretch(2,0);
// 	panel_title_layout->setRowStretch(3,1);
// 	panel_title_layout->setRowStretch(4,0);


	QGroupBox *panel_subtitle_groupbox = new QGroupBox(panel_title_groupbox);

	panel_subtitle_groupbox->setTitle("Subtitle");

// 	QWidget* panel_subtitle_widget=new QWidget(panel_subtitle_groupbox, "panel_subtitle_widget");

	QGridLayout *panel_subtitle_layout = new QGridLayout;
	panel_subtitle_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_subtitle_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* subtitle_text_label = new QLabel("Text: ");
	panel_subtitle_layout->addWidget(subtitle_text_label, 1, 0, Qt::AlignRight);
	_subtitle_fld = new QLineEdit;
	_subtitle_fld->setMinimumWidth(350);
	panel_subtitle_layout->addMultiCellWidget(_subtitle_fld, 1, 1, 1, 4, Qt::AlignLeft);
	connect( _subtitle_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setSubTitle(const QString&)) );

	QLabel* subtitle_char_size_label = new QLabel("Character size: ");
	panel_subtitle_layout->addWidget(subtitle_char_size_label, 3, 0, Qt::AlignRight);
	_subtitle_char_size_fld = new FloatSpinBox(0.01, 40., 1, .1, panel_subtitle_groupbox, "subtitle_char_size");
	_subtitle_char_size_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_char_size_fld, 3, 1, Qt::AlignLeft);
	_subtitle_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_SUBTITLE_CHAR_SIZE_ADD);

	connect(_subtitle_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleCharSize(double)));

	QLabel* subtitle_disp_label = new QLabel("Displacement: ");
	panel_subtitle_layout->addWidget(subtitle_disp_label, 3, 2, Qt::AlignRight);
	_subtitle_disp_fld = new FloatSpinBox(-10., 40., 1, .2, panel_subtitle_groupbox, "subtitle_disp");
	_subtitle_disp_fld->setFixedWidth(80);
	panel_subtitle_layout->addWidget(_subtitle_disp_fld, 3, 3, Qt::AlignLeft);
	_subtitle_disp_fld->setValue(QPUI_SUBTITLE_DISP);
	connect(_subtitle_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setSubTitleDisp(double)));

// 	panel_subtitle_layout->addRowSpacing(0,0);
// 	panel_subtitle_layout->addRowSpacing(1, _subtitle_fld->height());
// 	panel_subtitle_layout->addRowSpacing(2,0);
// 	panel_subtitle_layout->addRowSpacing(3, _subtitle_char_size_fld->height());
// 	panel_subtitle_layout->addRowSpacing(4,0);
// 
// 	panel_subtitle_layout->setRowStretch(0,0);
// 	panel_subtitle_layout->setRowStretch(1,1);
// 	panel_subtitle_layout->setRowStretch(2,0);
// 	panel_subtitle_layout->setRowStretch(3,1);
// 	panel_subtitle_layout->setRowStretch(4,0);
	
	panel_subtitle_groupbox->setLayout(panel_subtitle_layout);

	panel_title_layout->addMultiCellWidget(panel_subtitle_groupbox, 7, 11, 0, 4);
	
	QLabel* justification_label = new QLabel("Justification: ");
	panel_title_layout->addWidget(justification_label, 13, 0, Qt::AlignRight);
	_title_just_fld = new QComboBox(false);
	_title_just_fld->insertItem("centre", QPUI_HCENTRE);
	_title_just_fld->insertItem("left", QPUI_LEFT);
	_title_just_fld->insertItem("right", QPUI_RIGHT);
	QFontMetrics title_just_fm(_title_just_fld->font());
	int title_just_w = title_just_fm.width("centre");
	int title_just_h = title_just_fm.height();
	_title_just_fld->setMinimumSize(title_just_w, title_just_h);

	QToolTip::add( _title_just_fld, "Select title justification");
	panel_title_layout->addWidget(_title_just_fld, 13, 1, Qt::AlignLeft);

	connect( _title_just_fld, SIGNAL(activated(int)),this, SLOT(setTitleJust(int)) );

	QLabel* font_label = new QLabel("Font: ");
	panel_title_layout->addWidget(font_label, 15, 0, Qt::AlignRight);
	_title_font_fld = new FontButton;

	QToolTip::add( _title_font_fld, "Select title font");
	panel_title_layout->addWidget(_title_font_fld, 15, 1, Qt::AlignLeft);

	connect( _title_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setTitleFamily(int)) );
	connect( _title_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setTitleStyle(bool)) );
	connect( _title_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setTitleWeight(bool)) );
	
	QLabel* colour_label = new QLabel("Colour: ");
	panel_title_layout->addWidget(colour_label, 15, 2, Qt::AlignRight);
	_title_colour_fld = new QPushButton("Colour...");
	_title_colour_fld->setFixedWidth(100);
	_title_colour_fld->setFixedHeight(30);
	QFont f = QFont(qApp->font());
	f.setBold(TRUE);
	_title_colour_fld->setFont(f);
	panel_title_layout->addWidget(_title_colour_fld, 15, 3, Qt::AlignLeft);

	connect(_title_colour_fld, SIGNAL(clicked()), this, SLOT(setTitleColour()));

// 	panel_title_layout->addRowSpacing(13, _title_just_fld->height());
// 	panel_title_layout->addRowSpacing(14,0);
// 	panel_title_layout->addRowSpacing(15, _title_font_fld->height());
// 	panel_title_layout->addRowSpacing(16,0);
// 
// 	panel_title_layout->setRowStretch(13,1);
// 	panel_title_layout->setRowStretch(14,0);
// 	panel_title_layout->setRowStretch(15,1);
// 	panel_title_layout->setRowStretch(16,0);

	panel_title_groupbox->setLayout(panel_title_layout);
	
	panel_title_top_layout->addWidget(panel_title_groupbox);

	panel_title_top_layout->addSpacing(4);
	panel_title_top_layout->addStretch(10);
// 	panel_title_widget->adjustSize();
}
#endif


void QpTitleTab::setAutoTitle(bool bChoice)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTitleTab::setAutoTitle \n" );
		fflush(stdout);
	}

	if(bChoice) 
	{
		string title = I_Frame->getTitle();
		_title_fld->clearFocus(); // ensure field doesn't try to turn autotitle off
		_title_fld->setText(title.c_str());

	}
	setSlotColour(!bChoice);
	_title_fld->setPalette(GUIpalette);

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(AUTOTITLE, 0, 0.0, "", bChoice);
	
}

void QpTitleTab::setTitle(const QString &title)
{
	if(_title_fld->hasFocus())
	{
		// only change if user input
		_autotitle_fld->setChecked(false);
		
		// only tell list view to make changes to selected panel items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
		if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE, 0, 0.0, title);

	}	
}

void QpTitleTab::setTitleCharSize(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_CHAR_SIZE, 0, fChoice);

}

void QpTitleTab::setTitleDisp(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_DISP, 0, fChoice);

}

void QpTitleTab::setSubTitle(const QString &subTitle)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_DISP, 0, 0.0, subTitle);

}

void QpTitleTab::setSubTitleCharSize(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(SUBTITLE_CHAR_SIZE, 0, fChoice);

}

void QpTitleTab::setSubTitleDisp(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(SUBTITLE_DISP, 0, fChoice);

}

void QpTitleTab::setTitleJust(int iChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_JUST, iChoice);

}

void QpTitleTab::setTitleFamily(int iChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_FONT_FAMILY, iChoice);
}

void QpTitleTab::setTitleStyle(bool bChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_FONT_STYLE, 0, 0., "", bChoice);
}

void QpTitleTab::setTitleWeight(bool bChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_FONT_WEIGHT, 0, 0., "", bChoice);
}

void QpTitleTab::setTitleColour()
{
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(I_Frame->_title_colour, &changed, this);
	if(changed)
	{
		QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, c);
        this->_title_colour_fld->setPalette(line_palette);
        I_Frame->_title_colour = c;
	}

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TITLE_COLOUR, 0, 0., "", false, c);
}


//  QpDisplayTab Class
// ---------------------
#if QT_VERSION < 0x040000
QpDisplayTab::QpDisplayTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
    QpTab(tabWidget, name, QP_FRAME_TAB)

{
	// Constructor for the Display Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_display_top_layout = new QVBoxLayout(this);
	panel_display_top_layout->setSpacing(4);

	QGroupBox *panel_frame_groupbox = new QGroupBox(1, Qt::Vertical, this, "panel_frame_groupbox");

	if(I_Frame->frameType() == QP_TS) panel_frame_groupbox->setTitle("TS Frame");
	else  panel_frame_groupbox->setTitle("XY Frame");


	QWidget* panel_frame_widget = new QWidget(panel_frame_groupbox, "panel_frame_widget");

	QGridLayout *panel_frame_layout = new QGridLayout(panel_frame_widget, 7, 5);
	panel_frame_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_frame_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* line_width_label = new QLabel("Line width: ", panel_frame_widget,"line_width_label");
	panel_frame_layout->addWidget(line_width_label, 1, 0, Qt::AlignRight);
// 	_line_width_fld = new QSpinBox(1, 201, 1, panel_frame_widget, "line_width");
	_line_width_fld = new FloatSpinBox(0.1, 20., 1, 0.1, panel_frame_widget, "line_width");
	_line_width_fld->setValue(QSAS_profiles->current()->LineWidth);
	_line_width_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_line_width_fld, 1, 1, Qt::AlignLeft);

	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));

	QLabel* colour_label = new QLabel("Line colour: ", panel_frame_widget, "colour_label");
	panel_frame_layout->addWidget(colour_label, 1, 2, Qt::AlignRight);
	_line_colour_fld = new QPushButton("Colour...", panel_frame_widget, "colour");
	_line_colour_fld->setFixedWidth(100);
	_line_colour_fld->setFixedHeight(30);
	QFont f = _line_colour_fld->font();
	f.setBold(TRUE);
	_line_colour_fld->setFont(f);
	panel_frame_layout->addWidget(_line_colour_fld,1,3,Qt::AlignLeft);

	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));

	QLabel* ticks_length_label = new QLabel("Ticks length: ", panel_frame_widget, "ticks_length_label");
	panel_frame_layout->addWidget(ticks_length_label, 3, 0, Qt::AlignRight);
	_ticks_length_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_widget, "ticks_length");
	_ticks_length_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_length_fld, 3, 1, Qt::AlignLeft);
	_ticks_length_fld->setValue(QPUI_TICKS_LENGTH);
	connect(_ticks_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	
	QLabel* ticks_char_size_label = new QLabel("Ticks font size: ", panel_frame_widget, "ticks_char_size_label");
	panel_frame_layout->addWidget(ticks_char_size_label, 3, 2, Qt::AlignRight);
	_ticks_char_size_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_widget, "ticks_char_size");
	_ticks_char_size_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_char_size_fld, 3, 3, Qt::AlignLeft);
	_ticks_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	connect(_ticks_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksCharSize(double)));

	panel_frame_layout->addRowSpacing(0,0);
	panel_frame_layout->addRowSpacing(1, _line_width_fld->height());
	panel_frame_layout->addRowSpacing(2,0);
	panel_frame_layout->addRowSpacing(3, _ticks_length_fld->height());
	panel_frame_layout->addRowSpacing(4,0);

	panel_frame_layout->setRowStretch(0,0);
	panel_frame_layout->setRowStretch(1,1);
	panel_frame_layout->setRowStretch(2,0);
	panel_frame_layout->setRowStretch(3,1);
	panel_frame_layout->setRowStretch(4,0);

	panel_frame_layout->setColStretch(0,0);
	panel_frame_layout->setColStretch(1,0);
	panel_frame_layout->setColStretch(2,0);
	panel_frame_layout->setColStretch(3,0);
	panel_frame_layout->setColStretch(4,1);

	panel_display_top_layout->addWidget(panel_frame_groupbox);

	QGroupBox *panel_display_groupbox  =new QGroupBox(1, Qt::Vertical, this, "panel_display_groupbox");

	panel_display_groupbox->setTitle("Show");

	QWidget* panel_display_widget = new QWidget(panel_display_groupbox, "panel_display_widget");

	QGridLayout *panel_display_layout = new QGridLayout(panel_display_widget, 12, 8);
	panel_display_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_display_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* top_label = new QLabel("Top", panel_display_widget, "top_label");
	panel_display_layout->addWidget(top_label, 1, 1, Qt::AlignCenter);
	QLabel* bottom_label = new QLabel("Bottom", panel_display_widget, "bottom_label");
	panel_display_layout->addWidget(bottom_label, 1, 2, Qt::AlignCenter);
	QLabel* left_label = new QLabel("Left", panel_display_widget, "left_label");
	panel_display_layout->addWidget(left_label, 1, 3, Qt::AlignCenter);
	QLabel* right_label = new QLabel("Right", panel_display_widget, "right_label");
	panel_display_layout->addWidget(right_label, 1, 4, Qt::AlignCenter);
	QLabel* lr_alternating_label = new QLabel("L/R alternating", panel_display_widget, "lr_alternating_label");
	panel_display_layout->addWidget(lr_alternating_label, 1, 5, Qt::AlignCenter);
	QLabel* rl_alternating_label = new QLabel("R/L alternating", panel_display_widget, "rl_alternating_label");
	panel_display_layout->addWidget(rl_alternating_label, 1, 6, Qt::AlignCenter);

	QFrame *hline = new QFrame(panel_display_widget, "hline");
	hline->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline->setMinimumWidth(500);
	panel_display_layout->addMultiCellWidget(hline, 2, 2, 0, 6, Qt::AlignCenter);

	QLabel* axis_label = new QLabel("Axis", panel_display_widget, "axis_label");
	panel_display_layout->addWidget(axis_label, 3, 0, Qt::AlignCenter);

	_show_top_axis_fld = new QCheckBox(panel_display_widget,"show_top_axis");
	_show_top_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_top_axis_fld, 3, 1, Qt::AlignCenter);
	connect( _show_top_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setTopAxis(bool)) );

	_show_bottom_axis_fld = new QCheckBox(panel_display_widget,"show_bottom_axis");
	_show_bottom_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_axis_fld, 3, 2, Qt::AlignCenter);
	connect( _show_bottom_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomAxis(bool)) );

	_show_left_axis_fld = new QCheckBox(panel_display_widget, "show_left_axis");
	_show_left_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_left_axis_fld, 3, 3, Qt::AlignCenter);
	connect( _show_left_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftAxis(bool)) );

	_show_right_axis_fld = new QCheckBox(panel_display_widget, "show_right_axis");
	_show_right_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_right_axis_fld, 3, 4, Qt::AlignCenter);
	connect(_show_right_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRightAxis(bool)) );
	
	_show_lr_alternating_axis_fld = new QCheckBox(panel_display_widget, "show_lr_alternating_axis");
	_show_lr_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_lr_alternating_axis_fld, 3, 5, Qt::AlignCenter);
	connect(_show_lr_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingAxis(bool)) );
	
	_show_rl_alternating_axis_fld = new QCheckBox(panel_display_widget, "show_rl_alternating_axis");
	_show_rl_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_axis_fld, 3, 6, Qt::AlignCenter);
	connect(_show_rl_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingAxis(bool)) );

	QLabel* ticks_labels_label = new QLabel("Ticks Labels", panel_display_widget, "ticks_labels_label");
	panel_display_layout->addWidget(ticks_labels_label, 5, 0, Qt::AlignCenter);

	_show_top_ticks_labels_fld = new QCheckBox(panel_display_widget, "show_top_ticks_labels");
	_show_top_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_ticks_labels_fld, 5, 1, Qt::AlignCenter);
	connect( _show_top_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setTopTicksLabels(bool)) );

	_show_bottom_ticks_labels_fld=new QCheckBox(panel_display_widget, "show_bottom_ticks_labels");
	_show_bottom_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_ticks_labels_fld, 5, 2, Qt::AlignCenter);
	connect( _show_bottom_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomTicksLabels(bool)) );

	_show_left_ticks_labels_fld = new QCheckBox(panel_display_widget, "show_left_ticks_labels");
	_show_left_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_ticks_labels_fld, 5, 3, Qt::AlignCenter);
	connect( _show_left_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftTicksLabels(bool)) );

	_show_right_ticks_labels_fld = new QCheckBox(panel_display_widget, "show_right_ticks_labels");
	_show_right_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_ticks_labels_fld, 5, 4, Qt::AlignCenter);
	connect( _show_right_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightTicksLabels(bool)) );
	
	_show_lr_alternating_ticks_labels_fld = new QCheckBox(panel_display_widget, "show_lr_alternating_ticks_labels");
	_show_lr_alternating_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_ticks_labels_fld, 5, 5, Qt::AlignCenter);
	connect( _show_lr_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingTicksLabels(bool)) );
	
	_show_rl_alternating_ticks_labels_fld = new QCheckBox(panel_display_widget, "show_rl_alternating_ticks_labels");
	_show_rl_alternating_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_ticks_labels_fld, 5, 6, Qt::AlignCenter);
	connect( _show_rl_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingTicksLabels(bool)) );

	QLabel* labels_label = new QLabel("Labels", panel_display_widget, "labels_label");
	panel_display_layout->addWidget(labels_label, 7, 0, Qt::AlignCenter);

	_show_top_labels_fld = new QCheckBox(panel_display_widget, "show_top_labels");
	_show_top_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_labels_fld, 7, 1, Qt::AlignCenter);
	connect( _show_top_labels_fld, SIGNAL(toggled(bool)), this,SLOT(setTopLabels(bool)) );

	_show_bottom_labels_fld = new QCheckBox(panel_display_widget,"show_bottom_labels");
	_show_bottom_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_labels_fld, 7, 2, Qt::AlignCenter);
	connect( _show_bottom_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomLabels(bool)) );

	_show_left_labels_fld = new QCheckBox(panel_display_widget, "show_left_labels");
	_show_left_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_labels_fld, 7, 3, Qt::AlignCenter);
	connect( _show_left_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftLabels(bool)) );

	_show_right_labels_fld = new QCheckBox(panel_display_widget, "show_right_labels");
	_show_right_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_labels_fld, 7, 4, Qt::AlignCenter);
	connect( _show_right_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightLabels(bool)) );
	
	_show_lr_alternating_labels_fld = new QCheckBox(panel_display_widget, "show_lr_alternating_labels");
	_show_lr_alternating_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_labels_fld, 7, 5, Qt::AlignCenter);
	connect( _show_lr_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingLabels(bool)) );
	
	_show_rl_alternating_labels_fld = new QCheckBox(panel_display_widget, "show_rl_alternating_labels");
	_show_rl_alternating_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_labels_fld, 7, 6, Qt::AlignCenter);
	connect( _show_rl_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingLabels(bool)) );

	QFrame *hline2 = new QFrame(panel_display_widget, "hline2");
	hline2->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline2->setMinimumWidth(500);
	panel_display_layout->addMultiCellWidget(hline2, 8, 8, 0, 6, Qt::AlignCenter);

	QLabel* additional_lines_label = new QLabel("Additional Lines", panel_display_widget, "additional_lines_label");
	panel_display_layout->addMultiCellWidget(additional_lines_label, 9, 9, 0, 5, Qt::AlignCenter);

	QLabel* xiszero_label = new QLabel("X = 0", panel_display_widget, "xiszero_label");
	panel_display_layout->addWidget(xiszero_label, 10, 0, Qt::AlignRight);
	_show_xiszero_fld = new QCheckBox(panel_display_widget, "show_xiszero");
	_show_xiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_xiszero_fld, 10, 1, Qt::AlignCenter);
	connect( _show_xiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setXIsZero(bool)) );

	QLabel* yiszero_label = new QLabel("Y = 0", panel_display_widget, "xiszero_label");
	panel_display_layout->addWidget(yiszero_label, 10, 2, Qt::AlignRight);
	_show_yiszero_fld = new QCheckBox(panel_display_widget, "show_yiszero");
	_show_yiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_yiszero_fld, 10, 3, Qt::AlignCenter);
	connect( _show_yiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setYIsZero(bool)) );

	panel_display_layout->addRowSpacing(0,0);
	panel_display_layout->addRowSpacing(1,0);
	panel_display_layout->addRowSpacing(2,0);
	panel_display_layout->addRowSpacing(3, _show_top_axis_fld->height());
	panel_display_layout->addRowSpacing(4,0);
	panel_display_layout->addRowSpacing(5, _show_top_ticks_labels_fld->height());
	panel_display_layout->addRowSpacing(6,0);
	panel_display_layout->addRowSpacing(7, _show_top_labels_fld->height());
	panel_display_layout->addRowSpacing(8,0);
	panel_display_layout->addRowSpacing(9,0);
	panel_display_layout->addRowSpacing(10, _show_xiszero_fld->height());
	panel_display_layout->addRowSpacing(11,0);

	panel_display_layout->setRowStretch(0,0);
	panel_display_layout->setRowStretch(1,0);
	panel_display_layout->setRowStretch(2,0);
	panel_display_layout->setRowStretch(3,1);
	panel_display_layout->setRowStretch(4,0);
	panel_display_layout->setRowStretch(5,1);
	panel_display_layout->setRowStretch(6,0);
	panel_display_layout->setRowStretch(7,1);
	panel_display_layout->setRowStretch(8,0);
	panel_display_layout->setRowStretch(9,0);
	panel_display_layout->setRowStretch(10,1);
	panel_display_layout->setRowStretch(11,0);

	panel_display_layout->setColStretch(0,0);
	panel_display_layout->setColStretch(1,0);
	panel_display_layout->setColStretch(2,0);
	panel_display_layout->setColStretch(3,0);
	panel_display_layout->setColStretch(4,0);
	panel_display_layout->setColStretch(5,0);
	panel_display_layout->setColStretch(6,0);
	panel_display_layout->setColStretch(7,1);

	panel_display_top_layout->addWidget(panel_display_groupbox);

	panel_display_top_layout->addSpacing(4);
	panel_display_top_layout->addStretch(10);

	panel_frame_widget->adjustSize();
	panel_display_widget->adjustSize();

}
#else
QpDisplayTab::QpDisplayTab(QTabWidget *tabWidget,  QString name, QpFrameItem * _frame):
		QpTab(tabWidget, name, QP_FRAME_TAB)

{
	// Constructor for the Display Tab widget shown in the Panel Options

	I_Frame = _frame;

	QVBoxLayout *panel_display_top_layout = new QVBoxLayout(this);
	panel_display_top_layout->setSpacing(4);

	QGroupBox *panel_frame_groupbox = new QGroupBox(this);

	if(I_Frame->frameType() == QP_TS) panel_frame_groupbox->setTitle("TS Frame");
	else  panel_frame_groupbox->setTitle("XY Frame");


// 	QWidget* panel_frame_widget = new QWidget(panel_frame_groupbox, "panel_frame_widget");

	QGridLayout *panel_frame_layout = new QGridLayout;
	panel_frame_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_frame_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* line_width_label = new QLabel("Line width: ");
	panel_frame_layout->addWidget(line_width_label, 1, 0, Qt::AlignRight);

	_line_width_fld = new FloatSpinBox(0.1, 20., 1, 0.1, panel_frame_groupbox, "line_width");
	_line_width_fld->setValue(QSAS_profiles->current()->LineWidth);
	_line_width_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_line_width_fld, 1, 1, Qt::AlignLeft);

	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));

	QLabel* colour_label = new QLabel("Line colour: ");
	panel_frame_layout->addWidget(colour_label, 1, 2, Qt::AlignRight);
	_line_colour_fld = new QPushButton("Colour...");
	_line_colour_fld->setFixedWidth(100);
	_line_colour_fld->setFixedHeight(30);
	QFont f = _line_colour_fld->font();
	f.setBold(TRUE);
	_line_colour_fld->setFont(f);
	panel_frame_layout->addWidget(_line_colour_fld,1,3,Qt::AlignLeft);

	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));

	QLabel* ticks_length_label = new QLabel("Ticks length: ");
	panel_frame_layout->addWidget(ticks_length_label, 3, 0, Qt::AlignRight);
	_ticks_length_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_groupbox, "ticks_length");
	_ticks_length_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_length_fld, 3, 1, Qt::AlignLeft);
	_ticks_length_fld->setValue(QPUI_TICKS_LENGTH);
	connect(_ticks_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	
	QLabel* ticks_char_size_label = new QLabel("Ticks font size: ");
	panel_frame_layout->addWidget(ticks_char_size_label, 3, 2, Qt::AlignRight);
	_ticks_char_size_fld = new FloatSpinBox(0.01, 40., 1, .5, panel_frame_groupbox, "ticks_char_size");
	_ticks_char_size_fld->setFixedWidth(100);
	panel_frame_layout->addWidget(_ticks_char_size_fld, 3, 3, Qt::AlignLeft);
	_ticks_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	connect(_ticks_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksCharSize(double)));

// 	panel_frame_layout->addRowSpacing(0,0);
// 	panel_frame_layout->addRowSpacing(1, _line_width_fld->height());
// 	panel_frame_layout->addRowSpacing(2,0);
// 	panel_frame_layout->addRowSpacing(3, _ticks_length_fld->height());
// 	panel_frame_layout->addRowSpacing(4,0);
// 
	panel_frame_layout->setRowStretch(0,0);
	panel_frame_layout->setRowStretch(1,1);
	panel_frame_layout->setRowStretch(2,0);
	panel_frame_layout->setRowStretch(3,1);
	panel_frame_layout->setRowStretch(4,0);

	panel_frame_layout->setColStretch(0,0);
	panel_frame_layout->setColStretch(1,0);
	panel_frame_layout->setColStretch(2,0);
	panel_frame_layout->setColStretch(3,0);
	panel_frame_layout->setColStretch(4,1);
	
	panel_frame_groupbox->setLayout(panel_frame_layout);

	panel_display_top_layout->addWidget(panel_frame_groupbox);

	QGroupBox *panel_display_groupbox  =new QGroupBox(this);

	panel_display_groupbox->setTitle("Show");

// 	QWidget* panel_display_widget = new QWidget(panel_display_groupbox, "panel_display_widget");

	QGridLayout *panel_display_layout = new QGridLayout;
	panel_display_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	panel_display_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* top_label = new QLabel("Top");
	panel_display_layout->addWidget(top_label, 1, 1, Qt::AlignCenter);
	QLabel* bottom_label = new QLabel("Bottom");
	panel_display_layout->addWidget(bottom_label, 1, 2, Qt::AlignCenter);
	QLabel* left_label = new QLabel("Left");
	panel_display_layout->addWidget(left_label, 1, 3, Qt::AlignCenter);
	QLabel* right_label = new QLabel("Right");
	panel_display_layout->addWidget(right_label, 1, 4, Qt::AlignCenter);
	QLabel* lr_alternating_label = new QLabel("L/R alternating");
	panel_display_layout->addWidget(lr_alternating_label, 1, 5, Qt::AlignCenter);
	QLabel* rl_alternating_label = new QLabel("R/L alternating");
	panel_display_layout->addWidget(rl_alternating_label, 1, 6, Qt::AlignCenter);

	QFrame *hline = new QFrame;
	hline->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline->setMinimumWidth(500);
	panel_display_layout->addMultiCellWidget(hline, 2, 2, 0, 6, Qt::AlignCenter);

	QLabel* axis_label = new QLabel("Axis");
	panel_display_layout->addWidget(axis_label, 3, 0, Qt::AlignCenter);

	_show_top_axis_fld = new QCheckBox;
	_show_top_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_top_axis_fld, 3, 1, Qt::AlignCenter);
	connect( _show_top_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setTopAxis(bool)) );

	_show_bottom_axis_fld = new QCheckBox;
	_show_bottom_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_axis_fld, 3, 2, Qt::AlignCenter);
	connect( _show_bottom_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomAxis(bool)) );

	_show_left_axis_fld = new QCheckBox;
	_show_left_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_left_axis_fld, 3, 3, Qt::AlignCenter);
	connect( _show_left_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftAxis(bool)) );

	_show_right_axis_fld = new QCheckBox;
	_show_right_axis_fld->setChecked(true);
	panel_display_layout->addWidget(_show_right_axis_fld, 3, 4, Qt::AlignCenter);
	connect(_show_right_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRightAxis(bool)) );
	
	_show_lr_alternating_axis_fld = new QCheckBox;
	_show_lr_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_lr_alternating_axis_fld, 3, 5, Qt::AlignCenter);
	connect(_show_lr_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingAxis(bool)) );
	
	_show_rl_alternating_axis_fld = new QCheckBox;
	_show_rl_alternating_axis_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_axis_fld, 3, 6, Qt::AlignCenter);
	connect(_show_rl_alternating_axis_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingAxis(bool)) );

	QLabel* ticks_labels_label = new QLabel("Ticks Labels");
	panel_display_layout->addWidget(ticks_labels_label, 5, 0, Qt::AlignCenter);

	_show_top_ticks_labels_fld = new QCheckBox;
	_show_top_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_ticks_labels_fld, 5, 1, Qt::AlignCenter);
	connect( _show_top_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setTopTicksLabels(bool)) );

	_show_bottom_ticks_labels_fld=new QCheckBox;
	_show_bottom_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_ticks_labels_fld, 5, 2, Qt::AlignCenter);
	connect( _show_bottom_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomTicksLabels(bool)) );

	_show_left_ticks_labels_fld = new QCheckBox;
	_show_left_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_ticks_labels_fld, 5, 3, Qt::AlignCenter);
	connect( _show_left_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftTicksLabels(bool)) );

	_show_right_ticks_labels_fld = new QCheckBox;
	_show_right_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_ticks_labels_fld, 5, 4, Qt::AlignCenter);
	connect( _show_right_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightTicksLabels(bool)) );
	
	_show_lr_alternating_ticks_labels_fld = new QCheckBox;
	_show_lr_alternating_ticks_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_ticks_labels_fld, 5, 5, Qt::AlignCenter);
	connect( _show_lr_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingTicksLabels(bool)) );
	
	_show_rl_alternating_ticks_labels_fld = new QCheckBox;
	_show_rl_alternating_ticks_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_ticks_labels_fld, 5, 6, Qt::AlignCenter);
	connect( _show_rl_alternating_ticks_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingTicksLabels(bool)) );

	QLabel* labels_label = new QLabel("Labels");
	panel_display_layout->addWidget(labels_label, 7, 0, Qt::AlignCenter);

	_show_top_labels_fld = new QCheckBox;
	_show_top_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_top_labels_fld, 7, 1, Qt::AlignCenter);
	connect( _show_top_labels_fld, SIGNAL(toggled(bool)), this,SLOT(setTopLabels(bool)) );

	_show_bottom_labels_fld = new QCheckBox;
	_show_bottom_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_bottom_labels_fld, 7, 2, Qt::AlignCenter);
	connect( _show_bottom_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setBottomLabels(bool)) );

	_show_left_labels_fld = new QCheckBox;
	_show_left_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_left_labels_fld, 7, 3, Qt::AlignCenter);
	connect( _show_left_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLeftLabels(bool)) );

	_show_right_labels_fld = new QCheckBox;
	_show_right_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_right_labels_fld, 7, 4, Qt::AlignCenter);
	connect( _show_right_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRightLabels(bool)) );
	
	_show_lr_alternating_labels_fld = new QCheckBox;
	_show_lr_alternating_labels_fld->setChecked(true);
	panel_display_layout->addWidget(_show_lr_alternating_labels_fld, 7, 5, Qt::AlignCenter);
	connect( _show_lr_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setLRAlternatingLabels(bool)) );
	
	_show_rl_alternating_labels_fld = new QCheckBox;
	_show_rl_alternating_labels_fld->setChecked(false);
	panel_display_layout->addWidget(_show_rl_alternating_labels_fld, 7, 6, Qt::AlignCenter);
	connect( _show_rl_alternating_labels_fld, SIGNAL(toggled(bool)), this, SLOT(setRLAlternatingLabels(bool)) );

	QFrame *hline2 = new QFrame;
	hline2->setFrameStyle(QFrame::HLine | QFrame::Raised);
	hline2->setMinimumWidth(500);
	panel_display_layout->addMultiCellWidget(hline2, 8, 8, 0, 6, Qt::AlignCenter);

	QLabel* additional_lines_label = new QLabel("Additional Lines");
	panel_display_layout->addMultiCellWidget(additional_lines_label, 9, 9, 0, 5, Qt::AlignCenter);

	QLabel* xiszero_label = new QLabel("X = 0");
	panel_display_layout->addWidget(xiszero_label, 10, 0, Qt::AlignRight);
	_show_xiszero_fld = new QCheckBox;
	_show_xiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_xiszero_fld, 10, 1, Qt::AlignCenter);
	connect( _show_xiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setXIsZero(bool)) );

	QLabel* yiszero_label = new QLabel("Y = 0");
	panel_display_layout->addWidget(yiszero_label, 10, 2, Qt::AlignRight);
	_show_yiszero_fld = new QCheckBox;
	_show_yiszero_fld->setChecked(false);
	panel_display_layout->addWidget(_show_yiszero_fld, 10, 3, Qt::AlignCenter);
	connect( _show_yiszero_fld, SIGNAL(toggled(bool)), this, SLOT(setYIsZero(bool)) );

// 	panel_display_layout->addRowSpacing(0,0);
// 	panel_display_layout->addRowSpacing(1,0);
// 	panel_display_layout->addRowSpacing(2,0);
// 	panel_display_layout->addRowSpacing(3, _show_top_axis_fld->height());
// 	panel_display_layout->addRowSpacing(4,0);
// 	panel_display_layout->addRowSpacing(5, _show_top_ticks_labels_fld->height());
// 	panel_display_layout->addRowSpacing(6,0);
// 	panel_display_layout->addRowSpacing(7, _show_top_labels_fld->height());
// 	panel_display_layout->addRowSpacing(8,0);
// 	panel_display_layout->addRowSpacing(9,0);
// 	panel_display_layout->addRowSpacing(10, _show_xiszero_fld->height());
// 	panel_display_layout->addRowSpacing(11,0);

	panel_display_layout->setRowStretch(0,0);
	panel_display_layout->setRowStretch(1,0);
	panel_display_layout->setRowStretch(2,0);
	panel_display_layout->setRowStretch(3,1);
	panel_display_layout->setRowStretch(4,0);
	panel_display_layout->setRowStretch(5,1);
	panel_display_layout->setRowStretch(6,0);
	panel_display_layout->setRowStretch(7,1);
	panel_display_layout->setRowStretch(8,0);
	panel_display_layout->setRowStretch(9,0);
	panel_display_layout->setRowStretch(10,1);
	panel_display_layout->setRowStretch(11,0);

	panel_display_layout->setColStretch(0,0);
	panel_display_layout->setColStretch(1,0);
	panel_display_layout->setColStretch(2,0);
	panel_display_layout->setColStretch(3,0);
	panel_display_layout->setColStretch(4,0);
	panel_display_layout->setColStretch(5,0);
	panel_display_layout->setColStretch(6,0);
	panel_display_layout->setColStretch(7,1);

	panel_display_groupbox->setLayout(panel_display_layout);
	
	panel_display_top_layout->addWidget(panel_display_groupbox);

	panel_display_top_layout->addSpacing(4);
	panel_display_top_layout->addStretch(10);
	
// 	panel_frame_widget->adjustSize();
// 	panel_display_widget->adjustSize();

}
#endif

void QpDisplayTab::setLineWidth(double fChoice)
{
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(PANEL_LINE_WIDTH, 0, (float)fChoice);

}

void QpDisplayTab::setLineColour()
{
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(I_Frame->_line_colour, &changed, this);
	if(changed)
	{
		QPalette line_palette = QPalette(QPUI_normal_palette);
		line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, c);
		line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, c);
		this->_line_colour_fld->setPalette(line_palette);
		I_Frame->_line_colour = c;
	}

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(PANEL_LINE_COLOUR, 0, 0., "", false, c);
}

void QpDisplayTab::setTicksLength(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TICKS_LENGTH, 0, fChoice);
}

void QpDisplayTab::setTicksCharSize(double fChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TICKS_CHAR_SIZE, 0, fChoice);
}

void QpDisplayTab::setTopAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TOP_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(BOTTOM_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LEFT_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RIGHT_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LR_ALTERNATING_AXIS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingAxis(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RL_ALTERNATING_AXIS, 0, 0.0, "", bChoice);
}


void QpDisplayTab::setTopTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TOP_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(BOTTOM_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LEFT_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RIGHT_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LR_ALTERNATING_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingTicksLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RL_ALTERNATING_TICKS_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setTopLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(TOP_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setBottomLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(BOTTOM_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLeftLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LEFT_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRightLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RIGHT_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setLRAlternatingLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(LR_ALTERNATING_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setRLAlternatingLabels(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(RL_ALTERNATING_LABELS, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setYIsZero(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(Y_IS_ZERO, 0, 0.0, "", bChoice);
}

void QpDisplayTab::setXIsZero(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Frame->I_ListView->setFrameOption(X_IS_ZERO, 0, 0.0, "", bChoice);
}




