// qpui_panel.cc
//
// Plot Interface Implementation - Panel Object
//
// Janet Barnes
// July 2002
//
//  Tony Allen
//  Feb 2008

#include "qpui.h"
#include "qsas_plplot_time_helpers.h"

extern QPalette QPUI_normal_palette;
extern int qsas_restore_release;

void getXminXmaxTS(const Time& start, const Time& end, double * xmin, double * xmax, struct tm * tm_start, struct tm * tm_end)
{
	vector<string> decomp_time;
	
	start.split_srep(decomp_time);
	tm_start->tm_year=QString(decomp_time[0].c_str()).toInt()-1900;
	tm_start->tm_mon=QString(decomp_time[2].c_str()).toInt()-1;
	tm_start->tm_mday=QString(decomp_time[5].c_str()).toInt();
	tm_start->tm_hour=QString(decomp_time[6].c_str()).toInt();
	tm_start->tm_min=QString(decomp_time[7].c_str()).toInt();
	tm_start->tm_sec=QString(decomp_time[8].c_str()).toInt();
				//TODO handle msecs
	*xmin=mktimeutc(tm_start);
				
	end.split_srep(decomp_time);
	tm_end->tm_year=QString(decomp_time[0].c_str()).toInt()-1900;
	tm_end->tm_mon=QString(decomp_time[2].c_str()).toInt()-1;
	tm_end->tm_mday=QString(decomp_time[5].c_str()).toInt();
	tm_end->tm_hour=QString(decomp_time[6].c_str()).toInt();
	tm_end->tm_min=QString(decomp_time[7].c_str()).toInt();
	tm_end->tm_sec=QString(decomp_time[8].c_str()).toInt();
				//TODO handle msecs
	*xmax=mktimeutc(tm_end);
	
}

//     QpPanelItem CLASS
//     ----------------------

QpPanelItem::QpPanelItem(QpFrameItem *parent, QString name, QpPanelItem *after):
    QpListItem(parent,name,after,ListTypeGroup,"")
{
	// Constructor
  	  
	I_Frame = parent;
	I_Type = QP_PANEL;
	
	// initialise options tab widgets
   	LegendTab = new QpLegendTab(tabWidget, "LegendTab", this);
	LegendTab->hide();
	
   	yAxisTab = new QpYAxisTab(tabWidget, "yAxisTab", this);
	yAxisTab->hide();
	
}

// --------------------------------------

QpPanelItem::~QpPanelItem()
{
	// Destructor
	// empty panel of plot items
	QpPlotItem * child = dynamic_cast<QpPlotItem*>(this->firstChild());
	while(child)
	{
		QpPlotItem * next = dynamic_cast<QpPlotItem*>(child->nextSibling());
		delete child;
		child = next;
	}
	
	delete LegendTab;
	delete yAxisTab;
}

// ----------------------------------

void QpPanelItem::setDefaultOptions()
{

	yAxisTab->_axis_linlog_fld->setCurrentItem(QP_LINEAR);
	
	yAxisTab->_axis_from_xy_fld->setText("");	
	yAxisTab->_axis_to_xy_fld->setText("");
	yAxisTab->_label_fld->setText("");
		
	yAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_META);
	yAxisTab->_axis_major_ticks_num_fld->setValue(0);
	yAxisTab->_axis_minor_ticks_num_fld->setValue(0);
	yAxisTab->_axis_major_tickmarks_fld->setCurrentItem(QPUI_IN);
	yAxisTab->_axis_minor_tickmarks_fld->setCurrentItem(QPUI_IN);
	yAxisTab->_axis_autolabel_fld->setChecked(true);
	yAxisTab->_label_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	yAxisTab->_label_disp_fld->setValue(QPUI_LABELS_DISP);
	yAxisTab->_label_rotation_fld->setValue(0);
	yAxisTab->_label_font_fld->setSansFont();
	yAxisTab->_label_font_fld->setItalic(false);
	yAxisTab->_label_font_fld->setBold(false);

	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, Qt::black);
	line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, Qt::black);
	yAxisTab->_label_colour_fld->setPalette(line_palette);
    yAxisTab->_label_colour = Qt::black;
	
	LegendTab->_show_legend_fld->setChecked(false);
	LegendTab->_legend_border_fld->setCurrentItem(QPUI_LINE);
	LegendTab->_legend_background_fld->setCurrentItem(QPUI_TRANSPARENT);
	LegendTab->_legend_num_rows_fld->setValue(1);
	LegendTab->_legend_num_cols_fld->setValue(1);
	LegendTab->_legend_width_fld->setValue(QPUI_LEGEND_WIDTH);
	LegendTab->_legend_height_fld->setValue(QPUI_LEGEND_HEIGHT);
	LegendTab->_legend_max_char_length_fld->setValue(15);
	LegendTab->_legend_line_length_fld->setValue(0.2);
	LegendTab->_legend_char_size_fld->setValue(0.5);
	LegendTab->_legend_just_fld->setCurrentItem(QPUI_VCENTRE);
	LegendTab->_legend_font_fld->setSansFont();
	LegendTab->_legend_font_fld->setItalic(false);
	LegendTab->_legend_font_fld->setBold(false);
	LegendTab->_legend_align_fld->setCurrentItem(QPUI_INSIDE_TOP_RIGHT);
	LegendTab->_legend_loffset_fld->setValue(0.75);
	LegendTab->_legend_toffset_fld->setValue(0.);
}

// -------------------------------------------

void QpPanelItem::showTabs()
{	

	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--) 
	{
		if(tabWidget->page(i))
		{
			tabWidget->page(i)->hide();
			tabWidget->removePage(this->tabWidget->page(i));
		}
	}

	tabWidget->addTab(yAxisTab, "Y Axis");
	tabWidget->addTab(LegendTab, "Legend");
	// value return is always shown
	tabWidget->addTab(I_Frame->I_Page->ValueReturnTab, "Value Return");
	
	if(this->I_ListView->I_LastTab == QP_LEGEND_TAB)  tabWidget->showPage(LegendTab);
	else tabWidget->showPage(yAxisTab);
}

// -------------------------------------------

int QpPanelItem::getStackNumber()
{
	// get position in frame stack, 1 = first
	
	int cnt = 1;
	QpPanelItem * item = dynamic_cast<QpPanelItem*>(I_Frame->firstChild());
	while(item)
	{
		if(item == this) return cnt;
		item = dynamic_cast<QpPanelItem*>(item->nextSibling());
		cnt++;
	}
	return cnt; // just to be safe

}

// -------------------------------------------

int QpPanelItem::numberTraces()
{
	// get number of plot items that are traces in this panel
	
	int cnt = 0;
	QpPlotItem * item = dynamic_cast<QpPlotItem*>(this->firstChild());
	while(item)
	{
		QpTraceItem * traceItem = dynamic_cast<QpTraceItem*>(item);
		if(traceItem && traceItem->DataTab->_show_legend_fld->isChecked())  cnt++;
		item = dynamic_cast<QpPlotItem*>(item->nextSibling());
	}
	return cnt;

}

// ---------------------------

void QpPanelItem::changePlotType(QpPlotItem *item, QpItemType plotType)
{
	// this method is called when plot  type changes. Changes DataTab accordingly
	
	if(QuSlotDebug())
	{
		printf("QpPanelItem::changePlotType %ld\n", plotType);
		fflush(stdout);
	}
	
	if(item->I_Type == plotType) return; // no change

	QComboBox *typeFieldHeld;
	
	// get info from old item
	
	QpPlotItem * after = dynamic_cast<QpPlotItem*>(item->itemAbove());
	
	QString WLcontext = "";
	QString dataName;
	QpTraceItem * traceItem = dynamic_cast<QpTraceItem*>(item);
	QpSpectrumItem * spectrumItem = dynamic_cast<QpSpectrumItem*>(item);
	
    if(traceItem)
	{
		dataName = QString(traceItem->DataTab->_y_data_fld->text().latin1());
		WLcontext = baseObjectText(dataName);		
	}
	else if(spectrumItem)
	{
		dataName = QString(spectrumItem->DataTab->_z_data_fld->text().latin1());
		WLcontext = baseObjectText(dataName);
	}
	
	// delete item
	
	delete item;
	
    // create new fields
	
    switch(plotType)
    {
		default: case QP_TRACE: case QP_CONST_Y: case QP_CONST_XT:
		{
			QpTraceItem * newItem= new QpTraceItem(this, dataName, after, WLcontext, plotType);
			I_ListView->setCurrentItem(newItem);
			break;
		}
		case QP_SPECTROGRAM:
		{
			QdObject_var dobj = QuGetObjByName(WLcontext);
			QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(dobj);
			if(ms.is_nil()) 
			{
				QpSpectrumItem * newItem= new QpSpectrumItem(this, dataName, after, "");
				I_ListView->setCurrentItem(newItem);
			}
			else
			{
				QpSpectrumItem * newItem= new QpSpectrumItem(this, dataName, after, dataName);
				I_ListView->setCurrentItem(newItem);
			}
			
			break;
		}
	}	
	I_ListView->I_LastTab = QP_IGNORE_TAB; // tell updateOnSelection to ignore last tab and use DataTab
}

// -------------------------------------------------

string QpPanelItem::getTitle()
{
	if(QuSlotDebug())
	{
		printf("QpPanelItem::getTitle\n");
		fflush(stdout);
	}

	string title="";
	// get panel title from plot elements
	
	QpPlotItem * item = dynamic_cast<QpPlotItem*>(this->firstChild());
  
	while(item)
	{
		string item_title;
		QpTraceItem* traceItem = dynamic_cast<QpTraceItem*>(item);
		QpSpectrumItem* spectrumItem = dynamic_cast<QpSpectrumItem*>(item);      

		if(traceItem) item_title = traceItem->getTitle();
		else if(spectrumItem) item_title = spectrumItem->getTitle();
		
		if( title != item_title) 
		{
			if( title != "") title += "/";
			title += item_title;
		}
		item = dynamic_cast <QpPlotItem*>(item->nextSibling());
	}
	title = SafePGPString(title.c_str());
	return title;
}

// ---------------------------------------------

void QpPanelItem::prePlot()
{
	// pre plots panel (i.e. called BEFORE plot component is plotted)
	
	PLFLT xmin, xmax;
	if(frameType() == QP_TS)
	{
// 		struct tm dummy_s, dummy_e;
		Time start;
		start.set_time_rep(I_Frame->tAxisTab->_axis_from_time_fld->value());
		Time end;
		end.set_time_rep(I_Frame->tAxisTab->_axis_to_time_fld->value());
		xmin = (PLFLT) start.second_of_day();
		xmax = xmin + (PLFLT)Seconds(end - start);
// 		getXminXmaxTS(start, end, &xmin, &xmax, &dummy_s, &dummy_e);
	}
	else
	{
		xmin = I_Frame->xAxisTab->_axis_from_xy_fld->value();
		xmax = I_Frame->xAxisTab->_axis_to_xy_fld->value();
	}
	
	if(I_Frame->xtAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
	{
		xmin = SafeLog10(xmin);
		xmax = SafeLog10(xmax);
	}
	
	PLFLT ymin = yAxisTab->_axis_from_xy_fld->value();
	PLFLT ymax = yAxisTab->_axis_to_xy_fld->value();
	if(yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
	{
		ymin = (PLFLT)SafeLog10(ymin);
		ymax = (PLFLT)SafeLog10(ymax);
	}
	
	if(fabs(xmax - xmin) < 1.e-6)
	{
		xmin -= 0.5*xmin;
		xmax += 0.5*xmax;
	}
	if(fabs(ymax - ymin) < 1.e-6)
	{
		ymin -= 0.5*ymin;
		ymax += 0.5*ymax;
	}

	plwind(xmin, xmax, ymin, ymax);
	
	if(LegendTab->_show_legend_fld->isChecked())
	{
		// calculate width and height of legend entry
// 		string legend_text = SafePGPString(yAxisTab->_label_fld->text().latin1());
		plschr(QPUI_CHAR_HEIGHT, LegendTab->_legend_char_size_fld->value());
// 		PLFLT xch, ych;
// 		plgchr(&xch, &ych);
		_legend_entry = 0;
// 		PLFLT loffset, roffset, toffset, boffset;
// 		PLFLT x0vp_save, x1vp_save, y0vp_save, y1vp_save;
// 		plgvpd(&x0vp_save, &x1vp_save, &y0vp_save, &y1vp_save);

	}
	plschr(QPUI_CHAR_HEIGHT, I_Frame->DisplayTab->_ticks_length_fld->value());
}

// ------------------------------------------------

void QpPanelItem::postPlot(bool show_top_axis, bool show_bottom_axis)
{
	PLINT family, style, weight;
	
	float ymin = yAxisTab->_axis_from_xy_fld->value();
	float ymax = yAxisTab->_axis_to_xy_fld->value();
// 	if(ymin >= ymax) return;
	
	// draw frame

		// set colours, font, character size, line style and line width
	PLINT  line_colour_index = getPGPColourIndex(I_Frame->_line_colour);
	plcol0(line_colour_index);
	PLFLT _ticks_length = I_Frame->DisplayTab->_ticks_length_fld->value();
	PLFLT _ticks_label_size = I_Frame->DisplayTab->_ticks_char_size_fld->value();
// 		plschr(40., _ticks_length);
	pllsty(QP_SOLID);
	plwid(QtWidth2PlWidth(I_Frame->DisplayTab->_line_width_fld->value()));
		
	string xopt, yopt;
	string x_subopt, y_subopt;
	if(I_Frame->DisplayTab->_show_bottom_axis_fld->isChecked())
	{
		xopt+="b";
		x_subopt+="b";
	}
	if(I_Frame->DisplayTab->_show_left_axis_fld->isChecked())
	{
		yopt+="b";
		y_subopt+="b";
	}
	if(I_Frame->DisplayTab->_show_top_axis_fld->isChecked())
	{
		xopt+="c";
		x_subopt+="c";
	}
	if(I_Frame->DisplayTab->_show_right_axis_fld->isChecked())
	{
		yopt+="c";
		y_subopt+="c";
	}
	if(I_Frame->DisplayTab->_show_lr_alternating_axis_fld->isChecked())
	{
		if(getStackNumber()%2==1)
		{
			yopt+="b";
			y_subopt+="b";
		}
		else
		{
			yopt+="c";
			y_subopt+="c";
		}
	}
	if(I_Frame->DisplayTab->_show_rl_alternating_axis_fld->isChecked())
	{
		if(getStackNumber()%2==0)
		{
			yopt+="b";
			y_subopt+="b";
		}
		else
		{
			yopt+="c";
			y_subopt+="c";
		}
	}
				
	if(I_Frame->DisplayTab->_show_bottom_axis_fld->isChecked() || I_Frame->DisplayTab->_show_top_axis_fld->isChecked() || I_Frame->DisplayTab->_show_yiszero_fld->isChecked() )
	{

		I_Frame->xtAxisTab->_label_font_fld->getFontParams(family, style, weight);
		plsfont(family, style, weight);
		switch(I_Frame->xtAxisTab->_axis_minor_tickmarks_fld->currentItem())
		{
			case QPUI_IN:
				x_subopt+="st";
				break;
			case QPUI_OUT: default:
				x_subopt+="ist";
				break;
			case QPUI_NOTICKS:
				break;
		}
		switch(I_Frame->xtAxisTab->_axis_major_tickmarks_fld->currentItem())
		{
			case QPUI_IN: default:
				xopt += "t";
				break;
			case QPUI_OUT:
				xopt += "it";
				break;
			case QPUI_GRID:
				xopt+="gt";
				break;
			case QPUI_NOTICKS:
				break;
		}

		if(show_bottom_axis && I_Frame->DisplayTab->_show_bottom_ticks_labels_fld->isChecked()) xopt+="n";
			
		if(show_top_axis && I_Frame->DisplayTab->_show_top_ticks_labels_fld->isChecked()) xopt+="m";
			
		PLFLT xmin, xmax;
		PLINT nsubx = I_Frame->xtAxisTab->_axis_minor_ticks_num_fld->value();
		PLFLT xtick = 0.;
		int nx=I_Frame->xtAxisTab->_axis_major_ticks_num_fld->value();
		char fmt_ticks[50], fmt_label[50];

		if(frameType() == QP_TS)
		{
			PLINT auto_nsubx;
			PLFLT auto_xtick;
			char label[50];	
			Time start, end;
			struct tm tm_start, tm_end;
				
			xopt += "d";
			x_subopt += "d";
				
			start.set_time_rep(I_Frame->tAxisTab->_axis_from_time_fld->value());
			end.set_time_rep(I_Frame->tAxisTab->_axis_to_time_fld->value());

			getXminXmaxTS(start, end, &xmin, &xmax, &tm_start, &tm_end);
				
			int separator=(QSAS_profiles->current()->TimeRep=="Colon"?1:0);
			computeFormat(fabs(xmax-xmin), fmt_ticks, fmt_label, &auto_xtick, &auto_nsubx, separator);
				
			ymin = yAxisTab->_axis_from_xy_fld->value();
			ymax = yAxisTab->_axis_to_xy_fld->value();
			if(yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
			{
				ymin = (PLFLT)SafeLog10(ymin);
				ymax = (PLFLT)SafeLog10(ymax);
			}

			if(fabs(ymax - ymin) < 1.e-6)
			{
				ymin -= 0.5*ymin;
				ymax += 0.5*ymax;
			}

			plwind(xmin, xmax, ymin, ymax);
				
			if(nx!=0)
			{
				xtick=(xmax-xmin)/(PLFLT)nx;
			}
			else
			{
				xtick=auto_xtick;
			}
				
			if(nsubx==0) nsubx=auto_nsubx;
				
			if(I_Frame->tAxisTab->tick_time_fmt_auto_fld->isChecked())
			{
				pltimefmt(fmt_ticks);
				I_Frame->tAxisTab->tick_time_fmt_fld->setText(QString(fmt_ticks));
			}
			else
			{
				pltimefmt(I_Frame->tAxisTab->tick_time_fmt_fld->text().latin1());
			}
				
			// print labels
			
			I_Frame->tAxisTab->_label_font_fld->getFontParams(family, style, weight);
			plsfont(family, style, weight);
				
			plschr(0., _ticks_label_size);
				
			if(I_Frame->tAxisTab->time_labels_fmt_auto_fld->isChecked())
			{
				strftime(label,49,fmt_label, &tm_start);
				I_Frame->tAxisTab->time_labels_fmt_fld->setText(QString(fmt_label));
			}
			else
			{
				QString format=I_Frame->tAxisTab->time_labels_fmt_fld->text();
				strftime(label,49,format.latin1(), &tm_start);
			}
				
			if(show_bottom_axis && I_Frame->DisplayTab->_show_bottom_ticks_labels_fld->isChecked())
				plmtex("b",3.,0.0,0.3,label);
			if(show_top_axis && I_Frame->DisplayTab->_show_top_ticks_labels_fld->isChecked())
				plmtex("t",3.,0.0,0.3,label);
		}
		else
		{
			xmin = I_Frame->xAxisTab->_axis_from_xy_fld->value();
			xmax = I_Frame->xAxisTab->_axis_to_xy_fld->value();
			if(nx>0)
			{
				xtick=(xmax-xmin)/(PLFLT)nx;
			}
		}
			
		if(I_Frame->xtAxisTab->_axis_linlog_fld->currentItem() == QP_LOG){
			xopt+="l";
			x_subopt+="l";
			xmin = SafeLog10(xmin);
			xmax = SafeLog10(xmax);
		}
			
		plsmaj(0., _ticks_length);
		plschr(0., _ticks_label_size);
		plbox(xopt.c_str(), xtick, nsubx, "", 0, 0);
		plsmaj(0., 0.);
		plsmin(0., _ticks_length*0.6);
		plbox(x_subopt.c_str(), xtick, nsubx, "", 0, 0);
			
		// draw y=0
		if(I_Frame->DisplayTab->_show_yiszero_fld->isChecked() )
		{
			PLFLT yzero_x[2] = {xmin, xmax};
			PLFLT yzero_y[2]={0., 0.};
			plline(2,yzero_x,yzero_y);
		}
	}
	
	yAxisTab->_label_font_fld->getFontParams(family, style, weight);
	plsfont(family, style, weight);
	switch(yAxisTab->_axis_minor_tickmarks_fld->currentItem())
	{
		case QPUI_IN:
		default:
			y_subopt += "st";
			break;
		case QPUI_OUT:
			y_subopt += "ist";
			break;
		case QPUI_NOTICKS:
			break;
	}
		
	if(yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
	{
		yopt += "l";
		y_subopt+="l";
		ymin = SafeLog10(ymin);
		ymax = SafeLog10(ymax);
	}
		
	switch(yAxisTab->_axis_major_tickmarks_fld->currentItem())
	{
		case QPUI_IN: default:
			yopt += "t";
			break;
		case QPUI_OUT:
			yopt += "it";
			break;
		case QPUI_GRID:
			yopt += "gt";
			break;
		case QPUI_NOTICKS:
			break;
	}
		
	if(I_Frame->DisplayTab->_show_left_ticks_labels_fld->isChecked())
	{
		yopt += "nv";
	}
	if(I_Frame->DisplayTab->_show_right_ticks_labels_fld->isChecked())
	{
		yopt += "mv";
	}
	if(I_Frame->DisplayTab->_show_lr_alternating_ticks_labels_fld->isChecked())
	{
		if(getStackNumber()%2==1)
		{
			yopt += "nv";
		}
		else
		{
			yopt += "mv";
		}
	}
	if(I_Frame->DisplayTab->_show_rl_alternating_ticks_labels_fld->isChecked())
	{
		if(getStackNumber()%2==0)
		{
			yopt += "nv";
		}
		else
		{
			yopt += "mv";
		}
	}
		
	PLINT nsuby = yAxisTab->_axis_minor_ticks_num_fld->value();
	int ny=yAxisTab->_axis_major_ticks_num_fld->value();
	PLFLT ytick;
	if(ny!=0)
	{
		ytick=(ymax-ymin)/(PLFLT)ny;
	}
	else ytick=0.;
		
	plsmaj(0., _ticks_length);
	plschr(0., _ticks_label_size);
	plbox("", 0., 0, yopt.c_str(), ytick, nsuby);
	plsmaj(0., 0.);
	plsmin(0., _ticks_length*0.6);
	plbox("", 0., 0, y_subopt.c_str(), ytick, nsuby);
	plsmaj(0., _ticks_length);
		
	//draw x=0
	if(I_Frame->DisplayTab->_show_xiszero_fld->isChecked())
	{
		PLFLT xzero_x[2] = {0.,0.};
		PLFLT xzero_y[2] = {ymin, ymax};
		plline(2, xzero_x, xzero_y);
	}
		
	// draw labels - text line width needs to be adjustable
	int _text_line_width = 1;
	plwid(QtWidth2PlWidth(_text_line_width));
				
	PLFLT x0win_save, x1win_save, y0win_save, y1win_save;
	PLFLT x0vp_save, x1vp_save, y0vp_save, y1vp_save;
			
	plgvpw(&x0win_save, &x1win_save, &y0win_save, &y1win_save);
	plgvpd(&x0vp_save, &x1vp_save, &y0vp_save, &y1vp_save);
				
	float xch_ndc = yAxisTab->_label_char_size_fld->value()/40.;
	float ych_ndc = I_Frame->xtAxisTab->_label_char_size_fld->value()/40.;
			
	plvpor(0., 1., 0., 1.);
	plwind(0., 1., 0., 1.);
				
	if(show_top_axis || show_bottom_axis)
	{
		I_Frame->xtAxisTab->_label_font_fld->getFontParams(family, style, weight);
		plsfont(family, style, weight);
		plschr(QPUI_CHAR_HEIGHT, I_Frame->xtAxisTab->_label_char_size_fld->value());
		double p_def, p_ht;
		plgchr (&p_def, &p_ht);	
		PLINT labels_colour_index = getPGPColourIndex(I_Frame->xtAxisTab->_label_colour);
		plcol0(labels_colour_index);
			
		if(I_Frame->DisplayTab->_show_top_labels_fld->isChecked() && show_top_axis)
		{
			PLFLT dx, dy;
			dx=cos(I_Frame->xtAxisTab->_label_rotation_fld->value()*M_PI/180.);
			dy=sin(I_Frame->xtAxisTab->_label_rotation_fld->value()*M_PI/180.);
			plptex((x0vp_save+x1vp_save)/2.,
					y1vp_save+getCharHeight()*(1.+I_Frame->xtAxisTab->_label_disp_fld->value()),	
					dx, dy, 0.5,SafePGPString(I_Frame->xtAxisTab->_label_fld->text().latin1()).c_str());
		}
		if(I_Frame->DisplayTab->_show_bottom_labels_fld->isChecked() && show_bottom_axis)
		{
			PLFLT dx, dy;
			dx=cos(I_Frame->xtAxisTab->_label_rotation_fld->value()*M_PI/180.);
			dy=sin(I_Frame->xtAxisTab->_label_rotation_fld->value()*M_PI/180.);
			plptex((x0vp_save+x1vp_save)/2.,
					y0vp_save-getCharHeight()*(1.+I_Frame->xtAxisTab->_label_disp_fld->value()),
					 dx, dy
					,0.5,
					SafePGPString(I_Frame->xtAxisTab->_label_fld->text().latin1()).c_str());
		}
	}
				
	if(I_Frame->DisplayTab->_show_right_labels_fld->isChecked() || I_Frame->DisplayTab->_show_left_labels_fld->isChecked() || I_Frame->DisplayTab->_show_lr_alternating_labels_fld->isChecked() || I_Frame->DisplayTab->_show_rl_alternating_labels_fld->isChecked())
	{
		float y_label_just = 13./(1. + yAxisTab->_label_char_size_fld->value() * yAxisTab->_label_fld->text().length());
		if(y_label_just > 1.) y_label_just = 1.;
		
		PLINT family, style, weight;
		yAxisTab->_label_font_fld->getFontParams(family, style, weight);
		plsfont(family, style, weight);
		plschr(QPUI_CHAR_HEIGHT, yAxisTab->_label_char_size_fld->value());
				
		PLINT labels_colour_index = getPGPColourIndex(yAxisTab->_label_colour);
		plcol0(labels_colour_index);
				
		if(I_Frame->DisplayTab->_show_right_labels_fld->isChecked() ||
		(I_Frame->DisplayTab->_show_lr_alternating_labels_fld->isChecked() && getStackNumber()%2==0) ||
		(I_Frame->DisplayTab->_show_rl_alternating_labels_fld->isChecked() && getStackNumber()%2==1))
		{
			PLFLT dx, dy;
			dx=cos(yAxisTab->_label_rotation_fld->value()*M_PI/180.);
			dy=sin(yAxisTab->_label_rotation_fld->value()*M_PI/180.);
			plptex(x1vp_save+getCharHeight()*(1.+yAxisTab->_label_disp_fld->value()),
					(y0vp_save+y1vp_save)/2.,
					dx, dy,
					0.5,
					SafePGPString(yAxisTab->_label_fld->text().latin1()).c_str());
		}
		if(I_Frame->DisplayTab->_show_left_labels_fld->isChecked() ||
		(I_Frame->DisplayTab->_show_lr_alternating_labels_fld->isChecked() && getStackNumber()%2==1) ||
		(I_Frame->DisplayTab->_show_rl_alternating_labels_fld->isChecked() && getStackNumber()%2==0))
		{
			PLFLT dx, dy;
			dx=cos(yAxisTab->_label_rotation_fld->value()*M_PI/180.);
			dy=sin(yAxisTab->_label_rotation_fld->value()*M_PI/180.);
			plptex(x0vp_save-getCharHeight()*(1.+yAxisTab->_label_disp_fld->value()),
					(y0vp_save+y1vp_save)/2.,
					dx, dy,
					0.5,
				SafePGPString(yAxisTab->_label_fld->text().latin1()).c_str());
		}
	}
				
	plvpor(x0vp_save, x1vp_save, y0vp_save, y1vp_save);
	plwind(x0win_save, x1win_save, y0win_save, y1win_save);
			
	if(show_top_axis)
	{
		I_Frame->TitleTab->_title_font_fld->getFontParams(family, style, weight);
		plsfont(family, style, weight);
		plschr(QPUI_CHAR_HEIGHT, I_Frame->TitleTab->_title_char_size_fld->value());
		PLINT title_colour_index = getPGPColourIndex(I_Frame->_title_colour);
		plcol0(title_colour_index);
		PLFLT fjust;
		PLFLT coord;
		switch(I_Frame->TitleTab->_title_just_fld->currentItem())
		{
			case QPUI_HCENTRE:
			default:
				coord = 0.5;
				fjust = 0.5;
				break;
			case QPUI_LEFT:
				coord = 0.0;
				fjust = 0.0;
				break;
			case QPUI_RIGHT:
				coord = 1.0;
				fjust = 1.0;
				break;
		}
				
		plmtex("t", I_Frame->TitleTab->_title_disp_fld->value(), coord, fjust, SafePGPString(I_Frame->TitleTab->_title_fld->text().latin1()).c_str());
				
		plschr(QPUI_CHAR_HEIGHT, I_Frame->TitleTab->_subtitle_char_size_fld->value());
				
		plmtex("t", I_Frame->TitleTab->_subtitle_disp_fld->value(), coord, fjust, SafePGPString(I_Frame->TitleTab->_subtitle_fld->text().latin1()).c_str());
	}
	
	// draw legend if show_legend set
	if(LegendTab->_show_legend_fld->isChecked() && LegendTab->_legend_border_fld->currentItem()  != QPUI_NO_BORDER && numberTraces() > 0)
	{
		PLFLT x0win_save, x1win_save, y0win_save, y1win_save;
		PLFLT x0vp_save, x1vp_save, y0vp_save, y1vp_save;
	
		plgvpw(&x0win_save, &x1win_save, &y0win_save, &y1win_save);
		plgvpd(&x0vp_save, &x1vp_save, &y0vp_save, &y1vp_save);
	
	//         QPlot_window->save();
	
		if(LegendTab->_legend_border_fld->currentItem() == QPUI_LINE) pllsty(QP_SOLID);
		else pllsty(QP_NOLINE);
			
		plschr(QPUI_CHAR_HEIGHT, LegendTab->_legend_char_size_fld->value());
		PLFLT xch, ych;
		plgchr(&xch, &ych);
		PLFLT x0_vp = x0vp_save + LegendTab->_legend_loffset_fld->value() * (x1vp_save - x0vp_save);
		PLFLT x1_vp = x0_vp + LegendTab->_legend_width_fld->value() * (x1vp_save - x0vp_save);
		PLFLT y1_vp = y1vp_save - LegendTab->_legend_toffset_fld->value() * (y1vp_save - y0vp_save);
		PLFLT y0_vp = y1_vp - LegendTab->_legend_height_fld->value() * (y1vp_save - y0vp_save);
	
		plvpor(x0_vp, x1_vp, y0_vp, y1_vp);
		plwind(0., 1., 0., 1.);
		plwid(QtWidth2PlWidth(1));
		plbox("bc", 0., 0, "bc", 0., 0);
	
		plscol0(0, 255, 255, 255);
		plscol0(1, 0, 0, 0);
		plcol0(1);
		//         QPlot_window->unsave();
		plvpor(x0vp_save, x1vp_save, y0vp_save, y1vp_save);
		plwind(x0win_save, x1win_save, y0win_save, y1win_save);
	}
}

// ------------------------------------

void QpPanelItem::updateAxes()
{
	if(QuSlotDebug()) cout << "QpPanelItem::updateAxes " << endl;
	// this is called from QpFrame::updateAxes to ensure x or t axis set first
	// Note range done first to ensure _ydo exists for other calls
	
	if( yAxisTab->_axis_autoscale_fld->currentItem() != QP_FROM_USER)  yAxisTab->setAxisMinMax();
	
	if( yAxisTab->_axis_autolabel_fld->isChecked())  yAxisTab->setAutoLabelText();
	
	LegendTab->updateNumTraces();

}

// ---------------------------------------

void QpPanelItem::getLabel(const QpAxes& axis, string& label)
{
	// get label for axis

	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	label = "-";
	string itemLabel;

	while(plotItem)
	{
		QpTraceItem *traceItem = dynamic_cast<QpTraceItem*>(plotItem);
		QpSpectrumItem *spectrogramItem = dynamic_cast<QpSpectrumItem*>(plotItem);
		if(traceItem)
		{
			traceItem->getLabel(axis, itemLabel);
			if(label == "-") label = itemLabel.c_str();
			else{
				label = QuCommonLabl(label, itemLabel);
			}
		}
		else if(spectrogramItem)
		{
			// only allowed one spectrum in a panel
			spectrogramItem->getLabel(axis, label);
			break;
		}
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	
	QString greek = QString(label.c_str());
	greek.replace("theta","#gh");
	greek.replace("phi","#gf");
	label = greek.latin1();
}

// ---------------------------------------

int QpPanelItem::getXmetaIsLog()
{	
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	
	int isLog=0;
	// descend thru' panel hierarchy
	while(plotItem)
	{
		if(isLog == 0) isLog = plotItem->getXmetaIsLog(); // update until log is set 
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	return isLog;
}

// ---------------------------------------

bool QpPanelItem::getXmetaRange(double& panel_xmin, double& panel_xmax)
{
	// get range of values for x axis

	double xmin = DBL_MAX;
	double xmax = -DBL_MAX;
	
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	double item_xmin, item_xmax;
	bool rangeFound=false;
	
	// descend thru' panel hierarchy
	while(plotItem)
	{
		// single call to ensure _xdo, _ydo, _zdo exist where appropriate
		plotItem->get_dobjs();
		
		if(plotItem->getXmetaRange(item_xmin, item_xmax))
		{
			xmin = min(xmin, item_xmin);
			xmax = max(xmax, item_xmax);
			rangeFound = true;
		}
		
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	if(rangeFound)
	{
		panel_xmin = xmin;
		panel_xmax = xmax;
	}

	return rangeFound;
}


// ---------------------------------------

void QpPanelItem::getYObjects()
{
	// Get data objects so get Y range has them.
	// Called when XT axis is user set.
	
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
		
	// descend thru' panel hierarchy
	while(plotItem)
	{
		// single call to ensure _xdo, _ydo, _zdo exist where appropriate
		plotItem->get_dobjs();
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}

	return;
}

// ---------------------------------------

bool QpPanelItem::getXRange(double& panel_xmin, double& panel_xmax)
{
	// get range of values for x axis

	double xmin = DBL_MAX;
	double xmax = -DBL_MAX;
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	double item_xmin, item_xmax;
	bool rangeFound=false;
	
	// descend thru' panel hierarchy
	while(plotItem)
	{
		plotItem->get_dobjs();
		if(plotItem->getXRange(item_xmin, item_xmax))
		{
			xmin = min(xmin, item_xmin);
			xmax = max(xmax, item_xmax);
			rangeFound = true;
		}
		
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	if(rangeFound)
	{
		panel_xmin = xmin;
		panel_xmax = xmax;
	}

	return rangeFound;
}

// ---------------------------------------

bool QpPanelItem::getYRange(double& panel_ymin, double& panel_ymax)
{
	// get range of values for y axis
	if ( QuSlotDebug() )
	{
		printf( "QpPanelItem::getYRange this = %ld \n", this );
		fflush(stdout);
	}

	double ymin = DBL_MAX;
	double ymax =- DBL_MAX;
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());

	double item_ymin, item_ymax;
	bool rangeFound = false;
	while(plotItem)
	{
		QpTraceItem *traceItem = dynamic_cast<QpTraceItem*>(plotItem);
		QpSpectrumItem *spectrumItem = dynamic_cast<QpSpectrumItem*>(plotItem);
		if(traceItem)
		{
			if(traceItem->getYRange(item_ymin,item_ymax))
			{
				ymin = min(ymin, item_ymin);
				ymax = max(ymax, item_ymax);
				rangeFound = true;
			}
		}
		else if(spectrumItem)
		{
			if(spectrumItem->getYRange( item_ymin, item_ymax))
			{
				ymin = min(ymin, item_ymin);
				ymax = max(ymax, item_ymax);
				rangeFound = true;
			}
		}
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	
	if(!rangeFound) return rangeFound;
	
	if(yAxisTab->_axis_y_nudge_fld->isChecked())
	{
		if(yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			double nudgeUp = pow(fabs(ymax / ymin), 0.01);
			double nudgeDown = pow(fabs(ymax / ymin), 0.02);
			ymax *= nudgeUp;
			ymin /= nudgeDown;
		}
		else
		{
			double nudge = fabs(ymax - ymin)*0.02;
			ymax += nudge;
			ymin -= nudge;
		}
	}
	panel_ymin = ymin;
	panel_ymax = ymax;
	return rangeFound;
}

// ---------------------------------------

bool QpPanelItem::getYmetaRange(double& panel_ymin, double& panel_ymax)
{
	// get range of values for y axis
	double ymin = DBL_MAX;
	double ymax =- DBL_MAX;
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());

	double item_ymin, item_ymax;
	bool rangeFound = false;
	int isLog = 0;
	while(plotItem)
	{
		if(plotItem->getYmetaRange(item_ymin, item_ymax))
		{
			ymin = min(ymin, item_ymin);
			ymax = max(ymax, item_ymax);
			rangeFound = true;
		}

		if(isLog == 0) isLog = plotItem->getYmetaIsLog();
		
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	if(rangeFound)
	{
		panel_ymin = ymin;
		panel_ymax = ymax;
	}
	
	if(isLog == 1) yAxisTab->_axis_linlog_fld->setCurrentItem(QP_LOG);
	else yAxisTab->_axis_linlog_fld->setCurrentItem(QP_LINEAR);
	
	return rangeFound;
}

// ---------------------------------------

QpuiReturnStatus QpPanelItem::plot(double _height, double _width, double _toffset, double _loffset, PlotWindow* win)
{
	// set viewport
    PLFLT x0, x1, y0, y1;
    x0 = _loffset;
    x1 = x0 + _width;
    y1 = 1 - _toffset;
    y0 = y1 - _height;
    plvpor(x0, x1, y0, y1);
    
    // preplot
    this->prePlot();
    
    // now handle each plot object within this panel

    QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
    while(plotItem)
    {
	    QpuiReturnStatus plotRes;      
        win->setSmooth(false);
	    if(QPUI_FAILURE == plotItem->plot())
	    {
		    cout << "Problem plotting plot item " << (plotItem->text(0).latin1()) << endl;
	    }
	    win->setSmooth(true);
	    plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());

    }

    plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
    while(plotItem)
    {
        QpTraceItem* traceItem = dynamic_cast<QpTraceItem*>(plotItem);
        if(traceItem)
        {
            traceItem->plotLegend();
        }
	    plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());

    }

    bool show_top_axis = false;
    bool show_bottom_axis = false;
    int stackNumber = getStackNumber();
    if(stackNumber == 1) show_top_axis = true;
    if(stackNumber == I_Frame->numberInStack() ) show_bottom_axis = true;

    this->postPlot(show_top_axis, show_bottom_axis);

    return QPUI_SUCCESS;
}

void QpPanelItem::updateYZRange()
{
  	if ( QuSlotDebug() ) cout << "QpPanelItem::updateYZRange" << endl;

	// called when X or T axis range is changed
	QpPlotItem * plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	while(plotItem)
	{
		plotItem->clearObj_id();
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}
	this->yAxisTab->setAxisMinMax();
	
	// update colourscale in spectra
	plotItem = dynamic_cast<QpPlotItem*>(this->firstChild());
	while(plotItem)
	{
		QpSpectrumItem* spectrumItem = dynamic_cast<QpSpectrumItem*>(plotItem);
		if(spectrumItem) spectrumItem->ColourscaleTab->setColourscaleMinMax();
		
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}

}


// ---------------------------------------

Vistream& operator>>(Vistream& v_is, QpPanelItem& panel)
{
	if(QuSlotDebug()) cout <<  "QpPanelItem restore (>>)" << endl;

	// streams from virtual input stream v_is into panel object
	
	QpListItem& list_item = panel;
	v_is >> list_item;

	QpAxisTab *axisTab = panel.yAxisTab;
	
	v_is >> *axisTab; // do bits common to all axes

	int linlog;
	v_is >> linlog;
	panel.yAxisTab->_axis_linlog_fld->setCurrentItem(linlog);

	bool show_legend;
	v_is >> show_legend;
	panel.LegendTab->_show_legend_fld->setChecked(show_legend);
	
	int legend_border;
	v_is >> legend_border;
	panel.LegendTab->_legend_border_fld->setCurrentItem(legend_border);
	
	int legend_background;
	v_is >> legend_background;
	panel.LegendTab->_legend_background_fld->setCurrentItem(legend_background);
	
	int legend_num_rows;
	v_is >> legend_num_rows;
	panel.LegendTab->_legend_num_rows_fld->setValue(legend_num_rows);
	
	int legend_num_cols;
	v_is >> legend_num_cols;
	panel.LegendTab->_legend_num_cols_fld->setValue(legend_num_cols);
	
	float legend_char_size;
	v_is >> legend_char_size;
	panel.LegendTab->_legend_char_size_fld->setValue(legend_char_size);
	
	int legend_max_char_length;
	v_is >> legend_max_char_length;
	panel.LegendTab->_legend_max_char_length_fld->setValue(legend_max_char_length);
	
	float legend_line_length;
	v_is >> legend_line_length;
	panel.LegendTab->_legend_line_length_fld->setValue(legend_line_length);
	
	int legend_just;
	v_is >> legend_just;
	panel.LegendTab->_legend_just_fld->setCurrentItem(legend_just);
	
	if( qsas_restore_release >= 20306)
	{
		PLINT family, style, weight;
		v_is >> family;
		v_is >> style;
		v_is >> weight;
		switch(family)
		{
			case PL_FCI_SERIF: panel.LegendTab->_legend_font_fld->setSerifFont(); break;
			case PL_FCI_MONO: panel.LegendTab->_legend_font_fld->setMonotypeFont(); break;
			case PL_FCI_SCRIPT: panel.LegendTab->_legend_font_fld->setDecorativeFont(); break;
			default: panel.LegendTab->_legend_font_fld->setSansFont(); break;
		}
		panel.LegendTab->_legend_font_fld->setItalic(style==PL_FCI_ITALIC);
		panel.LegendTab->_legend_font_fld->setBold(weight==PL_FCI_BOLD);
	}
	else
	{
		int legend_font;
		v_is >> legend_font;
		switch(legend_font)
		{
			case 1: // Roman
				panel.LegendTab->_legend_font_fld->setSerifFont();
				panel.LegendTab->_legend_font_fld->setItalic(false);
				panel.LegendTab->_legend_font_fld->setBold(false);
				break;
				
			case 2: // Italic
				panel.LegendTab->_legend_font_fld->setSerifFont();
				panel.LegendTab->_legend_font_fld->setItalic(true);
				panel.LegendTab->_legend_font_fld->setBold(false);
				break;
				
			default: // Normal
				panel.LegendTab->_legend_font_fld->setSansFont();
				panel.LegendTab->_legend_font_fld->setItalic(false);
				panel.LegendTab->_legend_font_fld->setBold(false);
				break;
		}
	}
	
	int legend_align;
	v_is >> legend_align;
	panel.LegendTab->_legend_align_fld->setCurrentItem(legend_align);
  
	float legend_loffset;
	v_is >> legend_loffset;
	panel.LegendTab->_legend_loffset_fld->setValue(legend_loffset);
	
	float legend_toffset;
	v_is >> legend_toffset;
	panel.LegendTab->_legend_toffset_fld->setValue(legend_toffset);
	
	float legend_width;
	v_is >> legend_width;
	panel.LegendTab->_legend_width_fld->setValue(legend_width);
	
	float legend_height;
	v_is >> legend_height;
	panel.LegendTab->_legend_height_fld->setValue(legend_height);
	

	// read in the plot items
	int cnt;
	v_is >> cnt;
	QpPlotItem * after = 0;

	for(int i=0; i<cnt; i++)
	{
		int Type;
		v_is >> Type;
		if(Type == QP_TRACE)
		{
			QpTraceItem *traceItem = new QpTraceItem(&panel, "", after, "",  (QpItemType)Type);
			v_is >> *traceItem;
			after = (QpPlotItem*) traceItem;
		}
		else if(Type == QP_SPECTROGRAM)
		{
			QpSpectrumItem *spectrumItem = new QpSpectrumItem(&panel, "", after);
			v_is >> *spectrumItem;

			after = (QpPlotItem*) spectrumItem;
		}
		else if(Type == QP_CONST_Y)
		{
			QpTraceItem *constYItem = new QpTraceItem(&panel, "", after, "",  (QpItemType)Type);
			v_is >> *constYItem;
			after = (QpPlotItem*) constYItem;
		}
		else if(Type == QP_CONST_XT)
		{
			QpTraceItem *constXTItem = new QpTraceItem(&panel, "", after, "", (QpItemType)Type);
			v_is >> *constXTItem;
			after = (QpPlotItem*) constXTItem;
		}
		else if(Type == QP_EVENT)
		{
			QpEventItem *eventItem = new QpEventItem(&panel, "", after);
			v_is >> *eventItem;
			after = (QpPlotItem*) eventItem;
		}
	}

	return v_is;
}

// -----------------------------
 

Vostream& operator<<(Vostream& v_os, QpPanelItem& panel)
{
	if(QuSlotDebug()) cout << "QpPanelItem save (<<)" << endl;

	// streams panel object to virtual output stream v_os
	QpListItem& list_item = panel;
	v_os << list_item;
   
   	QpAxisTab *axisTab = panel.yAxisTab;
	
	v_os << *axisTab; // do bits common to all axes
	v_os << (int) panel.yAxisTab->_axis_linlog_fld->currentItem();

	v_os << panel.LegendTab->_show_legend_fld->isChecked();
	v_os << panel.LegendTab->_legend_border_fld->currentItem();
	v_os << (int)panel.LegendTab->_legend_background_fld->currentItem();
	v_os << (int)panel.LegendTab->_legend_num_rows_fld->value();
	v_os << (int)panel.LegendTab->_legend_num_cols_fld->value();
	v_os << (float)panel.LegendTab->_legend_char_size_fld->value();
	v_os << (int)panel.LegendTab->_legend_max_char_length_fld->value();
	v_os << (float)panel.LegendTab->_legend_line_length_fld->value();
	v_os << (int)panel.LegendTab->_legend_just_fld->currentItem();
	PLINT family, style, weight;
	panel.LegendTab->_legend_font_fld->getFontParams(family, style, weight);
	v_os << family;
	v_os << style;
	v_os << weight;
	v_os << (int)panel.LegendTab->_legend_align_fld->currentItem();
	v_os << (float)panel.LegendTab->_legend_loffset_fld->value();
	v_os << (float)panel.LegendTab->_legend_toffset_fld->value();
	v_os << (float)panel.LegendTab->_legend_width_fld->value();
	v_os << (float)panel.LegendTab->_legend_height_fld->value();
	
	// now output its children
	int cnt = panel.numberOfChildren();
	v_os << cnt;
	
	QpPlotItem *plotItem = dynamic_cast<QpPlotItem*>(panel.firstChild());
	while(plotItem)
	{
		v_os << plotItem->I_Type;
		if(plotItem->I_Type == QP_TRACE)
		{
			QpTraceItem *traceItem = dynamic_cast<QpTraceItem*>(plotItem);
			v_os << *traceItem;
		}
		else if(plotItem->I_Type == QP_SPECTROGRAM)
		{
			QpSpectrumItem *spectrumItem = dynamic_cast<QpSpectrumItem*>(plotItem);
			v_os << *spectrumItem;
		}
		else if(plotItem->I_Type == QP_CONST_Y)
		{
			QpTraceItem *constYItem = dynamic_cast<QpTraceItem*>(plotItem);
			v_os << *constYItem;
		}
		else if(plotItem->I_Type == QP_CONST_XT)
		{
			QpTraceItem *constXTItem = dynamic_cast<QpTraceItem*>(plotItem);
			v_os << *constXTItem;
		}
		else if(plotItem->I_Type == QP_EVENT)
		{
			QpEventItem *eventItem = dynamic_cast<QpEventItem*>(plotItem);
			v_os << *eventItem;
		}
		
		plotItem = dynamic_cast<QpPlotItem*>(plotItem->nextSibling());
	}

	return v_os;
}


