// qpui_spectrogram_options.cc
//
// Plot Interface Implementation - Spectrogram Options
//
// Janet Barnes
// July 2002
//
// Tony Allen
// Feb 2008

#include "qpui.h"
#include "qdutil.h"

#include <qtooltip.h>



// Data Tab for Spectrogram
// -------------------------

void QpDataTab::setXEndData()
{
	// slot called when x data name field changed on widget
	
	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setXEndData = " << _x_end_data_fld->text().latin1() << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;
	
	QString x_data = _x_end_data_fld->text();

	QdObject_var xdo = _x_end_data_fld->getSlotObject();

	if(xdo.is_nil())
	{
		I_Spectrum->_x_end_obj_id=0;
		return;
	}

	// only tell list view to make changes to selected spectrum items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
      	
	QString BaseName = baseObjectText(x_data); // new value
	QString oldBaseName = baseObjectText(I_Spectrum->x_data_end_str); // old value
	QString sChoice;
	if(BaseName != oldBaseName) sChoice = BaseName.latin1();
	else sChoice = QString(x_data.latin1()).remove(BaseName);
	
	// keep for comparison when changed
	I_Spectrum->x_data_end_str = x_data.latin1(); // used only for comparison when changed

	if(this->isVisible() ) I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_XDATA_END, 0, 0.0, sChoice);

	return;
 }

void QpDataTab::setXStartData()
{
	// slot called when x start data name field changed on widget
	
	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setXStartData = " << _x_start_data_fld->text().latin1() << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;
	
	QString x_data = _x_start_data_fld->text();

	QdObject_var xdo = _x_start_data_fld->getSlotObject();

	if(xdo.is_nil())
	{
		I_Spectrum->_x_start_obj_id = 0;
		_x_end_data_fld->setEnabled(false);
		if (_y_start_data_fld->text() == "") _XYtoggle->setEnabled(true);
		return;
	}

	_x_end_data_fld->setEnabled(true);
	_XYtoggle->setEnabled(false);
	_XYtoggle->setChecked(false);
	
	// update axes
	I_Spectrum->updateAxes();

	
	// only tell list view to make changes to selected spectrum items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	
	QString BaseName = baseObjectText(x_data); // new value
	QString oldBaseName = baseObjectText(I_Spectrum->x_data_start_str); // old value
	QString sChoice;
	if(BaseName != oldBaseName) sChoice = BaseName.latin1();
	else sChoice = QString(x_data.latin1()).remove(BaseName);
	
	// keep for comparison when changed
	I_Spectrum->x_data_start_str = x_data.latin1(); // used only for comparison when changed

	if(this->isVisible() ) I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_XDATA_START, 0, 0.0, sChoice);

	return;
}

void QpDataTab::setYEndData()
{
	// slot called when y data name field changed on widget
	
	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setYEndData = " << _y_end_data_fld->text().latin1() << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;

	QString y_data = _y_end_data_fld->text();
	
	QdObject_var ydo = _y_end_data_fld->getSlotObject();

	if(ydo.is_nil())
	{
		I_Spectrum->_y_end_obj_id = 0;
		return;
	}
	
	// only tell list view to make changes to selected spectrum items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
      	
	QString BaseName = baseObjectText(y_data); // new value
	QString oldBaseName = baseObjectText(I_Spectrum->y_data_end_str); // old value
	QString sChoice;
	if(BaseName != oldBaseName) sChoice = BaseName.latin1();
	else sChoice = QString(y_data.latin1()).remove(BaseName);
	
	// keep for comparison when changed
	I_Spectrum->y_data_end_str = y_data.latin1(); // used only for comparison when changed

	if(this->isVisible() ) I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_YDATA_END, 0, 0.0, sChoice);

	return;
}

void QpDataTab::setYStartData()
{
	// slot called when y data name field changed on widget
	
	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setYStartData = " << _y_start_data_fld->text().latin1() << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;

	QString y_data = _y_start_data_fld->text();
	
	QdObject_var ydo = _y_start_data_fld->getSlotObject();
	if(ydo.is_nil() ) 
	{
		I_Spectrum->_y_start_obj_id=0;
		_y_end_data_fld->setEnabled(false);
		if (_x_start_data_fld->text() == "") _XYtoggle->setEnabled(true);
		return;
	}

	_y_end_data_fld->setEnabled(true);
	_XYtoggle->setEnabled(false);
	_XYtoggle->setChecked(false);

	// only tell list view to make changes to selected spectrum items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	
	QString BaseName = baseObjectText(y_data); // new value
	QString oldBaseName = baseObjectText(I_Spectrum->y_data_start_str); // old value
	QString sChoice;
	if(BaseName != oldBaseName) sChoice = BaseName.latin1();
	else sChoice = QString(y_data.latin1()).remove(BaseName);
	
	// keep for comparison when changed
	I_Spectrum->y_data_start_str = y_data.latin1(); // used only for comparison when changed

	if(this->isVisible() ) I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_YDATA_START, 0, 0.0, sChoice);

	return;
}

void QpDataTab::setXYswap()
{
	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setXYswap " <<  endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;
	
	// set axes to default to ensure ranges try to reset sensibly
	int xLinlog = I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->currentItem();
	int yLinlog = I_Spectrum->I_Panel->yAxisTab->_axis_linlog_fld->currentItem();
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->setCurrentItem(yLinlog);
	I_Spectrum->I_Panel->yAxisTab->_axis_linlog_fld->setCurrentItem(xLinlog);

	string x_from(I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->text().latin1());
	string x_to(I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->text().latin1());
	string y_from(I_Spectrum->I_Panel->yAxisTab->_axis_from_xy_fld->text().latin1());
	string y_to(I_Spectrum->I_Panel->yAxisTab->_axis_to_xy_fld->text().latin1());
	 
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->setText(y_from.c_str());
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->setText(y_to.c_str());
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_META);
	I_Spectrum->I_Panel->yAxisTab->_axis_from_xy_fld->setText(x_from.c_str());
	I_Spectrum->I_Panel->yAxisTab->_axis_to_xy_fld->setText(x_to.c_str());
	I_Spectrum->I_Panel->yAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_META);
	
	// swap tick choices
	int xMajTick = I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_major_tickmarks_fld->currentItem();
	int xMinTick = I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_minor_tickmarks_fld->currentItem();
	
	int yMajTick = I_Spectrum->I_Panel->yAxisTab->_axis_major_tickmarks_fld->currentItem();
	int yMinTick = I_Spectrum->I_Panel->yAxisTab->_axis_minor_tickmarks_fld->currentItem();
	
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_major_tickmarks_fld->setCurrentItem(yMajTick);
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_minor_tickmarks_fld->setCurrentItem(yMinTick);

	I_Spectrum->I_Panel->yAxisTab->_axis_major_tickmarks_fld->setCurrentItem(xMajTick);
	I_Spectrum->I_Panel->yAxisTab->_axis_minor_tickmarks_fld->setCurrentItem(xMinTick);
	
	// swap num ticks
	int xMajTickNum = I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_major_ticks_num_fld->value();
	int xMinTickNum = I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_minor_ticks_num_fld->value();
	
	int yMajTickNum = I_Spectrum->I_Panel->yAxisTab->_axis_major_ticks_num_fld->value();
	int yMinTickNum = I_Spectrum->I_Panel->yAxisTab->_axis_minor_ticks_num_fld->value();
	
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_major_ticks_num_fld->setValue(yMajTickNum);
	I_Spectrum->I_Panel->I_Frame->xAxisTab->_axis_minor_ticks_num_fld->setValue(yMinTickNum);

	I_Spectrum->I_Panel->yAxisTab->_axis_major_ticks_num_fld->setValue(xMajTickNum);
	I_Spectrum->I_Panel->yAxisTab->_axis_minor_ticks_num_fld->setValue(xMinTickNum);
	
	
	// X and Y axes swapped on spectrum, force axis update to refill x, y and z data
	I_Spectrum->xdo_id_getXRange = 0;
	I_Spectrum->ydo_id_getYRange = 0;
	I_Spectrum->_z_obj_id = 0;
	setZData();
}

void QpDataTab::setZData()
{
	// slot called when z data name field changed on widget
	// or Colourscale tab brought to front

	if( QuSlotDebug() ) cout << "Spectrum QpDataTab::setZData = '" << _z_data_fld->text().latin1() << "'" << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	if( !I_Spectrum) return;
	
	I_Spectrum->updateAxes();

	// is change to base object or component...
	QString BaseName = baseObjectText(_z_data_fld->text()); // new value
	QString CompText = compText( _z_data_fld->text() ); // new value
// 	bool has_focus_z_data_fld=_z_data_fld->lineBox->hasFocus();
	bool compOnly = false;
	if(old_BaseName == BaseName && old_Comp != CompText) compOnly = true;
	old_BaseName = QString(BaseName.latin1());
	old_Comp = QString(CompText.latin1());

	// if item uses data name change it
	if( !(I_Spectrum->nameLock) && !(_z_data_fld->text().isEmpty())) {
		I_Spectrum->setText(NAME_COL, _z_data_fld->text());
	}
	I_Spectrum->WLcontext = BaseName.latin1();
	
	// only tell list view to make changes to selected spectrum items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

	if(this->isVisible() ) {
		// propogate to selected items 
		if(compOnly) I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_ZCOMP, 0, 0.0, CompText);
		else I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_ZDATA, 0, 0.0, BaseName);
	}
	// propogate to linked spectra, component selection is NOT propogated to linked items
	if(!(I_Spectrum->I_ListView->isLink)){
		bool discard = false; // used to find onceOnly linkage
		I_Spectrum->I_ListView->isLink = true; // stops infinite recursion;
		list <QpPlotItem*>::iterator it;
		for( it=I_Spectrum->linkedPlots.begin(); it != I_Spectrum->linkedPlots.end(); it++)
		{
			if( *it == I_Spectrum) continue;
			if(!compOnly)  (*it)->replaceData(BaseName);
			if((*it)->text(LINK_COL).find("**") != -1) 
			{
				// once only linkage
				(*it)->setText(LINK_COL, "");
				discard = true;
			}

		}
		if(discard) 
		{
			I_Spectrum->setText(LINK_COL, "");
			I_Spectrum->linkedPlots.clear();
		}
		I_Spectrum->I_ListView->isLink = false;
	}
	
//	I_Spectrum->I_ListView->clearSelection();
//	I_Spectrum->I_ListView->setCurrentItem(I_Spectrum);
//	I_Spectrum->I_ListView->updateOnSelection();

// 	if(has_focus_z_data_fld) _z_data_fld->lineBox->setFocus();
	
	return;
}

void QpDataTab::gap_toggle_changed()
{
	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);
	I_Spectrum->refreshX();
}

void QpDataTab::setZTextOnly()
{
	if( QuSlotDebug() ) cout << "QpDataTab::setZTextOnly  " << endl;

	QpSpectrumItem *I_Spectrum = dynamic_cast<QpSpectrumItem*>(I_Plot);

	I_Spectrum->WLcontext = baseObjectText(_z_data_fld->text());

	if(this->isVisible() ) {
		// propogate to selected items 
		I_Spectrum->I_Panel->I_ListView->setSpectrumOption(SPECTROGRAM_ZDATA, 0, 0.0, I_Spectrum->WLcontext);
	}

	// propogate to linked spectra
	if(!(I_Spectrum->I_ListView->isLink)){
		bool discard = false; // used to find onceOnly linkage
		I_Spectrum->I_ListView->isLink = true; // stops infinite recursion;
		list <QpPlotItem*>::iterator it;
		for( it=I_Spectrum->linkedPlots.begin(); it != I_Spectrum->linkedPlots.end(); it++)
		{
			if( *it == I_Spectrum) continue;
			(*it)->replaceData(I_Spectrum->WLcontext);
			if((*it)->text(LINK_COL).find("**") != -1) 
			{
				// once only linkage
				(*it)->setText(LINK_COL, "");
				discard = true;
			}

		}
		if(discard) 
		{
			I_Spectrum->setText(LINK_COL, "");
			I_Spectrum->linkedPlots.clear();
		}
		I_Spectrum->I_ListView->isLink = false;
	}

}



// ColourScale Tab
// -------------------

// -------------------------------------------

#if QT_VERSION < 0x040000
QpColourscaleTab::QpColourscaleTab(QTabWidget *tabWidget,  QString name, QpSpectrumItem * _spectro):
    QpTab(tabWidget, name, QP_COLOUR_TAB)
{
	// Constructor for the Colour Scale Tab widget shown in the SpectroGram Options

	I_Spectrum = _spectro;
	
	QVBoxLayout *colourscale_top_layout = new QVBoxLayout(this);
	colourscale_top_layout->setSpacing(4);

	QGroupBox *colourscale_scale_groupbox = new QGroupBox(1, Qt::Vertical, this, "colourscale_scale_groupbox");

	colourscale_scale_groupbox->setTitle("Scale");

	QWidget *colourscale_scale_widget = new QWidget(colourscale_scale_groupbox, "colourscale_scale_widget");

	QGridLayout *colourscale_scale_layout=new QGridLayout(colourscale_scale_widget, 5, 5);
	colourscale_scale_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_scale_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* auto_label=new QLabel("Use: ", colourscale_scale_widget, "auto_label");
	colourscale_scale_layout->addWidget(auto_label, 1, 0, Qt::AlignRight);	
	_colourscale_auto_fld = new QComboBox(false, colourscale_scale_widget, "colourscale_auto");
	_colourscale_auto_fld->insertItem("Metadata", QP_FROM_META);
	_colourscale_auto_fld->insertItem("Autoscale", QP_FROM_DATA);
	_colourscale_auto_fld->insertItem("User Set", QP_FROM_USER);
	colourscale_scale_layout->addWidget(_colourscale_auto_fld, 1, 1, Qt::AlignLeft);
	connect( _colourscale_auto_fld, SIGNAL(activated(int)), this, SLOT(setAutoScale(int)) );

	QLabel* linlog_label=new QLabel("  Type: ", colourscale_scale_widget, "linlog_label");
	colourscale_scale_layout->addWidget(linlog_label, 1, 2, Qt::AlignRight);
	_colourscale_linlog_fld = new QComboBox(false, colourscale_scale_widget, "colourscale_linlog");
	_colourscale_linlog_fld->insertItem("Linear", QP_LINEAR);
	_colourscale_linlog_fld->insertItem("Log", QP_LOG);
	QFontMetrics cs_linlog_fm(_colourscale_linlog_fld->font());
	int cs_linlog_w = cs_linlog_fm.width("Linear");
	int cs_linlog_h = cs_linlog_fm.height();
	_colourscale_linlog_fld->setMinimumSize(cs_linlog_w, cs_linlog_h);
#if QT_VERSION < 0x040000
	_colourscale_linlog_fld->listBox()->setMinimumSize(cs_linlog_w, cs_linlog_h);
#else
        _colourscale_linlog_fld->setMinimumSize(cs_linlog_w, cs_linlog_h);
#endif
	QToolTip::add( _colourscale_linlog_fld, "Select linear or log scaling");
	colourscale_scale_layout->addWidget(_colourscale_linlog_fld, 1, 3, Qt::AlignLeft);
	
	connect( _colourscale_linlog_fld, SIGNAL(activated(int)), this, SLOT(setLinLog(int)) );
	

	QLabel* min_label = new QLabel("Min: ",colourscale_scale_widget, "min_label");
	colourscale_scale_layout->addWidget(min_label,3,0,Qt::AlignRight);
	_colourscale_min_fld = new QguWLinput(colourscale_scale_widget, "colourscale_min");
	_colourscale_min_fld->setFixedWidth(120);
	_colourscale_min_fld->acceptNumber = true;
	setSlotColour(false);
	_colourscale_min_fld->lineBox->setPalette(GUIpalette);
	colourscale_scale_layout->addWidget(_colourscale_min_fld, 3, 1, Qt::AlignLeft);
	connect( _colourscale_min_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleMin(const QString&)) );

	QLabel* max_label=new QLabel("Max: ", colourscale_scale_widget, "max_label");
	colourscale_scale_layout->addWidget(max_label,3,2,Qt::AlignRight);
	_colourscale_max_fld = new QguWLinput(colourscale_scale_widget, "colourscale_max");
	_colourscale_max_fld->setFixedWidth(120);
	_colourscale_max_fld->acceptNumber = true;
	setSlotColour(false);
	_colourscale_max_fld->lineBox->setPalette(GUIpalette);
	colourscale_scale_layout->addWidget(_colourscale_max_fld, 3, 3, Qt::AlignLeft);
	connect( _colourscale_max_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleMax(const QString&)) );

	colourscale_scale_layout->addRowSpacing(0, 0);
	colourscale_scale_layout->addRowSpacing(1, _colourscale_linlog_fld->height());
	colourscale_scale_layout->addRowSpacing(2, 0);
	colourscale_scale_layout->addRowSpacing(3, _colourscale_min_fld->height());
	colourscale_scale_layout->addRowSpacing(4, 0);

	colourscale_scale_layout->setRowStretch(0,0);
	colourscale_scale_layout->setRowStretch(1,1);
	colourscale_scale_layout->setRowStretch(2,0);
	colourscale_scale_layout->setRowStretch(3,1);
	colourscale_scale_layout->setRowStretch(4,0);

	colourscale_scale_layout->setColStretch(0,0);
	colourscale_scale_layout->setColStretch(1,0);
	colourscale_scale_layout->setColStretch(2,0);
	colourscale_scale_layout->setColStretch(3,0);
	colourscale_scale_layout->setColStretch(4,1);

	colourscale_top_layout->addWidget(colourscale_scale_groupbox);

	QGroupBox *colourscale_colour_bar_groupbox=new QGroupBox(1, Qt::Vertical, this, "colourscale_colour_bar_groupbox");

	colourscale_colour_bar_groupbox->setTitle("Colour Bar");

	QWidget* colourscale_colour_bar_widget = new QWidget(colourscale_colour_bar_groupbox, "colourscale_colour_bar_widget");

	QGridLayout *colourscale_colour_bar_layout = new QGridLayout(colourscale_colour_bar_widget,5,5);
	colourscale_colour_bar_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_colour_bar_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_colourscale_show_colour_bar_fld = new QCheckBox("Show colour bar", colourscale_colour_bar_widget, "show_colour_bar");
	_colourscale_show_colour_bar_fld->setChecked(true);
	colourscale_colour_bar_layout->addWidget( _colourscale_show_colour_bar_fld, 1, 0, Qt::AlignLeft);

	connect( _colourscale_show_colour_bar_fld, SIGNAL(toggled(bool)), this, SLOT(setShowColourBar(bool)) );

	QLabel* colourscale_colour_bar_width = new QLabel("Width: ", colourscale_colour_bar_widget, "colour_bar_width_label");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_width, 3, 0, Qt::AlignRight);
	_colourscale_colour_bar_width_fld = new FloatSpinBox( 0.01, 40., 1, .5, colourscale_colour_bar_widget, "colour_bar_width");
	_colourscale_colour_bar_width_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_width_fld, 3, 1, Qt::AlignLeft);
	connect(_colourscale_colour_bar_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarWidth(double)));
	_colourscale_colour_bar_width_fld->setValue(QPUI_COLOUR_BAR_WIDTH);

	QLabel* colourscale_colour_bar_height = new QLabel("Height Fraction: ", colourscale_colour_bar_widget, "colour_bar_height_label");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_height, 3, 3, Qt::AlignRight);
	_colourscale_colour_bar_height_fld = new FloatSpinBox( 0.2,10.,2,.2,colourscale_colour_bar_widget, "colour_bar_height");
	_colourscale_colour_bar_height_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_height_fld, 3, 4, Qt::AlignLeft);
	connect(_colourscale_colour_bar_height_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarHeight(double)));
	_colourscale_colour_bar_height_fld->setValue(QPUI_COLOUR_BAR_HEIGHT);
	
	QLabel* colourscale_colour_bar_voffset = new QLabel("Vertical Offset: ", colourscale_colour_bar_widget, "colour_bar_voffset_label");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_voffset, 5, 3, Qt::AlignRight);
	_colourscale_colour_bar_voffset_fld = new FloatSpinBox( -10., 10., 2, .2, colourscale_colour_bar_widget, "colour_bar_voffset");
	_colourscale_colour_bar_voffset_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_voffset_fld, 5, 4, Qt::AlignLeft);
	connect(_colourscale_colour_bar_voffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarVoffset(double)));
	_colourscale_colour_bar_voffset_fld->setValue(QPUI_COLOUR_BAR_VOFFSET);
	
	QLabel* colour_table_label = new QLabel("Colour Table: ", colourscale_colour_bar_widget, "colour_table_label");
	colourscale_colour_bar_layout->addWidget(colour_table_label, 1, 3, Qt::AlignRight);
	_colourscale_table_fld = new QComboBox(false, colourscale_colour_bar_widget, "colourscale_table");
	_colourscale_table_fld->insertItem("Rainbow", QPUI_RAINBOW);
	_colourscale_table_fld->insertItem("Rainbow Extended", QPUI_RAINBOW_EXT);
	_colourscale_table_fld->insertItem("Gray White Low", QPUI_GRAY_WHITE_LOW);
	_colourscale_table_fld->insertItem("Gray White High", QPUI_GRAY_WHITE_HIGH);
	_colourscale_table_fld->insertItem("Blue to Red (dark ends)", QPUI_RED_BLUE);
	_colourscale_table_fld->insertItem("Blue to Red (light ends)", QPUI_INV_RED_BLUE);
	_colourscale_table_fld->insertItem("Earth", QPUI_EARTH);
	_colourscale_table_fld->insertItem("Profile-defined", QPUI_USER_TABLE);
	QFontMetrics cs_table_fm(_colourscale_table_fld->font());
	int cs_table_w = cs_table_fm.width("Blue to Red (light ends)");
	int cs_table_h = cs_table_fm.height();
	_colourscale_table_fld->setMinimumSize(cs_table_w, cs_table_h);
#if QT_VERSION < 0x040000
	_colourscale_table_fld->listBox()->setMinimumSize(cs_table_w,cs_table_h);
#else
        _colourscale_table_fld->setMinimumSize(cs_table_w,cs_table_h);
#endif
	QToolTip::add( _colourscale_table_fld, "Select colour table");
	colourscale_colour_bar_layout->addWidget(_colourscale_table_fld, 1, 4, Qt::AlignLeft);
	connect( _colourscale_table_fld, SIGNAL(activated(int)), this, SLOT(setColourtable(int)) );

	QLabel* colourscale_colour_bar_disp = new QLabel("Displacement: ", colourscale_colour_bar_widget, "colour_bar_disp_label");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_disp, 5, 0, Qt::AlignRight);
	_colourscale_colour_bar_disp_fld = new FloatSpinBox( 0.01,40.,1,.5,colourscale_colour_bar_widget, "colour_bar_disp");
	_colourscale_colour_bar_disp_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_disp_fld, 5, 1, Qt::AlignLeft);
	connect(_colourscale_colour_bar_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarDisplacement(double)));
	_colourscale_colour_bar_disp_fld->setValue(QPUI_COLOUR_BAR_DISP);

	QLabel* colourscale_ticklength_disp=new QLabel("Ticks length: ", colourscale_colour_bar_widget, "");
	colourscale_colour_bar_layout->addWidget(colourscale_ticklength_disp, 7, 0, Qt::AlignRight);
	_colourscale_ticklength_fld= new FloatSpinBox(0.01, 40., 1, .5, colourscale_colour_bar_widget, "ticks_length");
	_colourscale_ticklength_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_ticklength_fld, 7, 1, Qt::AlignLeft);
	_colourscale_ticklength_fld->setValue(QPUI_TICKS_LENGTH);
	
	connect(_colourscale_ticklength_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	colourscale_colour_bar_layout->addRowSpacing(0,0);
	colourscale_colour_bar_layout->addRowSpacing(1, _colourscale_table_fld->height());
	colourscale_colour_bar_layout->addRowSpacing(2,0);
	colourscale_colour_bar_layout->addRowSpacing(3, _colourscale_colour_bar_disp_fld->height());
	colourscale_colour_bar_layout->addRowSpacing(4,0);

	colourscale_colour_bar_layout->setRowStretch(0,0);
	colourscale_colour_bar_layout->setRowStretch(1,1);
	colourscale_colour_bar_layout->setRowStretch(2,0);
	colourscale_colour_bar_layout->setRowStretch(3,1);
	colourscale_colour_bar_layout->setRowStretch(4,0);
	colourscale_colour_bar_layout->setRowStretch(5,1);
	colourscale_colour_bar_layout->setRowStretch(6,0);

	colourscale_colour_bar_layout->setColStretch(0,0);
	colourscale_colour_bar_layout->setColStretch(1,0);
	colourscale_colour_bar_layout->setColStretch(2,0);
	colourscale_colour_bar_layout->setColStretch(3,0);
	colourscale_colour_bar_layout->setColStretch(4,1);

	colourscale_top_layout->addWidget(colourscale_colour_bar_groupbox);

	QGroupBox *colourscale_label_groupbox = new QGroupBox(1, Qt::Vertical, this, "colourscale_label_groupbox");
	colourscale_label_groupbox->setTitle("Label");

	QWidget* colourscale_label_widget = new QWidget(colourscale_label_groupbox, "colourscale_label_widget");

	QGridLayout *colourscale_label_layout = new QGridLayout(colourscale_label_widget, 3, 5);
	colourscale_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_colourscale_label_fld = new QLineEdit(colourscale_label_widget, "colourscale_label");
	_colourscale_label_fld->setMinimumWidth(350);
	colourscale_label_layout->addMultiCellWidget(_colourscale_label_fld, 1, 1, 0, 3, Qt::AlignLeft);
	connect( _colourscale_label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleLabel(const QString&)) );
	setSlotColour(false);
	_colourscale_label_fld->setPalette(GUIpalette);
	
	_colourscale_autolabel_fld = new QCheckBox( "Auto-label", colourscale_label_widget, "colourscale_autolabel");
	_colourscale_autolabel_fld->setChecked(true);
	colourscale_label_layout->addWidget(_colourscale_autolabel_fld, 1, 4, Qt::AlignLeft);
	connect( _colourscale_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setColourScaleAutoLabelToggle(bool)) );

	colourscale_label_layout->addRowSpacing(0,0);
	colourscale_label_layout->addRowSpacing(1,_colourscale_label_fld->height());
	colourscale_label_layout->addRowSpacing(2,0);

	colourscale_label_layout->setRowStretch(0,0);
	colourscale_label_layout->setRowStretch(1,1);
	colourscale_label_layout->setRowStretch(2,0);

	colourscale_label_layout->setColStretch(0,0);
	colourscale_label_layout->setColStretch(1,0);
	colourscale_label_layout->setColStretch(2,0);
	colourscale_label_layout->setColStretch(3,0);
	colourscale_label_layout->setColStretch(4,1);

	colourscale_top_layout->addWidget(colourscale_label_groupbox);

	colourscale_top_layout->addSpacing(4);
	colourscale_top_layout->addStretch(10);

	colourscale_scale_widget->adjustSize();

	colour_auto_last = QP_FROM_META;

	return;
}
#else
QpColourscaleTab::QpColourscaleTab(QTabWidget *tabWidget,  QString name, QpSpectrumItem * _spectro):
		QpTab(tabWidget, name, QP_COLOUR_TAB)
{
	// Constructor for the Colour Scale Tab widget shown in the SpectroGram Options

	I_Spectrum = _spectro;
	
	QVBoxLayout *colourscale_top_layout = new QVBoxLayout(this);
	colourscale_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *colourscale_scale_groupbox = new QGroupBox(this);

	colourscale_scale_groupbox->setTitle("Scale");

// 	QWidget *colourscale_scale_widget = new QWidget(colourscale_scale_groupbox, "colourscale_scale_widget");

	QGridLayout *colourscale_scale_layout=new QGridLayout;
	colourscale_scale_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_scale_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* auto_label=new QLabel("Use: ");
	colourscale_scale_layout->addWidget(auto_label, 1, 0, Qt::AlignRight);	
	_colourscale_auto_fld = new QComboBox(false);
	_colourscale_auto_fld->insertItem("Metadata", QP_FROM_META);
	_colourscale_auto_fld->insertItem("Autoscale", QP_FROM_DATA);
	_colourscale_auto_fld->insertItem("User Set", QP_FROM_USER);
	colourscale_scale_layout->addWidget(_colourscale_auto_fld, 1, 1, Qt::AlignLeft);
	connect( _colourscale_auto_fld, SIGNAL(activated(int)), this, SLOT(setAutoScale(int)) );

	QLabel* linlog_label=new QLabel("  Type: ");
	colourscale_scale_layout->addWidget(linlog_label, 1, 2, Qt::AlignRight);
	_colourscale_linlog_fld = new QComboBox(false);
	_colourscale_linlog_fld->insertItem("Linear", QP_LINEAR);
	_colourscale_linlog_fld->insertItem("Log", QP_LOG);
	QFontMetrics cs_linlog_fm(_colourscale_linlog_fld->font());
	int cs_linlog_w = cs_linlog_fm.width("Linear");
	int cs_linlog_h = cs_linlog_fm.height();
	_colourscale_linlog_fld->setMinimumSize(cs_linlog_w, cs_linlog_h);

	QToolTip::add( _colourscale_linlog_fld, "Select linear or log scaling");
	colourscale_scale_layout->addWidget(_colourscale_linlog_fld, 1, 3, Qt::AlignLeft);
	
	connect( _colourscale_linlog_fld, SIGNAL(activated(int)), this, SLOT(setLinLog(int)) );
	

	QLabel* min_label = new QLabel("Min: ");
	colourscale_scale_layout->addWidget(min_label,3,0,Qt::AlignRight);
	_colourscale_min_fld = new QguWLinput(colourscale_scale_groupbox, "colourscale_min");
	_colourscale_min_fld->setFixedWidth(120);
	_colourscale_min_fld->acceptNumber = true;
	setSlotColour(false);
	_colourscale_min_fld->lineBox->setPalette(GUIpalette);
	colourscale_scale_layout->addWidget(_colourscale_min_fld, 3, 1, Qt::AlignLeft);
	connect( _colourscale_min_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleMin(const QString&)) );

	QLabel* max_label=new QLabel("Max: ");
	colourscale_scale_layout->addWidget(max_label,3,2,Qt::AlignRight);
	_colourscale_max_fld = new QguWLinput(colourscale_scale_groupbox, "colourscale_max");
	_colourscale_max_fld->setFixedWidth(120);
	_colourscale_max_fld->acceptNumber = true;
	setSlotColour(false);
	_colourscale_max_fld->lineBox->setPalette(GUIpalette);
	colourscale_scale_layout->addWidget(_colourscale_max_fld, 3, 3, Qt::AlignLeft);
	connect( _colourscale_max_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleMax(const QString&)) );

// 	colourscale_scale_layout->addRowSpacing(0, 0);
// 	colourscale_scale_layout->addRowSpacing(1, _colourscale_linlog_fld->height());
// 	colourscale_scale_layout->addRowSpacing(2, 0);
// 	colourscale_scale_layout->addRowSpacing(3, _colourscale_min_fld->height());
// 	colourscale_scale_layout->addRowSpacing(4, 0);

	colourscale_scale_layout->setRowStretch(0,0);
	colourscale_scale_layout->setRowStretch(1,1);
	colourscale_scale_layout->setRowStretch(2,0);
	colourscale_scale_layout->setRowStretch(3,1);
	colourscale_scale_layout->setRowStretch(4,0);

	colourscale_scale_layout->setColStretch(0,0);
	colourscale_scale_layout->setColStretch(1,0);
	colourscale_scale_layout->setColStretch(2,0);
	colourscale_scale_layout->setColStretch(3,0);
	colourscale_scale_layout->setColStretch(4,1);
	
	colourscale_scale_groupbox->setLayout(colourscale_scale_layout);

	colourscale_top_layout->addWidget(colourscale_scale_groupbox);

	QGroupBox *colourscale_colour_bar_groupbox=new QGroupBox(this);

	colourscale_colour_bar_groupbox->setTitle("Colour Bar");

// 	QWidget* colourscale_colour_bar_widget = new QWidget(colourscale_colour_bar_groupbox, "colourscale_colour_bar_widget");

	QGridLayout *colourscale_colour_bar_layout = new QGridLayout;
	colourscale_colour_bar_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_colour_bar_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_colourscale_show_colour_bar_fld = new QCheckBox("Show colour bar");
	_colourscale_show_colour_bar_fld->setChecked(true);
	colourscale_colour_bar_layout->addWidget( _colourscale_show_colour_bar_fld, 1, 0, Qt::AlignLeft);

	connect( _colourscale_show_colour_bar_fld, SIGNAL(toggled(bool)), this, SLOT(setShowColourBar(bool)) );

	QLabel* colourscale_colour_bar_width = new QLabel("Width: ");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_width, 3, 0, Qt::AlignRight);
	_colourscale_colour_bar_width_fld = new FloatSpinBox( 0.01, 40., 1, .5, colourscale_colour_bar_groupbox, "colour_bar_width");
	_colourscale_colour_bar_width_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_width_fld, 3, 1, Qt::AlignLeft);
	connect(_colourscale_colour_bar_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarWidth(double)));
	_colourscale_colour_bar_width_fld->setValue(QPUI_COLOUR_BAR_WIDTH);

	QLabel* colourscale_colour_bar_height = new QLabel("Height Fraction: ");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_height, 3, 3, Qt::AlignRight);
	_colourscale_colour_bar_height_fld = new FloatSpinBox( 0.2,10.,2,.2,colourscale_colour_bar_groupbox, "colour_bar_height");
	_colourscale_colour_bar_height_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_height_fld, 3, 4, Qt::AlignLeft);
	connect(_colourscale_colour_bar_height_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarHeight(double)));
	_colourscale_colour_bar_height_fld->setValue(QPUI_COLOUR_BAR_HEIGHT);
	
	QLabel* colourscale_colour_bar_voffset = new QLabel("Vertical Offset: ");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_voffset, 5, 3, Qt::AlignRight);
	_colourscale_colour_bar_voffset_fld = new FloatSpinBox( -10., 10., 2, .2, colourscale_colour_bar_groupbox, "colour_bar_voffset");
	_colourscale_colour_bar_voffset_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_voffset_fld, 5, 4, Qt::AlignLeft);
	connect(_colourscale_colour_bar_voffset_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarVoffset(double)));
	_colourscale_colour_bar_voffset_fld->setValue(QPUI_COLOUR_BAR_VOFFSET);
	
	QLabel* colour_table_label = new QLabel("Colour Table: ");
	colourscale_colour_bar_layout->addWidget(colour_table_label, 1, 3, Qt::AlignRight);
	_colourscale_table_fld = new QComboBox(false);
	_colourscale_table_fld->insertItem("Rainbow", QPUI_RAINBOW);
	_colourscale_table_fld->insertItem("Rainbow Extended", QPUI_RAINBOW_EXT);
	_colourscale_table_fld->insertItem("Gray White Low", QPUI_GRAY_WHITE_LOW);
	_colourscale_table_fld->insertItem("Gray White High", QPUI_GRAY_WHITE_HIGH);
	_colourscale_table_fld->insertItem("Blue to Red (dark ends)", QPUI_RED_BLUE);
	_colourscale_table_fld->insertItem("Blue to Red (light ends)", QPUI_INV_RED_BLUE);
	_colourscale_table_fld->insertItem("Earth", QPUI_EARTH);
	_colourscale_table_fld->insertItem("Profile-defined", QPUI_USER_TABLE);
	QFontMetrics cs_table_fm(_colourscale_table_fld->font());
	int cs_table_w = cs_table_fm.width("Blue to Red (light ends)");
	int cs_table_h = cs_table_fm.height();
	_colourscale_table_fld->setMinimumSize(cs_table_w, cs_table_h);

	QToolTip::add( _colourscale_table_fld, "Select colour table");
	colourscale_colour_bar_layout->addWidget(_colourscale_table_fld, 1, 4, Qt::AlignLeft);
	connect( _colourscale_table_fld, SIGNAL(activated(int)), this, SLOT(setColourtable(int)) );

	QLabel* colourscale_colour_bar_disp = new QLabel("Displacement: ");
	colourscale_colour_bar_layout->addWidget( colourscale_colour_bar_disp, 5, 0, Qt::AlignRight);
	_colourscale_colour_bar_disp_fld = new FloatSpinBox( 0.01,40.,1,.5,colourscale_colour_bar_groupbox, "colour_bar_disp");
	_colourscale_colour_bar_disp_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_colour_bar_disp_fld, 5, 1, Qt::AlignLeft);
	connect(_colourscale_colour_bar_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setColourbarDisplacement(double)));
	_colourscale_colour_bar_disp_fld->setValue(QPUI_COLOUR_BAR_DISP);

	QLabel* colourscale_ticklength_disp=new QLabel("Ticks length: ");
	colourscale_colour_bar_layout->addWidget(colourscale_ticklength_disp, 7, 0, Qt::AlignRight);
	_colourscale_ticklength_fld= new FloatSpinBox(0.01, 40., 1, .5, this, "ticks_length");
	_colourscale_ticklength_fld->setFixedWidth(100);
	colourscale_colour_bar_layout->addWidget( _colourscale_ticklength_fld, 7, 1, Qt::AlignLeft);
	_colourscale_ticklength_fld->setValue(QPUI_TICKS_LENGTH);
	connect(_colourscale_ticklength_fld, SIGNAL(valueChanged(double)), this, SLOT(setTicksLength(double)));
	
// 	colourscale_colour_bar_layout->addRowSpacing(0,0);
// 	colourscale_colour_bar_layout->addRowSpacing(1, _colourscale_table_fld->height());
// 	colourscale_colour_bar_layout->addRowSpacing(2,0);
// 	colourscale_colour_bar_layout->addRowSpacing(3, _colourscale_colour_bar_disp_fld->height());
// 	colourscale_colour_bar_layout->addRowSpacing(4,0);

	colourscale_colour_bar_layout->setRowStretch(0,0);
	colourscale_colour_bar_layout->setRowStretch(1,1);
	colourscale_colour_bar_layout->setRowStretch(2,0);
	colourscale_colour_bar_layout->setRowStretch(3,1);
	colourscale_colour_bar_layout->setRowStretch(4,0);

	colourscale_colour_bar_layout->setColStretch(0,0);
	colourscale_colour_bar_layout->setColStretch(1,0);
	colourscale_colour_bar_layout->setColStretch(2,0);
	colourscale_colour_bar_layout->setColStretch(3,0);
	colourscale_colour_bar_layout->setColStretch(4,1);
	colourscale_colour_bar_layout->setColStretch(5,0);
	colourscale_colour_bar_layout->setColStretch(6,1);
	
	colourscale_colour_bar_groupbox->setLayout(colourscale_colour_bar_layout);

	colourscale_top_layout->addWidget(colourscale_colour_bar_groupbox);

	QGroupBox *colourscale_label_groupbox = new QGroupBox(this);
	colourscale_label_groupbox->setTitle("Label");

// 	QWidget* colourscale_label_widget = new QWidget(colourscale_label_groupbox, "colourscale_label_widget");

	QGridLayout *colourscale_label_layout = new QGridLayout;
	colourscale_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	colourscale_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_colourscale_label_fld = new QLineEdit;
	_colourscale_label_fld->setMinimumWidth(350);
	colourscale_label_layout->addMultiCellWidget(_colourscale_label_fld, 1, 1, 0, 3, Qt::AlignLeft);
	connect( _colourscale_label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setColourScaleLabel(const QString&)) );
	setSlotColour(false);
	_colourscale_label_fld->setPalette(GUIpalette);
	
	_colourscale_autolabel_fld = new QCheckBox( "Auto-label");
	_colourscale_autolabel_fld->setChecked(true);
	colourscale_label_layout->addWidget(_colourscale_autolabel_fld, 1, 4, Qt::AlignLeft);
	connect( _colourscale_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setColourScaleAutoLabelToggle(bool)) );

// 	colourscale_label_layout->addRowSpacing(0,0);
// 	colourscale_label_layout->addRowSpacing(1,_colourscale_label_fld->height());
// 	colourscale_label_layout->addRowSpacing(2,0);

	colourscale_label_layout->setRowStretch(0,0);
	colourscale_label_layout->setRowStretch(1,1);
	colourscale_label_layout->setRowStretch(2,0);

	colourscale_label_layout->setColStretch(0,0);
	colourscale_label_layout->setColStretch(1,0);
	colourscale_label_layout->setColStretch(2,0);
	colourscale_label_layout->setColStretch(3,0);
	colourscale_label_layout->setColStretch(4,1);
	
	colourscale_label_groupbox->setLayout(colourscale_label_layout);

	colourscale_top_layout->addWidget(colourscale_label_groupbox);

	colourscale_top_layout->addSpacing(4);
	colourscale_top_layout->addStretch(10);

// 	colourscale_scale_widget->adjustSize();

	colour_auto_last = QP_FROM_META;

	return;
}
#endif


void QpColourscaleTab::setColourScaleMin(const QString &value_str)
{
	if ( QuSlotDebug() )
	{
		printf( "QpColourscaleTab::setColourScaleMin value = %s \n", value_str.latin1() );
		fflush(stdout);
	}

	if(_colourscale_min_fld->lineBox->hasFocus())
	{
		// only change if user input
		_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(true);
		_colourscale_min_fld->lineBox->setPalette(GUIpalette);
		_colourscale_max_fld->lineBox->setPalette(GUIpalette);
		
		// set flag for next time (used in getZRange methods)
		colour_auto_last = _colourscale_auto_fld->currentItem();
	
		float value = (float) string_to_double(value_str.latin1());
		// propogate to other spectrum Items
		if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_MIN, 0, value);
	}
	
}

void QpColourscaleTab::setColourScaleMax(const QString &value_str)
{
	if ( QuSlotDebug() )
	{
		printf( "QpColourscaleTab::setColourScaleMax value = %s \n", value_str.latin1() );
		fflush(stdout);
	}

	if(_colourscale_max_fld->lineBox->hasFocus())
	{
		// only change if user input
		_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
		setSlotColour(true);
		_colourscale_min_fld->lineBox->setPalette(GUIpalette);
		_colourscale_max_fld->lineBox->setPalette(GUIpalette);
		
		// set flag for next time (used in getZRange methods)
		colour_auto_last = _colourscale_auto_fld->currentItem();
				
		float value = (float) string_to_double(value_str.latin1());
		// propogate to other spectrum Items
		if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_MAX, 0, value);

	}	
}


void QpColourscaleTab::setColourScaleLabel(const QString &txt)
{
	if ( QuSlotDebug() )
	{
		printf( "QpColourscaleTab::setColourScaleLabel value = %s \n", txt.latin1() );
		fflush(stdout);
	}

	if(_colourscale_label_fld->hasFocus())
	{
		// only change if user input
		_colourscale_autolabel_fld->setChecked(false);
		
		// propogate to other spectrum Items
		if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_LABEL, 0, 0.0, txt);

	}	
}


// ---------------------------------------

void QpColourscaleTab::setColourScaleAutoLabelToggle(bool On)
{
	if(On)
	{
		_colourscale_label_fld->clearFocus();		

		setColourScaleAutoLabel();
	}
	setSlotColour(!On);
	_colourscale_label_fld->setPalette(GUIpalette);

	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_AUTOLABEL, 0, 0.0, "", On);

}

// ---------------------------------------

void QpColourscaleTab::setColourScaleAutoLabel()
{
	if( !(_colourscale_autolabel_fld->isChecked()) ) return;
	
	string defaultLabel="";
	I_Spectrum->I_Panel->getLabel(COL_AXIS, defaultLabel);
	_colourscale_label_fld->setText(defaultLabel.c_str());
}

// ---------------------------------------

void QpColourscaleTab::setColourbarWidth(double width)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_COLOUR_BAR_WIDTH, 0, width);
}

// ---------------------------------------

void QpColourscaleTab::setColourbarVoffset(double offset)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_COLOUR_BAR_VOFFSET, 0, offset);
}

// ---------------------------------------

void QpColourscaleTab::setColourbarHeight(double height)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_COLOUR_BAR_HEIGHT, 0, height);
}
// ---------------------------------------

void QpColourscaleTab::setColourbarDisplacement(double disp)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_COLOUR_BAR_DISP, 0, disp);
}

// ---------------------------------------

void QpColourscaleTab::setColourtable(int choice)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_TABLE, choice);

}

// ---------------------------------------

void QpColourscaleTab::setAutoScale(int iChoice)
{
	if(iChoice == QP_FROM_USER)
	{
		setSlotColour(true);
	}
	else 
	{
		_colourscale_min_fld->clearFocus();
		_colourscale_max_fld->clearFocus();
		setColourscaleMinMax();
#if QT_VERSION < 0x040000
		qApp->processEvents(2000);
#else
		qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
		setSlotColour(false);
	}
	
	_colourscale_min_fld->lineBox->setPalette(GUIpalette);
	_colourscale_max_fld->lineBox->setPalette(GUIpalette);
	
	// set flag for next time (used in getZRange methods)
	colour_auto_last = _colourscale_auto_fld->currentItem();
	
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_AUTOSCALE, iChoice);

}

// ---------------------------------------

void QpColourscaleTab::setLinLog(int choice)
{
	// tell auto field user set
	_colourscale_auto_fld->setCurrentItem(QP_FROM_USER);
	
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_LINLOG, choice);

}
// ---------------------------------------

void QpColourscaleTab::setShowColourBar(bool On)
{

	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_SHOW_COLOUR_BAR, 0, 0.0, "", On);

}

void QpColourscaleTab::setTicksLength(double val)
{
	// propogate to other spectrum Items
	if(this->isVisible() ) I_Spectrum->I_ListView->setSpectrumOption(COLOUR_SCALE_TICK_LENGTH, 0, val);
}

// ---------------------------------------

void QpColourscaleTab::setColourscaleMinMax()
{
	
	if ( QuSlotDebug() )
	{
		printf( "QpColourscaleTab::setColourscaleMinMax \n");
		fflush(stdout);
	}

	switch( _colourscale_auto_fld->currentItem())
	{
	
		case QP_FROM_META:
		{
			
			// Set Colourscale if Use Metadata is on
			double _min=0, _max=0;
			if( I_Spectrum->getZmetaRange(_min, _max) )
			{
				_colourscale_min_fld->setText(double_to_string(_min).c_str());
				_colourscale_max_fld->setText(double_to_string(_max).c_str());
				 break;
			}
			// else fall through and use data

		}
		case QP_FROM_DATA: 
		{
			//  Set Colour Scale range if use autoscale is on

			// set wait cursor - could be a delay if large datasets
			QApplication::setOverrideCursor( Qt::waitCursor );
#if QT_VERSION < 0x040000
			qApp->processEvents(2000);
#else
                        qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
   
			bool rangeOK;
	
			double _min, _max;
			rangeOK = I_Spectrum->getZRange(_min, _max);
			
			// refresh from and to fields
			if(rangeOK)
			{
				_colourscale_min_fld->setText(double_to_string(_min).c_str());
				_colourscale_max_fld->setText(double_to_string(_max).c_str());
			}
			else
			{
				_colourscale_min_fld->setText("");
				_colourscale_max_fld->setText("");
			}
	    
   
			// restore cursor
			QApplication::restoreOverrideCursor();
#if QT_VERSION < 0x040000
                        qApp->processEvents(2000);
#else
                        qApp->processEvents(QEventLoop::AllEvents, 2000);
#endif
		}
		case QP_FROM_USER: Default:
		{
				// do nothing
		}
	}

}

