// qpui_trace.cc
//
// Plot Interface Implementation - Trace Object
//
// Janet Barnes
//
// July 2002
//
//  Tony Allen
//  Jan 2008

#include "qpui.h"
#include "qwui.h"
#include "TimeUtils.h"
#include "qdutil.h"


extern QPalette QPUI_normal_palette;

// --------------------------
//     QpTraceItem CLASS
//     ------------------

QpTraceItem::QpTraceItem(QpPanelItem *parent, QString name, QpPlotItem *after, QString WL, QpItemType plotType):
		QpPlotItem(parent,name,after, WL)
{
	// Constructor for trace which is a child of a panel
	
	if(QuSlotDebug())
	{
		printf("QpTraceItem::QpTraceItem of type %ld \n", plotType);
		fflush(stdout);
	}

	C1_colour = QColor(0,0,0);
	C2_colour = QColor(255,0,0);
	C3_colour = QColor(0,130,0);
	C4_colour = QColor(150,0,255);
	R_colour  = QColor(0,0,0);
	X_colour  = QColor(255,0,0);
	Y_colour  = QColor(0,180,0);
	Z_colour  = QColor(0,0,180);
	T_colour  = QColor(150,0,0);
	P_colour  = QColor(0,130,0);
	L_colour  = QColor(150,0,150);
	userLineColour = false;
	userSymbolColour = false;
	DataTab = new QpDataTab(tabWidget, "DataTab", this, plotType);
	DataTab->hide();

	TraceTab = new QpTraceTab(tabWidget, "TraceTab", this);
	TraceTab->hide();
	
	_line_colour = QColor(0, 0, 0);
	_symbol_colour = QColor(0, 0, 0);
	
	I_Type = plotType;
	
	switch(plotType)
	{
		default: case QP_TRACE:
		
			if(frameType() == QP_XY)
			{
				// set to xy trace
	
				DataTab->_y_label_fld->setText("Y: ");
				DataTab->_y_data_fld->acceptTime=false;
				DataTab->_y_data_fld->acceptTimeObject=false;
				DataTab->_y_data_fld->acceptTIvlObject=false;
				DataTab->_y_data_fld->acceptNumber=false;
				DataTab->_y_data_fld->acceptScalarObject=true;
				DataTab->_y_data_fld->acceptTimeSlice=true;
				DataTab->_y_data_fld->acceptTimeSubset=true;
		
				DataTab->_x_label_fld->show();
				DataTab->_x_data_fld->show();
				DataTab->_y_data_fld->setText(WL.latin1());
				DataTab->_y_data_fld->reValidate();
				DataTab->_show_legend_fld->setChecked(true);
				break;
			}
			else
			{
				// set to ts trace

				DataTab->_y_label_fld->setText("Y: ");
				DataTab->_y_data_fld->acceptTime=false;
				DataTab->_y_data_fld->acceptTimeObject=false;
				DataTab->_y_data_fld->acceptTIvlObject=false;
				DataTab->_y_data_fld->acceptNumber=false;
				DataTab->_y_data_fld->acceptScalarObject=true;
				DataTab->_y_data_fld->acceptTimeSlice=true;
				DataTab->_y_data_fld->acceptTimeSubset=true;

				DataTab->_y_data_fld->setText(WL.latin1());

				if(!WL.isEmpty()) DataTab->_y_data_fld->reValidate();

				DataTab->_x_label_fld->hide();
				DataTab->_x_data_fld->hide();
				DataTab->_show_legend_fld->setChecked(true);
				break;
			}
			
		case QP_CONST_XT:
	
			if(frameType() == QP_XY)
			{
				// change to const x trace

				DataTab->_y_label_fld->setText("X Constant: ");
				DataTab->_y_data_fld->acceptTime=false;
				DataTab->_y_data_fld->acceptTimeObject=false;
				DataTab->_y_data_fld->acceptTIvlObject=false;
				DataTab->_y_data_fld->acceptNumber=true;
				DataTab->_y_data_fld->acceptScalarObject=true;
				DataTab->_y_data_fld->acceptTimeSlice=true;
				DataTab->_y_data_fld->acceptTimeSubset=false;

				DataTab->_y_data_fld->setText(WL.latin1());
				DataTab->_y_data_fld->reValidate();

				DataTab->_x_label_fld->hide();
				DataTab->_x_data_fld->hide();
				DataTab->_show_legend_fld->setChecked(false);
				DataTab->timeAsSec_toggle->hide();

				break;
			}
			else
			{
				// set to const t trace

				DataTab->_y_label_fld->setText("Time Constant: ");
				DataTab->_y_data_fld->acceptTime=true;
				DataTab->_y_data_fld->acceptTimeObject=true;
				DataTab->_y_data_fld->acceptTIvlObject=true;
				DataTab->_y_data_fld->acceptNumber=false;
				DataTab->_y_data_fld->acceptScalarObject=false;
				DataTab->_y_data_fld->acceptTimeSlice=true;
				DataTab->_y_data_fld->acceptTimeSubset=false;
				if( !WL.isEmpty() )
				{
					Time time_val;
					if(valid_iso_time_str(WL.latin1(), time_val))
					{
						DataTab->_y_data_fld->setText(time_val.iso().c_str());
						DataTab->_y_data_fld->reValidate();
					}
					else
					{
						DataTab->_y_data_fld->setText(WL.latin1());
						DataTab->_y_data_fld->reValidate();
					}
				}
		
				DataTab->_x_label_fld->hide();
				DataTab->_x_data_fld->hide();
				DataTab->_show_legend_fld->setChecked(false);
				break;
			}
			
		case QP_CONST_Y:
			// set to const y trace
	  
			DataTab->_y_label_fld->setText("Y Constant: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=true;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=true;
			DataTab->_y_data_fld->acceptTimeSubset=false;
			DataTab->_y_data_fld->setText(WL.latin1());
			DataTab->_y_data_fld->reValidate();

			DataTab->_x_label_fld->hide();
			DataTab->_x_data_fld->hide();
			DataTab->_show_legend_fld->setChecked(false);
			DataTab->timeAsSec_toggle->hide();
	
			break;

	}


	// set Minimum Size for TabWidget
	tabWidget->adjustSize();
   
	_y_obj_id_getYRange = 0;
	_x_obj_id_getXRange = 0;

}

// ------------------------------------

QpTraceItem::~QpTraceItem()
{
   // Destructor
	
	if(DataTab) delete DataTab;
	if(TraceTab) delete TraceTab;
	
}


// ----------------------------------

void QpTraceItem::setDefaultOptions()
{
	DataTab->timeAsSec_toggle->setChecked(false);
	DataTab->_show_legend_fld->setChecked(true);
	
	TraceTab->_line_style_fld->setCurrentItem(QP_SOLID);
	TraceTab->_line_width_fld->setValue(1);
	TraceTab->_connect_style_fld->setCurrentItem(QPUI_STRAIGHT);
	
	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, Qt::black);
	line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, Qt::black);
	TraceTab->_line_colour_fld->setPalette(line_palette);
	_line_colour = Qt::black;

	TraceTab->_symbol_shape_fld->setCurrentItem(QPUI_NOSYMBOL);
	TraceTab->_symbol_size_fld->setValue(1.0);
	TraceTab->_symbol_colour_fld->setPalette(line_palette);
	_symbol_colour = Qt::black;
	
	updateAxes();
}

// ------------------------------------

void QpTraceItem::updateAxes()
{
	if(QuSlotDebug()) cout << "QpTraceItem::updateAxes " << endl;
	
	I_Panel->I_Frame->updateAxes();

}

// ------------------------------------

void QpTraceItem::clearObj_id()
{
   // called when we need to force a range update
	_y_obj_id_getYRange = 0;
	_x_obj_id_getXRange = 0;
}

// -------------------------------------------

void QpTraceItem::showTabs()
{
	if(QuSlotDebug()) cout << "QpTraceItem::showTabs '" << this->text(NAME_COL).latin1() << "'" << endl;
	
	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--)
	{
		if(tabWidget->page(i))
		{
			tabWidget->page(i)->hide();
			tabWidget->removePage(this->tabWidget->page(i));
		}
	}

	tabWidget->addTab(DataTab, "Data");

	tabWidget->addTab(TraceTab, "Trace");
	// value return is always shown
	tabWidget->addTab(I_Panel->I_Frame->I_Page->ValueReturnTab, "Value Return");
	if(this->I_ListView->I_LastTab == QP_TRACE_TAB)  tabWidget->showPage(TraceTab);
	else tabWidget->showPage(DataTab);
	
}

// -------------------------------------------

void QpTraceItem::replaceData(QString obj_name)
{
	if(QuSlotDebug()) cout << "QpTraceItem::replaceData " << endl;
	if(obj_name.isEmpty()) return;
	if(QuSlotDebug()) cout <<  obj_name.latin1() << endl;

	
	QString oldCompText = compText( DataTab->_y_data_fld->text() ); 
	if(oldCompText.isEmpty())
	{
		if(this->frameType() == QP_TS)
		{
			QString defComp = QuGetDefaultComp(obj_name, QSAS_profiles->current()->Array3DtoScalar);
			DataTab->_y_data_fld->setTextKeepComp(defComp); // set data slot
		}
		else
		{
			QString defComp = QuGetDefaultComp(obj_name, QSAS_profiles->current()->Array3DtoScalarXY); 
			DataTab->_y_data_fld->setTextKeepComp(defComp); // set data slot
		}
	}
	else
	{
		// setTextKeepComp can change text for comp or substitute string
		DataTab->_y_data_fld->setTextKeepComp(obj_name);  // set data slot	
	}

	
	if(!nameLock) this->setText(NAME_COL, DataTab->_y_data_fld->text().latin1());               // set item name
	this->WLcontext = QString(baseObjectText(DataTab->_y_data_fld->text()).latin1());

}

// -------------------------------------------

void QpTraceItem::replaceXData(QString obj_name)
{
	if(QuSlotDebug()) cout << "QpTraceItem::replaceXData" << endl;
	if(obj_name.isEmpty()) return;

	DataTab->_x_data_fld->setTextKeepComp(obj_name);  // set data slot
	// setTextKeepComp can change text for comp or substitute string

}

// -------------------------------------------

void QpTraceItem::replaceComponent(QString comp)
{
	if(QuSlotDebug()) cout << "QpTraceItem::replaceComponent" << endl;
	if(comp.isEmpty()) return;
	
	QString newName = baseObjectText(DataTab->_y_data_fld->text().latin1());
	newName += comp;
	DataTab->_y_data_fld->setText(newName);  // set data slot
	DataTab->_y_data_fld->reValidate();
//	if(!nameLock) this->setText(NAME_COL, newName);               // set item name 

}

// -------------------------------------------

void QpTraceItem::replaceXComponent(QString comp)
{
	if(QuSlotDebug()) cout << "QpTraceItem::replaceXComponent" << endl;
	if(comp.isEmpty()) return;
	
	QString newName = baseObjectText(DataTab->_x_data_fld->text().latin1());
	newName += comp;
	DataTab->_x_data_fld->setText(newName);  // set data slot
	
}

// -------------------------------------------

void QpTraceItem::refreshTabs()
{
	if(QuSlotDebug()) cout << "QpTraceItem::refreshTabs" << endl;

	DataTab->_y_data_fld->reValidate();

}


// ------------------------------------------

void QpTraceItem::changeFrameType(int frameType)
{
	// called when Frame changes type XY <-> TS
	
	if(frameType == QP_TS)
	{
	
		DataTab->_x_label_fld->hide();
		DataTab->_x_data_fld->hide();
		DataTab->timeAsSec_toggle->hide();

		if(DataTab->I_PlotType == QP_CONST_XT)
		{
			// set to const t trace

			DataTab->_y_label_fld->setText("Time Constant: ");
			DataTab->_y_data_fld->acceptTime=true;
			DataTab->_y_data_fld->acceptTimeObject=true;
			DataTab->_y_data_fld->acceptTIvlObject=true;
			DataTab->_y_data_fld->acceptNumber=false;
			DataTab->_y_data_fld->acceptScalarObject=false;
			DataTab->_y_data_fld->acceptTimeSlice=true;
			DataTab->_y_data_fld->acceptTimeSubset=false;
			DataTab->_show_legend_fld->setChecked(false);
		}
		else if(DataTab->I_PlotType == QP_CONST_Y)
		{
			DataTab->_y_label_fld->setText("Y Constant: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=true;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=true;
			DataTab->_y_data_fld->acceptTimeSubset=false;
			DataTab->_show_legend_fld->setChecked(false);
		}
		else
		{
			DataTab->_y_label_fld->setText("Y: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=false;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=false;
			DataTab->_y_data_fld->acceptTimeSubset=true;
			DataTab->_show_legend_fld->setChecked(true);
		}
	}
	else
	{
		DataTab->timeAsSec_toggle->show();
		
		if(DataTab->I_PlotType == QP_CONST_XT)
		{
			DataTab->_y_label_fld->setText("X Constant: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=true;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=true;
			DataTab->_y_data_fld->acceptTimeSubset=false;

			DataTab->_x_label_fld->hide();
			DataTab->_x_data_fld->hide();
			DataTab->timeAsSec_toggle->hide();
			DataTab->_show_legend_fld->setChecked(false);
		}
		else if(DataTab->I_PlotType == QP_CONST_Y)
		{
			DataTab->_y_label_fld->setText("Y Constant: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=true;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=true;
			DataTab->_y_data_fld->acceptTimeSubset=false;
			
			DataTab->_x_label_fld->hide();
			DataTab->_x_data_fld->hide();
			DataTab->timeAsSec_toggle->hide();
			DataTab->_show_legend_fld->setChecked(false);
		}
		else
		{
			DataTab->_y_label_fld->setText("Y: ");
			DataTab->_y_data_fld->acceptTime=false;
			DataTab->_y_data_fld->acceptTimeObject=false;
			DataTab->_y_data_fld->acceptTIvlObject=false;
			DataTab->_y_data_fld->acceptNumber=false;
			DataTab->_y_data_fld->acceptScalarObject=true;
			DataTab->_y_data_fld->acceptTimeSlice=false;
			DataTab->_y_data_fld->acceptTimeSubset=true;

			DataTab->_x_label_fld->show();
			DataTab->_x_data_fld->show();
			DataTab->timeAsSec_toggle->show();
			DataTab->_show_legend_fld->setChecked(false);
		}
		

	}
	DataTab->setDataTypeTitle();
	
}

// -------------------------------------------------

void QpTraceItem::getLabel(const QpAxes& axis, string& label)
{
	// fetch label for axis

	QdObject_var obj;
	string default_label;
	switch(axis)
	{
		case X_AXIS:
			// if time series plot default label is Universal Time
			if(frameType() == QP_TS)
			{
				label = "Universal Time";
				return;
			}
			if(DataTab->timeAsSec_toggle->isChecked())
			{
				label="Seconds";
				return;
			}
			// otherwise fetch x object and check if object needs refreshing
			label = "X Axis";
			obj = _xdo;
			if(obj.is_nil() )
			{
				if( !_ydo.is_nil() && _ydo->xref_exists(DEPEND_1) )  obj = _ydo->get_xref(DEPEND_1);
			}
			break;

		case Y_AXIS:
			// fetch y object and check if object needs refreshing
			label="Y Axis";
			obj = _ydo;
			break;

		default:
			return;
	}

	if(obj.is_nil())  return;
  
	// if object found set default label from LABLAXIS attribute if provided
	default_label = QuGetAttrText(LABLAXIS, obj);

	// if not use FIELDNAM attribute
	if(default_label.size()==0) default_label = QuGetAttrText(FIELDNAM, obj);
	
	// last resort use object name
	if(default_label.size()==0) default_label = QuGetAttrText(OBJECT_NAME, obj);
	if(default_label.size()!=0) label = default_label;
  
	// attach units
	string units = QuGetAttrText(UNITS, obj);
	if(units.size()!=0 && units!=" ")
	{
		label.append(" (");
		label.append(units);
		label.append(")");
	}
	// and spacecraft id
//	string scid=QuGetAttrText(SC_ID,obj);
//	if(scid.size()!=0 && scid!=" ")
//	{
//		label.append(" SC=");
//		label.append(scid);
//	}
	
	// convert theta and phi
	QString greek = QString(label.c_str());
	greek.replace("theta","#gh");
	greek.replace("phi","#gf");
	label = greek.latin1();

}


void QpTraceItem::getLegendText(string& legend_text)
{
	// fetch legend text
  
	if(_ydo.is_nil())  return;
  
	string default_label = QuGetAttrText(LABLAXIS, _ydo);
	// if not use FIELDNAM attribute
	if(default_label.size()==0) default_label = QuGetAttrText(FIELDNAM, _ydo);
	// else use object name
	if(default_label.size()==0) default_label = QuGetAttrText(OBJECT_NAME, _ydo);
	if(default_label.size()!=0) legend_text = default_label;
	else legend_text = "";
	
	// attach units
	string units = QuGetAttrText(UNITS, _ydo);
	if(units.size()!=0 && units!=" ")
	{
		legend_text.append(" (");
		legend_text.append(units);
		legend_text.append(")");
	}

	// and spacecraft id
	string scid=QuGetAttrText(SC_ID, _ydo);
	if(scid.size()!=0 && scid!=" ")
	{
		legend_text.append(" SC=");
		legend_text.append(scid);
	}
}

void QpTraceItem::getName(string& name)
{
	// fetch name of trace object (for display in listview)

	name = DataTab->_y_data_fld->text().latin1();
	int plotType = I_Type;
	
	if(plotType == QP_CONST_XT && frameType() == QP_TS)
	{
		Time time_val;
		if(valid_iso_time_str(name.c_str(), time_val))
			name="T="+time_val.iso();
		else
			name="Const T Trace";
		return;
	}
	if(plotType == QP_CONST_Y )
	{
		bool okay;
		double d = QString(name.c_str()).toDouble(&okay);
		if(okay)
		{
			name = "Constant=";
			name += name.c_str();
		}
		else
			name="Const Y Trace";
		return;
	}
	if(plotType == QP_CONST_XT && frameType() == QP_XY )
	{
		bool okay;
		double d = QString(name.c_str()).toDouble(&okay);
		if(okay)
		{
			name = "Constant=";
			name += name.c_str();
		}
		else
			name="Const X Trace";
		return;
	}
	if(plotType == QP_TRACE && frameType() == QP_TS)
	{
		if(name=="")  name="TS-Trace";
		return;
	}
	if(plotType == QP_TRACE && frameType() == QP_XY)
	{
		if(name=="") name="XY-Trace";
		return;
	}
}

string QpTraceItem::getTitle()
{
	// fetch title for current trace object

	int plotType = I_Type;

	string title ="";
	
	if(linkedPlots.size() != 0)
	{
		QdObject_var dobj = QuGetObjByName( baseObjectText(DataTab->_y_data_fld->text()) );
		if(!dobj.is_nil()) title = QuGetAttrText(FIELDNAM, dobj);
	}
	else
	{
		if(!_ydo.is_nil()) title = QuGetAttrText(FIELDNAM, _ydo);
	}
	
	if(title.size() == 0)  title += DataTab->_y_data_fld->text().latin1();
	
	if(frameType() == QP_XY && plotType == QP_TRACE)
	{
		if(DataTab->timeAsSec_toggle->isChecked() ) return title;
		
		string x_title;
		if(!_xdo.is_nil())  x_title = QuGetAttrText(FIELDNAM, _xdo);
  
		if(x_title.size() == 0) x_title += DataTab->_x_data_fld->text().latin1();
		if(x_title.size() != 0 && title.size() != 0)  title += " vs ";
		title += x_title;
	}
	return title;
}

// --------------------------------

int QpTraceItem::getXmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	
	int isLog=0;
	// find SCALETYP
		
	if (!_xdo.is_nil() && _xdo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_xdo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(strcmp(QuToUpper(scaleType->c_str()), "LOG") == 0) isLog = 1;
		}
	}
	return isLog;
}

// --------------------------------

void QpTraceItem::get_dobjs()
{
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::get_dobjs \n" );
		fflush(stdout);
	}
	_ydo = DataTab->_y_data_fld->getSlotObject();

	if(frameType() == QP_TS)
	{
		// if time series fetch timetags
		_xdo = get_timetags(_ydo);
	}
	else
	{
		if(DataTab->timeAsSec_toggle->isChecked())
		{
			QdTimeSeq_var tt = get_timetags(_ydo);
			QdTime_var tOffset = QdTime_var::narrow(DataTab->_x_data_fld->getSlotObject());
			if(tt.is_nil() || tOffset.is_nil()) _xdo = tt;
			else
			{
				double dOffset = Seconds(*tOffset);
				QdRScalarSeq_var ss = new QdRScalarSeq(tt->sequence_size());
				for(int i=0; i<tt->sequence_size(); i++)
				{
					ss[i] = ((double)Seconds(tt[i]) ) - dOffset;
				}
				_xdo = ss;
			}
		}
		else
		{
			_xdo = DataTab->_x_data_fld->getSlotObject();

			if( _xdo.is_nil() )
			{
				// try Depend_1
				if(!_ydo.is_nil() && _ydo->xref_exists(DEPEND_1) )  _xdo = _ydo->get_xref(DEPEND_1);
			}
		}
	}
	return;
}
// --------------------------------

bool QpTraceItem::getXmetaRange(double& min, double& max)
{
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::getXmetaRange \n" );
		fflush(stdout);
	}

  // gets x scalemin and scalemax values for this trace object
  // returns true if found
  // call with xmin and xmax set to values of range fields
  
	int plotType = I_Type;
	bool rangeFound = false;
	if(plotType == QP_CONST_Y || plotType == QP_CONST_XT)  return false;
	
	if(plotType == QP_TRACE && frameType() == QP_TS)
	{
		// if time series fetch timetags
		QdTimeSeq_var tdo = get_timetags(_ydo);
		// Time tags don't have scalemin/max so use 1st and last
		if(tdo.is_nil() || tdo->sequence_size() == 0) return false;
		min = Seconds(tdo[0]);
		max = Seconds(tdo[tdo->sequence_size()-1]);
		rangeFound = true;
	}
	else if(plotType == QP_TRACE && frameType() == QP_XY)
	{
		if( !(DataTab->timeAsSec_toggle->isChecked()) )
		{
			if(_xdo.is_nil() ) return false;
			if(_xdo->xref_exists(SCALEMIN) && _xdo->xref_exists(SCALEMAX))
			{
				QdRScalar_var minObj = QdRScalar_var::narrow(_xdo->get_xref(SCALEMIN));
				QdRScalar_var maxObj = QdRScalar_var::narrow(_xdo->get_xref(SCALEMAX));
				if(!minObj.is_nil() && !maxObj.is_nil() )
				{
					min = *minObj;
					max = *maxObj;
					rangeFound =  true;
				}
			}
		}
	}

	if (rangeFound)
	{
		// set current choice for use in y range
		_x_range_min = min;
		_x_range_max = max;
		return rangeFound;
	}
	else	return getXRange(min, max);

}

// --------------------------------

bool QpTraceItem::getXRange(double& min, double& max)
{
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::getXRange _xdo is nil %d\n", _xdo.is_nil());
		fflush(stdout);
	}

  // gets x min and max values for this trace object
  // returns true if range calculated
  // call with xmin and xmax set to values of range fields
  
	int plotType = I_Type;

	if(plotType == QP_CONST_Y)  return false;

	if(plotType == QP_CONST_XT && frameType() == QP_XY)
	{
		// In this case ydo is x value
		QdRScalar_var rscalar_var = QdRScalar_var::narrow(_ydo);
		if(!rscalar_var.is_nil())
		{
			min=rscalar_var->scalar();
			max=min;
			return true;
		}
		else return false;
		
	}
	else if(plotType == QP_CONST_XT && frameType() == QP_TS)
	{
		QdTime_var time_val = QdTime_var::narrow(_ydo);
		if(!time_val.is_nil())
		{
			min = time_val->get_time_rep();
			max = min;
			return true;
		}
		else return false;
	}
		
	if(_xdo.is_nil()) return false;

	int obj_id = _xdo->get_id();

	// check if object id has changed
	if(_x_obj_id_getXRange == obj_id)
	{
		// range has already been calculated just get previously stored values for x min and x max
		min = _x_from;
		max = _x_to;
		return true;
	}
	else
	{
        // if yes, refresh id

        _x_obj_id_getXRange = obj_id;

		// recalculate range for x data object
		bool res = QuGetDataRange(_xdo, min, max);
		if(res)
		{
			// store values 
			_x_from = min;
			_x_to = max;
		}
		return res;
	}
  
}

// --------------------------------

int QpTraceItem::getYmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::getYmetaIsLog \n" );
		fflush(stdout);
	}
	
	int isLog=0;
	// find SCALETYP
		
	if (!_ydo.is_nil() && _ydo->xref_exists(SCALETYP) )
	{
		QdString_var scaleType = QdString_var::narrow(_ydo->get_xref(SCALETYP));
		if(!scaleType.is_nil() )
		{
			if(strcmp(QuToUpper(scaleType->c_str()), "LOG") == 0) isLog = 1;
		}
	}
	return isLog;
}

//--------------------------

bool QpTraceItem::getYmetaRange(double& ymin, double& ymax)
{
	// gets y min and max values for all y values
	if ( QuSlotDebug() )
	{
		printf( "QpTraceItem::getYmetaRange (yobj is nil? %d)\n" ,  _ydo.is_nil() );
		fflush(stdout);
	}
	
	bool rangeFound = false;

	int plotType = I_Type;

	if(plotType == QP_CONST_XT || plotType == QP_CONST_Y) return false;
	if( _ydo.is_nil() ) return false; // can go no further
	
	if(_ydo->xref_exists(SCALEMIN) && _ydo->xref_exists(SCALEMAX) )
	{
		QdRScalar_var minObj = QdRScalar_var::narrow(_ydo->get_xref(SCALEMIN));
		QdRScalar_var maxObj = QdRScalar_var::narrow(_ydo->get_xref(SCALEMAX));
		
		if( !minObj.is_nil() && !maxObj.is_nil() )
		{
			ymin = *minObj;
			ymax = *maxObj;
			rangeFound = true;
		}
	}

	// choose default spacecraft colour
	if (_ydo->xref_exists(SC_ID) )
	{
		QdString_var sc_id_str = QdString_var::narrow(_ydo->get_xref(SC_ID));
		if(!sc_id_str.is_nil() )
		{
			if(strcmp(QuToUpper(sc_id_str->c_str()), "1") == 0 || strcmp(QuToUpper(sc_id_str->c_str()), "C1") == 0)
			{
				if(!userLineColour) TraceTab->setLineColour(C1_colour);
				if(!userSymbolColour) TraceTab->setSymbolColour(C1_colour);
			}
			if(strcmp(QuToUpper(sc_id_str->c_str()), "2") == 0 || strcmp(QuToUpper(sc_id_str->c_str()), "C2") == 0)
			{
				if(!userLineColour) TraceTab->setLineColour(C2_colour);
				if(!userSymbolColour) TraceTab->setSymbolColour(C2_colour);
			}
			if(strcmp(QuToUpper(sc_id_str->c_str()), "3") == 0 || strcmp(QuToUpper(sc_id_str->c_str()), "C3") == 0)
			{
				if(!userLineColour) TraceTab->setLineColour(C3_colour);
				if(!userSymbolColour) TraceTab->setSymbolColour(C3_colour);
			}
			if(strcmp(QuToUpper(sc_id_str->c_str()), "4") == 0 || strcmp(QuToUpper(sc_id_str->c_str()), "C4") == 0)
			{
				if(!userLineColour) TraceTab->setLineColour(C4_colour);
				if(!userSymbolColour) TraceTab->setSymbolColour(C4_colour);
			}
		}
		
	}
	if (rangeFound) return rangeFound;
	else{
		bool autoFound = getYRange(ymin, ymax);
		if (!I_Panel->yAxisTab->_axis_y_nudge_fld->isChecked()) return autoFound;
		
		// range is not nudged by panel getYmetaRange, so do it here
		// This is not be effective, however, if each trace in a panel 
		// has a small range relative to panel range
		if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			double nudgeUp = pow(fabs(ymax / ymin), 0.01);
			double nudgeDown = pow(fabs(ymax / ymin), 0.02);
			ymax *= nudgeUp;
			ymin /= nudgeDown;
		}
		else
		{
			double nudge = fabs(ymax - ymin)*0.02;
			ymax += nudge;
			ymin -= nudge;
		}
		return autoFound;
	}
}


//--------------------------

bool QpTraceItem::getYRange(double& ymin, double& ymax)
{
	// gets y min and max values for all y values in range xmin - xmax
	// call with ymin and ymax values set to current values of y range fields
	if ( QuSlotDebug() )
	{
		printf( " QpTraceItem::getYRange \n" );
		fflush(stdout);
	}
	
	double xmin, xmax;
	
	if(frameType() == QP_TS)
	{
		xmin=I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->value();
		xmax = I_Panel->I_Frame->tAxisTab->_axis_to_time_fld->value();
	}
	else
	{
		xmin=I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->value();
		xmax = I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->value();
	}
	if(xmin >= xmax) return false;
	
	bool recalc=false;
		
	int plotType = I_Type;

	if(plotType == QP_CONST_XT ) return false;
	if( _ydo.is_nil() ) return false; // can go no further
	  
	if(plotType == QP_CONST_Y)
	{
		QdRScalar_var rscalar_var = QdRScalar_var::narrow(_ydo);
		if(!rscalar_var.is_nil())
		{
			ymax = rscalar_var->scalar();
			ymin = rscalar_var->scalar();
			return true;
		}
		else return false;
	}
	
	// check if y object has changed
	int obj_id = _ydo->get_id();
	if(  _y_obj_id_getYRange != obj_id)
	{
		// if yes, store id
		recalc = true;
		_y_obj_id_getYRange = obj_id;
	}
 
	if(_xdo.is_nil()) return false;
	
	obj_id = _xdo->get_id();
	  
	// check if x range has changed since last call
	if(_x_range_min != xmin || _x_range_max != xmax)
	{
		_x_range_min = xmin;
		_x_range_max = xmax;
		recalc=true;
	}
	// if nothing has changed fetch range calculated last time
	if( !recalc )
	{
		ymin = _y_from;
		ymax = _y_to;
		return true;
	}
	
	// refresh y range

	// narrow x object to appropriate sequence type and fetch mask of entries in range
	QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(_xdo);
	QdTimeSeq_var timeseq_x = QdTimeSeq_var::narrow(_xdo);
	SequenceMask msk;
	if(!rscalarseq_x.is_nil()) msk = rscalarseq_x->get_mask(QdRScalarSeq::InRange(xmin, xmax));
	if(!timeseq_x.is_nil())
	{
		msk.resize(timeseq_x->size());
		for(register int i=0; i<timeseq_x->size(); i++)
		{
			double t = timeseq_x[i].get_time_rep();
			msk[i] = (t>=xmin && t<=xmax);
		}
	}
	
	bool res = false;
	
	// narrow y object to sequence of appropriate type and fetch subsequence using mask
	QdRScalarSeq_var ydo = QdRScalarSeq_var::narrow(_ydo);
	if(!ydo.is_nil())
	{
		if(msk.size())
		{
			QdRScalarSeq_var y_var = new QdRScalarSeq;
			ydo->get_subsequence(*y_var, msk);
			ydo = y_var;
		}
		// get range of values in subsequence
		res = QuGetDataRange(ydo, ymin, ymax);
	}
	
	// or narrow to matrix (but then cannot use msk)
	QdRMatrix_var ydm = QdRMatrix_var::narrow(_ydo);
	if(!ydm.is_nil()) res = QuGetDataRange(ydm, ymin, ymax);
	
	QdRMatrixSeq_var ydms = QdRMatrixSeq_var::narrow(_ydo);
	if(!ydms.is_nil()) res = QuGetDataRange(ydms, ymin, ymax);
	
	// store results if min and max found
	if(res)
	{
		_y_from = ymin;
		_y_to = ymax;
	}
	return res;
}


//--------------------------

QpuiReturnStatus QpTraceItem::plot()
{
	if(QuSlotDebug())
	{
		printf("QpTraceItem::plot \n");
		fflush(stdout);
	}
	int plotType = I_Type;
		
		// plots the Trace Object
	PLFLT *x_data=0;
	PLFLT *y_data=0;
	
	if(I_Panel->yAxisTab->_axis_from_xy_fld->text().isEmpty())
	{
		plmtex("lv",-10.0,0.5,0.0,"Y data ?? (check Y-axis range and component selection)");
		return QPUI_FAILURE;
	}
	
	if(frameType()==QP_TS)
	{
		if(I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->text().isEmpty())
		{
			plmtex("lv",-10.0,0.5,0.0,"T Data ?? (check T-axis range and DEPEND_0)");
			return QPUI_FAILURE;
		}
	}
	else
	{
		if(I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->text().isEmpty())
		{
			plmtex("lv",-10.0,0.5,0.0,"X Data ?? (check X-axis range, DEPEND_0 or X data object)");
			return QPUI_FAILURE;
		}
	}
	
	RScalarSeq x_diff;
	int n_data=0;
	double nominal_spacing = DBL_MAX;
	
	PLINT line_colour_index = getPGPColourIndex(_line_colour);
	PLINT symbol_colour_index = getPGPColourIndex(_symbol_colour);
	
		// Refresh y data object
	if(_ydo.is_nil())
	{
		plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Null Y Object");
		return QPUI_FAILURE;
	}
	
	if(plotType == QP_CONST_XT && frameType() == QP_TS)
	{
		QdTimeInterval_var tint = get_timeinterval(_ydo);
		QdTime_var t_plot = QdTime_var::narrow(_ydo);
		Time t_plot_start;
		if( !valid_iso_time_str(I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->text().latin1(), t_plot_start))
		{
			plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Invalid from time");
			return QPUI_FAILURE;
		}

		Time t_midnite(t_plot_start);
		// t_midnite is midnight at start of t_plot_start day...
		t_midnite -= Seconds(t_plot_start.second_of_day());

		if(!tint.is_nil())
		{
			x_data  =new PLFLT[4];
			y_data = new PLFLT[4];
			y_data[0] = y_data[3] = atof(I_Panel->yAxisTab->_axis_to_xy_fld->text().latin1());
			y_data[1] = y_data[2] = atof(I_Panel->yAxisTab->_axis_from_xy_fld->text().latin1());
			if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
			{
				y_data[0] = SafeLog10(y_data[0]);
				y_data[1] = SafeLog10(y_data[1]);
				y_data[2] = SafeLog10(y_data[2]);
				y_data[3] = SafeLog10(y_data[3]);
			}
			n_data = 4;

			x_data[0] = x_data[1] = (PLFLT)Seconds(tint->start()-t_midnite);
			x_data[2] = x_data[3] = (PLFLT)Seconds(tint->end()-t_midnite);
		}
		else
		{
			Time time_val;
			if(t_plot.is_nil())
			{
				plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Invalid constant time");
				return QPUI_FAILURE;
			}
			else time_val=*t_plot;
		 
			PLFLT t_constant = (PLFLT)Seconds(time_val - t_midnite);
			x_data = new PLFLT[2];
			x_data[0] = x_data[1] = t_constant;
			y_data = new PLFLT[2];
			y_data[0] = I_Panel->yAxisTab->_axis_from_xy_fld->value();
			y_data[1] = I_Panel->yAxisTab->_axis_to_xy_fld->value();
			if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
			{
				y_data[0] = SafeLog10(y_data[0]);
				y_data[1] = SafeLog10(y_data[1]);
			}
			n_data=2;
		}
	}
	else if(plotType == QP_CONST_Y)
	{
		PLFLT y_constant;
		QdRScalar_var rscalar_var = QdRScalar_var::narrow(_ydo);
		if(rscalar_var.is_nil())
		{
			QdRScalarSeq_var rscalarseq_var = QdRScalarSeq_var::narrow(_ydo);
			if(!rscalarseq_var.is_nil() && rscalarseq_var->sequence_size() == 1) rscalar_var = new QdRScalar(rscalarseq_var[0]);
		}

		if(rscalar_var.is_nil()) return QPUI_FAILURE;
		y_constant = (PLFLT)rscalar_var->scalar();
		
		n_data=2;
		y_data = new PLFLT[2];
		y_data[0] = y_data[1] = y_constant;
		if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			y_data[0] = SafeLog10(y_data[0]);
			y_data[1] = SafeLog10(y_data[1]);
		}
		x_data = new PLFLT[2];
		if(frameType() == QP_TS)
		{
			Time t_plot_start;
			if( !valid_iso_time_str(I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->text().latin1(), t_plot_start))
				return QPUI_FAILURE;

			Time t_plot_end;
			if( !valid_iso_time_str(I_Panel->I_Frame->tAxisTab->_axis_to_time_fld->text().latin1(), t_plot_end))
				return QPUI_FAILURE;

			Time t_midnite(t_plot_start);
			// t_midnite is midnight at start of t_plot_start day...
			t_midnite -= Seconds(t_plot_start.second_of_day());
			x_data[0] = (PLFLT)Seconds(t_plot_start - t_midnite);
			x_data[1] = (PLFLT)Seconds(t_plot_end - t_midnite);
		}
		else
		{
			x_data[0] = atof(I_Panel->I_Frame->xAxisTab->_axis_from_xy_fld->text().latin1());
			x_data[1] = atof(I_Panel->I_Frame->xAxisTab->_axis_to_xy_fld->text().latin1());
			if(I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
			{
				x_data[0] = SafeLog10(x_data[0]);
				x_data[1] = SafeLog10(x_data[1]);
			}
		}
	}
	else if(plotType == QP_CONST_XT && frameType() == QP_XY)
	{
		PLFLT x_constant;
		QdRScalar_var rscalar_var = QdRScalar_var::narrow(_ydo);
		if(rscalar_var.is_nil()) return QPUI_FAILURE;
		x_constant = (PLFLT)rscalar_var->scalar();
		
		n_data=2;
		x_data = new PLFLT[2];
		x_data[0] = x_data[1] = x_constant;
		y_data = new PLFLT[2];
		y_data[0] = atof(I_Panel->yAxisTab->_axis_from_xy_fld->text().latin1());
		y_data[1] = atof(I_Panel->yAxisTab->_axis_to_xy_fld->text().latin1());
		if(I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			x_data[0] = SafeLog10(x_data[0]);
			x_data[1] = SafeLog10(x_data[1]);
		}
		if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			y_data[0] = SafeLog10(y_data[0]);
			y_data[1] = SafeLog10(y_data[1]);
		}
	}
	else if(plotType == QP_TRACE && frameType() == QP_TS)
	{
		QdTimeSeq_var tdo = get_timetags(_ydo);
		if(tdo.is_nil() )
		{
			plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Null T Object");
			return QPUI_FAILURE;
		}
		// fetch number of elements in each sequence/array
		n_data = QuLength(_ydo);
		int nt_data = tdo->sequence_size();
		if(nt_data != n_data)
		{
			plmtex("lv",-10.0,0.5,0.0,"ERROR: Y and T series of different lengths");
			return QPUI_FAILURE;
		}
		
		// create float c-style arrays for plotting
		x_data = new PLFLT[n_data];
		y_data = new PLFLT[n_data];
		x_diff = RScalarSeq(n_data);
		
		// get y object data
		if(!QuSet_Array(n_data, &(y_data[0]), _ydo))
		{
			plmtex("lv",-10.0,0.5,0.0,"ERROR: Y Object not an Scalar, ScalarSeq or 1D matrix");
			return QPUI_FAILURE;
		}
		

		Time t_plot_start;
		if( !valid_iso_time_str(I_Panel->I_Frame->tAxisTab->_axis_from_time_fld->text().latin1(), t_plot_start))
			return QPUI_FAILURE;
		
		Time t_midnite(t_plot_start);
			// t_midnite is midnight at start of t_plot_start day...
		t_midnite -= Seconds(t_plot_start.second_of_day());
		Time half_spacing_t;
        if(0 == calc_tol_spacing(half_spacing_t, *tdo, QDOS::Seconds(0), QSAS_profiles->current()->gapTolerance)) nominal_spacing = half_spacing_t.get_time_rep();
// 		Time half_spacing2_t;
// 		TimeSeq toffseq(*tdo);
// 		toffseq.sub_assign(t_midnite);
// 		double nominal_spacing2 = DBL_MAX;
// 		if(0 == calc_tol_spacing(half_spacing2_t, toffseq)) nominal_spacing2 = half_spacing2_t.get_time_rep();
		
		// loop through data and set x and y c-style arrays
		double x_diff_sum = 0.;
		for(int i=0; i<n_data; i++)
		{
			// set time in number of seconds from t_midnite
			double s_i = Seconds(tdo[i] - t_midnite);
			if(i==0) x_diff[0] = s_i;
			else x_diff[i] = s_i - x_diff_sum;
			x_diff_sum += x_diff[i];
			x_data[i] = (PLFLT)s_i;
		}
		
		// calculate logarithmic values if logs selected
		if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			for(int i=0; i<n_data; i++) y_data[i] = (PLFLT) SafeLog10((double)y_data[i]);
		}
	}
	else if(plotType == QP_TRACE && frameType() == QP_XY)
	{
		// Data Series (XY trace)
		
		// fetch number of elements in each sequence/array
		n_data = QuLength(_ydo);
		int nx_data = QuLength(_xdo);

		if(nx_data != n_data)
		{
			plmtex("lv",-10.0,0.5,0.0,"ERROR: Y and X series of different lengths");
			return QPUI_FAILURE;
		}

		// create float c-style arrays for plotting
		x_data = new PLFLT[n_data];
		y_data = new PLFLT[n_data];

		// get y data
		if(!QuSet_Array(n_data, &(y_data[0]), _ydo))
		{
			plmtex("lv",-10.0,0.5,0.0,"ERROR: Y Object not an Scalar, ScalarSeq or 1D matrix");
			return QPUI_FAILURE;
		}
		
		// get x data
		QdRScalarSeq_var rscalarseq_x = QdRScalarSeq_var::narrow(_xdo);
		if(!QuSet_Array(n_data, &(x_data[0]), _xdo))
		{
			plmtex("lv",-10.0,0.5,0.0,"ERROR: X Object not an Scalar, ScalarSeq or 1D Matrix");
			return QPUI_FAILURE;
		}

		// loop through data and set x logs 
		if(I_Panel->I_Frame->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			for(int i=0; i<n_data; i++) x_data[i] = SafeLog10(x_data[i]);
		
		}
				
		// loop through data and set y logs 
		if(I_Panel->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			for(int i=0; i<n_data; i++) y_data[i] = SafeLog10(y_data[i]);
		
		}

	} // End if else over trace types
	
	int i=0;
	while (i < n_data)
	{
		int j=i;
		i++;
		if(plotType == QP_TRACE && frameType() == QP_TS)
		{
			// break into sub-sequences on gaps
			while(i < n_data && (double)x_diff[i] < nominal_spacing && x_diff[i] > 0. )
			{
				i++;  // increment i
			}
		}
		else if( plotType == QP_CONST_XT && frameType() == QP_TS)
		{
			i++;
		}
		else i = n_data; // most trace types ignore gaps
	
		// calculate no. points in this sub-sequence with no gaps
		int n_pts = i-j;
		if(TraceTab->_line_style_fld->currentItem() != QP_NOLINE)
		{
			plwid(QtWidth2PlWidth(TraceTab->_line_width_fld->value()));
			setDashStyle((QpLineStyle)TraceTab->_line_style_fld->currentItem());
			plcol0(line_colour_index);
			// draw line between points of sub-sequence of appropriate time
			switch(TraceTab->_connect_style_fld->currentItem())
			{
				case QPUI_STRAIGHT: default:
					plline(n_pts, x_data+j, y_data+j);
					break;
				case QPUI_HIST:
					plbin(n_pts, x_data+j, y_data+j, PL_BIN_CENTRED|PL_BIN_NOEXPAND);
					break;
				case QPUI_NOCONNECT:
					break;
			}
		}
		
		if(TraceTab->_symbol_shape_fld->currentItem() == QPUI_NOSYMBOL && n_pts==1)
		{
			// ensure single point is visible!
			plssym(0., TraceTab->_line_width_fld->value());
			plcol0(line_colour_index);
			// plot symbols in sub-sequence
			plpoin(1, x_data+j, y_data+j, (PLINT)QPUI_POINT);
		}
		
		if(TraceTab->_symbol_shape_fld->currentItem() != QPUI_NOSYMBOL)
		{
			plssym(0., TraceTab->_symbol_size_fld->value());
			plcol0(symbol_colour_index);
					// plot symbols in sub-sequence
			plpoin(n_pts, x_data+j, y_data+j, symbolIndexConversion(TraceTab->_symbol_shape_fld->currentItem()));
		}
	} // repeat until all data exhausted..
	
	delete x_data;
	delete y_data;
	return QPUI_SUCCESS;
}


QpuiReturnStatus QpTraceItem::plotLegend()
{
	if( !(DataTab->_show_legend_fld->isChecked()) || !(I_Panel->LegendTab->_show_legend_fld->isChecked()) ) return QPUI_SUCCESS;
    
	PLINT line_colour_index = getPGPColourIndex(_line_colour);
	PLINT symbol_colour_index = getPGPColourIndex(_symbol_colour);
    
	PLFLT x0win_save, x1win_save, y0win_save, y1win_save;
	PLFLT x0vp_save, x1vp_save, y0vp_save, y1vp_save;
    
	plgvpw(&x0win_save, &x1win_save, &y0win_save, &y1win_save);
	plgvpd(&x0vp_save, &x1vp_save, &y0vp_save, &y1vp_save);
    
//     QPlot_window->save();
    
	int row_num, col_num, num_cols, num_rows;
    
	row_num = I_Panel->_legend_entry / I_Panel->LegendTab->_legend_num_cols_fld->value();
	col_num = I_Panel->_legend_entry - row_num * I_Panel->LegendTab->_legend_num_cols_fld->value();
    
//     PLFLT xch, ych; 
	plschr(0., I_Panel->LegendTab->_legend_char_size_fld->value());
//     plgchr(&xch, &ych);
    
	float legend_width, legend_height;
	legend_width = I_Panel->LegendTab->_legend_width_fld->value() * (x1vp_save - x0vp_save);
	legend_height = I_Panel->LegendTab->_legend_height_fld->value() * (y1vp_save - y0vp_save);

	num_cols = I_Panel->LegendTab->_legend_num_cols_fld->value();
	num_rows = I_Panel->LegendTab->_legend_num_rows_fld->value();
	float _legend_cell_width, _legend_cell_height;
	_legend_cell_width = legend_width / num_cols;
	_legend_cell_height = legend_height / num_rows;
    
	float x0_vp, x1_vp, y0_vp, y1_vp;
	x0_vp = x0vp_save + I_Panel->LegendTab->_legend_loffset_fld->value() * (x1vp_save - x0vp_save);
	x0_vp += col_num * _legend_cell_width ;
	x1_vp = x0_vp + _legend_cell_width;

	y1_vp = y1vp_save - I_Panel->LegendTab->_legend_toffset_fld->value() * (y1vp_save - y0vp_save);
	y1_vp -= row_num * _legend_cell_height;
	y0_vp = y1_vp - _legend_cell_height;
    
	plvpor(x0_vp, x1_vp, y0_vp, y1_vp);
	plwind(0.,1.,0.,1.);
	plcol0(0);
	if((QpuiLegendBackground)I_Panel->LegendTab->_legend_background_fld->currentItem() == QPUI_OPAQUE)
	{
		PLFLT x_fill[]={0., 0., 1., 1.};
		PLFLT y_fill[]={0., 1., 1., 0.};
		plfill(4, x_fill, y_fill);
	}
	plcol0(1);
    
	float len, marg;
	len = I_Panel->LegendTab->_legend_line_length_fld->value();
	marg = 0.1*len;
    
	PLFLT x_lgd[2];
	x_lgd[0] = marg;
	x_lgd[1] = marg + len;
	PLFLT y_lgd[2] = {0.,0.};
	float txt_lgd = 0.;
	float ych_cell_height = getCharHeight();/*ych;*/
	if( _legend_cell_height > 1e-4) ych_cell_height /=  _legend_cell_height;
	if( ych_cell_height > 1.) ych_cell_height = 1.;
    
	switch((QpuiLegendJust)I_Panel->LegendTab->_legend_just_fld->currentItem())
	{
		case QPUI_VCENTRE: default:
			txt_lgd = 0.5;
			break;
		case QPUI_TOP:
			txt_lgd = 1. - ych_cell_height*1.2;
			break;
		case QPUI_BOTTOM:
			txt_lgd = 0.00 + ych_cell_height*1.2;
			break;
	}

    
	y_lgd[1] = txt_lgd;
	y_lgd[0] = y_lgd[1];
	if(TraceTab->_line_style_fld->currentItem() != QP_NOLINE)
	{
		plschr(QPUI_CHAR_HEIGHT, I_Panel->LegendTab->_legend_char_size_fld->value());
		plwid(QtWidth2PlWidth(TraceTab->_line_width_fld->value()));
		setDashStyle((QpLineStyle)TraceTab->_line_style_fld->currentItem());
		plcol0(line_colour_index);
		plline(2, x_lgd, y_lgd);
	}
	if(TraceTab->_symbol_shape_fld->currentItem() != QPUI_NOSYMBOL)
	{
		plssym(40., I_Panel->LegendTab->_legend_char_size_fld->value());
		plcol0(symbol_colour_index);
		PLFLT x=0.5*(x_lgd[0]+x_lgd[1]);
		PLFLT y=y_lgd[0];
		plpoin(1, &x, &y, symbolIndexConversion(TraceTab->_symbol_shape_fld->currentItem()));
	}
	plcol0(1);
    
	int _text_line_width = 1;
	plschr(0., I_Panel->LegendTab->_legend_char_size_fld->value());
	PLINT family, style, weight;
	I_Panel->LegendTab->_legend_font_fld->getFontParams(family, style, weight);
	plsfont(family, style, weight);
	
	plwid(QtWidth2PlWidth(_text_line_width));
	string legend_text = DataTab->_legend_text_fld->text().latin1();

	legend_text = SafePGPString(legend_text);
	QString greek = QString(legend_text.c_str());
	greek.replace("theta","#gh");
	greek.replace("phi","#gf");
	legend_text = greek.latin1();
	
	if(legend_text.size() > I_Panel->LegendTab->_legend_max_char_length_fld->value())
		legend_text = legend_text.substr(0, I_Panel->LegendTab->_legend_max_char_length_fld->value());
	plptex(x_lgd[1]+marg, txt_lgd, 1., 0., 0., legend_text.c_str());
	I_Panel->_legend_entry++;
	plscol0(0,255,255,255);
	plscol0(1,0,0,0);
	plcol0(1);

//     QPlot_window->unsave();
	plvpor(x0vp_save, x1vp_save, y0vp_save, y1vp_save);
	plwind(x0win_save, x1win_save, y0win_save, y1win_save);
    
	return QPUI_SUCCESS;
}

QpItemType QpRemapPlotType(int _type, QpFrameType &frameType){

	switch(_type){
		case 0:
			frameType = QP_XY;
			return QP_TRACE;
			break;
		
		case 1: default:
			frameType = QP_TS;
			return QP_TRACE;
			break;
		
		case 2:
			frameType = QP_TS;
			return QP_SPECTROGRAM;
			break;

		case 3:
			frameType = QP_XY;
			return QP_SPECTROGRAM;
			break;

		case 4:
			frameType = QP_TS;
			return QP_CONST_XT;
			break;

		case 5:
			frameType = QP_XY;
			return QP_CONST_Y;
			break;
		
		case 6:
			frameType = QP_XY;
			return QP_CONST_XT;
			break;
		
		case 7:
			frameType = QP_XY;
			return QP_TRACE;
			break;
		
		case 8:
			frameType = QP_TS;
			return QP_TRACE;
			break;
		
		case 9:
		// unknown type flag
			return QP_TRACE;
			break;
		
		case 10:
			frameType = QP_XY;
			return QP_EVENT;
			break;
		
		case 11:
			frameType = QP_TS;
			return QP_EVENT;
			break;
   
	}
   
}

// ----------------------------------

Vistream& operator>>(Vistream& v_is, QpTraceItem& trace)
{
	if(QuSlotDebug()) cout <<  "QpTraceItem restore (>>)" << endl;

	// streams from virtual input stream v_is into trace object

	QpPlotItem& plot_item = trace;
	v_is >> plot_item;

	string _y_data;
	string _x_data;
	v_is >> _y_data;
	v_is >> _x_data;
	trace.DataTab->_x_data_fld->setText(_x_data.c_str());
	trace.DataTab->_y_data_fld->setText(_y_data.c_str());
  
	string linkText;
	v_is >> linkText;
	trace.setText(LINK_COL, linkText.c_str());
	// do the relinking in listview restore after traces etc are there
	
	int _connect_style;
	v_is >> _connect_style;
	trace.TraceTab->_connect_style_fld->setCurrentItem(_connect_style);
	
	int _line_style;
	v_is >> _line_style;
	trace.TraceTab->_line_style_fld->setCurrentItem(_line_style);
	
	float _line_width;
	v_is >> _line_width;
	trace.TraceTab->_line_width_fld->setValue(_line_width);
	
	v_is >> trace.userLineColour;

	int r,g,b;
	v_is >> r;
	v_is >> g;
	v_is >> b;

	trace.TraceTab->setLineColour( QColor(r,g,b));
	
	int _symbol_shape;
	v_is >> _symbol_shape;
	trace.TraceTab->_symbol_shape_fld->setCurrentItem(_symbol_shape);
	
	float _symbol_size;
	v_is >> _symbol_size;
	trace.TraceTab->_symbol_size_fld->setValue(_symbol_size);
	
	v_is >> trace.userSymbolColour;

	v_is >> r;
	v_is >> g;
	v_is >> b;
	trace.TraceTab->setSymbolColour(QColor(r,g,b));

	bool autoLegend;
	v_is >> autoLegend;
	trace.DataTab->_auto_legend_fld->setChecked(autoLegend);
	
	string _legend_text;
	v_is >> _legend_text;
	trace.DataTab->_legend_text_fld->setText(_legend_text.c_str());

	bool _show_legend;
	v_is >> _show_legend;
	trace.DataTab->_show_legend_fld->setChecked(_show_legend);
	
	trace.DataTab->_x_data_fld->reValidate();
	trace.DataTab->_y_data_fld->reValidate();
	
	return v_is;
}

// ------------------------------------------


Vostream& operator<<(Vostream& v_os, QpTraceItem& trace)
{
	if(QuSlotDebug()) cout << "QpTraceItem save (<<) '" << trace.text(NAME_COL).latin1() << "'" << endl;

	// streams trace object to virtual output stream v_os
  
	QpPlotItem& plot_item = trace;
	v_os << plot_item;
	
	string _y_data = trace.DataTab->_y_data_fld->text().latin1();
	string _x_data = trace.DataTab->_x_data_fld->text().latin1();

	v_os << _y_data;
	v_os << _x_data;

	string linkText;
	if(!trace.text(LINK_COL).isEmpty())
	{
		linkText = string(trace.text(LINK_COL).latin1());
		linkText += "S"; // to avoid conflict with existing links on restore
	}
	else linkText="";
	v_os << linkText;
	
	
	int _connect_style = trace.TraceTab->_connect_style_fld->currentItem();
	v_os << _connect_style;
	
	int _line_style = trace.TraceTab->_line_style_fld->currentItem();
	v_os << _line_style;
	
	float _line_width = trace.TraceTab->_line_width_fld->value();
	v_os << _line_width;
	
	v_os << trace.userLineColour;

	v_os << trace._line_colour.red();
	v_os << trace._line_colour.green();
	v_os << trace._line_colour.blue();
	
	int _symbol_shape = trace.TraceTab->_symbol_shape_fld->currentItem();
	v_os << _symbol_shape;
	
	float _symbol_size = trace.TraceTab->_symbol_size_fld->value();
	v_os << _symbol_size;
	
	v_os << trace.userSymbolColour;

	v_os << trace._symbol_colour.red();
	v_os << trace._symbol_colour.green();
	v_os << trace._symbol_colour.blue();
	
	bool autoLegend = trace.DataTab->_auto_legend_fld->isChecked();
	v_os << autoLegend;


	string _legend_text = trace.DataTab->_legend_text_fld->text().latin1();
	v_os << _legend_text;

	bool _show_legend = trace.DataTab->_show_legend_fld->isChecked();
	v_os << _show_legend;
	
	return v_os;
}



