// qpui_value_return.cc
//
// Plot Interface Implementation - Value Return 
//
// Janet Barnes
//
// July 2002
// Modified Tony Allen 2008


#include "qpui.h"
#include "qwui.h"


//     QpValueReturnTab CLASS
//     ---------------------------  

#if QT_VERSION < 0x040000
QpValueReturnTab::QpValueReturnTab(QTabWidget *tabwidget, QString name, QpPageItem *page):
  QpTab(tabwidget, name, QP_VALUE_TAB)
{
  // Constructor
     
  QBoxLayout * value_return_layout=new QBoxLayout(/*value_return_widget*/this, QBoxLayout::TopToBottom);
  
  QGroupBox *getvalue_groupbox=new QGroupBox(1, Qt::Vertical, this,"getvalue_groupbox");
  getvalue_groupbox->setTitle("Get Values from Plot");
  
  QWidget* getvalue_widget=new QWidget(getvalue_groupbox, "getvalue_widget");

  QBoxLayout * getvalue_v_layout=new QBoxLayout(getvalue_widget, QBoxLayout::TopToBottom);

  getvalue_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
  getvalue_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

  QBoxLayout * getvalue_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight/*, getvalue_widget*/);
  QPushButton *get_values_pb=new QPushButton("Get Values...",getvalue_widget, "get_values_pb");
  get_values_pb->setFixedWidth(300);
  getvalue_h_layout_1->addWidget(get_values_pb);
  
  QBoxLayout * getvalue_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight/*, getvalue_widget*/);
  
  _xaxis_swap_fld=new QCheckBox("Preserve X,T direction", getvalue_widget,"xaxis_swap_fld");
  _xaxis_swap_fld->setChecked(true);
  getvalue_h_layout_2->addWidget(_xaxis_swap_fld);
  
  _yaxis_swap_fld=new QCheckBox("Preserve Y direction", getvalue_widget,"yaxis_swap_fld");
  _yaxis_swap_fld->setChecked(true);
  getvalue_h_layout_2->addWidget(_yaxis_swap_fld);
  
  getvalue_v_layout->addLayout(getvalue_h_layout_1);
  getvalue_v_layout->addLayout(getvalue_h_layout_2);
    
  value_return_layout->addWidget(getvalue_groupbox);
  
  QGroupBox *xaxis_groupbox=new QGroupBox(1, Qt::Vertical, this,"xaxis_groupbox");
  xaxis_groupbox->setTitle("Horizontal Axis");
  
  QWidget* xaxis_widget=new QWidget(xaxis_groupbox, "xaxis_widget");


  QBoxLayout *xaxis_v_layout=new QBoxLayout(xaxis_widget, QBoxLayout::TopToBottom); 

  xaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
  xaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

  QBoxLayout *xaxis_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QLabel* xaxis_from_label=new QLabel("From: ",xaxis_widget, "xaxis_from_label");
	xaxis_h_layout_1->addWidget(xaxis_from_label);
	
	_xaxis_from_fld=new QguWLinput(xaxis_widget, "xaxis_from");
	QFontMetrics axis_fm(_xaxis_from_fld->font());
	int axis_w = axis_fm.width("YYYYMMDD HH:MM:SS.SSS XX");
	int axis_h = axis_fm.height();
	_xaxis_from_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_xaxis_from_fld);
	
	_taxis_from_fld=new QbDateTimeWidget(xaxis_widget);
	_taxis_from_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_taxis_from_fld);
	_taxis_from_fld->hide();
  
	xaxis_h_layout_1->addStretch(1);
	
	QLabel* xaxis_to_label=new QLabel("To: ",xaxis_widget, "xaxis_to_label");
	xaxis_h_layout_1->addWidget(xaxis_to_label);
	
	_xaxis_to_fld=new QguWLinput(xaxis_widget, "xaxis_from");
  	_xaxis_to_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_xaxis_to_fld);
	
	_taxis_to_fld=new QbDateTimeWidget(xaxis_widget);
	_taxis_to_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_taxis_to_fld);
	_taxis_to_fld->hide();

	QBoxLayout *xaxis_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight/*, xaxis_widget*/);
  _xaxis_save_fld=new QLabel("Save as", xaxis_widget,"xaxis_save_fld");
  xaxis_h_layout_2->addWidget(_xaxis_save_fld);
  
  _xaxis_name_fld=new QguWLinput(xaxis_widget,"x_data");
  _xaxis_name_fld->setMinimumWidth(240);
  xaxis_h_layout_2->addWidget(_xaxis_name_fld);
  _xaxis_name_fld->acceptObject=true;
  _xaxis_name_fld->acceptText=true;
  _xaxis_name_fld->setText("Interval");

  _xaxis_overwrite_fld=new QCheckBox("Overwrite",  xaxis_widget,"xaxis_overwrite_fld");
  _xaxis_overwrite_fld->setChecked(true);
  xaxis_h_layout_2->addWidget(_xaxis_overwrite_fld);
   
  QBoxLayout *xaxis_h_layout_3=new QBoxLayout(QBoxLayout::LeftToRight/*, xaxis_widget*/);
  _saveXT = new QPushButton( "Save X T Interval", xaxis_widget, "saveXT" );
  connect( _saveXT, SIGNAL(clicked()), this, SLOT(saveXT()) );
  _saveXT->setFixedWidth(300);
  xaxis_h_layout_3->addWidget(_saveXT);
  
  xaxis_v_layout->addLayout(xaxis_h_layout_1);
  xaxis_v_layout->addLayout(xaxis_h_layout_2);
  xaxis_v_layout->addLayout(xaxis_h_layout_3);

  value_return_layout->addWidget(xaxis_groupbox);

  QGroupBox *yaxis_groupbox=new QGroupBox(1, Qt::Vertical, this,"yaxis_groupbox");
  yaxis_groupbox->setTitle("Y Axis");
  
  QWidget* yaxis_widget=new QWidget(yaxis_groupbox, "yaxis_widget");

  QBoxLayout *yaxis_v_layout=new QBoxLayout(yaxis_widget, QBoxLayout::TopToBottom); 

  yaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
  yaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

  QBoxLayout *yaxis_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QLabel* yaxis_from_label=new QLabel("From: ",yaxis_widget, "yaxis_from_label");
	yaxis_h_layout_1->addWidget(yaxis_from_label);
	
	_yaxis_from_fld=new QguWLinput(yaxis_widget, "yaxis_from");
	_yaxis_from_fld->acceptNumber=true;
	_yaxis_from_fld->setMinimumSize(axis_w, axis_h);
	yaxis_h_layout_1->addWidget(_yaxis_from_fld);
  
	yaxis_h_layout_1->addStretch(1);
	
	QLabel* yaxis_to_label=new QLabel("To: ",yaxis_widget, "yaxis_to_label");
	yaxis_h_layout_1->addWidget(yaxis_to_label);
	
	_yaxis_to_fld=new QguWLinput(yaxis_widget, "xaxis_from");
	_yaxis_to_fld->acceptNumber=true;
	_yaxis_to_fld->setMinimumSize(axis_w, axis_h);
	yaxis_h_layout_1->addWidget(_yaxis_to_fld);
  
	QBoxLayout *yaxis_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
  _yaxis_save_fld=new QLabel("Save as", yaxis_widget,"yaxis_save_fld");
  yaxis_h_layout_2->addWidget(_yaxis_save_fld);
  
  _yaxis_name_fld=new QguWLinput(yaxis_widget,"y_data");
  _yaxis_name_fld->setMinimumWidth(240);
  yaxis_h_layout_2->addWidget(_yaxis_name_fld);
  _yaxis_name_fld->acceptObject=true;
  _yaxis_name_fld->acceptText=true;
  _yaxis_name_fld->setText("Y_Range");
  
  _yaxis_overwrite_fld=new QCheckBox("Overwrite", yaxis_widget,"yaxis_overwrite_fld");
  _yaxis_overwrite_fld->setChecked(true);
  yaxis_h_layout_2->addWidget(_yaxis_overwrite_fld);
 
  QBoxLayout *yaxis_h_layout_3=new QBoxLayout(QBoxLayout::LeftToRight);
  _saveY = new QPushButton( "Save Y Range", yaxis_widget, "saveY" );
  connect( _saveY, SIGNAL(clicked()), this, SLOT(saveY()) );
  _saveY->setFixedWidth(300);
  yaxis_h_layout_3->addWidget(_saveY);

  yaxis_v_layout->addLayout(yaxis_h_layout_1);
  yaxis_v_layout->addLayout(yaxis_h_layout_2);
  yaxis_v_layout->addLayout(yaxis_h_layout_3);

  value_return_layout->addWidget(yaxis_groupbox);

  QGroupBox *replot_groupbox=new QGroupBox(1, Qt::Vertical, this,"replot_groupbox");
  replot_groupbox->setTitle("Apply to plot");
  
  QWidget* replot_widget=new QWidget(replot_groupbox, "replot_widget");

  QBoxLayout *replot_v_layout=new QBoxLayout(replot_widget, QBoxLayout::TopToBottom); 

  replot_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
  replot_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

  QBoxLayout *replot_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
  _xaxis_rescale_fld=new QCheckBox("Rescale X,T axis", replot_widget,"xaxis_rescale_fld");
  _xaxis_rescale_fld->setChecked(true);
  replot_h_layout_1->addWidget(_xaxis_rescale_fld);
  
  _yaxis_rescale_fld=new QCheckBox("Rescale Y axis", replot_widget,"yaxis_rescale_fld");
  _yaxis_rescale_fld->setChecked(true);
  replot_h_layout_1->addWidget(_yaxis_rescale_fld);
  
  QBoxLayout *replot_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
  _rePlot = new QPushButton( "RePlot", replot_widget, "rePlot" );
  connect( _rePlot, SIGNAL(clicked()), this, SLOT(rePlot()) );
  replot_h_layout_2->addWidget(_rePlot);

  _reset = new QPushButton( "Undo", replot_widget, "reset" );
  connect( _reset, SIGNAL(clicked()), this, SLOT(resetValues()) );
  replot_h_layout_2->addWidget(_reset);

  replot_v_layout->addLayout(replot_h_layout_1);
  replot_v_layout->addLayout(replot_h_layout_2);

  value_return_layout->addWidget(replot_groupbox);

  message_lbl = new QLabel("", this,"message_lbl");
  value_return_layout->addWidget(message_lbl);

  connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));  


  _page_item=page;
  _frame_item = 0;
  _panel_item = 0;
   
  // disable reset and apply push buttons
  _reset->setEnabled(false);
  _rePlot->setEnabled(false);
  _saveXT->setEnabled(false);
  _saveY->setEnabled(false);
  	
	connect(tabwidget, SIGNAL(currentChanged(QWidget*)), this, SLOT(catchValueRet(QWidget*)) );
}
#else

QpValueReturnTab::QpValueReturnTab(QTabWidget *tabwidget, QString name, QpPageItem *page):
		QpTab(tabwidget, name, QP_VALUE_TAB)
{
  // Constructor
     
	QBoxLayout * value_return_layout=new QBoxLayout(QBoxLayout::TopToBottom, this);
	
	QGroupBox *getvalue_groupbox=new QGroupBox(this);
	getvalue_groupbox->setTitle("Get Values from Plot");
  
// 	QWidget* getvalue_widget=new QWidget(getvalue_groupbox, "getvalue_widget");

	QBoxLayout * getvalue_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	getvalue_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	getvalue_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout * getvalue_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QPushButton *get_values_pb=new QPushButton("Get Values...");
	get_values_pb->setFixedWidth(300);
	getvalue_h_layout_1->addWidget(get_values_pb);
  
	QBoxLayout * getvalue_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
  
	_xaxis_swap_fld=new QCheckBox("Preserve X,T direction");
	_xaxis_swap_fld->setChecked(true);
	getvalue_h_layout_2->addWidget(_xaxis_swap_fld);
  
	_yaxis_swap_fld=new QCheckBox("Preserve Y direction");
	_yaxis_swap_fld->setChecked(true);
	getvalue_h_layout_2->addWidget(_yaxis_swap_fld);
  
	getvalue_v_layout->addLayout(getvalue_h_layout_1);
	getvalue_v_layout->addLayout(getvalue_h_layout_2);
    
	getvalue_groupbox->setLayout(getvalue_v_layout);
	
	value_return_layout->addWidget(getvalue_groupbox);
  
	QGroupBox *xaxis_groupbox=new QGroupBox(this);
	xaxis_groupbox->setTitle("Horizontal Axis");
  
// 	QWidget* xaxis_widget=new QWidget(xaxis_groupbox, "xaxis_widget");

	QBoxLayout *xaxis_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	xaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	xaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout *xaxis_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QLabel* xaxis_from_label=new QLabel("From: ");
	xaxis_h_layout_1->addWidget(xaxis_from_label);
	
	_xaxis_from_fld=new QguWLinput(xaxis_groupbox, "xaxis_from");
	QFontMetrics axis_fm(_xaxis_from_fld->font());
	int axis_w = axis_fm.width("YYYYMMDD HH:MM:SS.SSS XX");
	int axis_h = axis_fm.height();
	_xaxis_from_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_xaxis_from_fld);
	
	_taxis_from_fld=new QbDateTimeWidget(xaxis_groupbox);
	_taxis_from_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_taxis_from_fld);
	_taxis_from_fld->hide();
  
	xaxis_h_layout_1->addStretch(1);
	
	QLabel* xaxis_to_label=new QLabel("To: ");
	xaxis_h_layout_1->addWidget(xaxis_to_label);
	
	_xaxis_to_fld=new QguWLinput(xaxis_groupbox, "xaxis_from");
	_xaxis_to_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_xaxis_to_fld);
	
	_taxis_to_fld=new QbDateTimeWidget(xaxis_groupbox);
	_taxis_to_fld->setMinimumSize(axis_w, axis_h);
	xaxis_h_layout_1->addWidget(_taxis_to_fld);
	_taxis_to_fld->hide();

	QBoxLayout *xaxis_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
	_xaxis_save_fld=new QLabel("Save as");
	xaxis_h_layout_2->addWidget(_xaxis_save_fld);
  
	_xaxis_name_fld=new QguWLinput(xaxis_groupbox,"x_data");
	_xaxis_name_fld->setMinimumWidth(240);
	xaxis_h_layout_2->addWidget(_xaxis_name_fld);
	_xaxis_name_fld->acceptObject=true;
	_xaxis_name_fld->acceptText=true;
	_xaxis_name_fld->setText("Interval");

	_xaxis_overwrite_fld=new QCheckBox("Overwrite");
	_xaxis_overwrite_fld->setChecked(true);
	xaxis_h_layout_2->addWidget(_xaxis_overwrite_fld);
   
	QBoxLayout *xaxis_h_layout_3=new QBoxLayout(QBoxLayout::LeftToRight);
	_saveXT = new QPushButton( "Save X T Interval");
	connect( _saveXT, SIGNAL(clicked()), this, SLOT(saveXT()) );
	_saveXT->setFixedWidth(300);
	xaxis_h_layout_3->addWidget(_saveXT);
  
	xaxis_v_layout->addLayout(xaxis_h_layout_1);
	xaxis_v_layout->addLayout(xaxis_h_layout_2);
	xaxis_v_layout->addLayout(xaxis_h_layout_3);

	xaxis_groupbox->setLayout(xaxis_v_layout);
	
	value_return_layout->addWidget(xaxis_groupbox);

	QGroupBox *yaxis_groupbox=new QGroupBox(this);
	yaxis_groupbox->setTitle("Y Axis");
  
// 	QWidget* yaxis_widget=new QWidget(yaxis_groupbox, "yaxis_widget");

	QBoxLayout *yaxis_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	yaxis_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	yaxis_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout *yaxis_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QLabel* yaxis_from_label=new QLabel("From: ");
	yaxis_h_layout_1->addWidget(yaxis_from_label);
	
	_yaxis_from_fld=new QguWLinput(yaxis_groupbox, "yaxis_from");
	_yaxis_from_fld->acceptNumber=true;
	_yaxis_from_fld->setMinimumSize(axis_w, axis_h);
	yaxis_h_layout_1->addWidget(_yaxis_from_fld);
  
	yaxis_h_layout_1->addStretch(1);
	
	QLabel* yaxis_to_label=new QLabel("To: ");
	yaxis_h_layout_1->addWidget(yaxis_to_label);
	
	_yaxis_to_fld=new QguWLinput(yaxis_groupbox, "xaxis_from");
	_yaxis_to_fld->acceptNumber=true;
	_yaxis_to_fld->setMinimumSize(axis_w, axis_h);
	yaxis_h_layout_1->addWidget(_yaxis_to_fld);
  
	QBoxLayout *yaxis_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
	_yaxis_save_fld=new QLabel("Save as");
	yaxis_h_layout_2->addWidget(_yaxis_save_fld);
  
	_yaxis_name_fld=new QguWLinput(yaxis_groupbox,"y_data");
	_yaxis_name_fld->setMinimumWidth(240);
	yaxis_h_layout_2->addWidget(_yaxis_name_fld);
	_yaxis_name_fld->acceptObject=true;
	_yaxis_name_fld->acceptText=true;
	_yaxis_name_fld->setText("Y_Range");
  
	_yaxis_overwrite_fld=new QCheckBox("Overwrite");
	_yaxis_overwrite_fld->setChecked(true);
	yaxis_h_layout_2->addWidget(_yaxis_overwrite_fld);
 
	QBoxLayout *yaxis_h_layout_3=new QBoxLayout(QBoxLayout::LeftToRight);
	_saveY = new QPushButton( "Save Y Range");
	connect( _saveY, SIGNAL(clicked()), this, SLOT(saveY()) );
	_saveY->setFixedWidth(300);
	yaxis_h_layout_3->addWidget(_saveY);

	yaxis_v_layout->addLayout(yaxis_h_layout_1);
	yaxis_v_layout->addLayout(yaxis_h_layout_2);
	yaxis_v_layout->addLayout(yaxis_h_layout_3);

	yaxis_groupbox->setLayout(yaxis_v_layout);
	
	value_return_layout->addWidget(yaxis_groupbox);

	QGroupBox *replot_groupbox=new QGroupBox(this);
	replot_groupbox->setTitle("Apply to plot");
  
// 	QWidget* replot_widget=new QWidget(replot_groupbox, "replot_widget");

	QBoxLayout *replot_v_layout=new QBoxLayout(QBoxLayout::TopToBottom);

	replot_v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	replot_v_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QBoxLayout *replot_h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	_xaxis_rescale_fld=new QCheckBox("Rescale X,T axis");
	_xaxis_rescale_fld->setChecked(true);
	replot_h_layout_1->addWidget(_xaxis_rescale_fld);
  
	_yaxis_rescale_fld=new QCheckBox("Rescale Y axis");
	_yaxis_rescale_fld->setChecked(true);
	replot_h_layout_1->addWidget(_yaxis_rescale_fld);
  
	QBoxLayout *replot_h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
	_rePlot = new QPushButton( "RePlot");
	connect( _rePlot, SIGNAL(clicked()), this, SLOT(rePlot()) );
	replot_h_layout_2->addWidget(_rePlot);

	_reset = new QPushButton( "Undo");
	connect( _reset, SIGNAL(clicked()), this, SLOT(resetValues()) );
	replot_h_layout_2->addWidget(_reset);

	replot_v_layout->addLayout(replot_h_layout_1);
	replot_v_layout->addLayout(replot_h_layout_2);

	replot_groupbox->setLayout(replot_v_layout);
			
	value_return_layout->addWidget(replot_groupbox);

	message_lbl = new QLabel("", this,"message_lbl");
	value_return_layout->addWidget(message_lbl);

	connect(get_values_pb, SIGNAL(clicked()), this, SLOT(getValues()));  


	_page_item=page;
	_frame_item = 0;
	_panel_item = 0;
   
  // disable reset and apply push buttons
	_reset->setEnabled(false);
	_rePlot->setEnabled(false);
	_saveXT->setEnabled(false);
	_saveY->setEnabled(false);
  	
	connect(tabwidget, SIGNAL(currentChanged(QWidget*)), this, SLOT(catchValueRet(QWidget*)) );
}
#endif

// ----------------------------------

void QpValueReturnTab::catchValueRet(QWidget* tab)
{
	if (tab == this) _page_item->I_ListView->clearSelectedList(); 
}

// ----------------------------------

void QpValueReturnTab::saveXT()
{
	// Slot called when saveXT push button clicked

	if(QuSlotDebug()) cout << "QpValueReturnTab::saveXT" << endl;
	
	message_lbl->setText("");
			
   // Save X Axis Object?
   if( _frame_item)
   {
      // fetch name of object
      QString x_data = _xaxis_name_fld->text();
      // check not empty
      if(!x_data.isNull() && !x_data.isEmpty())
      {
        QdObject_var x_obj;
        if( _frame_item->frameType() == QP_XY )
        {
           // create an object to store axis ranges
	       QdRMatrix_var x_mat=new QdRMatrix(Dim(2));
	       (*x_mat)[0]=_xaxis_from_fld->value();
	       (*x_mat)[1]=_xaxis_to_fld->value();
	       x_obj=x_mat;
	       // Save to WL
	      if(_xaxis_overwrite_fld->isChecked()) x_data = QwuiPutOnWL(x_obj, x_data, "Constants", true);
	      else x_data = QwuiPutOnWL(x_obj, x_data, "Constants");
	      // get name actually saved on WL
		  if(!x_data.isNull() && !x_data.isEmpty())
          {
	         Name x_name(string(x_data.latin1()));
             x_name.delete_component(0);
             x_data=(x_name.srep()).c_str();
	      }
        }
        else if(_frame_item->frameType() == QP_TS )
        {
          Time from;
	      from.set_time_rep(_taxis_from_fld->value());
	      Time to;
	      to.set_time_rep(_taxis_to_fld->value());
	      // create time interval object to store time axis range
          QdTimeInterval_var x_tint= new QdTimeInterval(from, to);
          x_obj=x_tint;
	      // save to WL
	      if(_xaxis_overwrite_fld->isChecked()) x_data = QwuiPutOnWL(x_obj, x_data, "Time Intervals", true);
	      else x_data = QwuiPutOnWL(x_obj, x_data, "Time Intervals");
	      // get name of object actually saved on WL
          if(!x_data.isNull() && !x_data.isEmpty())
          {
	        Name x_name(string(x_data.latin1()));
            x_name.delete_component(0);
            x_data=(x_name.srep()).c_str();
	      }
        }
      }
      // refresh field with name of saved object
      if(!x_data.isNull() && !x_data.isEmpty()) _xaxis_name_fld->setText(x_data);
      else
      {
        (void)QMessageBox::warning( this, "Value Return", "Horizontal Axis Object not saved: no name given");
      }
   }
   
}

// ----------------------------------

void QpValueReturnTab::saveY()
{
	// Slot called when saveY push button clicked

	if(QuSlotDebug()) cout << "QpValueReturnTab::saveY" << endl;
	
	message_lbl->setText("");
			
   // Save Y Object ?
   if( _panel_item)
   {
      // fetch name
      QString y_data = _yaxis_name_fld->text();
      // check name nit empty
      if(!y_data.isNull() && !y_data.isEmpty())
      {
        // create an object for storing axis range
        QdObject_var y_obj;
       
	    QdRMatrix_var y_mat=new QdRMatrix(Dim(2));
		(*y_mat)[0]=_yaxis_from_fld->value();
		(*y_mat)[1]=_yaxis_to_fld->value();
		y_obj=y_mat;
		// save to WL
		if(_yaxis_overwrite_fld->isChecked()) y_data = QwuiPutOnWL(y_obj,y_data, "Constants", true);
		else y_data=QwuiPutOnWL(y_obj,y_data,"Constants");
		// get name actually saved on WL
		if(!y_data.isNull() && !y_data.isEmpty())
		{
			Name y_name(string(y_data.latin1()));
			y_name.delete_component(0);
			y_data=(y_name.srep()).c_str();
		}
        
      }
     // refresh field with name of saved object
     if(!y_data.isNull() && !y_data.isEmpty())
        _yaxis_name_fld->setText(y_data);
     else
     {
       (void)QMessageBox::warning(this,"Value Return","Y Axis Object not saved: no name given");
     }
   }

}
// ----------------------------------

void QpValueReturnTab::rePlot()
{
	// Slot called when rePlot push button clicked
	// If items are selected after values returned, then apply propogates down to them.

	if(QuSlotDebug()) cout << "QpValueReturnTab::rePlot" << endl;
	
	if( !_page_item || !_page_item->I_ListView->findPItem(_frame_item) || !_page_item->I_ListView->findPItem(_panel_item))
	{
		message_lbl->setText("Warning Frame or Panel no longer exists! Select new values.");
		return;
	}
	message_lbl->setText("");
	
	// make frame and panel lists and include panel used in selection
		
	vector <QpPanelItem*> panelList;
	vector <QpFrameItem*> frameList;
	
	// go through selected frames and panels excluding that used in selection
	for (int i=0; i<_page_item->I_ListView->localList.size(); i++)
	{
		QpFrameItem * frameItem = dynamic_cast<QpFrameItem*>(_page_item->I_ListView->localList[i]);
		if(frameItem && frameItem != _frame_item) frameList.push_back(frameItem);

		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(_page_item->I_ListView->localList[i]);
		if(panelItem && panelItem != _panel_item) panelList.push_back(panelItem);
	}
	
	// add frame and panel used in selection
	if( _xaxis_rescale_fld->isChecked()  ) frameList.push_back(_frame_item);
			
	if( _yaxis_rescale_fld->isChecked() )  panelList.push_back(_panel_item);

	// update frames
	if( _xaxis_rescale_fld->isChecked() ) 
	{
		num_old_frames.push_back(frameList.size());
		
		for(int i=0; i<frameList.size(); i++)
		{
			QpFrameItem *frameItem = frameList[i];
			
			// store original values on end of undo stack
			_frame_item_old.push_back(frameItem);

			if(frameItem->frameType() == QP_XY)
			{
				// XY Frame
				_x_from_old.push_back(frameItem->xAxisTab->_axis_from_xy_fld->value());
				_x_to_old.push_back(frameItem->xAxisTab->_axis_to_xy_fld->value());
				_frame_autoscale_old.push_back(frameItem->xAxisTab->_axis_autoscale_fld->currentItem());
				
				// replace with new values
				frameItem->xAxisTab->_axis_from_xy_fld->setDouble(_xaxis_from_fld->value());
				frameItem->xAxisTab->setAxisMin();
			
				frameItem->xAxisTab->_axis_to_xy_fld->setDouble(_xaxis_to_fld->value());
				frameItem->xAxisTab->setAxisMax();
				
				frameItem->xAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);
			}
			else
			{
				// TS Frame
				_x_from_old.push_back(frameItem->tAxisTab->_axis_from_time_fld->value());
				_x_to_old.push_back(frameItem->tAxisTab->_axis_to_time_fld->value());
				_frame_autoscale_old.push_back(frameItem->tAxisTab->_axis_autoscale_fld->currentItem());
				
				// replace with new values
				frameItem->tAxisTab->_axis_from_time_fld->setDouble(_taxis_from_fld->value());
				frameItem->tAxisTab->setAxisMin();			
				
				frameItem->tAxisTab->_axis_to_time_fld->setDouble(_taxis_to_fld->value());
				frameItem->tAxisTab->setAxisMax();
				
				frameItem->tAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

			}
		}
	}
	else 		num_old_frames.push_back(0);

	// update panels
	if( _yaxis_rescale_fld->isChecked() )
	{
		num_old_panels.push_back(panelList.size());

		for(int i=0; i<panelList.size(); i++)
		{
			QpPanelItem *panelItem = panelList[i];

			_y_from_old.push_back(panelItem->yAxisTab->_axis_from_time_fld->value());
			_y_to_old.push_back(panelItem->yAxisTab->_axis_to_time_fld->value());
			_panel_item_old.push_back(panelItem);
			_panel_autoscale_old.push_back(panelItem->yAxisTab->_axis_autoscale_fld->currentItem());
					
			// replace with new values
			panelItem->yAxisTab->_axis_from_xy_fld->setDouble(_yaxis_from_fld->value());
			panelItem->yAxisTab->setAxisMin();
			
			panelItem->yAxisTab->_axis_to_xy_fld->setDouble(_yaxis_to_fld->value());
			panelItem->yAxisTab->setAxisMax();
			panelItem->yAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_USER);

		}
	}
	else 	num_old_panels.push_back(0);

   // enable reset push button
   _reset->setEnabled(true);

	_page_item->I_ListView->plot();
   
}

void QpValueReturnTab::getValues()
{
	if(QuSlotDebug()) cout << "QpValueReturnTab::getValues" << endl;

      // check a plot page is currently selected
    if(!_page_item)
    {
        (void)QMessageBox::warning(this,"Value Return","No page selected!");
        return;
    }
    
      // clear listview selection to avoid user accidentally applying changes
  // to the wrong panel. Selection is from the plot.
    _page_item->I_ListView->localList.clear();
    _page_item->I_ListView->clearSelection();
    
      // check that the currently selected plot x window is open
    int plot_id=_page_item->PageTab->_xwin_num_fld->text().toInt();
    if(_page_item->PlotWindows.empty())
    {
		_page_item->I_ListView->setCurrentItem(_page_item);
	    _page_item->I_ListView->plotCurrentItem();
		plot_id=_page_item->PageTab->_xwin_num_fld->text().toInt();
    }
    
    PlotWindow* win=_page_item->focusWindow(plot_id);
    if(win==NULL)
    {
	    (void)QMessageBox::warning( this,"Value Return", "Error! Window id not found.");
	    return;
    }

    double x_start, y_start, x_end, y_end;
        win->plot->captureMouse(&x_start, &y_start, &x_end, &y_end);

	y_start=1.-y_start;
	y_end=1.-y_end;
    
  // swap range ends?
	if(x_end < x_start && _xaxis_swap_fld->isChecked())
	{
		double x_tmp=x_start;
		x_start=x_end;
		x_end=x_tmp;
	}
  
	if(y_end < y_start && _yaxis_swap_fld->isChecked())
	{
		double y_tmp=y_start;
		y_start=y_end;
		y_end=y_tmp;
	}
  
  // descend through frames in selected page to find frames
  // lying within range selected interactively
   	
	QpFrameItem *frameItem = dynamic_cast<QpFrameItem*>(_page_item->firstChild());
	_panel_item = 0;
	int nPanel=0;
	while(frameItem)
	{
		int numPanels = frameItem->numberOfChildren();
		float panelHt = frameItem->PropertiesTab->_height_fld->value() / numPanels;
		int num = 0;
		bool hasSpectrum = _page_item->containsSpectrum();
		double factor=(hasSpectrum?0.9:1.);
		QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(frameItem->firstChild());
		while(panelItem)
		{
		// check if selected start points lies within this panel
	    // if so this is selected panel so break out of loop
		// x-axis is on the frame and y-axis on the panel
			if(frameItem->PropertiesTab->_loffset_fld->value() <= x_start &&  
				x_start <= frameItem->PropertiesTab->_loffset_fld->value() + frameItem->PropertiesTab->_width_fld->value()*factor && 
				1. - frameItem->PropertiesTab->_toffset_fld->value() - num * panelHt >= y_start &&
				y_start >= 1. - frameItem->PropertiesTab->_toffset_fld->value() - (num+1) * panelHt)
			{
				_panel_item = panelItem;
				nPanel = num;
				break;
			}
			num++;
			panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
		}	    
		if(_panel_item) break; // panel has been found
		frameItem = dynamic_cast<QpFrameItem*>(frameItem->nextSibling());
	}
	_frame_item = frameItem;
  
  
  // Do we have a panel?
	if(!_panel_item)
	{
		(void)QMessageBox::warning(this, "Value Return", "Error! No panel selected");
		return;
	}
 
	bool hasSpectrum = _page_item->containsSpectrum();
	double factor=(hasSpectrum?0.9:1.);
  // check selected END points also lie within this frame 
	if(x_end > _frame_item->PropertiesTab->_loffset_fld->value() + _frame_item->PropertiesTab->_width_fld->value()*factor || 
		  x_end < _frame_item->PropertiesTab->_loffset_fld->value() || 
		  y_end > 1. - _frame_item->PropertiesTab->_toffset_fld->value() || 
		  y_end < 1. - _frame_item->PropertiesTab->_toffset_fld->value() - _frame_item->PropertiesTab->_height_fld->value())
	{
		_panel_item=0;
		_frame_item=0;
		(void)QMessageBox::warning(this, "Value Return", "Error! Start and finish in different frames");
		return;
	}
   

	double x_offset = _frame_item->PropertiesTab->_loffset_fld->value();
	double x_sz = _frame_item->PropertiesTab->_width_fld->value()*factor;
	double y_sz = _frame_item->PropertiesTab->_height_fld->value();
	y_sz /= _frame_item->numberOfChildren();
	double y_offset = 1. - _frame_item->PropertiesTab->_toffset_fld->value() - ( nPanel + 1 ) * y_sz;

  
  // calculate new x axis ranges from plot selection coordinates 
	if(_frame_item->frameType() == QP_XY)
	{
	
// 		_xaxis_from_fld->acceptTime=false;
// 		_xaxis_from_fld->acceptNumber=true;
// 		_xaxis_to_fld->acceptTime=false;
// 		_xaxis_to_fld->acceptNumber=true;
		_xaxis_from_fld->show();
		_taxis_from_fld->hide();
		_xaxis_to_fld->show();
		_taxis_to_fld->hide();

		double xmin = _frame_item->xAxisTab->_axis_from_xy_fld->value();
		double xmax = _frame_item->xAxisTab->_axis_to_xy_fld->value();
	
    // if logarithmic axis, convert to logs first
		if(_frame_item->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			xmin = SafeLog10(xmin);
			xmax = SafeLog10(xmax);
		}
	
		double x_range = xmax - xmin;
		_x_from = xmin + x_range * (x_start-x_offset)/x_sz;
		_x_to = xmin + x_range * (x_end-x_offset)/x_sz;
		if(_frame_item->xAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			_x_from=pow(10., _x_from);
			_x_to=pow(10., _x_to);
		}	    
		_xaxis_from_fld->setText(QString("%1").arg(_x_from));
		_xaxis_to_fld->setText(QString("%1").arg(_x_to));
	}
	else if(_frame_item->frameType() == QP_TS)
	{
// 		_xaxis_from_fld->acceptTime=true;
// 		_xaxis_from_fld->acceptNumber=false;
// 		_xaxis_to_fld->acceptTime=true;
// 		_xaxis_to_fld->acceptNumber=false;
		_xaxis_from_fld->hide();
		_taxis_from_fld->show();
		_xaxis_to_fld->hide();
		_taxis_to_fld->show();

		double tmin = _frame_item->tAxisTab->_axis_from_time_fld->value();
		double tmax = _frame_item->tAxisTab->_axis_to_time_fld->value();
		double t_range = tmax - tmin;
		_x_from = tmin + t_range * (x_start-x_offset)/x_sz;
		_x_to = tmin + t_range * (x_end-x_offset)/x_sz;
   
   	
		Seconds tFrom(_x_from);
		Seconds tTo(_x_to);
// 		Time tFrom(_x_from);
// 		Time tTo(_x_to);
		_taxis_from_fld->setTime(tFrom/*.iso_srep().c_str()*/);
		_taxis_to_fld->setTime(tTo/*.iso_srep().c_str()*/);

	} 
  
  // calculate new y axis ranges
	if(_panel_item->yAxisTab)
	{
		double ymin = _panel_item->yAxisTab->_axis_from_xy_fld->value();
		double ymax = _panel_item->yAxisTab->_axis_to_xy_fld->value();
    // if logarithmic axis convert to logs first
		if(_panel_item->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			ymin = SafeLog10(ymin);
			ymax = SafeLog10(ymax);
		}
		double y_range = ymax - ymin;
		_y_from = ymin + y_range*(y_start-y_offset)/y_sz;
		_y_to = ymin + y_range*(y_end-y_offset)/y_sz;
		if(_panel_item->yAxisTab->_axis_linlog_fld->currentItem() == QP_LOG)
		{
			_y_from = pow(10., _y_from);
			_y_to = pow(10., _y_to);
		}	    
		_yaxis_from_fld->setText(QString("%1").arg(_y_from));
		_yaxis_to_fld->setText(QString("%1").arg(_y_to));
	}
	
	// enable Apply push buttons
	_rePlot->setEnabled(true);
    _saveXT->setEnabled(true);
    _saveY->setEnabled(true);
	_panel_item->I_ListView->setCurrentItem((QguListItem*)_panel_item);
	_panel_item->I_ListView->raisePlotWindow();
    
    return;
} 

void QpValueReturnTab::resetValues()
{
	// slot called when undo button pushed, resets ranges to their last values 

	if(QuSlotDebug())  cout << "QpValueReturnTab::resetValues" << endl;
    
	if( !_page_item) {
		message_lbl->setText("No page available");
		return;
	}
	
	message_lbl->setText("");

	// undo frame changes
	for(int i=0; i<num_old_frames.back(); i++)
	{
		QpFrameItem *frameItem = _frame_item_old.back();
		
		if( _page_item->I_ListView->findPItem(frameItem) )
		{

			if(frameItem->frameType() == QP_XY)
			{
				// restore values
				frameItem->xAxisTab->_axis_from_xy_fld->setDouble(_x_from_old.back());
				frameItem->xAxisTab->_axis_to_xy_fld->setDouble(_x_to_old.back());      
				frameItem->xAxisTab->_axis_autoscale_fld->setCurrentItem(_frame_autoscale_old.back());
			}
			else if(_frame_item && _frame_item->frameType() == QP_TS)
			{
				// restore values
				frameItem->tAxisTab->_axis_from_time_fld->setDouble(_x_from_old.back());
				frameItem->tAxisTab->_axis_to_time_fld->setDouble(_x_to_old.back());	
				frameItem->tAxisTab->_axis_autoscale_fld->setCurrentItem(_frame_autoscale_old.back());
			}
		}
		else message_lbl->setText("Cannot Undo, frame deleted");

		_frame_item_old.pop_back();
		_x_from_old.pop_back();
		_x_to_old.pop_back();
		_frame_autoscale_old.pop_back();
	}
	num_old_frames.pop_back();

	// undo panel changes
	for(int i=0; i<num_old_panels.back(); i++)
	{
		QpPanelItem *panelItem = _panel_item_old.back();
		
		if( _page_item->I_ListView->findPItem(panelItem) )
		{
			// restore values
			panelItem->yAxisTab->_axis_from_xy_fld->setDouble(_y_from_old.back());
			panelItem->yAxisTab->_axis_to_xy_fld->setDouble(_y_to_old.back());
			panelItem->yAxisTab->_axis_autoscale_fld->setCurrentItem(_panel_autoscale_old.back());
		
		}
		else 	message_lbl->setText("Cannot Undo, panel deleted");

		_panel_item_old.pop_back();
		_y_from_old.pop_back();
		_y_to_old.pop_back();
		_panel_autoscale_old.pop_back();
		
	}
	num_old_panels.pop_back();

	// plot
	_page_item->I_ListView->plot();
		

	// disable reset
	if (num_old_frames.size() <= 0) _reset->setEnabled(false);

}  

