#include <stdio.h>
#include "qsw_helpwindow.h"
#include "qsas_version.h"

#include <qpixmap.h>
#include <qmainwindow.h>
#include <qtextbrowser.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qmenubar.h>

/* XPM */
static const char * p1_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #000000000000",
"X	c #FFFFFFFF0000",
"                ",
"                ",
"              . ",
"          .X .. ",
"         .XX... ",
"       .XXXX... ",
"     .XXXXXX... ",
"   .XXXXXXXXXX. ",
" .XXXXXXXXXXXX. ",
"   .XXXXXXXXXX. ",
"     .XXXXXX... ",
"       .XXXX... ",
"         .XX... ",
"          .X .. ",
"              . ",
"                "};

static const char * p2_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #000000000000",
"X	c #FFFFFFFF0000",
"                ",
"                ",
"       ..       ",
"      .XX.      ",
"     .XXXX.     ",
"    .XXXXXX.    ",
"   .XXXXXXXX.   ",
"  .XXXXXXXXXX.  ",
" .XXXXXXXXXXXX. ",
".XXXXXXXXXXXXXX.",
"  ....XXXX....  ",
" .....XXXX..... ",
"................",
"                ",
"                ",
"                "};

QswHelpWindow::QswHelpWindow( QWidget *parent, const char *name )
    : QMainWindow( parent, name )
{
// popups for menu bar
  
  QPopupMenu* qpum_file_popup = new QPopupMenu( this );
  qpum_file_popup->insertItem("&Close", this, SLOT(hide()) );
  
  QPopupMenu* qpum_help_popup = new QPopupMenu( this );
  qpum_help_popup->insertItem("&About",  this, SLOT(about()), CTRL+Key_H );

// set up the menu bar

  QMenuBar* qmb_menu_bar = this->menuBar();
  
  qmb_menu_bar->insertItem( "&File", qpum_file_popup );
  qmb_menu_bar->insertSeparator();
  qmb_menu_bar->insertItem( "&Help", qpum_help_popup );

  QToolBar * tool_bar = new QToolBar( this, "Navigation Tool Bar" );

  QPixmap back_icon( p1_xpm);
  QToolButton * back_button
	= new QToolButton( back_icon, "Back", QString::null,
			   this, SLOT(back()), tool_bar, "back_button" );

  tool_bar->addSeparator();
  
  QPixmap idx_icon( p2_xpm);
  QToolButton * idx_button
	= new QToolButton( idx_icon, "Index", QString::null,
			   this, SLOT(goIndex()), tool_bar, "idx_button" );
   
// Text browser for simple html help files

  help_browser = new QTextBrowser( this, "help_browser");

// also root directory stuff

  const char * home = getenv("QSAS_HOME");
  if(home==NULL) help_root_dir = "doc/html/"; // Safe: QTextBrowser warns user
  else help_root_dir = string(home)+"/doc/html/";

  this->setCentralWidget( help_browser );
  
  resize( 800, 600 );
}

void QswHelpWindow::about()
{
string about_txt("About QSAS version " + string(QSAS_VERSION_ID));
      about_txt+="\n\nThe QM Space Analysis System delivers a "
                 "flexible data selection, manipulation and display \n"
                 "environment tailored to the needs of space physics "
                 "in general and Cluster in particular.\n\n"
                 "The QSAS team is now at Imperial College, \n"
			  "but use csc_support@qmul.ac.uk";
      QMessageBox::about(this,"About QSAS",about_txt.c_str());  
}

void QswHelpWindow::goIndex()
{ 
    manEntry("index.html");
}

void QswHelpWindow::manEntry(const string &entry)
{ 
  FILE *fp;
  fp = fopen((help_root_dir+entry).c_str(), "r");
  if ( fp == NULL ){  
    QMessageBox::information( this, "Documentation not available", (help_root_dir+entry).c_str(), "OK" );    
  }
  else{
    fclose (fp);
    help_browser->setSource( (help_root_dir+entry).c_str() );
    this->show();
  }
}


void QswHelpWindow::back()
{
  help_browser->backward();
}
